/* PublishingPluginHost.c generated by valac 0.32.1, the Vala compiler
 * generated from PublishingPluginHost.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include <glib.h>
#include <glib-object.h>
#include "shotwell-plugin-dev-1.0.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>
#include <gio/gio.h>


#define PLUGINS_TYPE_STANDARD_HOST_INTERFACE (plugins_standard_host_interface_get_type ())
#define PLUGINS_STANDARD_HOST_INTERFACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGINS_TYPE_STANDARD_HOST_INTERFACE, PluginsStandardHostInterface))
#define PLUGINS_STANDARD_HOST_INTERFACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGINS_TYPE_STANDARD_HOST_INTERFACE, PluginsStandardHostInterfaceClass))
#define PLUGINS_IS_STANDARD_HOST_INTERFACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGINS_TYPE_STANDARD_HOST_INTERFACE))
#define PLUGINS_IS_STANDARD_HOST_INTERFACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGINS_TYPE_STANDARD_HOST_INTERFACE))
#define PLUGINS_STANDARD_HOST_INTERFACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGINS_TYPE_STANDARD_HOST_INTERFACE, PluginsStandardHostInterfaceClass))

typedef struct _PluginsStandardHostInterface PluginsStandardHostInterface;
typedef struct _PluginsStandardHostInterfaceClass PluginsStandardHostInterfaceClass;
typedef struct _PluginsStandardHostInterfacePrivate PluginsStandardHostInterfacePrivate;

#define SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST (spit_publishing_concrete_publishing_host_get_type ())
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost))
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHostClass))
#define SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST))
#define SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST))
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHostClass))

typedef struct _SpitPublishingConcretePublishingHost SpitPublishingConcretePublishingHost;
typedef struct _SpitPublishingConcretePublishingHostClass SpitPublishingConcretePublishingHostClass;
typedef struct _SpitPublishingConcretePublishingHostPrivate SpitPublishingConcretePublishingHostPrivate;

#define PUBLISHING_UI_TYPE_PUBLISHING_DIALOG (publishing_ui_publishing_dialog_get_type ())
#define PUBLISHING_UI_PUBLISHING_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_PUBLISHING_DIALOG, PublishingUIPublishingDialog))
#define PUBLISHING_UI_PUBLISHING_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_PUBLISHING_DIALOG, PublishingUIPublishingDialogClass))
#define PUBLISHING_UI_IS_PUBLISHING_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_PUBLISHING_DIALOG))
#define PUBLISHING_UI_IS_PUBLISHING_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_PUBLISHING_DIALOG))
#define PUBLISHING_UI_PUBLISHING_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_PUBLISHING_DIALOG, PublishingUIPublishingDialogClass))

typedef struct _PublishingUIPublishingDialog PublishingUIPublishingDialog;
typedef struct _PublishingUIPublishingDialogClass PublishingUIPublishingDialogClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER (publishing_glue_media_source_publishable_wrapper_get_type ())
#define PUBLISHING_GLUE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER, PublishingGlueMediaSourcePublishableWrapper))
#define PUBLISHING_GLUE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER, PublishingGlueMediaSourcePublishableWrapperClass))
#define PUBLISHING_GLUE_IS_MEDIA_SOURCE_PUBLISHABLE_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER))
#define PUBLISHING_GLUE_IS_MEDIA_SOURCE_PUBLISHABLE_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER))
#define PUBLISHING_GLUE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER, PublishingGlueMediaSourcePublishableWrapperClass))

typedef struct _PublishingGlueMediaSourcePublishableWrapper PublishingGlueMediaSourcePublishableWrapper;
typedef struct _PublishingGlueMediaSourcePublishableWrapperClass PublishingGlueMediaSourcePublishableWrapperClass;

#define PUBLISHING_UI_TYPE_CONCRETE_DIALOG_PANE (publishing_ui_concrete_dialog_pane_get_type ())
#define PUBLISHING_UI_CONCRETE_DIALOG_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_CONCRETE_DIALOG_PANE, PublishingUIConcreteDialogPane))
#define PUBLISHING_UI_CONCRETE_DIALOG_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_CONCRETE_DIALOG_PANE, PublishingUIConcreteDialogPaneClass))
#define PUBLISHING_UI_IS_CONCRETE_DIALOG_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_CONCRETE_DIALOG_PANE))
#define PUBLISHING_UI_IS_CONCRETE_DIALOG_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_CONCRETE_DIALOG_PANE))
#define PUBLISHING_UI_CONCRETE_DIALOG_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_CONCRETE_DIALOG_PANE, PublishingUIConcreteDialogPaneClass))

typedef struct _PublishingUIConcreteDialogPane PublishingUIConcreteDialogPane;
typedef struct _PublishingUIConcreteDialogPaneClass PublishingUIConcreteDialogPaneClass;

#define PUBLISHING_UI_TYPE_PROGRESS_PANE (publishing_ui_progress_pane_get_type ())
#define PUBLISHING_UI_PROGRESS_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_PROGRESS_PANE, PublishingUIProgressPane))
#define PUBLISHING_UI_PROGRESS_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_PROGRESS_PANE, PublishingUIProgressPaneClass))
#define PUBLISHING_UI_IS_PROGRESS_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_PROGRESS_PANE))
#define PUBLISHING_UI_IS_PROGRESS_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_PROGRESS_PANE))
#define PUBLISHING_UI_PROGRESS_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_PROGRESS_PANE, PublishingUIProgressPaneClass))

typedef struct _PublishingUIProgressPane PublishingUIProgressPane;
typedef struct _PublishingUIProgressPaneClass PublishingUIProgressPaneClass;
#define _g_free0(var) (var = (g_free (var), NULL))

#define PUBLISHING_UI_TYPE_STATIC_MESSAGE_PANE (publishing_ui_static_message_pane_get_type ())
#define PUBLISHING_UI_STATIC_MESSAGE_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_STATIC_MESSAGE_PANE, PublishingUIStaticMessagePane))
#define PUBLISHING_UI_STATIC_MESSAGE_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_STATIC_MESSAGE_PANE, PublishingUIStaticMessagePaneClass))
#define PUBLISHING_UI_IS_STATIC_MESSAGE_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_STATIC_MESSAGE_PANE))
#define PUBLISHING_UI_IS_STATIC_MESSAGE_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_STATIC_MESSAGE_PANE))
#define PUBLISHING_UI_STATIC_MESSAGE_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_STATIC_MESSAGE_PANE, PublishingUIStaticMessagePaneClass))

typedef struct _PublishingUIStaticMessagePane PublishingUIStaticMessagePane;
typedef struct _PublishingUIStaticMessagePaneClass PublishingUIStaticMessagePaneClass;

#define PUBLISHING_UI_TYPE_SUCCESS_PANE (publishing_ui_success_pane_get_type ())
#define PUBLISHING_UI_SUCCESS_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_SUCCESS_PANE, PublishingUISuccessPane))
#define PUBLISHING_UI_SUCCESS_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_SUCCESS_PANE, PublishingUISuccessPaneClass))
#define PUBLISHING_UI_IS_SUCCESS_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_SUCCESS_PANE))
#define PUBLISHING_UI_IS_SUCCESS_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_SUCCESS_PANE))
#define PUBLISHING_UI_SUCCESS_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_SUCCESS_PANE, PublishingUISuccessPaneClass))

typedef struct _PublishingUISuccessPane PublishingUISuccessPane;
typedef struct _PublishingUISuccessPaneClass PublishingUISuccessPaneClass;

#define PUBLISHING_UI_TYPE_ACCOUNT_FETCH_WAIT_PANE (publishing_ui_account_fetch_wait_pane_get_type ())
#define PUBLISHING_UI_ACCOUNT_FETCH_WAIT_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_ACCOUNT_FETCH_WAIT_PANE, PublishingUIAccountFetchWaitPane))
#define PUBLISHING_UI_ACCOUNT_FETCH_WAIT_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_ACCOUNT_FETCH_WAIT_PANE, PublishingUIAccountFetchWaitPaneClass))
#define PUBLISHING_UI_IS_ACCOUNT_FETCH_WAIT_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_ACCOUNT_FETCH_WAIT_PANE))
#define PUBLISHING_UI_IS_ACCOUNT_FETCH_WAIT_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_ACCOUNT_FETCH_WAIT_PANE))
#define PUBLISHING_UI_ACCOUNT_FETCH_WAIT_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_ACCOUNT_FETCH_WAIT_PANE, PublishingUIAccountFetchWaitPaneClass))

typedef struct _PublishingUIAccountFetchWaitPane PublishingUIAccountFetchWaitPane;
typedef struct _PublishingUIAccountFetchWaitPaneClass PublishingUIAccountFetchWaitPaneClass;

#define PUBLISHING_UI_TYPE_LOGIN_WAIT_PANE (publishing_ui_login_wait_pane_get_type ())
#define PUBLISHING_UI_LOGIN_WAIT_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_LOGIN_WAIT_PANE, PublishingUILoginWaitPane))
#define PUBLISHING_UI_LOGIN_WAIT_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_LOGIN_WAIT_PANE, PublishingUILoginWaitPaneClass))
#define PUBLISHING_UI_IS_LOGIN_WAIT_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_LOGIN_WAIT_PANE))
#define PUBLISHING_UI_IS_LOGIN_WAIT_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_LOGIN_WAIT_PANE))
#define PUBLISHING_UI_LOGIN_WAIT_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_LOGIN_WAIT_PANE, PublishingUILoginWaitPaneClass))

typedef struct _PublishingUILoginWaitPane PublishingUILoginWaitPane;
typedef struct _PublishingUILoginWaitPaneClass PublishingUILoginWaitPaneClass;

#define PUBLISHING_UI_TYPE_LOGIN_WELCOME_PANE (publishing_ui_login_welcome_pane_get_type ())
#define PUBLISHING_UI_LOGIN_WELCOME_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_UI_TYPE_LOGIN_WELCOME_PANE, PublishingUILoginWelcomePane))
#define PUBLISHING_UI_LOGIN_WELCOME_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_UI_TYPE_LOGIN_WELCOME_PANE, PublishingUILoginWelcomePaneClass))
#define PUBLISHING_UI_IS_LOGIN_WELCOME_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_UI_TYPE_LOGIN_WELCOME_PANE))
#define PUBLISHING_UI_IS_LOGIN_WELCOME_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_UI_TYPE_LOGIN_WELCOME_PANE))
#define PUBLISHING_UI_LOGIN_WELCOME_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_UI_TYPE_LOGIN_WELCOME_PANE, PublishingUILoginWelcomePaneClass))

typedef struct _PublishingUILoginWelcomePane PublishingUILoginWelcomePane;
typedef struct _PublishingUILoginWelcomePaneClass PublishingUILoginWelcomePaneClass;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

struct _PluginsStandardHostInterface {
	GObject parent_instance;
	PluginsStandardHostInterfacePrivate * priv;
};

struct _PluginsStandardHostInterfaceClass {
	GObjectClass parent_class;
};

struct _SpitPublishingConcretePublishingHost {
	PluginsStandardHostInterface parent_instance;
	SpitPublishingConcretePublishingHostPrivate * priv;
};

struct _SpitPublishingConcretePublishingHostClass {
	PluginsStandardHostInterfaceClass parent_class;
};

struct _SpitPublishingConcretePublishingHostPrivate {
	PublishingUIPublishingDialog* dialog;
	SpitPublishingPublisher* active_publisher;
	SpitPublishingPublishable** publishables;
	gint publishables_length1;
	gint _publishables_size_;
	SpitPublishingLoginCallback current_login_callback;
	gpointer current_login_callback_target;
	gboolean publishing_halted;
	SpitPublishingPublisherMediaType media_type;
};


static gpointer spit_publishing_concrete_publishing_host_parent_class = NULL;
static SpitPublishingPluginHostIface* spit_publishing_concrete_publishing_host_spit_publishing_plugin_host_parent_iface = NULL;

GType plugins_standard_host_interface_get_type (void) G_GNUC_CONST;
GType spit_publishing_concrete_publishing_host_get_type (void) G_GNUC_CONST;
GType publishing_ui_publishing_dialog_get_type (void) G_GNUC_CONST;
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHostPrivate))
enum  {
	SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_DUMMY_PROPERTY
};
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_PREPARE_STATUS_DESCRIPTION _ ("Preparing for upload")
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_UPLOAD_STATUS_DESCRIPTION _ ("Uploading %d of %d")
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_STATUS_PREPARATION_FRACTION 0.3
#define SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_STATUS_UPLOAD_FRACTION 0.7
SpitPublishingConcretePublishingHost* spit_publishing_concrete_publishing_host_new (SpitPublishingService* service, PublishingUIPublishingDialog* dialog, SpitPublishingPublishable** publishables, int publishables_length1);
SpitPublishingConcretePublishingHost* spit_publishing_concrete_publishing_host_construct (GType object_type, SpitPublishingService* service, PublishingUIPublishingDialog* dialog, SpitPublishingPublishable** publishables, int publishables_length1);
PluginsStandardHostInterface* plugins_standard_host_interface_new (SpitPluggable* pluggable, const gchar* config_domain);
PluginsStandardHostInterface* plugins_standard_host_interface_construct (GType object_type, SpitPluggable* pluggable, const gchar* config_domain);
static SpitPublishingPublishable** _vala_array_dup20 (SpitPublishingPublishable** self, int length);
static void spit_publishing_concrete_publishing_host_on_login_clicked (SpitPublishingConcretePublishingHost* self);
static void spit_publishing_concrete_publishing_host_clean_up (SpitPublishingConcretePublishingHost* self);
GType publishing_glue_media_source_publishable_wrapper_get_type (void) G_GNUC_CONST;
void publishing_glue_media_source_publishable_wrapper_clean_up (PublishingGlueMediaSourcePublishableWrapper* self);
static void spit_publishing_concrete_publishing_host_report_plugin_upload_progress (SpitPublishingConcretePublishingHost* self, gint file_number, gdouble fraction_complete);
SpitPublishingDialogPane* publishing_ui_publishing_dialog_get_active_pane (PublishingUIPublishingDialog* self);
GType publishing_ui_concrete_dialog_pane_get_type (void) G_GNUC_CONST;
GType publishing_ui_progress_pane_get_type (void) G_GNUC_CONST;
void publishing_ui_progress_pane_set_status (PublishingUIProgressPane* self, const gchar* status_text, gdouble progress);
static void spit_publishing_concrete_publishing_host_install_progress_pane (SpitPublishingConcretePublishingHost* self);
PublishingUIProgressPane* publishing_ui_progress_pane_new (void);
PublishingUIProgressPane* publishing_ui_progress_pane_construct (GType object_type);
void publishing_ui_publishing_dialog_install_pane (PublishingUIPublishingDialog* self, SpitPublishingDialogPane* pane);
void spit_publishing_concrete_publishing_host_set_button_mode (SpitPublishingConcretePublishingHost* self, SpitPublishingPluginHostButtonMode mode);
static void spit_publishing_concrete_publishing_host_real_install_dialog_pane (SpitPublishingPluginHost* base, SpitPublishingDialogPane* pane, SpitPublishingPluginHostButtonMode button_mode);
static void spit_publishing_concrete_publishing_host_real_post_error (SpitPublishingPluginHost* base, GError* err);
PublishingUIStaticMessagePane* publishing_ui_static_message_pane_new (const gchar* message_string, gboolean enable_markup);
PublishingUIStaticMessagePane* publishing_ui_static_message_pane_construct (GType object_type, const gchar* message_string, gboolean enable_markup);
GType publishing_ui_static_message_pane_get_type (void) G_GNUC_CONST;
void publishing_ui_publishing_dialog_set_close_button_mode (PublishingUIPublishingDialog* self);
void publishing_ui_publishing_dialog_unlock_service (PublishingUIPublishingDialog* self);
static void spit_publishing_concrete_publishing_host_real_stop_publishing (SpitPublishingPluginHost* base);
void spit_publishing_concrete_publishing_host_start_publishing (SpitPublishingConcretePublishingHost* self);
static SpitPublishingPublisher* spit_publishing_concrete_publishing_host_real_get_publisher (SpitPublishingPluginHost* base);
static void spit_publishing_concrete_publishing_host_real_install_static_message_pane (SpitPublishingPluginHost* base, const gchar* message, SpitPublishingPluginHostButtonMode button_mode);
static void spit_publishing_concrete_publishing_host_real_install_pango_message_pane (SpitPublishingPluginHost* base, const gchar* markup, SpitPublishingPluginHostButtonMode button_mode);
static void spit_publishing_concrete_publishing_host_real_install_success_pane (SpitPublishingPluginHost* base);
PublishingUISuccessPane* publishing_ui_success_pane_new (SpitPublishingPublisherMediaType published_media, gint num_uploaded);
PublishingUISuccessPane* publishing_ui_success_pane_construct (GType object_type, SpitPublishingPublisherMediaType published_media, gint num_uploaded);
GType publishing_ui_success_pane_get_type (void) G_GNUC_CONST;
static void spit_publishing_concrete_publishing_host_real_install_account_fetch_wait_pane (SpitPublishingPluginHost* base);
PublishingUIAccountFetchWaitPane* publishing_ui_account_fetch_wait_pane_new (void);
PublishingUIAccountFetchWaitPane* publishing_ui_account_fetch_wait_pane_construct (GType object_type);
GType publishing_ui_account_fetch_wait_pane_get_type (void) G_GNUC_CONST;
static void spit_publishing_concrete_publishing_host_real_install_login_wait_pane (SpitPublishingPluginHost* base);
PublishingUILoginWaitPane* publishing_ui_login_wait_pane_new (void);
PublishingUILoginWaitPane* publishing_ui_login_wait_pane_construct (GType object_type);
GType publishing_ui_login_wait_pane_get_type (void) G_GNUC_CONST;
static void spit_publishing_concrete_publishing_host_real_install_welcome_pane (SpitPublishingPluginHost* base, const gchar* welcome_message, SpitPublishingLoginCallback login_clicked_callback, void* login_clicked_callback_target);
GType publishing_ui_login_welcome_pane_get_type (void) G_GNUC_CONST;
PublishingUILoginWelcomePane* publishing_ui_login_welcome_pane_new (const gchar* service_welcome_message);
PublishingUILoginWelcomePane* publishing_ui_login_welcome_pane_construct (GType object_type, const gchar* service_welcome_message);
static void _spit_publishing_concrete_publishing_host_on_login_clicked_publishing_ui_login_welcome_pane_login_requested (PublishingUILoginWelcomePane* _sender, gpointer self);
static void spit_publishing_concrete_publishing_host_real_set_service_locked (SpitPublishingPluginHost* base, gboolean locked);
void publishing_ui_publishing_dialog_lock_service (PublishingUIPublishingDialog* self);
void publishing_ui_publishing_dialog_set_cancel_button_mode (PublishingUIPublishingDialog* self);
static void spit_publishing_concrete_publishing_host_real_set_dialog_default_widget (SpitPublishingPluginHost* base, GtkWidget* widget);
static SpitPublishingPublisherMediaType spit_publishing_concrete_publishing_host_real_get_publishable_media_type (SpitPublishingPluginHost* base);
static SpitPublishingPublishable** spit_publishing_concrete_publishing_host_real_get_publishables (SpitPublishingPluginHost* base, int* result_length1);
static SpitPublishingPublishable** _vala_array_dup21 (SpitPublishingPublishable** self, int length);
static SpitPublishingProgressCallback spit_publishing_concrete_publishing_host_real_serialize_publishables (SpitPublishingPluginHost* base, gint content_major_axis, gboolean strip_metadata, void** result_target, GDestroyNotify* result_target_destroy_notify);
void spin_event_loop (void);
GFile* publishing_glue_media_source_publishable_wrapper_serialize_for_publishing (PublishingGlueMediaSourcePublishableWrapper* self, gint content_major_axis, gboolean strip_metadata, GError** error);
static void _spit_publishing_concrete_publishing_host_report_plugin_upload_progress_spit_publishing_progress_callback (gint file_number, gdouble fraction_complete, gpointer self);
static void spit_publishing_concrete_publishing_host_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


static gpointer _g_object_ref0 (gpointer self) {
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return self ? g_object_ref (self) : NULL;
#line 250 "PublishingPluginHost.c"
}


static SpitPublishingPublishable** _vala_array_dup20 (SpitPublishingPublishable** self, int length) {
	SpitPublishingPublishable** result;
	int i;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	result = g_new0 (SpitPublishingPublishable*, length + 1);
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	for (i = 0; i < length; i++) {
#line 261 "PublishingPluginHost.c"
		SpitPublishingPublishable* _tmp0_ = NULL;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp0_ = _g_object_ref0 (self[i]);
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		result[i] = _tmp0_;
#line 267 "PublishingPluginHost.c"
	}
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return result;
#line 271 "PublishingPluginHost.c"
}


SpitPublishingConcretePublishingHost* spit_publishing_concrete_publishing_host_construct (GType object_type, SpitPublishingService* service, PublishingUIPublishingDialog* dialog, SpitPublishingPublishable** publishables, int publishables_length1) {
	SpitPublishingConcretePublishingHost * self = NULL;
	SpitPublishingService* _tmp0_ = NULL;
	PublishingUIPublishingDialog* _tmp1_ = NULL;
	SpitPublishingPublishable** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	SpitPublishingPublishable** _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	SpitPublishingPublishable** _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
	SpitPublishingService* _tmp9_ = NULL;
	SpitPublishingPublisher* _tmp10_ = NULL;
#line 24 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_val_if_fail (SPIT_PUBLISHING_IS_SERVICE (service), NULL);
#line 24 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_val_if_fail (PUBLISHING_UI_IS_PUBLISHING_DIALOG (dialog), NULL);
#line 26 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = service;
#line 26 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = (SpitPublishingConcretePublishingHost*) plugins_standard_host_interface_construct (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_PLUGGABLE, SpitPluggable), "sharing");
#line 27 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = dialog;
#line 27 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->dialog = _tmp1_;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = publishables;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2__length1 = publishables_length1;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = (_tmp2_ != NULL) ? _vala_array_dup20 (_tmp2_, _tmp2__length1) : ((gpointer) _tmp2_);
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3__length1 = _tmp2__length1;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishables = (_vala_array_free (self->priv->publishables, self->priv->publishables_length1, (GDestroyNotify) g_object_unref), NULL);
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishables = _tmp3_;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishables_length1 = _tmp3__length1;
#line 28 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->_publishables_size_ = self->priv->publishables_length1;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = publishables;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4__length1 = publishables_length1;
#line 319 "PublishingPluginHost.c"
	{
		SpitPublishingPublishable** curr_publishable_collection = NULL;
		gint curr_publishable_collection_length1 = 0;
		gint _curr_publishable_collection_size_ = 0;
		gint curr_publishable_it = 0;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		curr_publishable_collection = _tmp4_;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		curr_publishable_collection_length1 = _tmp4__length1;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		for (curr_publishable_it = 0; curr_publishable_it < _tmp4__length1; curr_publishable_it = curr_publishable_it + 1) {
#line 331 "PublishingPluginHost.c"
			SpitPublishingPublishable* _tmp5_ = NULL;
			SpitPublishingPublishable* curr_publishable = NULL;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			_tmp5_ = _g_object_ref0 (curr_publishable_collection[curr_publishable_it]);
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			curr_publishable = _tmp5_;
#line 338 "PublishingPluginHost.c"
			{
				SpitPublishingPublisherMediaType _tmp6_ = 0;
				SpitPublishingPublishable* _tmp7_ = NULL;
				SpitPublishingPublisherMediaType _tmp8_ = 0;
#line 31 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp6_ = self->priv->media_type;
#line 31 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp7_ = curr_publishable;
#line 31 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp8_ = spit_publishing_publishable_get_media_type (_tmp7_);
#line 31 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				self->priv->media_type = _tmp6_ | _tmp8_;
#line 30 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_g_object_unref0 (curr_publishable);
#line 353 "PublishingPluginHost.c"
			}
		}
	}
#line 33 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp9_ = service;
#line 33 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp10_ = spit_publishing_service_create_publisher (_tmp9_, G_TYPE_CHECK_INSTANCE_CAST (self, SPIT_PUBLISHING_TYPE_PLUGIN_HOST, SpitPublishingPluginHost));
#line 33 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (self->priv->active_publisher);
#line 33 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->active_publisher = _tmp10_;
#line 24 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return self;
#line 367 "PublishingPluginHost.c"
}


SpitPublishingConcretePublishingHost* spit_publishing_concrete_publishing_host_new (SpitPublishingService* service, PublishingUIPublishingDialog* dialog, SpitPublishingPublishable** publishables, int publishables_length1) {
#line 24 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return spit_publishing_concrete_publishing_host_construct (SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, service, dialog, publishables, publishables_length1);
#line 374 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_on_login_clicked (SpitPublishingConcretePublishingHost* self) {
	SpitPublishingLoginCallback _tmp0_ = NULL;
	void* _tmp0__target = NULL;
#line 36 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST (self));
#line 37 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->current_login_callback;
#line 37 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0__target = self->priv->current_login_callback_target;
#line 37 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp0_ != NULL) {
#line 389 "PublishingPluginHost.c"
		SpitPublishingLoginCallback _tmp1_ = NULL;
		void* _tmp1__target = NULL;
#line 38 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp1_ = self->priv->current_login_callback;
#line 38 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp1__target = self->priv->current_login_callback_target;
#line 38 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp1_ (_tmp1__target);
#line 398 "PublishingPluginHost.c"
	}
}


static void spit_publishing_concrete_publishing_host_clean_up (SpitPublishingConcretePublishingHost* self) {
	SpitPublishingPublishable** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
#line 41 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST (self));
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->publishables;
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0__length1 = self->priv->publishables_length1;
#line 412 "PublishingPluginHost.c"
	{
		SpitPublishingPublishable** publishable_collection = NULL;
		gint publishable_collection_length1 = 0;
		gint _publishable_collection_size_ = 0;
		gint publishable_it = 0;
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishable_collection = _tmp0_;
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishable_collection_length1 = _tmp0__length1;
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		for (publishable_it = 0; publishable_it < _tmp0__length1; publishable_it = publishable_it + 1) {
#line 424 "PublishingPluginHost.c"
			SpitPublishingPublishable* _tmp1_ = NULL;
			SpitPublishingPublishable* publishable = NULL;
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			_tmp1_ = _g_object_ref0 (publishable_collection[publishable_it]);
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			publishable = _tmp1_;
#line 431 "PublishingPluginHost.c"
			{
				SpitPublishingPublishable* _tmp2_ = NULL;
#line 43 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp2_ = publishable;
#line 43 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				publishing_glue_media_source_publishable_wrapper_clean_up (G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER, PublishingGlueMediaSourcePublishableWrapper));
#line 42 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_g_object_unref0 (publishable);
#line 440 "PublishingPluginHost.c"
			}
		}
	}
}


static void spit_publishing_concrete_publishing_host_report_plugin_upload_progress (SpitPublishingConcretePublishingHost* self, gint file_number, gdouble fraction_complete) {
	PublishingUIPublishingDialog* _tmp0_ = NULL;
	SpitPublishingDialogPane* _tmp1_ = NULL;
	SpitPublishingDialogPane* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
	PublishingUIProgressPane* pane = NULL;
	PublishingUIPublishingDialog* _tmp4_ = NULL;
	SpitPublishingDialogPane* _tmp5_ = NULL;
	gchar* status_string = NULL;
	gint _tmp6_ = 0;
	SpitPublishingPublishable** _tmp7_ = NULL;
	gint _tmp7__length1 = 0;
	gchar* _tmp8_ = NULL;
	gdouble status_fraction = 0.0;
	gdouble _tmp9_ = 0.0;
	PublishingUIProgressPane* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	gdouble _tmp12_ = 0.0;
#line 46 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST (self));
#line 48 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->dialog;
#line 48 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = publishing_ui_publishing_dialog_get_active_pane (_tmp0_);
#line 48 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = _tmp1_;
#line 48 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = !G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, PUBLISHING_UI_TYPE_PROGRESS_PANE);
#line 48 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp2_);
#line 48 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp3_) {
#line 49 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		return;
#line 481 "PublishingPluginHost.c"
	}
#line 51 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = self->priv->dialog;
#line 51 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp5_ = publishing_ui_publishing_dialog_get_active_pane (_tmp4_);
#line 51 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	pane = G_TYPE_CHECK_INSTANCE_CAST (_tmp5_, PUBLISHING_UI_TYPE_PROGRESS_PANE, PublishingUIProgressPane);
#line 53 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp6_ = file_number;
#line 53 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp7_ = self->priv->publishables;
#line 53 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp7__length1 = self->priv->publishables_length1;
#line 53 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp8_ = g_strdup_printf (SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_UPLOAD_STATUS_DESCRIPTION, _tmp6_, _tmp7__length1);
#line 53 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	status_string = _tmp8_;
#line 55 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp9_ = fraction_complete;
#line 55 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	status_fraction = SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_STATUS_PREPARATION_FRACTION + (SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_STATUS_UPLOAD_FRACTION * _tmp9_);
#line 58 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp10_ = pane;
#line 58 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp11_ = status_string;
#line 58 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp12_ = status_fraction;
#line 58 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_progress_pane_set_status (_tmp10_, _tmp11_, _tmp12_);
#line 46 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_free0 (status_string);
#line 46 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (pane);
#line 515 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_install_progress_pane (SpitPublishingConcretePublishingHost* self) {
	PublishingUIProgressPane* progress_pane = NULL;
	PublishingUIProgressPane* _tmp0_ = NULL;
	PublishingUIPublishingDialog* _tmp1_ = NULL;
#line 61 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST (self));
#line 62 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = publishing_ui_progress_pane_new ();
#line 62 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	progress_pane = _tmp0_;
#line 64 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = self->priv->dialog;
#line 64 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp1_, G_TYPE_CHECK_INSTANCE_CAST (progress_pane, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 65 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_set_button_mode (self, SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CANCEL);
#line 61 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (progress_pane);
#line 537 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_dialog_pane (SpitPublishingPluginHost* base, SpitPublishingDialogPane* pane, SpitPublishingPluginHostButtonMode button_mode) {
	SpitPublishingConcretePublishingHost * self;
	gboolean _tmp0_ = FALSE;
	SpitPublishingPublisher* _tmp1_ = NULL;
	PublishingUIPublishingDialog* _tmp4_ = NULL;
	SpitPublishingDialogPane* _tmp5_ = NULL;
	SpitPublishingPluginHostButtonMode _tmp6_ = 0;
#line 68 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 68 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_DIALOG_PANE (pane));
#line 70 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_debug ("PublishingPluginHost.vala:70: Publishing.PluginHost: install_dialog_pa" \
"ne( ): invoked.");
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = self->priv->active_publisher;
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp1_ == NULL) {
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp0_ = TRUE;
#line 560 "PublishingPluginHost.c"
	} else {
		SpitPublishingPublisher* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp2_ = self->priv->active_publisher;
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp3_ = spit_publishing_publisher_is_running (_tmp2_);
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp0_ = !_tmp3_;
#line 570 "PublishingPluginHost.c"
	}
#line 72 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp0_) {
#line 73 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		return;
#line 576 "PublishingPluginHost.c"
	}
#line 75 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = self->priv->dialog;
#line 75 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp5_ = pane;
#line 75 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp4_, _tmp5_);
#line 77 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp6_ = button_mode;
#line 77 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_set_button_mode (self, _tmp6_);
#line 588 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_post_error (SpitPublishingPluginHost* base, GError* err) {
	SpitPublishingConcretePublishingHost * self;
	gchar* msg = NULL;
	const gchar* _tmp0_ = NULL;
	SpitPublishingPublisher* _tmp1_ = NULL;
	SpitPublishingService* _tmp2_ = NULL;
	SpitPublishingService* _tmp3_ = NULL;
	const gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	const gchar* _tmp7_ = NULL;
	GError* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	const gchar* _tmp13_ = NULL;
	const gchar* _tmp14_ = NULL;
	gchar* _tmp15_ = NULL;
	PublishingUIPublishingDialog* _tmp16_ = NULL;
	const gchar* _tmp17_ = NULL;
	PublishingUIStaticMessagePane* _tmp18_ = NULL;
	PublishingUIStaticMessagePane* _tmp19_ = NULL;
	PublishingUIPublishingDialog* _tmp20_ = NULL;
	PublishingUIPublishingDialog* _tmp21_ = NULL;
	SpitPublishingPublisher* _tmp22_ = NULL;
#line 80 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = _ ("Publishing to %s can't continue because an error occurred:");
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = self->priv->active_publisher;
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = spit_publishing_publisher_get_service (_tmp1_);
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = _tmp2_;
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = spit_pluggable_get_pluggable_name (G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, SPIT_TYPE_PLUGGABLE, SpitPluggable));
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp5_ = g_strdup_printf (_tmp0_, _tmp4_);
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp6_ = _tmp5_;
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp3_);
#line 81 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	msg = _tmp6_;
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp7_ = msg;
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp8_ = err;
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp9_ = _tmp8_->message;
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp10_ = g_markup_printf_escaped ("\n\n<i>%s</i>\n\n", _tmp9_);
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp11_ = _tmp10_;
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp12_ = g_strconcat (_tmp7_, _tmp11_, NULL);
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_free0 (msg);
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	msg = _tmp12_;
#line 83 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_free0 (_tmp11_);
#line 84 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp13_ = msg;
#line 84 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp14_ = _ ("To try publishing to another service, select one from the above menu.");
#line 84 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp15_ = g_strconcat (_tmp13_, _tmp14_, NULL);
#line 84 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_free0 (msg);
#line 84 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	msg = _tmp15_;
#line 86 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp16_ = self->priv->dialog;
#line 86 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp17_ = msg;
#line 86 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp18_ = publishing_ui_static_message_pane_new (_tmp17_, TRUE);
#line 86 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp19_ = _tmp18_;
#line 86 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp16_, G_TYPE_CHECK_INSTANCE_CAST (_tmp19_, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 86 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp19_);
#line 87 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp20_ = self->priv->dialog;
#line 87 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_set_close_button_mode (_tmp20_);
#line 88 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp21_ = self->priv->dialog;
#line 88 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_unlock_service (_tmp21_);
#line 90 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp22_ = self->priv->active_publisher;
#line 90 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_publisher_stop (_tmp22_);
#line 95 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_clean_up (self);
#line 80 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_free0 (msg);
#line 694 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_stop_publishing (SpitPublishingPluginHost* base) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingPublisher* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 98 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 99 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_debug ("PublishingPluginHost.vala:99: ConcretePublishingHost.stop_publishing( " \
"): invoked.");
#line 101 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->active_publisher;
#line 101 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = spit_publishing_publisher_is_running (_tmp0_);
#line 101 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp1_) {
#line 712 "PublishingPluginHost.c"
		SpitPublishingPublisher* _tmp2_ = NULL;
#line 102 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp2_ = self->priv->active_publisher;
#line 102 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		spit_publishing_publisher_stop (_tmp2_);
#line 718 "PublishingPluginHost.c"
	}
#line 104 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_clean_up (self);
#line 106 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishing_halted = TRUE;
#line 724 "PublishingPluginHost.c"
}


void spit_publishing_concrete_publishing_host_start_publishing (SpitPublishingConcretePublishingHost* self) {
	SpitPublishingPublisher* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	SpitPublishingPublisher* _tmp2_ = NULL;
#line 109 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST (self));
#line 110 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->active_publisher;
#line 110 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = spit_publishing_publisher_is_running (_tmp0_);
#line 110 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp1_) {
#line 111 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		return;
#line 742 "PublishingPluginHost.c"
	}
#line 113 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_debug ("PublishingPluginHost.vala:113: ConcretePublishingHost.start_publishing" \
"( ): invoked.");
#line 115 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = self->priv->active_publisher;
#line 115 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_publisher_start (_tmp2_);
#line 750 "PublishingPluginHost.c"
}


static SpitPublishingPublisher* spit_publishing_concrete_publishing_host_real_get_publisher (SpitPublishingPluginHost* base) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingPublisher* result = NULL;
	SpitPublishingPublisher* _tmp0_ = NULL;
	SpitPublishingPublisher* _tmp1_ = NULL;
#line 118 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 119 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->active_publisher;
#line 119 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 119 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	result = _tmp1_;
#line 119 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return result;
#line 769 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_static_message_pane (SpitPublishingPluginHost* base, const gchar* message, SpitPublishingPluginHostButtonMode button_mode) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingPluginHostButtonMode _tmp0_ = 0;
	PublishingUIPublishingDialog* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	PublishingUIStaticMessagePane* _tmp3_ = NULL;
	PublishingUIStaticMessagePane* _tmp4_ = NULL;
#line 122 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 122 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (message != NULL);
#line 125 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = button_mode;
#line 125 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_set_button_mode (self, _tmp0_);
#line 127 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = self->priv->dialog;
#line 127 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = message;
#line 127 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = publishing_ui_static_message_pane_new (_tmp2_, FALSE);
#line 127 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = _tmp3_;
#line 127 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp1_, G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 127 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp4_);
#line 800 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_pango_message_pane (SpitPublishingPluginHost* base, const gchar* markup, SpitPublishingPluginHostButtonMode button_mode) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingPluginHostButtonMode _tmp0_ = 0;
	PublishingUIPublishingDialog* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	PublishingUIStaticMessagePane* _tmp3_ = NULL;
	PublishingUIStaticMessagePane* _tmp4_ = NULL;
#line 130 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 130 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (markup != NULL);
#line 132 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = button_mode;
#line 132 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_set_button_mode (self, _tmp0_);
#line 134 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = self->priv->dialog;
#line 134 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = markup;
#line 134 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = publishing_ui_static_message_pane_new (_tmp2_, TRUE);
#line 134 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = _tmp3_;
#line 134 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp1_, G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 134 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp4_);
#line 831 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_success_pane (SpitPublishingPluginHost* base) {
	SpitPublishingConcretePublishingHost * self;
	PublishingUIPublishingDialog* _tmp0_ = NULL;
	SpitPublishingPublisherMediaType _tmp1_ = 0;
	SpitPublishingPublishable** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	PublishingUISuccessPane* _tmp3_ = NULL;
	PublishingUISuccessPane* _tmp4_ = NULL;
	PublishingUIPublishingDialog* _tmp5_ = NULL;
#line 137 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->dialog;
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = spit_publishing_plugin_host_get_publishable_media_type (G_TYPE_CHECK_INSTANCE_CAST (self, SPIT_PUBLISHING_TYPE_PLUGIN_HOST, SpitPublishingPluginHost));
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = self->priv->publishables;
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2__length1 = self->priv->publishables_length1;
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = publishing_ui_success_pane_new (_tmp1_, _tmp2__length1);
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp4_ = _tmp3_;
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp0_, G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 138 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp4_);
#line 140 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp5_ = self->priv->dialog;
#line 140 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_set_close_button_mode (_tmp5_);
#line 144 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_clean_up (self);
#line 868 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_account_fetch_wait_pane (SpitPublishingPluginHost* base) {
	SpitPublishingConcretePublishingHost * self;
	PublishingUIPublishingDialog* _tmp0_ = NULL;
	PublishingUIAccountFetchWaitPane* _tmp1_ = NULL;
	PublishingUIAccountFetchWaitPane* _tmp2_ = NULL;
#line 147 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 148 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->dialog;
#line 148 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = publishing_ui_account_fetch_wait_pane_new ();
#line 148 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = _tmp1_;
#line 148 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp0_, G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 148 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp2_);
#line 149 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_set_button_mode (self, SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CANCEL);
#line 891 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_login_wait_pane (SpitPublishingPluginHost* base) {
	SpitPublishingConcretePublishingHost * self;
	PublishingUIPublishingDialog* _tmp0_ = NULL;
	PublishingUILoginWaitPane* _tmp1_ = NULL;
	PublishingUILoginWaitPane* _tmp2_ = NULL;
#line 152 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 153 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->dialog;
#line 153 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = publishing_ui_login_wait_pane_new ();
#line 153 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = _tmp1_;
#line 153 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp0_, G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 153 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (_tmp2_);
#line 912 "PublishingPluginHost.c"
}


static void _spit_publishing_concrete_publishing_host_on_login_clicked_publishing_ui_login_welcome_pane_login_requested (PublishingUILoginWelcomePane* _sender, gpointer self) {
#line 160 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_on_login_clicked ((SpitPublishingConcretePublishingHost*) self);
#line 919 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_install_welcome_pane (SpitPublishingPluginHost* base, const gchar* welcome_message, SpitPublishingLoginCallback login_clicked_callback, void* login_clicked_callback_target) {
	SpitPublishingConcretePublishingHost * self;
	PublishingUILoginWelcomePane* login_pane = NULL;
	const gchar* _tmp0_ = NULL;
	PublishingUILoginWelcomePane* _tmp1_ = NULL;
	SpitPublishingLoginCallback _tmp2_ = NULL;
	void* _tmp2__target = NULL;
	PublishingUIPublishingDialog* _tmp3_ = NULL;
#line 156 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 156 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (welcome_message != NULL);
#line 157 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = welcome_message;
#line 157 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = publishing_ui_login_welcome_pane_new (_tmp0_);
#line 157 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	login_pane = _tmp1_;
#line 159 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = login_clicked_callback;
#line 159 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2__target = login_clicked_callback_target;
#line 159 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->current_login_callback = _tmp2_;
#line 159 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->current_login_callback_target = _tmp2__target;
#line 160 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_signal_connect_object (login_pane, "login-requested", (GCallback) _spit_publishing_concrete_publishing_host_on_login_clicked_publishing_ui_login_welcome_pane_login_requested, self, 0);
#line 162 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_set_button_mode (self, SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CLOSE);
#line 164 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp3_ = self->priv->dialog;
#line 164 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	publishing_ui_publishing_dialog_install_pane (_tmp3_, G_TYPE_CHECK_INSTANCE_CAST (login_pane, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane));
#line 156 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (login_pane);
#line 959 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_real_set_service_locked (SpitPublishingPluginHost* base, gboolean locked) {
	SpitPublishingConcretePublishingHost * self;
	gboolean _tmp0_ = FALSE;
#line 167 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 168 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = locked;
#line 168 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp0_) {
#line 972 "PublishingPluginHost.c"
		PublishingUIPublishingDialog* _tmp1_ = NULL;
#line 169 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp1_ = self->priv->dialog;
#line 169 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishing_ui_publishing_dialog_lock_service (_tmp1_);
#line 978 "PublishingPluginHost.c"
	} else {
		PublishingUIPublishingDialog* _tmp2_ = NULL;
#line 171 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp2_ = self->priv->dialog;
#line 171 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishing_ui_publishing_dialog_unlock_service (_tmp2_);
#line 985 "PublishingPluginHost.c"
	}
}


void spit_publishing_concrete_publishing_host_set_button_mode (SpitPublishingConcretePublishingHost* self, SpitPublishingPluginHostButtonMode mode) {
	SpitPublishingPluginHostButtonMode _tmp0_ = 0;
#line 174 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (SPIT_PUBLISHING_IS_CONCRETE_PUBLISHING_HOST (self));
#line 175 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = mode;
#line 175 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (_tmp0_ == SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CLOSE) {
#line 998 "PublishingPluginHost.c"
		PublishingUIPublishingDialog* _tmp1_ = NULL;
#line 176 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp1_ = self->priv->dialog;
#line 176 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishing_ui_publishing_dialog_set_close_button_mode (_tmp1_);
#line 1004 "PublishingPluginHost.c"
	} else {
		SpitPublishingPluginHostButtonMode _tmp2_ = 0;
#line 177 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp2_ = mode;
#line 177 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		if (_tmp2_ == SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CANCEL) {
#line 1011 "PublishingPluginHost.c"
			PublishingUIPublishingDialog* _tmp3_ = NULL;
#line 178 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			_tmp3_ = self->priv->dialog;
#line 178 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			publishing_ui_publishing_dialog_set_cancel_button_mode (_tmp3_);
#line 1017 "PublishingPluginHost.c"
		} else {
#line 180 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			g_error ("PublishingPluginHost.vala:180: unrecognized button mode enumeration va" \
"lue");
#line 1021 "PublishingPluginHost.c"
		}
	}
}


static void spit_publishing_concrete_publishing_host_real_set_dialog_default_widget (SpitPublishingPluginHost* base, GtkWidget* widget) {
	SpitPublishingConcretePublishingHost * self;
	GtkWidget* _tmp0_ = NULL;
	PublishingUIPublishingDialog* _tmp1_ = NULL;
	GtkWidget* _tmp2_ = NULL;
#line 183 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 183 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_return_if_fail (GTK_IS_WIDGET (widget));
#line 184 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = widget;
#line 184 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	gtk_widget_set_can_default (_tmp0_, TRUE);
#line 185 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = self->priv->dialog;
#line 185 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = widget;
#line 185 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	gtk_window_set_default (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, gtk_window_get_type (), GtkWindow), _tmp2_);
#line 1046 "PublishingPluginHost.c"
}


static SpitPublishingPublisherMediaType spit_publishing_concrete_publishing_host_real_get_publishable_media_type (SpitPublishingPluginHost* base) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingPublisherMediaType result = 0;
	SpitPublishingPublisherMediaType _tmp0_ = 0;
#line 188 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 189 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->media_type;
#line 189 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	result = _tmp0_;
#line 189 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return result;
#line 1062 "PublishingPluginHost.c"
}


static SpitPublishingPublishable** _vala_array_dup21 (SpitPublishingPublishable** self, int length) {
	SpitPublishingPublishable** result;
	int i;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	result = g_new0 (SpitPublishingPublishable*, length + 1);
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	for (i = 0; i < length; i++) {
#line 1073 "PublishingPluginHost.c"
		SpitPublishingPublishable* _tmp0_ = NULL;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		_tmp0_ = _g_object_ref0 (self[i]);
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		result[i] = _tmp0_;
#line 1079 "PublishingPluginHost.c"
	}
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return result;
#line 1083 "PublishingPluginHost.c"
}


static SpitPublishingPublishable** spit_publishing_concrete_publishing_host_real_get_publishables (SpitPublishingPluginHost* base, int* result_length1) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingPublishable** result = NULL;
	SpitPublishingPublishable** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	SpitPublishingPublishable** _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
	SpitPublishingPublishable** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
#line 192 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->publishables;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0__length1 = self->priv->publishables_length1;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = (_tmp0_ != NULL) ? _vala_array_dup21 (_tmp0_, _tmp0__length1) : ((gpointer) _tmp0_);
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1__length1 = _tmp0__length1;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = _tmp1_;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2__length1 = _tmp1__length1;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	if (result_length1) {
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		*result_length1 = _tmp2__length1;
#line 1114 "PublishingPluginHost.c"
	}
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	result = _tmp2_;
#line 193 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return result;
#line 1120 "PublishingPluginHost.c"
}


static void _spit_publishing_concrete_publishing_host_report_plugin_upload_progress_spit_publishing_progress_callback (gint file_number, gdouble fraction_complete, gpointer self) {
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_report_plugin_upload_progress ((SpitPublishingConcretePublishingHost*) self, file_number, fraction_complete);
#line 1127 "PublishingPluginHost.c"
}


static SpitPublishingProgressCallback spit_publishing_concrete_publishing_host_real_serialize_publishables (SpitPublishingPluginHost* base, gint content_major_axis, gboolean strip_metadata, void** result_target, GDestroyNotify* result_target_destroy_notify) {
	SpitPublishingConcretePublishingHost * self;
	SpitPublishingProgressCallback result = NULL;
	PublishingUIProgressPane* progress_pane = NULL;
	PublishingUIPublishingDialog* _tmp0_ = NULL;
	SpitPublishingDialogPane* _tmp1_ = NULL;
	gint i = 0;
	SpitPublishingPublishable** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	SpitPublishingProgressCallback _tmp25_ = NULL;
	void* _tmp25__target = NULL;
	GDestroyNotify _tmp25__target_destroy_notify = NULL;
	GError * _inner_error_ = NULL;
#line 196 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 198 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_install_progress_pane (self);
#line 199 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp0_ = self->priv->dialog;
#line 199 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp1_ = publishing_ui_publishing_dialog_get_active_pane (_tmp0_);
#line 199 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	progress_pane = G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, PUBLISHING_UI_TYPE_PROGRESS_PANE, PublishingUIProgressPane);
#line 205 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spin_event_loop ();
#line 207 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	i = 0;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2_ = self->priv->publishables;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp2__length1 = self->priv->publishables_length1;
#line 1162 "PublishingPluginHost.c"
	{
		SpitPublishingPublishable** publishable_collection = NULL;
		gint publishable_collection_length1 = 0;
		gint _publishable_collection_size_ = 0;
		gint publishable_it = 0;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishable_collection = _tmp2_;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		publishable_collection_length1 = _tmp2__length1;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
		for (publishable_it = 0; publishable_it < _tmp2__length1; publishable_it = publishable_it + 1) {
#line 1174 "PublishingPluginHost.c"
			SpitPublishingPublishable* _tmp3_ = NULL;
			SpitPublishingPublishable* publishable = NULL;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			_tmp3_ = _g_object_ref0 (publishable_collection[publishable_it]);
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
			publishable = _tmp3_;
#line 1181 "PublishingPluginHost.c"
			{
				gboolean _tmp4_ = FALSE;
				gboolean _tmp5_ = FALSE;
				gdouble phase_fraction_complete = 0.0;
				gint _tmp18_ = 0;
				SpitPublishingPublishable** _tmp19_ = NULL;
				gint _tmp19__length1 = 0;
				gdouble fraction_complete = 0.0;
				gdouble _tmp20_ = 0.0;
				gdouble _tmp21_ = 0.0;
				PublishingUIProgressPane* _tmp22_ = NULL;
				gdouble _tmp23_ = 0.0;
				gint _tmp24_ = 0;
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp5_ = self->priv->publishing_halted;
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				if (_tmp5_) {
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp4_ = TRUE;
#line 1201 "PublishingPluginHost.c"
				} else {
					SpitPublishingPublisher* _tmp6_ = NULL;
					gboolean _tmp7_ = FALSE;
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp6_ = self->priv->active_publisher;
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp7_ = spit_publishing_publisher_is_running (_tmp6_);
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp4_ = !_tmp7_;
#line 1211 "PublishingPluginHost.c"
				}
#line 209 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				if (_tmp4_) {
#line 1215 "PublishingPluginHost.c"
					SpitPublishingProgressCallback _tmp8_ = NULL;
					void* _tmp8__target = NULL;
					GDestroyNotify _tmp8__target_destroy_notify = NULL;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp8_ = NULL;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp8__target = NULL;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp8__target_destroy_notify = NULL;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					*result_target = _tmp8__target;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					*result_target_destroy_notify = _tmp8__target_destroy_notify;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					result = _tmp8_;
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (publishable);
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (progress_pane);
#line 210 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					return result;
#line 1237 "PublishingPluginHost.c"
				}
				{
					PublishingGlueMediaSourcePublishableWrapper* wrapper = NULL;
					SpitPublishingPublishable* _tmp9_ = NULL;
					PublishingGlueMediaSourcePublishableWrapper* _tmp10_ = NULL;
					PublishingGlueMediaSourcePublishableWrapper* _tmp11_ = NULL;
					gint _tmp12_ = 0;
					gboolean _tmp13_ = FALSE;
					GFile* _tmp14_ = NULL;
					GFile* _tmp15_ = NULL;
#line 213 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp9_ = publishable;
#line 213 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp10_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, PUBLISHING_GLUE_TYPE_MEDIA_SOURCE_PUBLISHABLE_WRAPPER, PublishingGlueMediaSourcePublishableWrapper));
#line 213 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					wrapper = _tmp10_;
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp11_ = wrapper;
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp12_ = content_major_axis;
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp13_ = strip_metadata;
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp14_ = publishing_glue_media_source_publishable_wrapper_serialize_for_publishing (_tmp11_, _tmp12_, _tmp13_, &_inner_error_);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp15_ = _tmp14_;
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (_tmp15_);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						_g_object_unref0 (wrapper);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 1272 "PublishingPluginHost.c"
							goto __catch34_spit_publishing_publishing_error;
						}
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						_g_object_unref0 (wrapper);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						_g_object_unref0 (publishable);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						_g_object_unref0 (progress_pane);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						g_clear_error (&_inner_error_);
#line 215 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
						return NULL;
#line 1287 "PublishingPluginHost.c"
					}
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (wrapper);
#line 1291 "PublishingPluginHost.c"
				}
				goto __finally34;
				__catch34_spit_publishing_publishing_error:
				{
					GError* err = NULL;
					GError* _tmp16_ = NULL;
					SpitPublishingProgressCallback _tmp17_ = NULL;
					void* _tmp17__target = NULL;
					GDestroyNotify _tmp17__target_destroy_notify = NULL;
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					err = _inner_error_;
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_inner_error_ = NULL;
#line 217 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp16_ = err;
#line 217 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					spit_publishing_plugin_host_post_error (G_TYPE_CHECK_INSTANCE_CAST (self, SPIT_PUBLISHING_TYPE_PLUGIN_HOST, SpitPublishingPluginHost), _tmp16_);
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp17_ = NULL;
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp17__target = NULL;
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_tmp17__target_destroy_notify = NULL;
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					*result_target = _tmp17__target;
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					*result_target_destroy_notify = _tmp17__target_destroy_notify;
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					result = _tmp17_;
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_error_free0 (err);
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (publishable);
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (progress_pane);
#line 218 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					return result;
#line 1329 "PublishingPluginHost.c"
				}
				__finally34:
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (publishable);
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					_g_object_unref0 (progress_pane);
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					g_clear_error (&_inner_error_);
#line 212 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
					return NULL;
#line 1344 "PublishingPluginHost.c"
				}
#line 221 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp18_ = i;
#line 221 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp19_ = self->priv->publishables;
#line 221 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp19__length1 = self->priv->publishables_length1;
#line 221 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				phase_fraction_complete = ((gdouble) (_tmp18_ + 1)) / ((gdouble) _tmp19__length1);
#line 222 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp20_ = phase_fraction_complete;
#line 222 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				fraction_complete = _tmp20_ * SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_STATUS_PREPARATION_FRACTION;
#line 224 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp21_ = fraction_complete;
#line 224 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				g_debug ("PublishingPluginHost.vala:224: serialize_publishables( ): fraction_com" \
"plete = %f.", _tmp21_);
#line 226 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp22_ = progress_pane;
#line 226 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp23_ = fraction_complete;
#line 226 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				publishing_ui_progress_pane_set_status (_tmp22_, SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_PREPARE_STATUS_DESCRIPTION, _tmp23_);
#line 228 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				spin_event_loop ();
#line 230 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_tmp24_ = i;
#line 230 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				i = _tmp24_ + 1;
#line 208 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
				_g_object_unref0 (publishable);
#line 1376 "PublishingPluginHost.c"
			}
		}
	}
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp25_ = _spit_publishing_concrete_publishing_host_report_plugin_upload_progress_spit_publishing_progress_callback;
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp25__target = g_object_ref (self);
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_tmp25__target_destroy_notify = g_object_unref;
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	*result_target = _tmp25__target;
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	*result_target_destroy_notify = _tmp25__target_destroy_notify;
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	result = _tmp25_;
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (progress_pane);
#line 233 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	return result;
#line 1396 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_class_init (SpitPublishingConcretePublishingHostClass * klass) {
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_parent_class = g_type_class_peek_parent (klass);
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	g_type_class_add_private (klass, sizeof (SpitPublishingConcretePublishingHostPrivate));
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	G_OBJECT_CLASS (klass)->finalize = spit_publishing_concrete_publishing_host_finalize;
#line 1407 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_spit_publishing_plugin_host_interface_init (SpitPublishingPluginHostIface * iface) {
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	spit_publishing_concrete_publishing_host_spit_publishing_plugin_host_parent_iface = g_type_interface_peek_parent (iface);
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_dialog_pane = (void (*)(SpitPublishingPluginHost*, SpitPublishingDialogPane*, SpitPublishingPluginHostButtonMode)) spit_publishing_concrete_publishing_host_real_install_dialog_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->post_error = (void (*)(SpitPublishingPluginHost*, GError*)) spit_publishing_concrete_publishing_host_real_post_error;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->stop_publishing = (void (*)(SpitPublishingPluginHost*)) spit_publishing_concrete_publishing_host_real_stop_publishing;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->get_publisher = (SpitPublishingPublisher* (*)(SpitPublishingPluginHost*)) spit_publishing_concrete_publishing_host_real_get_publisher;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_static_message_pane = (void (*)(SpitPublishingPluginHost*, const gchar*, SpitPublishingPluginHostButtonMode)) spit_publishing_concrete_publishing_host_real_install_static_message_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_pango_message_pane = (void (*)(SpitPublishingPluginHost*, const gchar*, SpitPublishingPluginHostButtonMode)) spit_publishing_concrete_publishing_host_real_install_pango_message_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_success_pane = (void (*)(SpitPublishingPluginHost*)) spit_publishing_concrete_publishing_host_real_install_success_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_account_fetch_wait_pane = (void (*)(SpitPublishingPluginHost*)) spit_publishing_concrete_publishing_host_real_install_account_fetch_wait_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_login_wait_pane = (void (*)(SpitPublishingPluginHost*)) spit_publishing_concrete_publishing_host_real_install_login_wait_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->install_welcome_pane = (void (*)(SpitPublishingPluginHost*, const gchar*, SpitPublishingLoginCallback, void*)) spit_publishing_concrete_publishing_host_real_install_welcome_pane;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->set_service_locked = (void (*)(SpitPublishingPluginHost*, gboolean)) spit_publishing_concrete_publishing_host_real_set_service_locked;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->set_dialog_default_widget = (void (*)(SpitPublishingPluginHost*, GtkWidget*)) spit_publishing_concrete_publishing_host_real_set_dialog_default_widget;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->get_publishable_media_type = (SpitPublishingPublisherMediaType (*)(SpitPublishingPluginHost*)) spit_publishing_concrete_publishing_host_real_get_publishable_media_type;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->get_publishables = (SpitPublishingPublishable** (*)(SpitPublishingPluginHost*, int*)) spit_publishing_concrete_publishing_host_real_get_publishables;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	iface->serialize_publishables = (SpitPublishingProgressCallback (*)(SpitPublishingPluginHost*, gint, gboolean, void**, GDestroyNotify*)) spit_publishing_concrete_publishing_host_real_serialize_publishables;
#line 1444 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_instance_init (SpitPublishingConcretePublishingHost * self) {
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv = SPIT_PUBLISHING_CONCRETE_PUBLISHING_HOST_GET_PRIVATE (self);
#line 16 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->dialog = NULL;
#line 17 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->active_publisher = NULL;
#line 18 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishables = NULL;
#line 18 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishables_length1 = 0;
#line 18 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->_publishables_size_ = self->priv->publishables_length1;
#line 19 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->current_login_callback = NULL;
#line 20 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishing_halted = FALSE;
#line 21 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->media_type = SPIT_PUBLISHING_PUBLISHER_MEDIA_TYPE_NONE;
#line 1467 "PublishingPluginHost.c"
}


static void spit_publishing_concrete_publishing_host_finalize (GObject* obj) {
	SpitPublishingConcretePublishingHost * self;
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SPIT_PUBLISHING_TYPE_CONCRETE_PUBLISHING_HOST, SpitPublishingConcretePublishingHost);
#line 17 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	_g_object_unref0 (self->priv->active_publisher);
#line 18 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	self->priv->publishables = (_vala_array_free (self->priv->publishables, self->priv->publishables_length1, (GDestroyNotify) g_object_unref), NULL);
#line 9 "/home/jens/Source/shotwell/src/publishing/PublishingPluginHost.vala"
	G_OBJECT_CLASS (spit_publishing_concrete_publishing_host_parent_class)->finalize (obj);
#line 1481 "PublishingPluginHost.c"
}


GType spit_publishing_concrete_publishing_host_get_type (void) {
	static volatile gsize spit_publishing_concrete_publishing_host_type_id__volatile = 0;
	if (g_once_init_enter (&spit_publishing_concrete_publishing_host_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SpitPublishingConcretePublishingHostClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) spit_publishing_concrete_publishing_host_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SpitPublishingConcretePublishingHost), 0, (GInstanceInitFunc) spit_publishing_concrete_publishing_host_instance_init, NULL };
		static const GInterfaceInfo spit_publishing_plugin_host_info = { (GInterfaceInitFunc) spit_publishing_concrete_publishing_host_spit_publishing_plugin_host_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType spit_publishing_concrete_publishing_host_type_id;
		spit_publishing_concrete_publishing_host_type_id = g_type_register_static (PLUGINS_TYPE_STANDARD_HOST_INTERFACE, "SpitPublishingConcretePublishingHost", &g_define_type_info, 0);
		g_type_add_interface_static (spit_publishing_concrete_publishing_host_type_id, SPIT_PUBLISHING_TYPE_PLUGIN_HOST, &spit_publishing_plugin_host_info);
		g_once_init_leave (&spit_publishing_concrete_publishing_host_type_id__volatile, spit_publishing_concrete_publishing_host_type_id);
	}
	return spit_publishing_concrete_publishing_host_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



