/* AppWindow.c generated by valac 0.32.1, the Vala compiler
 * generated from AppWindow.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <time.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <cairo.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gee.h>


#define TYPE_PAGE_WINDOW (page_window_get_type ())
#define PAGE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE_WINDOW, PageWindow))
#define PAGE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE_WINDOW, PageWindowClass))
#define IS_PAGE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE_WINDOW))
#define IS_PAGE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE_WINDOW))
#define PAGE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE_WINDOW, PageWindowClass))

typedef struct _PageWindow PageWindow;
typedef struct _PageWindowClass PageWindowClass;
typedef struct _PageWindowPrivate PageWindowPrivate;

#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;

#define TYPE_FULLSCREEN_WINDOW (fullscreen_window_get_type ())
#define FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindow))
#define FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))
#define IS_FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FULLSCREEN_WINDOW))
#define IS_FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FULLSCREEN_WINDOW))
#define FULLSCREEN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))

typedef struct _FullscreenWindow FullscreenWindow;
typedef struct _FullscreenWindowClass FullscreenWindowClass;
typedef struct _FullscreenWindowPrivate FullscreenWindowPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_CONFIGURATION_FACADE (configuration_facade_get_type ())
#define CONFIGURATION_FACADE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONFIGURATION_FACADE, ConfigurationFacade))
#define CONFIGURATION_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONFIGURATION_FACADE, ConfigurationFacadeClass))
#define IS_CONFIGURATION_FACADE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONFIGURATION_FACADE))
#define IS_CONFIGURATION_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONFIGURATION_FACADE))
#define CONFIGURATION_FACADE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONFIGURATION_FACADE, ConfigurationFacadeClass))

typedef struct _ConfigurationFacade ConfigurationFacade;
typedef struct _ConfigurationFacadeClass ConfigurationFacadeClass;

#define CONFIG_TYPE_FACADE (config_facade_get_type ())
#define CONFIG_FACADE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONFIG_TYPE_FACADE, ConfigFacade))
#define CONFIG_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONFIG_TYPE_FACADE, ConfigFacadeClass))
#define CONFIG_IS_FACADE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONFIG_TYPE_FACADE))
#define CONFIG_IS_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONFIG_TYPE_FACADE))
#define CONFIG_FACADE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONFIG_TYPE_FACADE, ConfigFacadeClass))

typedef struct _ConfigFacade ConfigFacade;
typedef struct _ConfigFacadeClass ConfigFacadeClass;

#define TYPE_SINGLE_PHOTO_PAGE (single_photo_page_get_type ())
#define SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPage))
#define SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))
#define IS_SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SINGLE_PHOTO_PAGE))
#define IS_SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SINGLE_PHOTO_PAGE))
#define SINGLE_PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))

typedef struct _SinglePhotoPage SinglePhotoPage;
typedef struct _SinglePhotoPageClass SinglePhotoPageClass;

#define TYPE_SLIDESHOW_PAGE (slideshow_page_get_type ())
#define SLIDESHOW_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SLIDESHOW_PAGE, SlideshowPage))
#define SLIDESHOW_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SLIDESHOW_PAGE, SlideshowPageClass))
#define IS_SLIDESHOW_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SLIDESHOW_PAGE))
#define IS_SLIDESHOW_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SLIDESHOW_PAGE))
#define SLIDESHOW_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SLIDESHOW_PAGE, SlideshowPageClass))

typedef struct _SlideshowPage SlideshowPage;
typedef struct _SlideshowPageClass SlideshowPageClass;
typedef struct _AppWindowPrivate AppWindowPrivate;

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_COMMAND_MANAGER (command_manager_get_type ())
#define COMMAND_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMMAND_MANAGER, CommandManager))
#define COMMAND_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMMAND_MANAGER, CommandManagerClass))
#define IS_COMMAND_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMMAND_MANAGER))
#define IS_COMMAND_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMMAND_MANAGER))
#define COMMAND_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMMAND_MANAGER, CommandManagerClass))

typedef struct _CommandManager CommandManager;
typedef struct _CommandManagerClass CommandManagerClass;

#define TYPE_LIBRARY_WINDOW (library_window_get_type ())
#define LIBRARY_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LIBRARY_WINDOW, LibraryWindow))
#define LIBRARY_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LIBRARY_WINDOW, LibraryWindowClass))
#define IS_LIBRARY_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LIBRARY_WINDOW))
#define IS_LIBRARY_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LIBRARY_WINDOW))
#define LIBRARY_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LIBRARY_WINDOW, LibraryWindowClass))

typedef struct _LibraryWindow LibraryWindow;
typedef struct _LibraryWindowClass LibraryWindowClass;

#define TYPE_DIRECT_WINDOW (direct_window_get_type ())
#define DIRECT_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DIRECT_WINDOW, DirectWindow))
#define DIRECT_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DIRECT_WINDOW, DirectWindowClass))
#define IS_DIRECT_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DIRECT_WINDOW))
#define IS_DIRECT_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DIRECT_WINDOW))
#define DIRECT_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DIRECT_WINDOW, DirectWindowClass))

typedef struct _DirectWindow DirectWindow;
typedef struct _DirectWindowClass DirectWindowClass;
#define _command_manager_unref0(var) ((var == NULL) ? NULL : (var = (command_manager_unref (var), NULL)))

#define TYPE_APPLICATION (application_get_type ())
#define APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APPLICATION, Application))
#define APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APPLICATION, ApplicationClass))
#define IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APPLICATION))
#define IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APPLICATION))
#define APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APPLICATION, ApplicationClass))

typedef struct _Application Application;
typedef struct _ApplicationClass ApplicationClass;
#define _application_unref0(var) ((var == NULL) ? NULL : (var = (application_unref (var), NULL)))

#define TYPE_DATA_COLLECTION (data_collection_get_type ())
#define DATA_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_COLLECTION, DataCollection))
#define DATA_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_COLLECTION, DataCollectionClass))
#define IS_DATA_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_COLLECTION))
#define IS_DATA_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_COLLECTION))
#define DATA_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_COLLECTION, DataCollectionClass))

typedef struct _DataCollection DataCollection;
typedef struct _DataCollectionClass DataCollectionClass;

#define TYPE_VIEW_COLLECTION (view_collection_get_type ())
#define VIEW_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_VIEW_COLLECTION, ViewCollection))
#define VIEW_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_VIEW_COLLECTION, ViewCollectionClass))
#define IS_VIEW_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_VIEW_COLLECTION))
#define IS_VIEW_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_VIEW_COLLECTION))
#define VIEW_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_VIEW_COLLECTION, ViewCollectionClass))

typedef struct _ViewCollection ViewCollection;
typedef struct _ViewCollectionClass ViewCollectionClass;
#define _data_collection_unref0(var) ((var == NULL) ? NULL : (var = (data_collection_unref (var), NULL)))

#define TYPE_DATA_OBJECT (data_object_get_type ())
#define DATA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_OBJECT, DataObject))
#define DATA_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_OBJECT, DataObjectClass))
#define IS_DATA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_OBJECT))
#define IS_DATA_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_OBJECT))
#define DATA_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_OBJECT, DataObjectClass))

typedef struct _DataObject DataObject;
typedef struct _DataObjectClass DataObjectClass;

#define TYPE_DATA_SOURCE (data_source_get_type ())
#define DATA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_SOURCE, DataSource))
#define DATA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_SOURCE, DataSourceClass))
#define IS_DATA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_SOURCE))
#define IS_DATA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_SOURCE))
#define DATA_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_SOURCE, DataSourceClass))

typedef struct _DataSource DataSource;
typedef struct _DataSourceClass DataSourceClass;

#define TYPE_THUMBNAIL_SOURCE (thumbnail_source_get_type ())
#define THUMBNAIL_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL_SOURCE, ThumbnailSource))
#define THUMBNAIL_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL_SOURCE, ThumbnailSourceClass))
#define IS_THUMBNAIL_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL_SOURCE))
#define IS_THUMBNAIL_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL_SOURCE))
#define THUMBNAIL_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL_SOURCE, ThumbnailSourceClass))

typedef struct _ThumbnailSource ThumbnailSource;
typedef struct _ThumbnailSourceClass ThumbnailSourceClass;

#define TYPE_MEDIA_SOURCE (media_source_get_type ())
#define MEDIA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEDIA_SOURCE, MediaSource))
#define MEDIA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEDIA_SOURCE, MediaSourceClass))
#define IS_MEDIA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEDIA_SOURCE))
#define IS_MEDIA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEDIA_SOURCE))
#define MEDIA_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEDIA_SOURCE, MediaSourceClass))

typedef struct _MediaSource MediaSource;
typedef struct _MediaSourceClass MediaSourceClass;

#define TYPE_DATA_VIEW (data_view_get_type ())
#define DATA_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_VIEW, DataView))
#define DATA_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_VIEW, DataViewClass))
#define IS_DATA_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_VIEW))
#define IS_DATA_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_VIEW))
#define DATA_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_VIEW, DataViewClass))

typedef struct _DataView DataView;
typedef struct _DataViewClass DataViewClass;
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;

#define TYPE_COMMAND_DESCRIPTION (command_description_get_type ())
#define COMMAND_DESCRIPTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMMAND_DESCRIPTION, CommandDescription))
#define IS_COMMAND_DESCRIPTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMMAND_DESCRIPTION))
#define COMMAND_DESCRIPTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), TYPE_COMMAND_DESCRIPTION, CommandDescriptionIface))

typedef struct _CommandDescription CommandDescription;
typedef struct _CommandDescriptionIface CommandDescriptionIface;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _PageWindow {
	GtkWindow parent_instance;
	PageWindowPrivate * priv;
	GtkUIManager* ui;
};

struct _PageWindowClass {
	GtkWindowClass parent_class;
	void (*switched_pages) (PageWindow* self, Page* old_page, Page* new_page);
	void (*set_current_page) (PageWindow* self, Page* page);
	void (*clear_current_page) (PageWindow* self);
};

struct _FullscreenWindow {
	PageWindow parent_instance;
	FullscreenWindowPrivate * priv;
};

struct _FullscreenWindowClass {
	PageWindowClass parent_class;
};

struct _FullscreenWindowPrivate {
	GtkOverlay* overlay;
	GtkToolbar* toolbar;
	GtkToolButton* close_button;
	GtkToggleToolButton* pin_button;
	gboolean is_toolbar_shown;
	gboolean waiting_for_invoke;
	time_t left_toolbar_time;
	gboolean switched_to;
	gboolean is_toolbar_dismissal_enabled;
};

struct _PageWindowPrivate {
	Page* current_page;
	gint busy_counter;
};

struct _Dimensions {
	gint width;
	gint height;
};

struct _AppWindow {
	PageWindow parent_instance;
	AppWindowPrivate * priv;
	GtkActionGroup** common_action_groups;
	gint common_action_groups_length1;
	gboolean maximized;
	Dimensions dimensions;
	gint pos_x;
	gint pos_y;
};

struct _AppWindowClass {
	PageWindowClass parent_class;
	void (*on_fullscreen) (AppWindow* self);
	gchar* (*get_app_role) (AppWindow* self);
	void (*on_quit) (AppWindow* self);
	GtkActionGroup** (*create_common_action_groups) (AppWindow* self, int* result_length1);
	void (*replace_common_placeholders) (AppWindow* self, GtkUIManager* ui);
	void (*update_common_action_availability) (AppWindow* self, Page* old_page, Page* new_page);
	void (*update_common_actions) (AppWindow* self, Page* page, gint selected_count, gint count);
};

struct _AppWindowPrivate {
	GtkActionGroup* common_action_group;
};

typedef enum  {
	DATABASE_ERROR_ERROR,
	DATABASE_ERROR_BACKING,
	DATABASE_ERROR_MEMORY,
	DATABASE_ERROR_ABORT,
	DATABASE_ERROR_LIMITS,
	DATABASE_ERROR_TYPESPEC
} DatabaseError;
#define DATABASE_ERROR database_error_quark ()
struct _CommandDescriptionIface {
	GTypeInterface parent_iface;
	gchar* (*get_name) (CommandDescription* self);
	gchar* (*get_explanation) (CommandDescription* self);
};


static gpointer fullscreen_window_parent_class = NULL;
static gpointer page_window_parent_class = NULL;
static gpointer app_window_parent_class = NULL;
extern AppWindow* app_window_instance;
AppWindow* app_window_instance = NULL;
static FullscreenWindow* app_window_fullscreen_window;
static FullscreenWindow* app_window_fullscreen_window = NULL;
static CommandManager* app_window_command_manager;
static CommandManager* app_window_command_manager = NULL;

GType page_window_get_type (void) G_GNUC_CONST;
GType page_get_type (void) G_GNUC_CONST;
GType fullscreen_window_get_type (void) G_GNUC_CONST;
#define FULLSCREEN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_FULLSCREEN_WINDOW, FullscreenWindowPrivate))
enum  {
	FULLSCREEN_WINDOW_DUMMY_PROPERTY
};
#define FULLSCREEN_WINDOW_TOOLBAR_INVOCATION_MSEC 250
#define FULLSCREEN_WINDOW_TOOLBAR_DISMISSAL_SEC 2
#define FULLSCREEN_WINDOW_TOOLBAR_CHECK_DISMISSAL_MSEC 500
FullscreenWindow* fullscreen_window_new (Page* page);
FullscreenWindow* fullscreen_window_construct (GType object_type, Page* page);
PageWindow* page_window_construct (GType object_type);
void page_window_set_current_page (PageWindow* self, Page* page);
GFile* resources_get_ui (const gchar* filename);
static GtkActionEntry* fullscreen_window_create_actions (FullscreenWindow* self, int* result_length1);
GType app_window_get_type (void) G_GNUC_CONST;
AppWindow* app_window_get_instance (void);
static void fullscreen_window_get_monitor_geometry (FullscreenWindow* self, GdkRectangle* result);
GType configuration_facade_get_type (void) G_GNUC_CONST;
GType config_facade_get_type (void) G_GNUC_CONST;
ConfigFacade* config_facade_get_instance (void);
gboolean configuration_facade_get_pin_toolbar_state (ConfigurationFacade* self);
void fullscreen_window_update_toolbar_dismissal (FullscreenWindow* self);
static void _fullscreen_window_update_toolbar_dismissal_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void fullscreen_window_on_close (FullscreenWindow* self);
static void _fullscreen_window_on_close_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
GtkToolbar* page_get_toolbar (Page* self);
#define RESOURCES_TRANSIENT_WINDOW_OPACITY 0.90
GType single_photo_page_get_type (void) G_GNUC_CONST;
GType slideshow_page_get_type (void) G_GNUC_CONST;
static void fullscreen_window_hide_toolbar (FullscreenWindow* self);
static void _fullscreen_window_hide_toolbar_slideshow_page_hide_toolbar (SlideshowPage* _sender, gpointer self);
void page_set_cursor_hide_time (Page* self, gint hide_time);
void page_start_cursor_hiding (Page* self);
static void fullscreen_window_invoke_toolbar (FullscreenWindow* self);
void fullscreen_window_disable_toolbar_dismissal (FullscreenWindow* self);
static gboolean fullscreen_window_real_configure_event (GtkWidget* base, GdkEventConfigure* event);
Page* page_window_get_current_page (PageWindow* self);
void page_switched_to (Page* self);
#define RESOURCES_LEAVE_FULLSCREEN_LABEL _ ("Leave _Fullscreen")
#define TRANSLATABLE "translatable"
static void _fullscreen_window_on_close_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add158 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static gboolean fullscreen_window_real_key_press_event (GtkWidget* base, GdkEventKey* event);
void configuration_facade_set_pin_toolbar_state (ConfigurationFacade* self, gboolean state);
void app_window_end_fullscreen (AppWindow* self);
void fullscreen_window_close (FullscreenWindow* self);
static void fullscreen_window_real_destroy (GtkWidget* base);
void page_window_clear_current_page (PageWindow* self);
void page_stop_cursor_hiding (Page* self);
void page_switching_from (Page* self);
static gboolean fullscreen_window_real_delete_event (GtkWidget* base, GdkEventAny* event);
static gboolean fullscreen_window_real_motion_notify_event (GtkWidget* base, GdkEventMotion* event);
static gboolean fullscreen_window_is_pointer_in_toolbar (FullscreenWindow* self);
static gboolean fullscreen_window_on_check_toolbar_invocation (FullscreenWindow* self);
static gboolean _fullscreen_window_on_check_toolbar_invocation_gsource_func (gpointer self);
static gboolean fullscreen_window_on_check_toolbar_dismissal (FullscreenWindow* self);
static gboolean _fullscreen_window_on_check_toolbar_dismissal_gsource_func (gpointer self);
static void fullscreen_window_finalize (GObject* obj);
#define PAGE_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PAGE_WINDOW, PageWindowPrivate))
enum  {
	PAGE_WINDOW_DUMMY_PROPERTY
};
void page_window_switched_pages (PageWindow* self, Page* old_page, Page* new_page);
static void page_window_real_switched_pages (PageWindow* self, Page* old_page, Page* new_page);
GtkUIManager* page_window_get_ui_manager (PageWindow* self);
static void page_window_real_set_current_page (PageWindow* self, Page* page);
void page_clear_container (Page* self);
void page_set_container (Page* self, GtkWindow* container);
static void page_window_real_clear_current_page (PageWindow* self);
static gboolean page_window_real_key_press_event (GtkWidget* base, GdkEventKey* event);
gboolean page_notify_app_key_pressed (Page* self, GdkEventKey* event);
static gboolean page_window_real_key_release_event (GtkWidget* base, GdkEventKey* event);
gboolean page_notify_app_key_released (Page* self, GdkEventKey* event);
static gboolean page_window_real_focus_in_event (GtkWidget* base, GdkEventFocus* event);
gboolean page_notify_app_focus_in (Page* self, GdkEventFocus* event);
static gboolean page_window_real_focus_out_event (GtkWidget* base, GdkEventFocus* event);
gboolean page_notify_app_focus_out (Page* self, GdkEventFocus* event);
static gboolean page_window_real_configure_event (GtkWidget* base, GdkEventConfigure* event);
gboolean page_notify_configure_event (Page* self, GdkEventConfigure* event);
void page_window_set_busy_cursor (PageWindow* self);
void spin_event_loop (void);
void page_window_set_normal_cursor (PageWindow* self);
static void page_window_finalize (GObject* obj);
GType dimensions_get_type (void) G_GNUC_CONST;
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
#define APP_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_APP_WINDOW, AppWindowPrivate))
enum  {
	APP_WINDOW_DUMMY_PROPERTY
};
gpointer command_manager_ref (gpointer instance);
void command_manager_unref (gpointer instance);
GParamSpec* param_spec_command_manager (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_command_manager (GValue* value, gpointer v_object);
void value_take_command_manager (GValue* value, gpointer v_object);
gpointer value_get_command_manager (const GValue* value);
GType command_manager_get_type (void) G_GNUC_CONST;
#define APP_WINDOW_DND_ICON_SCALE 128
AppWindow* app_window_construct (GType object_type);
#define RESOURCES_APP_TITLE "Shotwell"
GType library_window_get_type (void) G_GNUC_CONST;
void configuration_facade_get_library_window_state (ConfigurationFacade* self, gboolean* maximize, Dimensions* dimensions);
GType direct_window_get_type (void) G_GNUC_CONST;
void configuration_facade_get_direct_window_state (ConfigurationFacade* self, gboolean* maximize, Dimensions* dimensions);
#define COMMAND_MANAGER_DEFAULT_DEPTH 20
CommandManager* command_manager_new (gint depth);
CommandManager* command_manager_construct (GType object_type, gint depth);
static void app_window_on_command_manager_altered (AppWindow* self);
static void _app_window_on_command_manager_altered_command_manager_altered (CommandManager* _sender, gboolean can_undo, gboolean can_redo, gpointer self);
GtkActionGroup** app_window_create_common_action_groups (AppWindow* self, int* result_length1);
gchar* build_dummy_ui_string (GtkActionGroup** groups, int groups_length1);
#define RESOURCES_CUSTOM_CSS "LibraryWindow * {\n" \
"               -GtkPaned-handle-size: 1;\n" \
"            }\n" \
"            LibraryWindow .pane-separator {\n" \
"               background-color: @borders;\n" \
"            }\n" \
"            SearchFilterToolbar {\n" \
"               border-width: 0 0 1px 0;\n" \
"               border-style: solid;\n" \
"               border-color: @borders;\n" \
"            }"
static GtkActionEntry* app_window_create_common_actions (AppWindow* self, int* result_length1);
#define RESOURCES_QUIT_LABEL _ ("_Quit")
void app_window_on_quit (AppWindow* self);
static void _app_window_on_quit_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add159 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_ABOUT_LABEL _ ("_About")
void app_window_on_about (AppWindow* self);
static void _app_window_on_about_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add160 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_FULLSCREEN_LABEL _ ("Fulls_creen")
void app_window_on_fullscreen (AppWindow* self);
static void _app_window_on_fullscreen_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add161 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_HELP_LABEL _ ("_Help")
static void app_window_on_help_contents (AppWindow* self);
static void _app_window_on_help_contents_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add162 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static void app_window_on_help_faq (AppWindow* self);
static void _app_window_on_help_faq_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add163 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static void app_window_on_help_report_problem (AppWindow* self);
static void _app_window_on_help_report_problem_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add164 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_UNDO_MENU _ ("_Undo")
static void app_window_on_undo (AppWindow* self);
static void _app_window_on_undo_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add165 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_REDO_MENU _ ("_Redo")
static void app_window_on_redo (AppWindow* self);
static void _app_window_on_redo_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add166 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_JUMP_TO_FILE_MENU _ ("Show in File Mana_ger")
void app_window_on_jump_to_file (AppWindow* self);
static void _app_window_on_jump_to_file_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add167 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_SELECT_ALL_MENU _ ("Select _All")
static void app_window_on_select_all (AppWindow* self);
static void _app_window_on_select_all_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add168 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static void app_window_on_select_none (AppWindow* self);
static void _app_window_on_select_none_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add169 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static void app_window_real_on_fullscreen (AppWindow* self);
gboolean app_window_has_instance (void);
FullscreenWindow* app_window_get_fullscreen (void);
GtkBuilder* app_window_create_builder (const gchar* glade_filename, void* user);
GFile* app_dirs_get_resources_dir (void);
void app_window_error_message (const gchar* message, GtkWindow* parent);
void app_window_error_message_with_title (const gchar* title, const gchar* message, GtkWindow* parent, gboolean should_escape);
gchar* build_alert_body_text (const gchar* primary_text, const gchar* secondary_text, gboolean should_escape);
gboolean app_window_negate_affirm_question (const gchar* message, const gchar* negative, const gchar* affirmative, const gchar* title, GtkWindow* parent);
GtkResponseType app_window_negate_affirm_cancel_question (const gchar* message, const gchar* negative, const gchar* affirmative, const gchar* title, GtkWindow* parent);
GtkResponseType app_window_affirm_cancel_question (const gchar* message, const gchar* affirmative, const gchar* title, GtkWindow* parent);
GtkResponseType app_window_negate_affirm_all_cancel_question (const gchar* message, const gchar* negative, const gchar* affirmative, const gchar* affirmative_all, const gchar* title, GtkWindow* parent);
GQuark database_error_quark (void);
void app_window_database_error (GError* err);
void app_window_panic (const gchar* msg);
gpointer application_ref (gpointer instance);
void application_unref (gpointer instance);
GParamSpec* param_spec_application (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_application (GValue* value, gpointer v_object);
void value_take_application (GValue* value, gpointer v_object);
gpointer value_get_application (const GValue* value);
GType application_get_type (void) G_GNUC_CONST;
Application* application_get_instance (void);
void application_panic (Application* self);
gchar* app_window_get_app_role (AppWindow* self);
static gchar* app_window_real_get_app_role (AppWindow* self);
#define RESOURCES_APP_VERSION _VERSION
#define RESOURCES_COPYRIGHT _ ("Copyright 2016 Software Freedom Conservancy Inc.")
#define RESOURCES_HOME_URL "https://wiki.gnome.org/Apps/Shotwell"
#define RESOURCES_LICENSE "\n" \
"Shotwell is free software; you can redistribute it and/or modify it un" \
"der the \n" \
"terms of the GNU Lesser General Public License as published by the Fre" \
"e \n" \
"Software Foundation; either version 2.1 of the License, or (at your op" \
"tion) \n" \
"any later version.\n" \
"\n" \
"Shotwell is distributed in the hope that it will be useful, but WITHOU" \
"T \n" \
"ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or " \
"FITNESS\n" \
"FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License f" \
"or \n" \
"more details.\n" \
"\n" \
"You should have received a copy of the GNU Lesser General Public Licen" \
"se \n" \
"along with Shotwell; if not, write to the Free Software Foundation, In" \
"c., \n" \
"51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA\n"
GdkPixbuf* resources_get_icon (const gchar* name, gint scale);
#define RESOURCES_ICON_ABOUT_LOGO "shotwell-street.jpg"
void resources_launch_help (GdkScreen* screen, const gchar* anchor, GError** error);
void app_window_show_uri (AppWindow* self, const gchar* url, GError** error);
#define RESOURCES_BUG_DB_URL "https://wiki.gnome.org/Apps/Shotwell/ReportingABug"
#define RESOURCES_FAQ_URL "https://wiki.gnome.org/Apps/Shotwell/FAQ"
static void app_window_real_on_quit (AppWindow* self);
void application_exit (Application* self);
gpointer data_collection_ref (gpointer instance);
void data_collection_unref (gpointer instance);
GParamSpec* param_spec_data_collection (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_data_collection (GValue* value, gpointer v_object);
void value_take_data_collection (GValue* value, gpointer v_object);
gpointer value_get_data_collection (const GValue* value);
GType data_collection_get_type (void) G_GNUC_CONST;
GType view_collection_get_type (void) G_GNUC_CONST;
ViewCollection* page_get_view (Page* self);
gint view_collection_get_selected_count (ViewCollection* self);
GType data_object_get_type (void) G_GNUC_CONST;
GType data_source_get_type (void) G_GNUC_CONST;
GType thumbnail_source_get_type (void) G_GNUC_CONST;
GType media_source_get_type (void) G_GNUC_CONST;
GType data_view_get_type (void) G_GNUC_CONST;
DataView* view_collection_get_selected_at (ViewCollection* self, gint index);
DataSource* data_view_get_source (DataView* self);
void app_window_show_file_uri (AppWindow* self, GFile* file, GError** error);
GFile* media_source_get_master_file (MediaSource* self);
gchar* resources_jump_to_file_failed (GError* err);
static void app_window_real_destroy (GtkWidget* base);
gchar* get_nautilus_install_location (void);
void show_file_in_nautilus (const gchar* filename, GError** error);
void sys_show_uri (GdkScreen* screen, const gchar* uri, GError** error);
static GtkActionGroup** app_window_real_create_common_action_groups (AppWindow* self, int* result_length1);
static void _vala_array_add170 (GtkActionGroup*** array, int* length, int* size, GtkActionGroup* value);
GtkActionGroup** app_window_get_common_action_groups (AppWindow* self, int* result_length1);
static GtkActionGroup** _vala_array_dup28 (GtkActionGroup** self, int length);
void app_window_replace_common_placeholders (AppWindow* self, GtkUIManager* ui);
static void app_window_real_replace_common_placeholders (AppWindow* self, GtkUIManager* ui);
void app_window_go_fullscreen (AppWindow* self, Page* page);
void page_switching_to_fullscreen (Page* self, FullscreenWindow* fsw);
void page_returning_from_fullscreen (Page* self, FullscreenWindow* fsw);
GtkAction* app_window_get_common_action (AppWindow* self, const gchar* name);
void app_window_set_common_action_sensitive (AppWindow* self, const gchar* name, gboolean sensitive);
void app_window_set_common_action_important (AppWindow* self, const gchar* name, gboolean important);
void app_window_set_common_action_visible (AppWindow* self, const gchar* name, gboolean visible);
static void app_window_real_switched_pages (PageWindow* base, Page* old_page, Page* new_page);
void app_window_update_common_action_availability (AppWindow* self, Page* old_page, Page* new_page);
static void app_window_on_update_common_actions (AppWindow* self);
static void _app_window_on_update_common_actions_data_collection_contents_altered (DataCollection* _sender, GeeIterable* added, GeeIterable* removed, gpointer self);
static void _app_window_on_update_common_actions_view_collection_selection_group_altered (ViewCollection* _sender, gpointer self);
static void _app_window_on_update_common_actions_view_collection_items_state_changed (ViewCollection* _sender, GeeIterable* changed, gpointer self);
void app_window_update_common_actions (AppWindow* self, Page* page, gint selected_count, gint count);
gint data_collection_get_count (DataCollection* self);
static void app_window_real_update_common_action_availability (AppWindow* self, Page* old_page, Page* new_page);
GType checkerboard_page_get_type (void) G_GNUC_CONST;
static void app_window_real_update_common_actions (AppWindow* self, Page* page, gint selected_count, gint count);
void app_window_decorate_undo_action (AppWindow* self);
void app_window_decorate_redo_action (AppWindow* self);
CommandManager* app_window_get_command_manager (void);
GType command_description_get_type (void) G_GNUC_CONST;
static void app_window_decorate_command_manager_action (AppWindow* self, const gchar* name, const gchar* prefix, const gchar* default_explanation, CommandDescription* desc);
gchar* command_description_get_name (CommandDescription* self);
gchar* command_description_get_explanation (CommandDescription* self);
CommandDescription* command_manager_get_undo_description (CommandManager* self);
CommandDescription* command_manager_get_redo_description (CommandManager* self);
gboolean command_manager_undo (CommandManager* self);
gboolean command_manager_redo (CommandManager* self);
void view_collection_select_all (ViewCollection* self);
void view_collection_unselect_all (ViewCollection* self);
static gboolean app_window_real_configure_event (GtkWidget* base, GdkEventConfigure* event);
static void app_window_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

extern const gchar* RESOURCES_AUTHORS[6];

static gpointer _g_object_ref0 (gpointer self) {
#line 38 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return self ? g_object_ref (self) : NULL;
#line 613 "AppWindow.c"
}


static void _fullscreen_window_update_toolbar_dismissal_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 57 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_update_toolbar_dismissal ((FullscreenWindow*) self);
#line 620 "AppWindow.c"
}


static void _fullscreen_window_on_close_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 61 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_on_close ((FullscreenWindow*) self);
#line 627 "AppWindow.c"
}


static void _fullscreen_window_hide_toolbar_slideshow_page_hide_toolbar (SlideshowPage* _sender, gpointer self) {
#line 72 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_hide_toolbar ((FullscreenWindow*) self);
#line 634 "AppWindow.c"
}


FullscreenWindow* fullscreen_window_construct (GType object_type, Page* page) {
	FullscreenWindow * self = NULL;
	Page* _tmp0_ = NULL;
	GFile* ui_file = NULL;
	GFile* _tmp1_ = NULL;
	GtkActionGroup* action_group = NULL;
	GtkActionGroup* _tmp8_ = NULL;
	GtkActionGroup* _tmp9_ = NULL;
	gint _tmp10_ = 0;
	GtkActionEntry* _tmp11_ = NULL;
	GtkActionEntry* _tmp12_ = NULL;
	gint _tmp12__length1 = 0;
	GtkUIManager* _tmp13_ = NULL;
	GtkActionGroup* _tmp14_ = NULL;
	GtkUIManager* _tmp15_ = NULL;
	GtkAccelGroup* accel_group = NULL;
	GtkUIManager* _tmp16_ = NULL;
	GtkAccelGroup* _tmp17_ = NULL;
	GtkAccelGroup* _tmp18_ = NULL;
	GtkAccelGroup* _tmp19_ = NULL;
	AppWindow* _tmp21_ = NULL;
	AppWindow* _tmp22_ = NULL;
	GdkScreen* _tmp23_ = NULL;
	GdkRectangle monitor = {0};
	GdkRectangle _tmp24_ = {0};
	GdkRectangle _tmp25_ = {0};
	gint _tmp26_ = 0;
	GdkRectangle _tmp27_ = {0};
	gint _tmp28_ = 0;
	ConfigFacade* _tmp29_ = NULL;
	ConfigFacade* _tmp30_ = NULL;
	gboolean _tmp31_ = FALSE;
	GtkToggleToolButton* _tmp32_ = NULL;
	GtkToggleToolButton* _tmp33_ = NULL;
	const gchar* _tmp34_ = NULL;
	GtkToggleToolButton* _tmp35_ = NULL;
	const gchar* _tmp36_ = NULL;
	GtkToggleToolButton* _tmp37_ = NULL;
	gboolean _tmp38_ = FALSE;
	GtkToggleToolButton* _tmp39_ = NULL;
	GtkToolButton* _tmp40_ = NULL;
	GtkToolButton* _tmp41_ = NULL;
	const gchar* _tmp42_ = NULL;
	GtkToolButton* _tmp43_ = NULL;
	Page* _tmp44_ = NULL;
	GtkToolbar* _tmp45_ = NULL;
	GtkToolbar* _tmp46_ = NULL;
	GtkToolbar* _tmp47_ = NULL;
	GtkToolbar* _tmp48_ = NULL;
	GtkToolbar* _tmp49_ = NULL;
	GtkToolbar* _tmp50_ = NULL;
	Page* _tmp51_ = NULL;
	Page* _tmp55_ = NULL;
	Page* _tmp56_ = NULL;
	GtkToolbar* _tmp57_ = NULL;
	GtkToolButton* _tmp58_ = NULL;
	GtkOverlay* _tmp59_ = NULL;
	GtkOverlay* _tmp60_ = NULL;
	Page* _tmp61_ = NULL;
	GtkOverlay* _tmp62_ = NULL;
	GtkToolbar* _tmp63_ = NULL;
	GdkRectangle _tmp64_ = {0};
	gint _tmp65_ = 0;
	GdkRectangle _tmp66_ = {0};
	gint _tmp67_ = 0;
	GError * _inner_error_ = NULL;
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_PAGE (page), NULL);
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = (FullscreenWindow*) page_window_construct (object_type);
#line 23 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page;
#line 23 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_window_set_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), _tmp0_);
#line 25 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = resources_get_ui ("fullscreen.ui");
#line 25 "/home/jens/Source/shotwell/src/AppWindow.vala"
	ui_file = _tmp1_;
#line 716 "AppWindow.c"
	{
		GtkUIManager* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 28 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 28 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = g_file_get_path (ui_file);
#line 28 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp3_;
#line 28 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_ui_manager_add_ui_from_file (_tmp2_, _tmp4_, &_inner_error_);
#line 28 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp4_);
#line 28 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 733 "AppWindow.c"
			goto __catch211_g_error;
		}
	}
	goto __finally211;
	__catch211_g_error:
	{
		GError* err = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 30 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = g_file_get_path (ui_file);
#line 30 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = _tmp5_;
#line 30 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = err->message;
#line 30 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_error ("AppWindow.vala:30: Error loading UI file %s: %s", _tmp6_, _tmp7_);
#line 30 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp6_);
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 760 "AppWindow.c"
	}
	__finally211:
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (ui_file);
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 27 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return NULL;
#line 773 "AppWindow.c"
	}
#line 33 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = gtk_action_group_new ("FullscreenActionGroup");
#line 33 "/home/jens/Source/shotwell/src/AppWindow.vala"
	action_group = _tmp8_;
#line 34 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = action_group;
#line 34 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = fullscreen_window_create_actions (self, &_tmp10_);
#line 34 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = _tmp11_;
#line 34 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12__length1 = _tmp10_;
#line 34 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_action_group_add_actions (_tmp9_, _tmp12_, _tmp10_, self);
#line 34 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = (g_free (_tmp12_), NULL);
#line 35 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 35 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = action_group;
#line 35 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_ui_manager_insert_action_group (_tmp13_, _tmp14_, 0);
#line 36 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 36 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_ui_manager_ensure_update (_tmp15_);
#line 38 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 38 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = gtk_ui_manager_get_accel_group (_tmp16_);
#line 38 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_ = _g_object_ref0 (_tmp17_);
#line 38 "/home/jens/Source/shotwell/src/AppWindow.vala"
	accel_group = _tmp18_;
#line 39 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp19_ = accel_group;
#line 39 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp19_ != NULL) {
#line 813 "AppWindow.c"
		GtkAccelGroup* _tmp20_ = NULL;
#line 40 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp20_ = accel_group;
#line 40 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_window_add_accel_group (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp20_);
#line 819 "AppWindow.c"
	}
#line 42 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp21_ = app_window_get_instance ();
#line 42 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_ = _tmp21_;
#line 42 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp23_ = gtk_window_get_screen (G_TYPE_CHECK_INSTANCE_CAST (_tmp22_, gtk_window_get_type (), GtkWindow));
#line 42 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_screen (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp23_);
#line 42 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp22_);
#line 45 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_get_monitor_geometry (self, &_tmp24_);
#line 45 "/home/jens/Source/shotwell/src/AppWindow.vala"
	monitor = _tmp24_;
#line 46 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_ = monitor;
#line 46 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp26_ = _tmp25_.x;
#line 46 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp27_ = monitor;
#line 46 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_ = _tmp27_.y;
#line 46 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_move (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp26_, _tmp28_);
#line 48 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_container_set_border_width (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_container_get_type (), GtkContainer), (guint) 0);
#line 51 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp29_ = config_facade_get_instance ();
#line 51 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp30_ = _tmp29_;
#line 51 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_ = configuration_facade_get_pin_toolbar_state (G_TYPE_CHECK_INSTANCE_CAST (_tmp30_, TYPE_CONFIGURATION_FACADE, ConfigurationFacade));
#line 51 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->is_toolbar_dismissal_enabled = _tmp31_;
#line 51 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp30_);
#line 53 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp32_ = self->priv->pin_button;
#line 53 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_tool_button_set_icon_name (G_TYPE_CHECK_INSTANCE_CAST (_tmp32_, gtk_tool_button_get_type (), GtkToolButton), "pin-toolbar");
#line 54 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp33_ = self->priv->pin_button;
#line 54 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_ = _ ("Pin Toolbar");
#line 54 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_tool_button_set_label (G_TYPE_CHECK_INSTANCE_CAST (_tmp33_, gtk_tool_button_get_type (), GtkToolButton), _tmp34_);
#line 55 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp35_ = self->priv->pin_button;
#line 55 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp36_ = _ ("Pin the toolbar open");
#line 55 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_tool_item_set_tooltip_text (G_TYPE_CHECK_INSTANCE_CAST (_tmp35_, gtk_tool_item_get_type (), GtkToolItem), _tmp36_);
#line 56 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp37_ = self->priv->pin_button;
#line 56 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp38_ = self->priv->is_toolbar_dismissal_enabled;
#line 56 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_toggle_tool_button_set_active (_tmp37_, !_tmp38_);
#line 57 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp39_ = self->priv->pin_button;
#line 57 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp39_, gtk_tool_button_get_type (), GtkToolButton), "clicked", (GCallback) _fullscreen_window_update_toolbar_dismissal_gtk_tool_button_clicked, self, 0);
#line 59 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp40_ = self->priv->close_button;
#line 59 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_tool_button_set_icon_name (_tmp40_, "view-restore");
#line 60 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp41_ = self->priv->close_button;
#line 60 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp42_ = _ ("Leave fullscreen");
#line 60 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_tool_item_set_tooltip_text (G_TYPE_CHECK_INSTANCE_CAST (_tmp41_, gtk_tool_item_get_type (), GtkToolItem), _tmp42_);
#line 61 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp43_ = self->priv->close_button;
#line 61 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_signal_connect_object (_tmp43_, "clicked", (GCallback) _fullscreen_window_on_close_gtk_tool_button_clicked, self, 0);
#line 63 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp44_ = page;
#line 63 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp45_ = page_get_toolbar (_tmp44_);
#line 63 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->toolbar);
#line 63 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->toolbar = _tmp45_;
#line 64 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp46_ = self->priv->toolbar;
#line 64 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_toolbar_set_show_arrow (_tmp46_, FALSE);
#line 65 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp47_ = self->priv->toolbar;
#line 65 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_set_valign (G_TYPE_CHECK_INSTANCE_CAST (_tmp47_, gtk_widget_get_type (), GtkWidget), GTK_ALIGN_END);
#line 66 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp48_ = self->priv->toolbar;
#line 66 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_set_halign (G_TYPE_CHECK_INSTANCE_CAST (_tmp48_, gtk_widget_get_type (), GtkWidget), GTK_ALIGN_CENTER);
#line 67 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp49_ = self->priv->toolbar;
#line 67 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp49_, gtk_widget_get_type (), GtkWidget), "expand", FALSE, NULL);
#line 68 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp50_ = self->priv->toolbar;
#line 68 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_set_opacity (G_TYPE_CHECK_INSTANCE_CAST (_tmp50_, gtk_widget_get_type (), GtkWidget), RESOURCES_TRANSIENT_WINDOW_OPACITY);
#line 70 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp51_ = page;
#line 70 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp51_, TYPE_SLIDESHOW_PAGE)) {
#line 929 "AppWindow.c"
		Page* _tmp52_ = NULL;
#line 72 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp52_ = page;
#line 72 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp52_, TYPE_SLIDESHOW_PAGE, SlideshowPage), "hide-toolbar", (GCallback) _fullscreen_window_hide_toolbar_slideshow_page_hide_toolbar, self, 0);
#line 935 "AppWindow.c"
	} else {
		GtkToolbar* _tmp53_ = NULL;
		GtkToggleToolButton* _tmp54_ = NULL;
#line 75 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp53_ = self->priv->toolbar;
#line 75 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp54_ = self->priv->pin_button;
#line 75 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_toolbar_insert (_tmp53_, G_TYPE_CHECK_INSTANCE_CAST (_tmp54_, gtk_tool_item_get_type (), GtkToolItem), -1);
#line 945 "AppWindow.c"
	}
#line 78 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp55_ = page;
#line 78 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_set_cursor_hide_time (_tmp55_, FULLSCREEN_WINDOW_TOOLBAR_DISMISSAL_SEC * 1000);
#line 79 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp56_ = page;
#line 79 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_start_cursor_hiding (_tmp56_);
#line 81 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp57_ = self->priv->toolbar;
#line 81 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp58_ = self->priv->close_button;
#line 81 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_toolbar_insert (_tmp57_, G_TYPE_CHECK_INSTANCE_CAST (_tmp58_, gtk_tool_item_get_type (), GtkToolItem), -1);
#line 83 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp59_ = self->priv->overlay;
#line 83 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_container_add (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_container_get_type (), GtkContainer), G_TYPE_CHECK_INSTANCE_CAST (_tmp59_, gtk_widget_get_type (), GtkWidget));
#line 84 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp60_ = self->priv->overlay;
#line 84 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp61_ = page;
#line 84 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_container_add (G_TYPE_CHECK_INSTANCE_CAST (_tmp60_, gtk_container_get_type (), GtkContainer), G_TYPE_CHECK_INSTANCE_CAST (_tmp61_, gtk_widget_get_type (), GtkWidget));
#line 85 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp62_ = self->priv->overlay;
#line 85 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp63_ = self->priv->toolbar;
#line 85 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_overlay_add_overlay (_tmp62_, G_TYPE_CHECK_INSTANCE_CAST (_tmp63_, gtk_widget_get_type (), GtkWidget));
#line 90 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp64_ = monitor;
#line 90 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp65_ = _tmp64_.width;
#line 90 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp66_ = monitor;
#line 90 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp67_ = _tmp66_.height;
#line 90 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_default_size (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp65_, _tmp67_);
#line 93 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_fullscreen (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 94 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_show_all (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 97 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_add_events (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget), (gint) GDK_POINTER_MOTION_MASK);
#line 100 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_invoke_toolbar (self);
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (accel_group);
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (action_group);
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (ui_file);
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return self;
#line 1003 "AppWindow.c"
}


FullscreenWindow* fullscreen_window_new (Page* page) {
#line 22 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return fullscreen_window_construct (TYPE_FULLSCREEN_WINDOW, page);
#line 1010 "AppWindow.c"
}


void fullscreen_window_disable_toolbar_dismissal (FullscreenWindow* self) {
#line 103 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 104 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->is_toolbar_dismissal_enabled = FALSE;
#line 1019 "AppWindow.c"
}


void fullscreen_window_update_toolbar_dismissal (FullscreenWindow* self) {
	GtkToggleToolButton* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 107 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 108 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->pin_button;
#line 108 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gtk_toggle_tool_button_get_active (_tmp0_);
#line 108 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->is_toolbar_dismissal_enabled = !_tmp1_;
#line 1034 "AppWindow.c"
}


static void fullscreen_window_get_monitor_geometry (FullscreenWindow* self, GdkRectangle* result) {
	GdkRectangle monitor = {0};
	GdkScreen* _tmp0_ = NULL;
	GdkScreen* _tmp1_ = NULL;
	AppWindow* _tmp2_ = NULL;
	AppWindow* _tmp3_ = NULL;
	GdkWindow* _tmp4_ = NULL;
	gint _tmp5_ = 0;
	GdkRectangle _tmp6_ = {0};
#line 111 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_window_get_screen (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gtk_window_get_screen (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = app_window_get_instance ();
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _tmp2_;
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = gtk_widget_get_window (G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, gtk_widget_get_type (), GtkWidget));
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = gdk_screen_get_monitor_at_window (_tmp1_, _tmp4_);
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gdk_screen_get_monitor_geometry (_tmp0_, _tmp5_, &_tmp6_);
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	monitor = _tmp6_;
#line 114 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp3_);
#line 117 "/home/jens/Source/shotwell/src/AppWindow.vala"
	*result = monitor;
#line 117 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return;
#line 1071 "AppWindow.c"
}


static gboolean fullscreen_window_real_configure_event (GtkWidget* base, GdkEventConfigure* event) {
	FullscreenWindow * self;
	gboolean result = FALSE;
	gboolean _result_ = FALSE;
	GdkEventConfigure* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
#line 120 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_FULLSCREEN_WINDOW, FullscreenWindow);
#line 120 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 121 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = event;
#line 121 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = GTK_WIDGET_CLASS (fullscreen_window_parent_class)->configure_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), gtk_widget_get_type (), GtkWidget), _tmp0_);
#line 121 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_result_ = _tmp1_;
#line 123 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = self->priv->switched_to;
#line 123 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (!_tmp2_) {
#line 1096 "AppWindow.c"
		Page* _tmp3_ = NULL;
		Page* _tmp4_ = NULL;
#line 124 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 124 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp3_;
#line 124 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_switched_to (_tmp4_);
#line 124 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp4_);
#line 125 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->priv->switched_to = TRUE;
#line 1109 "AppWindow.c"
	}
#line 128 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _result_;
#line 128 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1115 "AppWindow.c"
}


static void _fullscreen_window_on_close_gtk_action_callback (GtkAction* action, gpointer self) {
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_on_close ((FullscreenWindow*) self);
#line 1122 "AppWindow.c"
}


static void _vala_array_add158 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 1133 "AppWindow.c"
	}
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 1137 "AppWindow.c"
}


static GtkActionEntry* fullscreen_window_create_actions (FullscreenWindow* self, int* result_length1) {
	GtkActionEntry* result = NULL;
	GtkActionEntry* actions = NULL;
	GtkActionEntry* _tmp0_ = NULL;
	gint actions_length1 = 0;
	gint _actions_size_ = 0;
	GtkActionEntry leave_fullscreen = {0};
	GtkActionEntry _tmp1_ = {0};
	GtkActionEntry* _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	GtkActionEntry _tmp3_ = {0};
	GtkActionEntry* _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
#line 131 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), NULL);
#line 132 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = g_new0 (GtkActionEntry, 0);
#line 132 "/home/jens/Source/shotwell/src/AppWindow.vala"
	actions = _tmp0_;
#line 132 "/home/jens/Source/shotwell/src/AppWindow.vala"
	actions_length1 = 0;
#line 132 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_actions_size_ = actions_length1;
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.name = "LeaveFullscreen";
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.stock_id = RESOURCES_LEAVE_FULLSCREEN_LABEL;
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.label = TRANSLATABLE;
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.accelerator = "F11";
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.tooltip = TRANSLATABLE;
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.callback = (GCallback) _fullscreen_window_on_close_gtk_action_callback;
#line 134 "/home/jens/Source/shotwell/src/AppWindow.vala"
	leave_fullscreen = _tmp1_;
#line 136 "/home/jens/Source/shotwell/src/AppWindow.vala"
	leave_fullscreen.label = RESOURCES_LEAVE_FULLSCREEN_LABEL;
#line 137 "/home/jens/Source/shotwell/src/AppWindow.vala"
	leave_fullscreen.tooltip = RESOURCES_LEAVE_FULLSCREEN_LABEL;
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = actions;
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2__length1 = actions_length1;
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = leave_fullscreen;
#line 138 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add158 (&actions, &actions_length1, &_actions_size_, &_tmp3_);
#line 140 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = actions;
#line 140 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4__length1 = actions_length1;
#line 140 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (result_length1) {
#line 140 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*result_length1 = _tmp4__length1;
#line 1198 "AppWindow.c"
	}
#line 140 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp4_;
#line 140 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1204 "AppWindow.c"
}


static gboolean fullscreen_window_real_key_press_event (GtkWidget* base, GdkEventKey* event) {
	FullscreenWindow * self;
	gboolean result = FALSE;
	GdkEventKey* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	const gchar* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
	AppWindow* _tmp6_ = NULL;
	AppWindow* _tmp7_ = NULL;
	GdkEventKey* _tmp8_ = NULL;
	gboolean _tmp9_ = FALSE;
	gboolean _tmp10_ = FALSE;
#line 143 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_FULLSCREEN_WINDOW, FullscreenWindow);
#line 143 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 145 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = event;
#line 145 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_->keyval;
#line 145 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = gdk_keyval_name (_tmp1_);
#line 145 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (g_strcmp0 (_tmp2_, "Escape") == 0) {
#line 146 "/home/jens/Source/shotwell/src/AppWindow.vala"
		fullscreen_window_on_close (self);
#line 148 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 148 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1238 "AppWindow.c"
	}
#line 152 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (fullscreen_window_parent_class)->key_press_event != NULL) {
#line 1242 "AppWindow.c"
		GdkEventKey* _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
#line 152 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = event;
#line 152 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = GTK_WIDGET_CLASS (fullscreen_window_parent_class)->key_press_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), gtk_widget_get_type (), GtkWidget), _tmp4_);
#line 152 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _tmp5_;
#line 1251 "AppWindow.c"
	} else {
#line 152 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = FALSE;
#line 1255 "AppWindow.c"
	}
#line 152 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp3_) {
#line 153 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 153 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1263 "AppWindow.c"
	}
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = app_window_get_instance ();
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = _tmp6_;
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = event;
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_signal_emit_by_name (G_TYPE_CHECK_INSTANCE_CAST (_tmp7_, gtk_widget_get_type (), GtkWidget), "key-press-event", _tmp8_, &_tmp9_);
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = _tmp9_;
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp7_);
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp10_;
#line 156 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1281 "AppWindow.c"
}


static void fullscreen_window_on_close (FullscreenWindow* self) {
	ConfigFacade* _tmp0_ = NULL;
	ConfigFacade* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	AppWindow* _tmp3_ = NULL;
	AppWindow* _tmp4_ = NULL;
#line 159 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 160 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = config_facade_get_instance ();
#line 160 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 160 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = self->priv->is_toolbar_dismissal_enabled;
#line 160 "/home/jens/Source/shotwell/src/AppWindow.vala"
	configuration_facade_set_pin_toolbar_state (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_CONFIGURATION_FACADE, ConfigurationFacade), _tmp2_);
#line 160 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp1_);
#line 161 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_hide_toolbar (self);
#line 163 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = app_window_get_instance ();
#line 163 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = _tmp3_;
#line 163 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_end_fullscreen (_tmp4_);
#line 163 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp4_);
#line 1313 "AppWindow.c"
}


void fullscreen_window_close (FullscreenWindow* self) {
#line 166 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 167 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_on_close (self);
#line 1322 "AppWindow.c"
}


static void fullscreen_window_real_destroy (GtkWidget* base) {
	FullscreenWindow * self;
	Page* page = NULL;
	Page* _tmp0_ = NULL;
	Page* _tmp1_ = NULL;
#line 170 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_FULLSCREEN_WINDOW, FullscreenWindow);
#line 171 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 171 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page = _tmp0_;
#line 172 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_window_clear_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 174 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = page;
#line 174 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 1343 "AppWindow.c"
		Page* _tmp2_ = NULL;
		Page* _tmp3_ = NULL;
#line 175 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = page;
#line 175 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_stop_cursor_hiding (_tmp2_);
#line 176 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = page;
#line 176 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_switching_from (_tmp3_);
#line 1354 "AppWindow.c"
	}
#line 179 "/home/jens/Source/shotwell/src/AppWindow.vala"
	GTK_WIDGET_CLASS (fullscreen_window_parent_class)->destroy (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), gtk_widget_get_type (), GtkWidget));
#line 170 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (page);
#line 1360 "AppWindow.c"
}


static gboolean fullscreen_window_real_delete_event (GtkWidget* base, GdkEventAny* event) {
	FullscreenWindow * self;
	gboolean result = FALSE;
	AppWindow* _tmp0_ = NULL;
	AppWindow* _tmp1_ = NULL;
#line 182 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_FULLSCREEN_WINDOW, FullscreenWindow);
#line 182 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 183 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_on_close (self);
#line 184 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_get_instance ();
#line 184 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 184 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, gtk_widget_get_type (), GtkWidget));
#line 184 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp1_);
#line 186 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = TRUE;
#line 186 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1387 "AppWindow.c"
}


static gboolean _fullscreen_window_on_check_toolbar_invocation_gsource_func (gpointer self) {
	gboolean result;
	result = fullscreen_window_on_check_toolbar_invocation ((FullscreenWindow*) self);
#line 195 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1396 "AppWindow.c"
}


static gboolean fullscreen_window_real_motion_notify_event (GtkWidget* base, GdkEventMotion* event) {
	FullscreenWindow * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp4_ = FALSE;
#line 189 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_FULLSCREEN_WINDOW, FullscreenWindow);
#line 189 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 190 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->is_toolbar_shown;
#line 190 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (!_tmp0_) {
#line 1413 "AppWindow.c"
		gboolean _tmp1_ = FALSE;
		gboolean _tmp2_ = FALSE;
#line 194 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = self->priv->waiting_for_invoke;
#line 194 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (!_tmp2_) {
#line 1420 "AppWindow.c"
			gboolean _tmp3_ = FALSE;
#line 194 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_tmp3_ = fullscreen_window_is_pointer_in_toolbar (self);
#line 194 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_tmp1_ = _tmp3_;
#line 1426 "AppWindow.c"
		} else {
#line 194 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_tmp1_ = FALSE;
#line 1430 "AppWindow.c"
		}
#line 194 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (_tmp1_) {
#line 195 "/home/jens/Source/shotwell/src/AppWindow.vala"
			g_timeout_add_full (G_PRIORITY_DEFAULT, (guint) FULLSCREEN_WINDOW_TOOLBAR_INVOCATION_MSEC, _fullscreen_window_on_check_toolbar_invocation_gsource_func, g_object_ref (self), g_object_unref);
#line 196 "/home/jens/Source/shotwell/src/AppWindow.vala"
			self->priv->waiting_for_invoke = TRUE;
#line 1438 "AppWindow.c"
		}
	}
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (fullscreen_window_parent_class)->motion_notify_event != NULL) {
#line 1443 "AppWindow.c"
		GdkEventMotion* _tmp5_ = NULL;
		gboolean _tmp6_ = FALSE;
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = event;
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = GTK_WIDGET_CLASS (fullscreen_window_parent_class)->motion_notify_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), gtk_widget_get_type (), GtkWidget), _tmp5_);
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp6_;
#line 1452 "AppWindow.c"
	} else {
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = FALSE;
#line 1456 "AppWindow.c"
	}
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp4_;
#line 200 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1462 "AppWindow.c"
}


static gboolean fullscreen_window_is_pointer_in_toolbar (FullscreenWindow* self) {
	gboolean result = FALSE;
	GdkDeviceManager* devmgr = NULL;
	GdkDisplay* _tmp0_ = NULL;
	GdkDeviceManager* _tmp1_ = NULL;
	GdkDeviceManager* _tmp2_ = NULL;
	GdkDeviceManager* _tmp3_ = NULL;
	gint py = 0;
	GdkDeviceManager* _tmp4_ = NULL;
	GdkDevice* _tmp5_ = NULL;
	gint _tmp6_ = 0;
	gint wy = 0;
	GtkToolbar* _tmp7_ = NULL;
	GdkWindow* _tmp8_ = NULL;
	gint _tmp9_ = 0;
	gint _tmp10_ = 0;
	gint _tmp11_ = 0;
#line 203 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 204 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_widget_get_display (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 204 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gdk_display_get_device_manager (_tmp0_);
#line 204 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 204 "/home/jens/Source/shotwell/src/AppWindow.vala"
	devmgr = _tmp2_;
#line 205 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = devmgr;
#line 205 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp3_ == NULL) {
#line 206 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_debug ("AppWindow.vala:206: No device manager for display");
#line 208 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = FALSE;
#line 208 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (devmgr);
#line 208 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1505 "AppWindow.c"
	}
#line 212 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = devmgr;
#line 212 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = gdk_device_manager_get_client_pointer (_tmp4_);
#line 212 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gdk_device_get_position (_tmp5_, NULL, NULL, &_tmp6_);
#line 212 "/home/jens/Source/shotwell/src/AppWindow.vala"
	py = _tmp6_;
#line 215 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = self->priv->toolbar;
#line 215 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = gtk_widget_get_window (G_TYPE_CHECK_INSTANCE_CAST (_tmp7_, gtk_widget_get_type (), GtkWidget));
#line 215 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gdk_window_get_geometry (_tmp8_, NULL, &_tmp9_, NULL, NULL);
#line 215 "/home/jens/Source/shotwell/src/AppWindow.vala"
	wy = _tmp9_;
#line 217 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = py;
#line 217 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = wy;
#line 217 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp10_ >= _tmp11_;
#line 217 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (devmgr);
#line 217 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1533 "AppWindow.c"
}


static gboolean fullscreen_window_on_check_toolbar_invocation (FullscreenWindow* self) {
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
#line 220 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 221 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->waiting_for_invoke = FALSE;
#line 223 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->is_toolbar_shown;
#line 223 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_) {
#line 224 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = FALSE;
#line 224 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1553 "AppWindow.c"
	}
#line 226 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = fullscreen_window_is_pointer_in_toolbar (self);
#line 226 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (!_tmp1_) {
#line 227 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = FALSE;
#line 227 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1563 "AppWindow.c"
	}
#line 229 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_invoke_toolbar (self);
#line 231 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = FALSE;
#line 231 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1571 "AppWindow.c"
}


static gboolean _fullscreen_window_on_check_toolbar_dismissal_gsource_func (gpointer self) {
	gboolean result;
	result = fullscreen_window_on_check_toolbar_dismissal ((FullscreenWindow*) self);
#line 239 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1580 "AppWindow.c"
}


static void fullscreen_window_invoke_toolbar (FullscreenWindow* self) {
	GtkToolbar* _tmp0_ = NULL;
#line 234 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 235 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->toolbar;
#line 235 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_show_all (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 237 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->is_toolbar_shown = TRUE;
#line 239 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_timeout_add_full (G_PRIORITY_DEFAULT, (guint) FULLSCREEN_WINDOW_TOOLBAR_CHECK_DISMISSAL_MSEC, _fullscreen_window_on_check_toolbar_dismissal_gsource_func, g_object_ref (self), g_object_unref);
#line 1596 "AppWindow.c"
}


static gboolean fullscreen_window_on_check_toolbar_dismissal (FullscreenWindow* self) {
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	time_t _tmp3_ = 0;
	time_t now = 0;
	time_t _tmp5_ = 0;
	time_t _tmp6_ = 0;
	time_t _tmp7_ = 0;
	time_t _tmp8_ = 0;
	time_t _tmp9_ = 0;
#line 242 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 243 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->is_toolbar_shown;
#line 243 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (!_tmp0_) {
#line 244 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = FALSE;
#line 244 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1622 "AppWindow.c"
	}
#line 247 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = self->priv->is_toolbar_dismissal_enabled;
#line 247 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (!_tmp1_) {
#line 248 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 248 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1632 "AppWindow.c"
	}
#line 251 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = fullscreen_window_is_pointer_in_toolbar (self);
#line 251 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_) {
#line 252 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->priv->left_toolbar_time = (time_t) 0;
#line 254 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 254 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1644 "AppWindow.c"
	}
#line 258 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = self->priv->left_toolbar_time;
#line 258 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp3_ == ((time_t) 0)) {
#line 1650 "AppWindow.c"
		time_t _tmp4_ = 0;
#line 259 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = time (NULL);
#line 259 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->priv->left_toolbar_time = _tmp4_;
#line 261 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 261 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1660 "AppWindow.c"
	}
#line 265 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = time (NULL);
#line 265 "/home/jens/Source/shotwell/src/AppWindow.vala"
	now = _tmp5_;
#line 266 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = now;
#line 266 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = self->priv->left_toolbar_time;
#line 266 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_assert (_tmp6_ >= _tmp7_, "now >= left_toolbar_time");
#line 268 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = now;
#line 268 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = self->priv->left_toolbar_time;
#line 268 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((_tmp8_ - _tmp9_) < ((time_t) FULLSCREEN_WINDOW_TOOLBAR_DISMISSAL_SEC)) {
#line 269 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 269 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 1682 "AppWindow.c"
	}
#line 271 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_hide_toolbar (self);
#line 273 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = FALSE;
#line 273 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1690 "AppWindow.c"
}


static void fullscreen_window_hide_toolbar (FullscreenWindow* self) {
	GtkToolbar* _tmp0_ = NULL;
#line 276 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 277 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->toolbar;
#line 277 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_hide (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 278 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->is_toolbar_shown = FALSE;
#line 1704 "AppWindow.c"
}


static void fullscreen_window_class_init (FullscreenWindowClass * klass) {
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen_window_parent_class = g_type_class_peek_parent (klass);
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_type_class_add_private (klass, sizeof (FullscreenWindowPrivate));
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->configure_event = fullscreen_window_real_configure_event;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->key_press_event = fullscreen_window_real_key_press_event;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->destroy = fullscreen_window_real_destroy;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->delete_event = fullscreen_window_real_delete_event;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->motion_notify_event = fullscreen_window_real_motion_notify_event;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	G_OBJECT_CLASS (klass)->finalize = fullscreen_window_finalize;
#line 1725 "AppWindow.c"
}


static void fullscreen_window_instance_init (FullscreenWindow * self) {
	GtkOverlay* _tmp0_ = NULL;
	GtkToolButton* _tmp1_ = NULL;
	GtkToggleToolButton* _tmp2_ = NULL;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv = FULLSCREEN_WINDOW_GET_PRIVATE (self);
#line 12 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = (GtkOverlay*) gtk_overlay_new ();
#line 12 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp0_);
#line 12 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->overlay = _tmp0_;
#line 13 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->toolbar = NULL;
#line 14 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = (GtkToolButton*) gtk_tool_button_new (NULL, NULL);
#line 14 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp1_);
#line 14 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->close_button = _tmp1_;
#line 15 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = (GtkToggleToolButton*) gtk_toggle_tool_button_new ();
#line 15 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp2_);
#line 15 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->pin_button = _tmp2_;
#line 16 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->is_toolbar_shown = FALSE;
#line 17 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->waiting_for_invoke = FALSE;
#line 18 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->left_toolbar_time = (time_t) 0;
#line 19 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->switched_to = FALSE;
#line 1763 "AppWindow.c"
}


static void fullscreen_window_finalize (GObject* obj) {
	FullscreenWindow * self;
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_FULLSCREEN_WINDOW, FullscreenWindow);
#line 12 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->overlay);
#line 13 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->toolbar);
#line 14 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->close_button);
#line 15 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->pin_button);
#line 7 "/home/jens/Source/shotwell/src/AppWindow.vala"
	G_OBJECT_CLASS (fullscreen_window_parent_class)->finalize (obj);
#line 1781 "AppWindow.c"
}


GType fullscreen_window_get_type (void) {
	static volatile gsize fullscreen_window_type_id__volatile = 0;
	if (g_once_init_enter (&fullscreen_window_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (FullscreenWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) fullscreen_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FullscreenWindow), 0, (GInstanceInitFunc) fullscreen_window_instance_init, NULL };
		GType fullscreen_window_type_id;
		fullscreen_window_type_id = g_type_register_static (TYPE_PAGE_WINDOW, "FullscreenWindow", &g_define_type_info, 0);
		g_once_init_leave (&fullscreen_window_type_id__volatile, fullscreen_window_type_id);
	}
	return fullscreen_window_type_id__volatile;
}


static void page_window_real_switched_pages (PageWindow* self, Page* old_page, Page* new_page) {
#line 295 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((old_page == NULL) || IS_PAGE (old_page));
#line 295 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((new_page == NULL) || IS_PAGE (new_page));
#line 1802 "AppWindow.c"
}


void page_window_switched_pages (PageWindow* self, Page* old_page, Page* new_page) {
#line 295 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE_WINDOW (self));
#line 295 "/home/jens/Source/shotwell/src/AppWindow.vala"
	PAGE_WINDOW_GET_CLASS (self)->switched_pages (self, old_page, new_page);
#line 1811 "AppWindow.c"
}


PageWindow* page_window_construct (GType object_type) {
	PageWindow * self = NULL;
#line 298 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = (PageWindow*) g_object_new (object_type, NULL);
#line 300 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_add_events (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget), (gint) ((GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK) | GDK_STRUCTURE_MASK));
#line 303 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_has_resize_grip (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), FALSE);
#line 298 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return self;
#line 1825 "AppWindow.c"
}


GtkUIManager* page_window_get_ui_manager (PageWindow* self) {
	GtkUIManager* result = NULL;
	GtkUIManager* _tmp0_ = NULL;
	GtkUIManager* _tmp1_ = NULL;
#line 306 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_PAGE_WINDOW (self), NULL);
#line 307 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->ui;
#line 307 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 307 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp1_;
#line 307 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1843 "AppWindow.c"
}


Page* page_window_get_current_page (PageWindow* self) {
	Page* result = NULL;
	Page* _tmp0_ = NULL;
	Page* _tmp1_ = NULL;
#line 310 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_PAGE_WINDOW (self), NULL);
#line 311 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->current_page;
#line 311 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 311 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp1_;
#line 311 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 1861 "AppWindow.c"
}


static void page_window_real_set_current_page (PageWindow* self, Page* page) {
	Page* _tmp0_ = NULL;
	Page* old_page = NULL;
	Page* _tmp2_ = NULL;
	Page* _tmp3_ = NULL;
	Page* _tmp4_ = NULL;
	Page* _tmp5_ = NULL;
	Page* _tmp6_ = NULL;
	Page* _tmp7_ = NULL;
#line 314 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 315 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->current_page;
#line 315 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ != NULL) {
#line 1880 "AppWindow.c"
		Page* _tmp1_ = NULL;
#line 316 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = self->priv->current_page;
#line 316 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_clear_container (_tmp1_);
#line 1886 "AppWindow.c"
	}
#line 318 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = self->priv->current_page;
#line 318 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 318 "/home/jens/Source/shotwell/src/AppWindow.vala"
	old_page = _tmp3_;
#line 319 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = page;
#line 319 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = _g_object_ref0 (_tmp4_);
#line 319 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->current_page);
#line 319 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->current_page = _tmp5_;
#line 320 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = self->priv->current_page;
#line 320 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_set_container (_tmp6_, G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 322 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = page;
#line 322 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_window_switched_pages (self, old_page, _tmp7_);
#line 314 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (old_page);
#line 1912 "AppWindow.c"
}


void page_window_set_current_page (PageWindow* self, Page* page) {
#line 314 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE_WINDOW (self));
#line 314 "/home/jens/Source/shotwell/src/AppWindow.vala"
	PAGE_WINDOW_GET_CLASS (self)->set_current_page (self, page);
#line 1921 "AppWindow.c"
}


static void page_window_real_clear_current_page (PageWindow* self) {
	Page* _tmp0_ = NULL;
	Page* old_page = NULL;
	Page* _tmp2_ = NULL;
	Page* _tmp3_ = NULL;
#line 326 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->current_page;
#line 326 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ != NULL) {
#line 1934 "AppWindow.c"
		Page* _tmp1_ = NULL;
#line 327 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = self->priv->current_page;
#line 327 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_clear_container (_tmp1_);
#line 1940 "AppWindow.c"
	}
#line 329 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = self->priv->current_page;
#line 329 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 329 "/home/jens/Source/shotwell/src/AppWindow.vala"
	old_page = _tmp3_;
#line 330 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->current_page);
#line 330 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->current_page = NULL;
#line 332 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_window_switched_pages (self, old_page, NULL);
#line 325 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (old_page);
#line 1956 "AppWindow.c"
}


void page_window_clear_current_page (PageWindow* self) {
#line 325 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE_WINDOW (self));
#line 325 "/home/jens/Source/shotwell/src/AppWindow.vala"
	PAGE_WINDOW_GET_CLASS (self)->clear_current_page (self);
#line 1965 "AppWindow.c"
}


static gboolean page_window_real_key_press_event (GtkWidget* base, GdkEventKey* event) {
	PageWindow * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	GtkWidget* _tmp1_ = NULL;
	gboolean _tmp5_ = FALSE;
	Page* _tmp6_ = NULL;
	gboolean _tmp10_ = FALSE;
#line 335 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_PAGE_WINDOW, PageWindow);
#line 335 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gtk_window_get_focus (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp1_, gtk_entry_get_type ())) {
#line 1985 "AppWindow.c"
		GtkWidget* _tmp2_ = NULL;
		GdkEventKey* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = gtk_window_get_focus (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = event;
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_emit_by_name (_tmp2_, "key-press-event", _tmp3_, &_tmp4_);
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp4_;
#line 1997 "AppWindow.c"
	} else {
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = FALSE;
#line 2001 "AppWindow.c"
	}
#line 336 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_) {
#line 337 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 337 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 2009 "AppWindow.c"
	}
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = self->priv->current_page;
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp6_ != NULL) {
#line 2015 "AppWindow.c"
		Page* _tmp7_ = NULL;
		GdkEventKey* _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = self->priv->current_page;
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = event;
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = page_notify_app_key_pressed (_tmp7_, _tmp8_);
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp9_;
#line 2027 "AppWindow.c"
	} else {
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = FALSE;
#line 2031 "AppWindow.c"
	}
#line 339 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp5_) {
#line 340 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 340 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 2039 "AppWindow.c"
	}
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (page_window_parent_class)->key_press_event != NULL) {
#line 2043 "AppWindow.c"
		GdkEventKey* _tmp11_ = NULL;
		gboolean _tmp12_ = FALSE;
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = event;
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp12_ = GTK_WIDGET_CLASS (page_window_parent_class)->key_press_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), gtk_widget_get_type (), GtkWidget), _tmp11_);
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _tmp12_;
#line 2052 "AppWindow.c"
	} else {
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = FALSE;
#line 2056 "AppWindow.c"
	}
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp10_;
#line 342 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 2062 "AppWindow.c"
}


static gboolean page_window_real_key_release_event (GtkWidget* base, GdkEventKey* event) {
	PageWindow * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	GtkWidget* _tmp1_ = NULL;
	gboolean _tmp5_ = FALSE;
	Page* _tmp6_ = NULL;
	gboolean _tmp10_ = FALSE;
#line 345 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_PAGE_WINDOW, PageWindow);
#line 345 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gtk_window_get_focus (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp1_, gtk_entry_get_type ())) {
#line 2082 "AppWindow.c"
		GtkWidget* _tmp2_ = NULL;
		GdkEventKey* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = gtk_window_get_focus (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = event;
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_emit_by_name (_tmp2_, "key-release-event", _tmp3_, &_tmp4_);
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp4_;
#line 2094 "AppWindow.c"
	} else {
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = FALSE;
#line 2098 "AppWindow.c"
	}
#line 346 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_) {
#line 347 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 347 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 2106 "AppWindow.c"
	}
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = self->priv->current_page;
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp6_ != NULL) {
#line 2112 "AppWindow.c"
		Page* _tmp7_ = NULL;
		GdkEventKey* _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = self->priv->current_page;
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = event;
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = page_notify_app_key_released (_tmp7_, _tmp8_);
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp9_;
#line 2124 "AppWindow.c"
	} else {
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = FALSE;
#line 2128 "AppWindow.c"
	}
#line 349 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp5_) {
#line 350 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 350 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 2136 "AppWindow.c"
	}
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (page_window_parent_class)->key_release_event != NULL) {
#line 2140 "AppWindow.c"
		GdkEventKey* _tmp11_ = NULL;
		gboolean _tmp12_ = FALSE;
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = event;
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp12_ = GTK_WIDGET_CLASS (page_window_parent_class)->key_release_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), gtk_widget_get_type (), GtkWidget), _tmp11_);
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _tmp12_;
#line 2149 "AppWindow.c"
	} else {
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = FALSE;
#line 2153 "AppWindow.c"
	}
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp10_;
#line 352 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 2159 "AppWindow.c"
}


static gboolean page_window_real_focus_in_event (GtkWidget* base, GdkEventFocus* event) {
	PageWindow * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	Page* _tmp1_ = NULL;
	gboolean _tmp5_ = FALSE;
#line 355 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_PAGE_WINDOW, PageWindow);
#line 355 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = self->priv->current_page;
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 2177 "AppWindow.c"
		Page* _tmp2_ = NULL;
		GdkEventFocus* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = self->priv->current_page;
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = event;
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = page_notify_app_focus_in (_tmp2_, _tmp3_);
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp4_;
#line 2189 "AppWindow.c"
	} else {
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = FALSE;
#line 2193 "AppWindow.c"
	}
#line 356 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_) {
#line 357 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 357 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 2201 "AppWindow.c"
	}
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (page_window_parent_class)->focus_in_event != NULL) {
#line 2205 "AppWindow.c"
		GdkEventFocus* _tmp6_ = NULL;
		gboolean _tmp7_ = FALSE;
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = event;
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = GTK_WIDGET_CLASS (page_window_parent_class)->focus_in_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), gtk_widget_get_type (), GtkWidget), _tmp6_);
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp7_;
#line 2214 "AppWindow.c"
	} else {
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = FALSE;
#line 2218 "AppWindow.c"
	}
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp5_;
#line 359 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 2224 "AppWindow.c"
}


static gboolean page_window_real_focus_out_event (GtkWidget* base, GdkEventFocus* event) {
	PageWindow * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	Page* _tmp1_ = NULL;
	gboolean _tmp5_ = FALSE;
#line 362 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_PAGE_WINDOW, PageWindow);
#line 362 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = self->priv->current_page;
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 2242 "AppWindow.c"
		Page* _tmp2_ = NULL;
		GdkEventFocus* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = self->priv->current_page;
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = event;
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = page_notify_app_focus_out (_tmp2_, _tmp3_);
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp4_;
#line 2254 "AppWindow.c"
	} else {
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = FALSE;
#line 2258 "AppWindow.c"
	}
#line 363 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_) {
#line 364 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result = TRUE;
#line 364 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return result;
#line 2266 "AppWindow.c"
	}
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (page_window_parent_class)->focus_out_event != NULL) {
#line 2270 "AppWindow.c"
		GdkEventFocus* _tmp6_ = NULL;
		gboolean _tmp7_ = FALSE;
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = event;
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = GTK_WIDGET_CLASS (page_window_parent_class)->focus_out_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), gtk_widget_get_type (), GtkWidget), _tmp6_);
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp7_;
#line 2279 "AppWindow.c"
	} else {
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = FALSE;
#line 2283 "AppWindow.c"
	}
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp5_;
#line 366 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 2289 "AppWindow.c"
}


static gboolean page_window_real_configure_event (GtkWidget* base, GdkEventConfigure* event) {
	PageWindow * self;
	gboolean result = FALSE;
	Page* _tmp0_ = NULL;
	gboolean _tmp4_ = FALSE;
#line 369 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_PAGE_WINDOW, PageWindow);
#line 369 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 370 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->current_page;
#line 370 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ != NULL) {
#line 2306 "AppWindow.c"
		Page* _tmp1_ = NULL;
		GdkEventConfigure* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
#line 371 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = self->priv->current_page;
#line 371 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = event;
#line 371 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = page_notify_configure_event (_tmp1_, _tmp2_);
#line 371 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (_tmp3_) {
#line 372 "/home/jens/Source/shotwell/src/AppWindow.vala"
			result = TRUE;
#line 372 "/home/jens/Source/shotwell/src/AppWindow.vala"
			return result;
#line 2322 "AppWindow.c"
		}
	}
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (GTK_WIDGET_CLASS (page_window_parent_class)->configure_event != NULL) {
#line 2327 "AppWindow.c"
		GdkEventConfigure* _tmp5_ = NULL;
		gboolean _tmp6_ = FALSE;
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = event;
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = GTK_WIDGET_CLASS (page_window_parent_class)->configure_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), gtk_widget_get_type (), GtkWidget), _tmp5_);
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp6_;
#line 2336 "AppWindow.c"
	} else {
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = FALSE;
#line 2340 "AppWindow.c"
	}
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp4_;
#line 375 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 2346 "AppWindow.c"
}


void page_window_set_busy_cursor (PageWindow* self) {
	gint _tmp0_ = 0;
	GdkWindow* _tmp1_ = NULL;
	GdkCursor* _tmp2_ = NULL;
	GdkCursor* _tmp3_ = NULL;
#line 378 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE_WINDOW (self));
#line 379 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->busy_counter;
#line 379 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->busy_counter = _tmp0_ + 1;
#line 379 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ > 0) {
#line 380 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 2365 "AppWindow.c"
	}
#line 382 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gtk_widget_get_window (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 382 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = gdk_cursor_new (GDK_WATCH);
#line 382 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _tmp2_;
#line 382 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gdk_window_set_cursor (_tmp1_, _tmp3_);
#line 382 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp3_);
#line 383 "/home/jens/Source/shotwell/src/AppWindow.vala"
	spin_event_loop ();
#line 2379 "AppWindow.c"
}


void page_window_set_normal_cursor (PageWindow* self) {
	gint _tmp0_ = 0;
	GdkWindow* _tmp3_ = NULL;
	GdkCursor* _tmp4_ = NULL;
	GdkCursor* _tmp5_ = NULL;
#line 386 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE_WINDOW (self));
#line 387 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->priv->busy_counter;
#line 387 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ <= 0) {
#line 388 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->priv->busy_counter = 0;
#line 389 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 2398 "AppWindow.c"
	} else {
		gint _tmp1_ = 0;
		gint _tmp2_ = 0;
#line 390 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = self->priv->busy_counter;
#line 390 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->priv->busy_counter = _tmp1_ - 1;
#line 390 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = self->priv->busy_counter;
#line 390 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (_tmp2_ > 0) {
#line 391 "/home/jens/Source/shotwell/src/AppWindow.vala"
			return;
#line 2412 "AppWindow.c"
		}
	}
#line 394 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = gtk_widget_get_window (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 394 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = gdk_cursor_new (GDK_LEFT_PTR);
#line 394 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = _tmp4_;
#line 394 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gdk_window_set_cursor (_tmp3_, _tmp5_);
#line 394 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp5_);
#line 395 "/home/jens/Source/shotwell/src/AppWindow.vala"
	spin_event_loop ();
#line 2427 "AppWindow.c"
}


static void page_window_class_init (PageWindowClass * klass) {
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page_window_parent_class = g_type_class_peek_parent (klass);
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_type_class_add_private (klass, sizeof (PageWindowPrivate));
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((PageWindowClass *) klass)->switched_pages = page_window_real_switched_pages;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((PageWindowClass *) klass)->set_current_page = page_window_real_set_current_page;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((PageWindowClass *) klass)->clear_current_page = page_window_real_clear_current_page;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->key_press_event = page_window_real_key_press_event;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->key_release_event = page_window_real_key_release_event;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->focus_in_event = page_window_real_focus_in_event;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->focus_out_event = page_window_real_focus_out_event;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->configure_event = page_window_real_configure_event;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	G_OBJECT_CLASS (klass)->finalize = page_window_finalize;
#line 2454 "AppWindow.c"
}


static void page_window_instance_init (PageWindow * self) {
	GtkUIManager* _tmp0_ = NULL;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv = PAGE_WINDOW_GET_PRIVATE (self);
#line 290 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_ui_manager_new ();
#line 290 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->ui = _tmp0_;
#line 292 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->current_page = NULL;
#line 293 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->busy_counter = 0;
#line 2470 "AppWindow.c"
}


static void page_window_finalize (GObject* obj) {
	PageWindow * self;
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_PAGE_WINDOW, PageWindow);
#line 290 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->ui);
#line 292 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->current_page);
#line 289 "/home/jens/Source/shotwell/src/AppWindow.vala"
	G_OBJECT_CLASS (page_window_parent_class)->finalize (obj);
#line 2484 "AppWindow.c"
}


GType page_window_get_type (void) {
	static volatile gsize page_window_type_id__volatile = 0;
	if (g_once_init_enter (&page_window_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PageWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) page_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PageWindow), 0, (GInstanceInitFunc) page_window_instance_init, NULL };
		GType page_window_type_id;
		page_window_type_id = g_type_register_static (gtk_window_get_type (), "PageWindow", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&page_window_type_id__volatile, page_window_type_id);
	}
	return page_window_type_id__volatile;
}


static void _app_window_on_command_manager_altered_command_manager_altered (CommandManager* _sender, gboolean can_undo, gboolean can_redo, gpointer self) {
#line 447 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_command_manager_altered ((AppWindow*) self);
#line 2503 "AppWindow.c"
}


AppWindow* app_window_construct (GType object_type) {
	AppWindow * self = NULL;
	AppWindow* _tmp0_ = NULL;
	AppWindow* _tmp1_ = NULL;
	Dimensions _tmp10_ = {0};
	gint _tmp11_ = 0;
	Dimensions _tmp12_ = {0};
	gint _tmp13_ = 0;
	gboolean _tmp14_ = FALSE;
	CommandManager* _tmp15_ = NULL;
	CommandManager* _tmp16_ = NULL;
	CommandManager* _tmp17_ = NULL;
	gint _tmp18_ = 0;
	GtkActionGroup** _tmp19_ = NULL;
	GtkActionGroup** _tmp20_ = NULL;
	gint _tmp20__length1 = 0;
	GtkUIManager* _tmp29_ = NULL;
	GtkUIManager* _tmp30_ = NULL;
	GtkAccelGroup* _tmp31_ = NULL;
	GtkCssProvider* provider = NULL;
	GtkCssProvider* _tmp32_ = NULL;
	GError * _inner_error_ = NULL;
#line 424 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = (AppWindow*) page_window_construct (object_type);
#line 426 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_instance;
#line 426 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_assert (_tmp0_ == NULL, "instance == null");
#line 427 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _g_object_ref0 (self);
#line 427 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (app_window_instance);
#line 427 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_instance = _tmp1_;
#line 429 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_title (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), RESOURCES_APP_TITLE);
#line 430 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_default_icon_name ("shotwell");
#line 433 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (self, TYPE_LIBRARY_WINDOW)) {
#line 2547 "AppWindow.c"
		ConfigFacade* _tmp2_ = NULL;
		ConfigFacade* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
		Dimensions _tmp5_ = {0};
#line 434 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = config_facade_get_instance ();
#line 434 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _tmp2_;
#line 434 "/home/jens/Source/shotwell/src/AppWindow.vala"
		configuration_facade_get_library_window_state (G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, TYPE_CONFIGURATION_FACADE, ConfigurationFacade), &_tmp4_, &_tmp5_);
#line 434 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->maximized = _tmp4_;
#line 434 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->dimensions = _tmp5_;
#line 434 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp3_);
#line 2564 "AppWindow.c"
	} else {
		ConfigFacade* _tmp6_ = NULL;
		ConfigFacade* _tmp7_ = NULL;
		gboolean _tmp8_ = FALSE;
		Dimensions _tmp9_ = {0};
#line 436 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (self, TYPE_DIRECT_WINDOW), "this is DirectWindow");
#line 437 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = config_facade_get_instance ();
#line 437 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = _tmp6_;
#line 437 "/home/jens/Source/shotwell/src/AppWindow.vala"
		configuration_facade_get_direct_window_state (G_TYPE_CHECK_INSTANCE_CAST (_tmp7_, TYPE_CONFIGURATION_FACADE, ConfigurationFacade), &_tmp8_, &_tmp9_);
#line 437 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->maximized = _tmp8_;
#line 437 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->dimensions = _tmp9_;
#line 437 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp7_);
#line 2584 "AppWindow.c"
	}
#line 440 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = self->dimensions;
#line 440 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = _tmp10_.width;
#line 440 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = self->dimensions;
#line 440 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = _tmp12_.height;
#line 440 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_default_size (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp11_, _tmp13_);
#line 442 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = self->maximized;
#line 442 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp14_) {
#line 443 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_window_maximize (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 2602 "AppWindow.c"
	}
#line 445 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = app_window_command_manager;
#line 445 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_assert (_tmp15_ == NULL, "command_manager == null");
#line 446 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = command_manager_new (COMMAND_MANAGER_DEFAULT_DEPTH);
#line 446 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_command_manager_unref0 (app_window_command_manager);
#line 446 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_command_manager = _tmp16_;
#line 447 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = app_window_command_manager;
#line 447 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_signal_connect_object (_tmp17_, "altered", (GCallback) _app_window_on_command_manager_altered_command_manager_altered, self, 0);
#line 454 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp19_ = app_window_create_common_action_groups (self, &_tmp18_);
#line 454 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->common_action_groups = (_vala_array_free (self->common_action_groups, self->common_action_groups_length1, (GDestroyNotify) g_object_unref), NULL);
#line 454 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->common_action_groups = _tmp19_;
#line 454 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->common_action_groups_length1 = _tmp18_;
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp20_ = self->common_action_groups;
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp20__length1 = self->common_action_groups_length1;
#line 2630 "AppWindow.c"
	{
		GtkActionGroup** group_collection = NULL;
		gint group_collection_length1 = 0;
		gint _group_collection_size_ = 0;
		gint group_it = 0;
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
		group_collection = _tmp20_;
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
		group_collection_length1 = _tmp20__length1;
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
		for (group_it = 0; group_it < _tmp20__length1; group_it = group_it + 1) {
#line 2642 "AppWindow.c"
			GtkActionGroup* _tmp21_ = NULL;
			GtkActionGroup* group = NULL;
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_tmp21_ = _g_object_ref0 (group_collection[group_it]);
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
			group = _tmp21_;
#line 2649 "AppWindow.c"
			{
				GtkUIManager* _tmp22_ = NULL;
				GtkActionGroup* _tmp23_ = NULL;
#line 456 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp22_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 456 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp23_ = group;
#line 456 "/home/jens/Source/shotwell/src/AppWindow.vala"
				gtk_ui_manager_insert_action_group (_tmp22_, _tmp23_, 0);
#line 455 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_g_object_unref0 (group);
#line 2661 "AppWindow.c"
			}
		}
	}
	{
		GtkUIManager* _tmp24_ = NULL;
		GtkActionGroup** _tmp25_ = NULL;
		gint _tmp25__length1 = 0;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp24_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp25_ = self->common_action_groups;
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp25__length1 = self->common_action_groups_length1;
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp26_ = build_dummy_ui_string (_tmp25_, _tmp25__length1);
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp27_ = _tmp26_;
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_ui_manager_add_ui_from_string (_tmp24_, _tmp27_, (gssize) -1, &_inner_error_);
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp27_);
#line 459 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2687 "AppWindow.c"
			goto __catch212_g_error;
		}
	}
	goto __finally212;
	__catch212_g_error:
	{
		GError* err = NULL;
		const gchar* _tmp28_ = NULL;
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 461 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp28_ = err->message;
#line 461 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_error ("AppWindow.vala:461: Unable to add AppWindow UI: %s", _tmp28_);
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 2706 "AppWindow.c"
	}
	__finally212:
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 458 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return NULL;
#line 2717 "AppWindow.c"
	}
#line 464 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp29_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 464 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_ui_manager_ensure_update (_tmp29_);
#line 465 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp30_ = G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow)->ui;
#line 465 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_ = gtk_ui_manager_get_accel_group (_tmp30_);
#line 465 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_add_accel_group (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp31_);
#line 467 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp32_ = gtk_css_provider_new ();
#line 467 "/home/jens/Source/shotwell/src/AppWindow.vala"
	provider = _tmp32_;
#line 2733 "AppWindow.c"
	{
		GdkScreen* _tmp33_ = NULL;
#line 469 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_css_provider_load_from_data (provider, RESOURCES_CUSTOM_CSS, (gssize) -1, &_inner_error_);
#line 469 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2740 "AppWindow.c"
			goto __catch213_g_error;
		}
#line 470 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp33_ = gdk_screen_get_default ();
#line 470 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_style_context_add_provider_for_screen (_tmp33_, G_TYPE_CHECK_INSTANCE_CAST (provider, GTK_TYPE_STYLE_PROVIDER, GtkStyleProvider), (guint) GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
#line 2747 "AppWindow.c"
	}
	goto __finally213;
	__catch213_g_error:
	{
		GError* err = NULL;
		GError* _tmp34_ = NULL;
		const gchar* _tmp35_ = NULL;
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 472 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp34_ = err;
#line 472 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp35_ = _tmp34_->message;
#line 472 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_debug ("AppWindow.vala:472: Unable to load custom CSS: %s", _tmp35_);
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 2767 "AppWindow.c"
	}
	__finally213:
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (provider);
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 468 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return NULL;
#line 2780 "AppWindow.c"
	}
#line 424 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (provider);
#line 424 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return self;
#line 2786 "AppWindow.c"
}


static void _app_window_on_quit_gtk_action_callback (GtkAction* action, gpointer self) {
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_quit ((AppWindow*) self);
#line 2793 "AppWindow.c"
}


static void _vala_array_add159 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2804 "AppWindow.c"
	}
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2808 "AppWindow.c"
}


static void _app_window_on_about_gtk_action_callback (GtkAction* action, gpointer self) {
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_about ((AppWindow*) self);
#line 2815 "AppWindow.c"
}


static void _vala_array_add160 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2826 "AppWindow.c"
	}
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2830 "AppWindow.c"
}


static void _app_window_on_fullscreen_gtk_action_callback (GtkAction* action, gpointer self) {
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_fullscreen ((AppWindow*) self);
#line 2837 "AppWindow.c"
}


static void _vala_array_add161 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2848 "AppWindow.c"
	}
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2852 "AppWindow.c"
}


static void _app_window_on_help_contents_gtk_action_callback (GtkAction* action, gpointer self) {
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_help_contents ((AppWindow*) self);
#line 2859 "AppWindow.c"
}


static void _vala_array_add162 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2870 "AppWindow.c"
	}
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2874 "AppWindow.c"
}


static void _app_window_on_help_faq_gtk_action_callback (GtkAction* action, gpointer self) {
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_help_faq ((AppWindow*) self);
#line 2881 "AppWindow.c"
}


static void _vala_array_add163 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2892 "AppWindow.c"
	}
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2896 "AppWindow.c"
}


static void _app_window_on_help_report_problem_gtk_action_callback (GtkAction* action, gpointer self) {
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_help_report_problem ((AppWindow*) self);
#line 2903 "AppWindow.c"
}


static void _vala_array_add164 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2914 "AppWindow.c"
	}
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2918 "AppWindow.c"
}


static void _app_window_on_undo_gtk_action_callback (GtkAction* action, gpointer self) {
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_undo ((AppWindow*) self);
#line 2925 "AppWindow.c"
}


static void _vala_array_add165 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2936 "AppWindow.c"
	}
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2940 "AppWindow.c"
}


static void _app_window_on_redo_gtk_action_callback (GtkAction* action, gpointer self) {
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_redo ((AppWindow*) self);
#line 2947 "AppWindow.c"
}


static void _vala_array_add166 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2958 "AppWindow.c"
	}
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2962 "AppWindow.c"
}


static void _app_window_on_jump_to_file_gtk_action_callback (GtkAction* action, gpointer self) {
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_jump_to_file ((AppWindow*) self);
#line 2969 "AppWindow.c"
}


static void _vala_array_add167 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 2980 "AppWindow.c"
	}
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 2984 "AppWindow.c"
}


static void _app_window_on_select_all_gtk_action_callback (GtkAction* action, gpointer self) {
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_select_all ((AppWindow*) self);
#line 2991 "AppWindow.c"
}


static void _vala_array_add168 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 3002 "AppWindow.c"
	}
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 3006 "AppWindow.c"
}


static void _app_window_on_select_none_gtk_action_callback (GtkAction* action, gpointer self) {
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_select_none ((AppWindow*) self);
#line 3013 "AppWindow.c"
}


static void _vala_array_add169 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 3024 "AppWindow.c"
	}
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = *value;
#line 3028 "AppWindow.c"
}


static GtkActionEntry* app_window_create_common_actions (AppWindow* self, int* result_length1) {
	GtkActionEntry* result = NULL;
	GtkActionEntry* actions = NULL;
	GtkActionEntry* _tmp0_ = NULL;
	gint actions_length1 = 0;
	gint _actions_size_ = 0;
	GtkActionEntry quit = {0};
	GtkActionEntry _tmp1_ = {0};
	GtkActionEntry* _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	GtkActionEntry _tmp3_ = {0};
	GtkActionEntry about = {0};
	GtkActionEntry _tmp4_ = {0};
	GtkActionEntry* _tmp5_ = NULL;
	gint _tmp5__length1 = 0;
	GtkActionEntry _tmp6_ = {0};
	GtkActionEntry fullscreen = {0};
	GtkActionEntry _tmp7_ = {0};
	GtkActionEntry* _tmp8_ = NULL;
	gint _tmp8__length1 = 0;
	GtkActionEntry _tmp9_ = {0};
	GtkActionEntry help_contents = {0};
	GtkActionEntry _tmp10_ = {0};
	const gchar* _tmp11_ = NULL;
	GtkActionEntry* _tmp12_ = NULL;
	gint _tmp12__length1 = 0;
	GtkActionEntry _tmp13_ = {0};
	GtkActionEntry help_faq = {0};
	GtkActionEntry _tmp14_ = {0};
	const gchar* _tmp15_ = NULL;
	GtkActionEntry* _tmp16_ = NULL;
	gint _tmp16__length1 = 0;
	GtkActionEntry _tmp17_ = {0};
	GtkActionEntry help_report_problem = {0};
	GtkActionEntry _tmp18_ = {0};
	const gchar* _tmp19_ = NULL;
	GtkActionEntry* _tmp20_ = NULL;
	gint _tmp20__length1 = 0;
	GtkActionEntry _tmp21_ = {0};
	GtkActionEntry undo = {0};
	GtkActionEntry _tmp22_ = {0};
	GtkActionEntry* _tmp23_ = NULL;
	gint _tmp23__length1 = 0;
	GtkActionEntry _tmp24_ = {0};
	GtkActionEntry redo = {0};
	GtkActionEntry _tmp25_ = {0};
	GtkActionEntry* _tmp26_ = NULL;
	gint _tmp26__length1 = 0;
	GtkActionEntry _tmp27_ = {0};
	GtkActionEntry jump_to_file = {0};
	GtkActionEntry _tmp28_ = {0};
	GtkActionEntry* _tmp29_ = NULL;
	gint _tmp29__length1 = 0;
	GtkActionEntry _tmp30_ = {0};
	GtkActionEntry select_all = {0};
	GtkActionEntry _tmp31_ = {0};
	GtkActionEntry* _tmp32_ = NULL;
	gint _tmp32__length1 = 0;
	GtkActionEntry _tmp33_ = {0};
	GtkActionEntry select_none = {0};
	GtkActionEntry _tmp34_ = {0};
	GtkActionEntry* _tmp35_ = NULL;
	gint _tmp35__length1 = 0;
	GtkActionEntry _tmp36_ = {0};
	GtkActionEntry* _tmp37_ = NULL;
	gint _tmp37__length1 = 0;
#line 476 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 477 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = g_new0 (GtkActionEntry, 0);
#line 477 "/home/jens/Source/shotwell/src/AppWindow.vala"
	actions = _tmp0_;
#line 477 "/home/jens/Source/shotwell/src/AppWindow.vala"
	actions_length1 = 0;
#line 477 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_actions_size_ = actions_length1;
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.name = "CommonQuit";
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.stock_id = RESOURCES_QUIT_LABEL;
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.label = TRANSLATABLE;
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.accelerator = "<Ctrl>Q";
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.tooltip = TRANSLATABLE;
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_.callback = (GCallback) _app_window_on_quit_gtk_action_callback;
#line 479 "/home/jens/Source/shotwell/src/AppWindow.vala"
	quit = _tmp1_;
#line 481 "/home/jens/Source/shotwell/src/AppWindow.vala"
	quit.label = RESOURCES_QUIT_LABEL;
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = actions;
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2__length1 = actions_length1;
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = quit;
#line 482 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add159 (&actions, &actions_length1, &_actions_size_, &_tmp3_);
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_.name = "CommonAbout";
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_.stock_id = RESOURCES_ABOUT_LABEL;
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_.label = TRANSLATABLE;
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_.accelerator = NULL;
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_.tooltip = TRANSLATABLE;
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_.callback = (GCallback) _app_window_on_about_gtk_action_callback;
#line 484 "/home/jens/Source/shotwell/src/AppWindow.vala"
	about = _tmp4_;
#line 486 "/home/jens/Source/shotwell/src/AppWindow.vala"
	about.label = RESOURCES_ABOUT_LABEL;
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = actions;
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5__length1 = actions_length1;
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = about;
#line 487 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add160 (&actions, &actions_length1, &_actions_size_, &_tmp6_);
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_.name = "CommonFullscreen";
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_.stock_id = RESOURCES_FULLSCREEN_LABEL;
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_.label = TRANSLATABLE;
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_.accelerator = "F11";
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_.tooltip = TRANSLATABLE;
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_.callback = (GCallback) _app_window_on_fullscreen_gtk_action_callback;
#line 489 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen = _tmp7_;
#line 491 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fullscreen.label = RESOURCES_FULLSCREEN_LABEL;
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = actions;
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8__length1 = actions_length1;
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = fullscreen;
#line 492 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add161 (&actions, &actions_length1, &_actions_size_, &_tmp9_);
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_.name = "CommonHelpContents";
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_.stock_id = RESOURCES_HELP_LABEL;
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_.label = TRANSLATABLE;
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_.accelerator = "F1";
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_.tooltip = TRANSLATABLE;
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_.callback = (GCallback) _app_window_on_help_contents_gtk_action_callback;
#line 494 "/home/jens/Source/shotwell/src/AppWindow.vala"
	help_contents = _tmp10_;
#line 496 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = _ ("_Contents");
#line 496 "/home/jens/Source/shotwell/src/AppWindow.vala"
	help_contents.label = _tmp11_;
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = actions;
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12__length1 = actions_length1;
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = help_contents;
#line 497 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add162 (&actions, &actions_length1, &_actions_size_, &_tmp13_);
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_.name = "CommonHelpFAQ";
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_.stock_id = NULL;
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_.label = TRANSLATABLE;
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_.accelerator = NULL;
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_.tooltip = TRANSLATABLE;
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_.callback = (GCallback) _app_window_on_help_faq_gtk_action_callback;
#line 499 "/home/jens/Source/shotwell/src/AppWindow.vala"
	help_faq = _tmp14_;
#line 501 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = _ ("_Frequently Asked Questions");
#line 501 "/home/jens/Source/shotwell/src/AppWindow.vala"
	help_faq.label = _tmp15_;
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = actions;
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16__length1 = actions_length1;
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = help_faq;
#line 502 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add163 (&actions, &actions_length1, &_actions_size_, &_tmp17_);
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_.name = "CommonHelpReportProblem";
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_.stock_id = NULL;
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_.label = TRANSLATABLE;
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_.accelerator = NULL;
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_.tooltip = TRANSLATABLE;
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_.callback = (GCallback) _app_window_on_help_report_problem_gtk_action_callback;
#line 504 "/home/jens/Source/shotwell/src/AppWindow.vala"
	help_report_problem = _tmp18_;
#line 506 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp19_ = _ ("_Report a Problem...");
#line 506 "/home/jens/Source/shotwell/src/AppWindow.vala"
	help_report_problem.label = _tmp19_;
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp20_ = actions;
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp20__length1 = actions_length1;
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp21_ = help_report_problem;
#line 507 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add164 (&actions, &actions_length1, &_actions_size_, &_tmp21_);
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_.name = "CommonUndo";
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_.stock_id = RESOURCES_UNDO_MENU;
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_.label = TRANSLATABLE;
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_.accelerator = "<Ctrl>Z";
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_.tooltip = TRANSLATABLE;
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp22_.callback = (GCallback) _app_window_on_undo_gtk_action_callback;
#line 509 "/home/jens/Source/shotwell/src/AppWindow.vala"
	undo = _tmp22_;
#line 511 "/home/jens/Source/shotwell/src/AppWindow.vala"
	undo.label = RESOURCES_UNDO_MENU;
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp23_ = actions;
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp23__length1 = actions_length1;
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp24_ = undo;
#line 512 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add165 (&actions, &actions_length1, &_actions_size_, &_tmp24_);
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_.name = "CommonRedo";
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_.stock_id = RESOURCES_REDO_MENU;
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_.label = TRANSLATABLE;
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_.accelerator = "<Ctrl><Shift>Z";
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_.tooltip = TRANSLATABLE;
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp25_.callback = (GCallback) _app_window_on_redo_gtk_action_callback;
#line 514 "/home/jens/Source/shotwell/src/AppWindow.vala"
	redo = _tmp25_;
#line 516 "/home/jens/Source/shotwell/src/AppWindow.vala"
	redo.label = RESOURCES_REDO_MENU;
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp26_ = actions;
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp26__length1 = actions_length1;
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp27_ = redo;
#line 517 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add166 (&actions, &actions_length1, &_actions_size_, &_tmp27_);
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_.name = "CommonJumpToFile";
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_.stock_id = RESOURCES_JUMP_TO_FILE_MENU;
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_.label = TRANSLATABLE;
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_.accelerator = "<Ctrl><Shift>M";
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_.tooltip = TRANSLATABLE;
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp28_.callback = (GCallback) _app_window_on_jump_to_file_gtk_action_callback;
#line 519 "/home/jens/Source/shotwell/src/AppWindow.vala"
	jump_to_file = _tmp28_;
#line 521 "/home/jens/Source/shotwell/src/AppWindow.vala"
	jump_to_file.label = RESOURCES_JUMP_TO_FILE_MENU;
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp29_ = actions;
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp29__length1 = actions_length1;
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp30_ = jump_to_file;
#line 522 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add167 (&actions, &actions_length1, &_actions_size_, &_tmp30_);
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_.name = "CommonSelectAll";
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_.stock_id = RESOURCES_SELECT_ALL_MENU;
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_.label = TRANSLATABLE;
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_.accelerator = "<Ctrl>A";
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_.tooltip = TRANSLATABLE;
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp31_.callback = (GCallback) _app_window_on_select_all_gtk_action_callback;
#line 524 "/home/jens/Source/shotwell/src/AppWindow.vala"
	select_all = _tmp31_;
#line 526 "/home/jens/Source/shotwell/src/AppWindow.vala"
	select_all.label = RESOURCES_SELECT_ALL_MENU;
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp32_ = actions;
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp32__length1 = actions_length1;
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp33_ = select_all;
#line 527 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add168 (&actions, &actions_length1, &_actions_size_, &_tmp33_);
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_.name = "CommonSelectNone";
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_.stock_id = NULL;
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_.label = NULL;
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_.accelerator = "<Ctrl><Shift>A";
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_.tooltip = TRANSLATABLE;
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_.callback = (GCallback) _app_window_on_select_none_gtk_action_callback;
#line 529 "/home/jens/Source/shotwell/src/AppWindow.vala"
	select_none = _tmp34_;
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp35_ = actions;
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp35__length1 = actions_length1;
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp36_ = select_none;
#line 531 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add169 (&actions, &actions_length1, &_actions_size_, &_tmp36_);
#line 533 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp37_ = actions;
#line 533 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp37__length1 = actions_length1;
#line 533 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (result_length1) {
#line 533 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*result_length1 = _tmp37__length1;
#line 3384 "AppWindow.c"
	}
#line 533 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp37_;
#line 533 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3390 "AppWindow.c"
}


static void app_window_real_on_fullscreen (AppWindow* self) {
#line 536 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_critical ("Type `%s' does not implement abstract method `app_window_on_fullscreen'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
#line 536 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return;
#line 3399 "AppWindow.c"
}


void app_window_on_fullscreen (AppWindow* self) {
#line 536 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 536 "/home/jens/Source/shotwell/src/AppWindow.vala"
	APP_WINDOW_GET_CLASS (self)->on_fullscreen (self);
#line 3408 "AppWindow.c"
}


gboolean app_window_has_instance (void) {
	gboolean result = FALSE;
	AppWindow* _tmp0_ = NULL;
#line 539 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_instance;
#line 539 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp0_ != NULL;
#line 539 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3421 "AppWindow.c"
}


AppWindow* app_window_get_instance (void) {
	AppWindow* result = NULL;
	AppWindow* _tmp0_ = NULL;
	AppWindow* _tmp1_ = NULL;
#line 543 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_instance;
#line 543 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 543 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp1_;
#line 543 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3437 "AppWindow.c"
}


FullscreenWindow* app_window_get_fullscreen (void) {
	FullscreenWindow* result = NULL;
	FullscreenWindow* _tmp0_ = NULL;
	FullscreenWindow* _tmp1_ = NULL;
#line 547 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_fullscreen_window;
#line 547 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 547 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp1_;
#line 547 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3453 "AppWindow.c"
}


GtkBuilder* app_window_create_builder (const gchar* glade_filename, void* user) {
	GtkBuilder* result = NULL;
	GtkBuilder* builder = NULL;
	GtkBuilder* _tmp0_ = NULL;
	void* _tmp12_ = NULL;
	GError * _inner_error_ = NULL;
#line 550 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (glade_filename != NULL, NULL);
#line 551 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_builder_new ();
#line 551 "/home/jens/Source/shotwell/src/AppWindow.vala"
	builder = _tmp0_;
#line 3469 "AppWindow.c"
	{
		GFile* _tmp1_ = NULL;
		GFile* _tmp2_ = NULL;
		GFile* _tmp3_ = NULL;
		GFile* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		GFile* _tmp6_ = NULL;
		GFile* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = app_dirs_get_resources_dir ();
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = _tmp1_;
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = g_file_get_child (_tmp2_, "ui");
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp3_;
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = glade_filename;
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = g_file_get_child (_tmp4_, _tmp5_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = _tmp6_;
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = g_file_get_path (_tmp7_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = _tmp8_;
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_builder_add_from_file (builder, _tmp9_, &_inner_error_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp9_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp7_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp4_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp2_);
#line 553 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3510 "AppWindow.c"
			goto __catch214_g_error;
		}
	}
	goto __finally214;
	__catch214_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp10_ = NULL;
		const gchar* _tmp11_ = NULL;
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_error_ = _inner_error_;
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 556 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _error_;
#line 556 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = _tmp10_->message;
#line 556 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_warning ("AppWindow.vala:556: Unable to create Gtk.Builder: %s\n", _tmp11_);
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (_error_);
#line 3532 "AppWindow.c"
	}
	__finally214:
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (builder);
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 552 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return NULL;
#line 3545 "AppWindow.c"
	}
#line 559 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = user;
#line 559 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_builder_connect_signals (builder, _tmp12_);
#line 561 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = builder;
#line 561 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3555 "AppWindow.c"
}


void app_window_error_message (const gchar* message, GtkWindow* parent) {
	const gchar* _tmp0_ = NULL;
	GtkWindow* _tmp1_ = NULL;
#line 564 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (message != NULL);
#line 564 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent));
#line 565 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = message;
#line 565 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = parent;
#line 565 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_error_message_with_title (RESOURCES_APP_TITLE, _tmp0_, _tmp1_, TRUE);
#line 3572 "AppWindow.c"
}


void app_window_error_message_with_title (const gchar* title, const gchar* message, GtkWindow* parent, gboolean should_escape) {
	GtkWindow* _tmp0_ = NULL;
	GtkWindow* _tmp1_ = NULL;
	GtkMessageDialog* dialog = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	gboolean _tmp7_ = FALSE;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	GtkMessageDialog* _tmp10_ = NULL;
	GtkMessageDialog* _tmp11_ = NULL;
	const gchar* _tmp12_ = NULL;
	const gchar* _tmp13_ = NULL;
	gboolean _tmp14_ = FALSE;
	gchar* _tmp15_ = NULL;
	gchar* _tmp16_ = NULL;
#line 568 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (title != NULL);
#line 568 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (message != NULL);
#line 568 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent));
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = parent;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 3602 "AppWindow.c"
		GtkWindow* _tmp2_ = NULL;
		GtkWindow* _tmp3_ = NULL;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = parent;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _g_object_ref0 (_tmp2_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp3_;
#line 3613 "AppWindow.c"
	} else {
		AppWindow* _tmp4_ = NULL;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = app_window_get_instance ();
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_window_get_type (), GtkWindow);
#line 3622 "AppWindow.c"
	}
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = title;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = message;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = should_escape;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = build_alert_body_text (_tmp5_, _tmp6_, _tmp7_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = _tmp8_;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = (GtkMessageDialog*) gtk_message_dialog_new_with_markup (_tmp0_, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", _tmp9_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp10_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = _tmp10_;
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp9_);
#line 572 "/home/jens/Source/shotwell/src/AppWindow.vala"
	dialog = _tmp11_;
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = title;
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = message;
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = should_escape;
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = build_alert_body_text (_tmp12_, _tmp13_, _tmp14_);
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = _tmp15_;
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_message_dialog_set_markup (dialog, _tmp16_);
#line 576 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp16_);
#line 578 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_set (dialog, "use-markup", TRUE, NULL);
#line 579 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_dialog_run (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_dialog_get_type (), GtkDialog));
#line 580 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_widget_get_type (), GtkWidget));
#line 568 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (dialog);
#line 568 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp0_);
#line 3668 "AppWindow.c"
}


gboolean app_window_negate_affirm_question (const gchar* message, const gchar* negative, const gchar* affirmative, const gchar* title, GtkWindow* parent) {
	gboolean result = FALSE;
	GtkWindow* _tmp0_ = NULL;
	GtkWindow* _tmp1_ = NULL;
	GtkMessageDialog* dialog = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	GtkMessageDialog* _tmp9_ = NULL;
	GtkMessageDialog* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	const gchar* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
	gchar* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	const gchar* _tmp16_ = NULL;
	gboolean response = FALSE;
	gint _tmp17_ = 0;
#line 583 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (message != NULL, FALSE);
#line 583 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (negative != NULL, FALSE);
#line 583 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (affirmative != NULL, FALSE);
#line 583 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent), FALSE);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = parent;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 3703 "AppWindow.c"
		GtkWindow* _tmp2_ = NULL;
		GtkWindow* _tmp3_ = NULL;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = parent;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _g_object_ref0 (_tmp2_);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp3_;
#line 3714 "AppWindow.c"
	} else {
		AppWindow* _tmp4_ = NULL;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = app_window_get_instance ();
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_window_get_type (), GtkWindow);
#line 3723 "AppWindow.c"
	}
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = title;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = message;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = build_alert_body_text (_tmp5_, _tmp6_, TRUE);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = _tmp7_;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = (GtkMessageDialog*) gtk_message_dialog_new (_tmp0_, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", _tmp8_);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp9_);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = _tmp9_;
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp8_);
#line 585 "/home/jens/Source/shotwell/src/AppWindow.vala"
	dialog = _tmp10_;
#line 588 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = title;
#line 588 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = message;
#line 588 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = build_alert_body_text (_tmp11_, _tmp12_, TRUE);
#line 588 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = _tmp13_;
#line 588 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_message_dialog_set_markup (dialog, _tmp14_);
#line 588 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp14_);
#line 589 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = negative;
#line 589 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = affirmative;
#line 589 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_dialog_add_buttons (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_dialog_get_type (), GtkDialog), _tmp15_, GTK_RESPONSE_NO, _tmp16_, GTK_RESPONSE_YES, NULL);
#line 590 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_urgency_hint (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_window_get_type (), GtkWindow), TRUE);
#line 592 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = gtk_dialog_run (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_dialog_get_type (), GtkDialog));
#line 592 "/home/jens/Source/shotwell/src/AppWindow.vala"
	response = _tmp17_ == ((gint) GTK_RESPONSE_YES);
#line 594 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_widget_get_type (), GtkWidget));
#line 596 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = response;
#line 596 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (dialog);
#line 596 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp0_);
#line 596 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3777 "AppWindow.c"
}


GtkResponseType app_window_negate_affirm_cancel_question (const gchar* message, const gchar* negative, const gchar* affirmative, const gchar* title, GtkWindow* parent) {
	GtkResponseType result = 0;
	GtkWindow* _tmp0_ = NULL;
	GtkWindow* _tmp1_ = NULL;
	GtkMessageDialog* dialog = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	GtkMessageDialog* _tmp9_ = NULL;
	GtkMessageDialog* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	const gchar* _tmp12_ = NULL;
	const gchar* _tmp13_ = NULL;
	const gchar* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	gchar* _tmp16_ = NULL;
	gchar* _tmp17_ = NULL;
	gint response = 0;
	gint _tmp18_ = 0;
#line 599 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (message != NULL, 0);
#line 599 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (negative != NULL, 0);
#line 599 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (affirmative != NULL, 0);
#line 599 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent), 0);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = parent;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 3813 "AppWindow.c"
		GtkWindow* _tmp2_ = NULL;
		GtkWindow* _tmp3_ = NULL;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = parent;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _g_object_ref0 (_tmp2_);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp3_;
#line 3824 "AppWindow.c"
	} else {
		AppWindow* _tmp4_ = NULL;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = app_window_get_instance ();
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_window_get_type (), GtkWindow);
#line 3833 "AppWindow.c"
	}
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = title;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = message;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = build_alert_body_text (_tmp5_, _tmp6_, TRUE);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = _tmp7_;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = (GtkMessageDialog*) gtk_message_dialog_new_with_markup (_tmp0_, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", _tmp8_);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp9_);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = _tmp9_;
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp8_);
#line 601 "/home/jens/Source/shotwell/src/AppWindow.vala"
	dialog = _tmp10_;
#line 604 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = negative;
#line 604 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = affirmative;
#line 604 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = _ ("_Cancel");
#line 604 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_dialog_add_buttons (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_dialog_get_type (), GtkDialog), _tmp11_, GTK_RESPONSE_NO, _tmp12_, GTK_RESPONSE_YES, _tmp13_, GTK_RESPONSE_CANCEL, NULL);
#line 608 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = title;
#line 608 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = message;
#line 608 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = build_alert_body_text (_tmp14_, _tmp15_, TRUE);
#line 608 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = _tmp16_;
#line 608 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_message_dialog_set_markup (dialog, _tmp17_);
#line 608 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp17_);
#line 609 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_set (dialog, "use-markup", TRUE, NULL);
#line 611 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_ = gtk_dialog_run (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_dialog_get_type (), GtkDialog));
#line 611 "/home/jens/Source/shotwell/src/AppWindow.vala"
	response = _tmp18_;
#line 613 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (dialog, gtk_widget_get_type (), GtkWidget));
#line 615 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = (GtkResponseType) response;
#line 615 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (dialog);
#line 615 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp0_);
#line 615 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 3889 "AppWindow.c"
}


GtkResponseType app_window_affirm_cancel_question (const gchar* message, const gchar* affirmative, const gchar* title, GtkWindow* parent) {
	GtkResponseType result = 0;
	GtkWindow* _tmp0_ = NULL;
	GtkWindow* _tmp1_ = NULL;
	GtkMessageDialog* dialog = NULL;
	const gchar* _tmp5_ = NULL;
	GtkMessageDialog* _tmp6_ = NULL;
	GtkMessageDialog* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	GtkMessageDialog* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	GtkMessageDialog* _tmp13_ = NULL;
	GtkMessageDialog* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	const gchar* _tmp16_ = NULL;
	gint response = 0;
	GtkMessageDialog* _tmp17_ = NULL;
	gint _tmp18_ = 0;
	GtkMessageDialog* _tmp19_ = NULL;
#line 618 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (message != NULL, 0);
#line 618 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (affirmative != NULL, 0);
#line 618 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent), 0);
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = parent;
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 3923 "AppWindow.c"
		GtkWindow* _tmp2_ = NULL;
		GtkWindow* _tmp3_ = NULL;
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = parent;
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _g_object_ref0 (_tmp2_);
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp3_;
#line 3934 "AppWindow.c"
	} else {
		AppWindow* _tmp4_ = NULL;
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = app_window_get_instance ();
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_window_get_type (), GtkWindow);
#line 3943 "AppWindow.c"
	}
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = message;
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = (GtkMessageDialog*) gtk_message_dialog_new_with_markup (_tmp0_, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", _tmp5_);
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp6_);
#line 620 "/home/jens/Source/shotwell/src/AppWindow.vala"
	dialog = _tmp6_;
#line 623 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = dialog;
#line 623 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = message;
#line 623 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_message_dialog_set_markup (_tmp7_, _tmp8_);
#line 624 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = dialog;
#line 624 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_set (_tmp9_, "use-markup", TRUE, NULL);
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = title;
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp11_ != NULL) {
#line 3967 "AppWindow.c"
		const gchar* _tmp12_ = NULL;
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp12_ = title;
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _tmp12_;
#line 3973 "AppWindow.c"
	} else {
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = RESOURCES_APP_TITLE;
#line 3977 "AppWindow.c"
	}
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = dialog;
#line 625 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_title (G_TYPE_CHECK_INSTANCE_CAST (_tmp13_, gtk_window_get_type (), GtkWindow), _tmp10_);
#line 626 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = dialog;
#line 626 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = affirmative;
#line 626 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = _ ("_Cancel");
#line 626 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_dialog_add_buttons (G_TYPE_CHECK_INSTANCE_CAST (_tmp14_, gtk_dialog_get_type (), GtkDialog), _tmp15_, GTK_RESPONSE_YES, _tmp16_, GTK_RESPONSE_CANCEL, NULL);
#line 629 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = dialog;
#line 629 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_ = gtk_dialog_run (G_TYPE_CHECK_INSTANCE_CAST (_tmp17_, gtk_dialog_get_type (), GtkDialog));
#line 629 "/home/jens/Source/shotwell/src/AppWindow.vala"
	response = _tmp18_;
#line 631 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp19_ = dialog;
#line 631 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (_tmp19_, gtk_widget_get_type (), GtkWidget));
#line 633 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = (GtkResponseType) response;
#line 633 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (dialog);
#line 633 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp0_);
#line 633 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 4009 "AppWindow.c"
}


GtkResponseType app_window_negate_affirm_all_cancel_question (const gchar* message, const gchar* negative, const gchar* affirmative, const gchar* affirmative_all, const gchar* title, GtkWindow* parent) {
	GtkResponseType result = 0;
	GtkWindow* _tmp0_ = NULL;
	GtkWindow* _tmp1_ = NULL;
	GtkMessageDialog* dialog = NULL;
	const gchar* _tmp5_ = NULL;
	GtkMessageDialog* _tmp6_ = NULL;
	const gchar* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	GtkMessageDialog* _tmp10_ = NULL;
	GtkMessageDialog* _tmp11_ = NULL;
	const gchar* _tmp12_ = NULL;
	const gchar* _tmp13_ = NULL;
	const gchar* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	gint response = 0;
	GtkMessageDialog* _tmp16_ = NULL;
	gint _tmp17_ = 0;
	GtkMessageDialog* _tmp18_ = NULL;
#line 636 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (message != NULL, 0);
#line 636 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (negative != NULL, 0);
#line 636 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (affirmative != NULL, 0);
#line 636 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (affirmative_all != NULL, 0);
#line 636 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent), 0);
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = parent;
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 4046 "AppWindow.c"
		GtkWindow* _tmp2_ = NULL;
		GtkWindow* _tmp3_ = NULL;
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = parent;
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _g_object_ref0 (_tmp2_);
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _tmp3_;
#line 4057 "AppWindow.c"
	} else {
		AppWindow* _tmp4_ = NULL;
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = app_window_get_instance ();
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_window_get_type (), GtkWindow);
#line 4066 "AppWindow.c"
	}
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = message;
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = (GtkMessageDialog*) gtk_message_dialog_new (_tmp0_, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", _tmp5_);
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp6_);
#line 639 "/home/jens/Source/shotwell/src/AppWindow.vala"
	dialog = _tmp6_;
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = title;
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp8_ != NULL) {
#line 4080 "AppWindow.c"
		const gchar* _tmp9_ = NULL;
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = title;
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = _tmp9_;
#line 4086 "AppWindow.c"
	} else {
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = RESOURCES_APP_TITLE;
#line 4090 "AppWindow.c"
	}
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = dialog;
#line 641 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_set_title (G_TYPE_CHECK_INSTANCE_CAST (_tmp10_, gtk_window_get_type (), GtkWindow), _tmp7_);
#line 642 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = dialog;
#line 642 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = negative;
#line 642 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = affirmative;
#line 642 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = affirmative_all;
#line 642 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = _ ("_Cancel");
#line 642 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_dialog_add_buttons (G_TYPE_CHECK_INSTANCE_CAST (_tmp11_, gtk_dialog_get_type (), GtkDialog), _tmp12_, GTK_RESPONSE_NO, _tmp13_, GTK_RESPONSE_YES, _tmp14_, GTK_RESPONSE_APPLY, _tmp15_, GTK_RESPONSE_CANCEL, NULL);
#line 645 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp16_ = dialog;
#line 645 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp17_ = gtk_dialog_run (G_TYPE_CHECK_INSTANCE_CAST (_tmp16_, gtk_dialog_get_type (), GtkDialog));
#line 645 "/home/jens/Source/shotwell/src/AppWindow.vala"
	response = _tmp17_;
#line 647 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp18_ = dialog;
#line 647 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (_tmp18_, gtk_widget_get_type (), GtkWidget));
#line 649 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = (GtkResponseType) response;
#line 649 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (dialog);
#line 649 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp0_);
#line 649 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 4126 "AppWindow.c"
}


void app_window_database_error (GError* err) {
	const gchar* _tmp0_ = NULL;
	GError* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = _ ("A fatal error occurred when accessing Shotwell's library.  Shotwell ca" \
"nnot continue.\n" \
"\n" \
"%s");
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = err;
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = _tmp1_->message;
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = g_strdup_printf (_tmp0_, _tmp2_);
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = _tmp3_;
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_panic (_tmp4_);
#line 653 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp4_);
#line 4150 "AppWindow.c"
}


void app_window_panic (const gchar* msg) {
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	Application* _tmp2_ = NULL;
	Application* _tmp3_ = NULL;
#line 657 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (msg != NULL);
#line 658 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = msg;
#line 658 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_critical ("AppWindow.vala:658: %s", _tmp0_);
#line 659 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = msg;
#line 659 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_error_message (_tmp1_, NULL);
#line 661 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = application_get_instance ();
#line 661 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _tmp2_;
#line 661 "/home/jens/Source/shotwell/src/AppWindow.vala"
	application_panic (_tmp3_);
#line 661 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_application_unref0 (_tmp3_);
#line 4177 "AppWindow.c"
}


static gchar* app_window_real_get_app_role (AppWindow* self) {
#line 664 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_critical ("Type `%s' does not implement abstract method `app_window_get_app_role'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
#line 664 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return NULL;
#line 4186 "AppWindow.c"
}


gchar* app_window_get_app_role (AppWindow* self) {
#line 664 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 664 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return APP_WINDOW_GET_CLASS (self)->get_app_role (self);
#line 4195 "AppWindow.c"
}


void app_window_on_about (AppWindow* self) {
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	GdkPixbuf* _tmp3_ = NULL;
	GdkPixbuf* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
#line 666 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_get_app_role (self);
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = _ ("Visit the Shotwell web site");
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = resources_get_icon (RESOURCES_ICON_ABOUT_LOGO, -1);
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = _tmp3_;
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = _ ("translator-credits");
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_show_about_dialog (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), "version", RESOURCES_APP_VERSION, "comments", _tmp1_, "copyright", RESOURCES_COPYRIGHT, "website", RESOURCES_HOME_URL, "license", RESOURCES_LICENSE, "website-label", _tmp2_, "authors", RESOURCES_AUTHORS, "logo", _tmp4_, "translator-credits", _tmp5_, NULL, NULL);
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp4_);
#line 667 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp1_);
#line 4226 "AppWindow.c"
}


static void app_window_on_help_contents (AppWindow* self) {
	GError * _inner_error_ = NULL;
#line 681 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 4234 "AppWindow.c"
	{
		GdkScreen* _tmp0_ = NULL;
#line 683 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = gtk_window_get_screen (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 683 "/home/jens/Source/shotwell/src/AppWindow.vala"
		resources_launch_help (_tmp0_, NULL, &_inner_error_);
#line 683 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4243 "AppWindow.c"
			goto __catch215_g_error;
		}
	}
	goto __finally215;
	__catch215_g_error:
	{
		GError* err = NULL;
		const gchar* _tmp1_ = NULL;
		GError* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = _ ("Unable to display help: %s");
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = err;
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = _tmp2_->message;
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = g_strdup_printf (_tmp1_, _tmp3_);
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp4_;
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_error_message (_tmp5_, NULL);
#line 685 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp5_);
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 4276 "AppWindow.c"
	}
	__finally215:
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 682 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4287 "AppWindow.c"
	}
}


static void app_window_on_help_report_problem (AppWindow* self) {
	GError * _inner_error_ = NULL;
#line 689 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 4296 "AppWindow.c"
	{
#line 691 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_show_uri (self, RESOURCES_BUG_DB_URL, &_inner_error_);
#line 691 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4302 "AppWindow.c"
			goto __catch216_g_error;
		}
	}
	goto __finally216;
	__catch216_g_error:
	{
		GError* err = NULL;
		const gchar* _tmp0_ = NULL;
		GError* _tmp1_ = NULL;
		const gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _ ("Unable to navigate to bug database: %s");
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = err;
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = _tmp1_->message;
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = g_strdup_printf (_tmp0_, _tmp2_);
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp3_;
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_error_message (_tmp4_, NULL);
#line 693 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp4_);
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 4335 "AppWindow.c"
	}
	__finally216:
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 690 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4346 "AppWindow.c"
	}
}


static void app_window_on_help_faq (AppWindow* self) {
	GError * _inner_error_ = NULL;
#line 697 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 4355 "AppWindow.c"
	{
#line 699 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_show_uri (self, RESOURCES_FAQ_URL, &_inner_error_);
#line 699 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4361 "AppWindow.c"
			goto __catch217_g_error;
		}
	}
	goto __finally217;
	__catch217_g_error:
	{
		GError* err = NULL;
		const gchar* _tmp0_ = NULL;
		GError* _tmp1_ = NULL;
		const gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _ ("Unable to display FAQ: %s");
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = err;
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = _tmp1_->message;
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = g_strdup_printf (_tmp0_, _tmp2_);
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = _tmp3_;
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_error_message (_tmp4_, NULL);
#line 701 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp4_);
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 4394 "AppWindow.c"
	}
	__finally217:
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 698 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4405 "AppWindow.c"
	}
}


static void app_window_real_on_quit (AppWindow* self) {
	Application* _tmp0_ = NULL;
	Application* _tmp1_ = NULL;
#line 706 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = application_get_instance ();
#line 706 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 706 "/home/jens/Source/shotwell/src/AppWindow.vala"
	application_exit (_tmp1_);
#line 706 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_application_unref0 (_tmp1_);
#line 4421 "AppWindow.c"
}


void app_window_on_quit (AppWindow* self) {
#line 705 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 705 "/home/jens/Source/shotwell/src/AppWindow.vala"
	APP_WINDOW_GET_CLASS (self)->on_quit (self);
#line 4430 "AppWindow.c"
}


void app_window_on_jump_to_file (AppWindow* self) {
	Page* _tmp0_ = NULL;
	Page* _tmp1_ = NULL;
	ViewCollection* _tmp2_ = NULL;
	ViewCollection* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gboolean _tmp5_ = FALSE;
	MediaSource* media = NULL;
	Page* _tmp6_ = NULL;
	Page* _tmp7_ = NULL;
	ViewCollection* _tmp8_ = NULL;
	ViewCollection* _tmp9_ = NULL;
	DataView* _tmp10_ = NULL;
	DataView* _tmp11_ = NULL;
	DataSource* _tmp12_ = NULL;
	MediaSource* _tmp13_ = NULL;
	MediaSource* _tmp14_ = NULL;
	MediaSource* _tmp15_ = NULL;
	GError * _inner_error_ = NULL;
#line 709 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = page_get_view (_tmp1_);
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _tmp2_;
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = view_collection_get_selected_count (_tmp3_);
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = _tmp4_ != 1;
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_data_collection_unref0 (_tmp3_);
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp1_);
#line 710 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp5_) {
#line 711 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4475 "AppWindow.c"
	}
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = _tmp6_;
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = page_get_view (_tmp7_);
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = _tmp8_;
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = view_collection_get_selected_at (_tmp9_, 0);
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp11_ = _tmp10_;
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = data_view_get_source (_tmp11_);
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp12_, TYPE_MEDIA_SOURCE) ? ((MediaSource*) _tmp12_) : NULL;
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp13_ == NULL) {
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp12_);
#line 4497 "AppWindow.c"
	}
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = _tmp13_;
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp11_);
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_data_collection_unref0 (_tmp9_);
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp7_);
#line 713 "/home/jens/Source/shotwell/src/AppWindow.vala"
	media = _tmp14_;
#line 715 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = media;
#line 715 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp15_ == NULL) {
#line 716 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (media);
#line 716 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4517 "AppWindow.c"
	}
	{
		AppWindow* _tmp16_ = NULL;
		AppWindow* _tmp17_ = NULL;
		MediaSource* _tmp18_ = NULL;
		GFile* _tmp19_ = NULL;
		GFile* _tmp20_ = NULL;
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp16_ = app_window_get_instance ();
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp17_ = _tmp16_;
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp18_ = media;
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp19_ = media_source_get_master_file (_tmp18_);
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp20_ = _tmp19_;
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_show_file_uri (_tmp17_, _tmp20_, &_inner_error_);
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp20_);
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp17_);
#line 719 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4543 "AppWindow.c"
			goto __catch218_g_error;
		}
	}
	goto __finally218;
	__catch218_g_error:
	{
		GError* err = NULL;
		GError* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		err = _inner_error_;
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_inner_error_ = NULL;
#line 721 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp21_ = err;
#line 721 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp22_ = resources_jump_to_file_failed (_tmp21_);
#line 721 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp23_ = _tmp22_;
#line 721 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_error_message (_tmp23_, NULL);
#line 721 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp23_);
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_error_free0 (err);
#line 4570 "AppWindow.c"
	}
	__finally218:
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (media);
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_clear_error (&_inner_error_);
#line 718 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4583 "AppWindow.c"
	}
#line 709 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (media);
#line 4587 "AppWindow.c"
}


static void app_window_real_destroy (GtkWidget* base) {
	AppWindow * self;
#line 725 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_APP_WINDOW, AppWindow);
#line 726 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_quit (self);
#line 4597 "AppWindow.c"
}


static gchar* string_replace (const gchar* self, const gchar* old, const gchar* replacement) {
	gchar* result = NULL;
	GError * _inner_error_ = NULL;
#line 1380 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1380 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 1380 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 4610 "AppWindow.c"
	{
		GRegex* regex = NULL;
		const gchar* _tmp0_ = NULL;
		gchar* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		GRegex* _tmp3_ = NULL;
		GRegex* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		GRegex* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp0_ = old;
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp1_ = g_regex_escape_string (_tmp0_, -1);
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp2_ = _tmp1_;
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp3_ = g_regex_new (_tmp2_, 0, 0, &_inner_error_);
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp4_ = _tmp3_;
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_g_free0 (_tmp2_);
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		regex = _tmp4_;
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 4641 "AppWindow.c"
				goto __catch219_g_regex_error;
			}
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1382 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			return NULL;
#line 4650 "AppWindow.c"
		}
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp6_ = regex;
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp7_ = replacement;
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp8_ = g_regex_replace_literal (_tmp6_, self, (gssize) -1, 0, _tmp7_, 0, &_inner_error_);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp5_ = _tmp8_;
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 4666 "AppWindow.c"
				goto __catch219_g_regex_error;
			}
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
			return NULL;
#line 4677 "AppWindow.c"
		}
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp9_ = _tmp5_;
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_tmp5_ = NULL;
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		result = _tmp9_;
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_g_free0 (_tmp5_);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_g_regex_unref0 (regex);
#line 1383 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		return result;
#line 4691 "AppWindow.c"
	}
	goto __finally219;
	__catch219_g_regex_error:
	{
		GError* e = NULL;
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		e = _inner_error_;
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_inner_error_ = NULL;
#line 1385 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		g_assert_not_reached ();
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		_g_error_free0 (e);
#line 4705 "AppWindow.c"
	}
	__finally219:
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		g_clear_error (&_inner_error_);
#line 1381 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
		return NULL;
#line 4716 "AppWindow.c"
	}
}


void app_window_show_file_uri (AppWindow* self, GFile* file, GError** error) {
	gchar* tmp = NULL;
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	GError * _inner_error_ = NULL;
#line 729 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 729 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (G_IS_FILE (file));
#line 734 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = get_nautilus_install_location ();
#line 734 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 734 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = _tmp1_ != NULL;
#line 734 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (_tmp1_);
#line 734 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_) {
#line 4741 "AppWindow.c"
		GFile* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = file;
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = g_file_get_uri (_tmp3_);
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp4_;
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = string_replace (_tmp5_, "'", "\\\'");
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (tmp);
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		tmp = _tmp6_;
#line 735 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp5_);
#line 736 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = tmp;
#line 736 "/home/jens/Source/shotwell/src/AppWindow.vala"
		show_file_in_nautilus (_tmp7_, &_inner_error_);
#line 736 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 736 "/home/jens/Source/shotwell/src/AppWindow.vala"
			g_propagate_error (error, _inner_error_);
#line 736 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_g_free0 (tmp);
#line 736 "/home/jens/Source/shotwell/src/AppWindow.vala"
			return;
#line 4773 "AppWindow.c"
		}
	} else {
		GFile* _tmp8_ = NULL;
		GFile* _tmp9_ = NULL;
		GFile* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		const gchar* _tmp14_ = NULL;
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = file;
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = g_file_get_parent (_tmp8_);
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _tmp9_;
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = g_file_get_uri (_tmp10_);
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp12_ = _tmp11_;
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp13_ = string_replace (_tmp12_, "'", "\\\'");
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (tmp);
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		tmp = _tmp13_;
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp12_);
#line 738 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp10_);
#line 739 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp14_ = tmp;
#line 739 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_show_uri (self, _tmp14_, &_inner_error_);
#line 739 "/home/jens/Source/shotwell/src/AppWindow.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 739 "/home/jens/Source/shotwell/src/AppWindow.vala"
			g_propagate_error (error, _inner_error_);
#line 739 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_g_free0 (tmp);
#line 739 "/home/jens/Source/shotwell/src/AppWindow.vala"
			return;
#line 4815 "AppWindow.c"
		}
	}
#line 729 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_free0 (tmp);
#line 4820 "AppWindow.c"
}


void app_window_show_uri (AppWindow* self, const gchar* url, GError** error) {
	GdkWindow* _tmp0_ = NULL;
	GdkScreen* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	GError * _inner_error_ = NULL;
#line 743 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 743 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (url != NULL);
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_widget_get_window (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gdk_window_get_screen (_tmp0_);
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = url;
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
	sys_show_uri (_tmp1_, _tmp2_, &_inner_error_);
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_propagate_error (error, _inner_error_);
#line 744 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 4847 "AppWindow.c"
	}
}


static void _vala_array_add170 (GtkActionGroup*** array, int* length, int* size, GtkActionGroup* value) {
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if ((*length) == (*size)) {
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*array = g_renew (GtkActionGroup*, *array, (*size) + 1);
#line 4859 "AppWindow.c"
	}
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[(*length)++] = value;
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	(*array)[*length] = NULL;
#line 4865 "AppWindow.c"
}


static GtkActionGroup** app_window_real_create_common_action_groups (AppWindow* self, int* result_length1) {
	GtkActionGroup** result = NULL;
	GtkActionGroup** groups = NULL;
	GtkActionGroup** _tmp0_ = NULL;
	gint groups_length1 = 0;
	gint _groups_size_ = 0;
	GtkActionGroup* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GtkActionEntry* _tmp3_ = NULL;
	GtkActionEntry* _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
	GtkActionGroup** _tmp5_ = NULL;
	gint _tmp5__length1 = 0;
	GtkActionGroup* _tmp6_ = NULL;
	GtkActionGroup* _tmp7_ = NULL;
	GtkActionGroup** _tmp8_ = NULL;
	gint _tmp8__length1 = 0;
#line 748 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = g_new0 (GtkActionGroup*, 0 + 1);
#line 748 "/home/jens/Source/shotwell/src/AppWindow.vala"
	groups = _tmp0_;
#line 748 "/home/jens/Source/shotwell/src/AppWindow.vala"
	groups_length1 = 0;
#line 748 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_groups_size_ = groups_length1;
#line 750 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = self->priv->common_action_group;
#line 750 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = app_window_create_common_actions (self, &_tmp2_);
#line 750 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = _tmp3_;
#line 750 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4__length1 = _tmp2_;
#line 750 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_action_group_add_actions (_tmp1_, _tmp4_, _tmp2_, self);
#line 750 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = (g_free (_tmp4_), NULL);
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = groups;
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5__length1 = groups_length1;
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = self->priv->common_action_group;
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = _g_object_ref0 (_tmp6_);
#line 751 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_vala_array_add170 (&groups, &groups_length1, &_groups_size_, _tmp7_);
#line 753 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = groups;
#line 753 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8__length1 = groups_length1;
#line 753 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (result_length1) {
#line 753 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*result_length1 = _tmp8__length1;
#line 4924 "AppWindow.c"
	}
#line 753 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp8_;
#line 753 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 4930 "AppWindow.c"
}


GtkActionGroup** app_window_create_common_action_groups (AppWindow* self, int* result_length1) {
#line 747 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 747 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return APP_WINDOW_GET_CLASS (self)->create_common_action_groups (self, result_length1);
#line 4939 "AppWindow.c"
}


static GtkActionGroup** _vala_array_dup28 (GtkActionGroup** self, int length) {
	GtkActionGroup** result;
	int i;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = g_new0 (GtkActionGroup*, length + 1);
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	for (i = 0; i < length; i++) {
#line 4950 "AppWindow.c"
		GtkActionGroup* _tmp0_ = NULL;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp0_ = _g_object_ref0 (self[i]);
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
		result[i] = _tmp0_;
#line 4956 "AppWindow.c"
	}
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 4960 "AppWindow.c"
}


GtkActionGroup** app_window_get_common_action_groups (AppWindow* self, int* result_length1) {
	GtkActionGroup** result = NULL;
	GtkActionGroup** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	GtkActionGroup** _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
	GtkActionGroup** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
#line 756 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->common_action_groups;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0__length1 = self->common_action_groups_length1;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = (_tmp0_ != NULL) ? _vala_array_dup28 (_tmp0_, _tmp0__length1) : ((gpointer) _tmp0_);
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1__length1 = _tmp0__length1;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = _tmp1_;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2__length1 = _tmp1__length1;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (result_length1) {
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
		*result_length1 = _tmp2__length1;
#line 4990 "AppWindow.c"
	}
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp2_;
#line 757 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 4996 "AppWindow.c"
}


static void app_window_real_replace_common_placeholders (AppWindow* self, GtkUIManager* ui) {
#line 760 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (GTK_IS_UI_MANAGER (ui));
#line 5003 "AppWindow.c"
}


void app_window_replace_common_placeholders (AppWindow* self, GtkUIManager* ui) {
#line 760 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 760 "/home/jens/Source/shotwell/src/AppWindow.vala"
	APP_WINDOW_GET_CLASS (self)->replace_common_placeholders (self, ui);
#line 5012 "AppWindow.c"
}


void app_window_go_fullscreen (AppWindow* self, Page* page) {
	FullscreenWindow* _tmp0_ = NULL;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	FullscreenWindow* fsw = NULL;
	Page* _tmp4_ = NULL;
	FullscreenWindow* _tmp5_ = NULL;
	Page* _tmp6_ = NULL;
	Page* _tmp7_ = NULL;
	gboolean _tmp8_ = FALSE;
	FullscreenWindow* _tmp12_ = NULL;
	FullscreenWindow* _tmp13_ = NULL;
	FullscreenWindow* _tmp14_ = NULL;
#line 763 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 763 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 765 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_fullscreen_window;
#line 765 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ != NULL) {
#line 5037 "AppWindow.c"
		FullscreenWindow* _tmp1_ = NULL;
#line 766 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = app_window_fullscreen_window;
#line 766 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_window_present (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, gtk_window_get_type (), GtkWindow));
#line 768 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 5045 "AppWindow.c"
	}
#line 771 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_get_position (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), &_tmp2_, &_tmp3_);
#line 771 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->pos_x = _tmp2_;
#line 771 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->pos_y = _tmp3_;
#line 772 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_hide (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 774 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = page;
#line 774 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = fullscreen_window_new (_tmp4_);
#line 774 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_object_ref_sink (_tmp5_);
#line 774 "/home/jens/Source/shotwell/src/AppWindow.vala"
	fsw = _tmp5_;
#line 776 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 776 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = _tmp6_;
#line 776 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp8_ = _tmp7_ != NULL;
#line 776 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp7_);
#line 776 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp8_) {
#line 5073 "AppWindow.c"
		Page* _tmp9_ = NULL;
		Page* _tmp10_ = NULL;
		FullscreenWindow* _tmp11_ = NULL;
#line 777 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 777 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _tmp9_;
#line 777 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = fsw;
#line 777 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_switching_to_fullscreen (_tmp10_, _tmp11_);
#line 777 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp10_);
#line 5087 "AppWindow.c"
	}
#line 779 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp12_ = fsw;
#line 779 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp13_ = _g_object_ref0 (_tmp12_);
#line 779 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (app_window_fullscreen_window);
#line 779 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_fullscreen_window = _tmp13_;
#line 780 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp14_ = app_window_fullscreen_window;
#line 780 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_present (G_TYPE_CHECK_INSTANCE_CAST (_tmp14_, gtk_window_get_type (), GtkWindow));
#line 763 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (fsw);
#line 5103 "AppWindow.c"
}


void app_window_end_fullscreen (AppWindow* self) {
	FullscreenWindow* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	Page* _tmp3_ = NULL;
	Page* _tmp4_ = NULL;
	gboolean _tmp5_ = FALSE;
	FullscreenWindow* _tmp9_ = NULL;
	FullscreenWindow* _tmp10_ = NULL;
#line 783 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 784 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_fullscreen_window;
#line 784 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp0_ == NULL) {
#line 785 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 5124 "AppWindow.c"
	}
#line 787 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = self->pos_x;
#line 787 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = self->pos_y;
#line 787 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_move (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), _tmp1_, _tmp2_);
#line 789 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_show_all (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 791 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 791 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp4_ = _tmp3_;
#line 791 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = _tmp4_ != NULL;
#line 791 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp4_);
#line 791 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp5_) {
#line 5144 "AppWindow.c"
		Page* _tmp6_ = NULL;
		Page* _tmp7_ = NULL;
		FullscreenWindow* _tmp8_ = NULL;
#line 792 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 792 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = _tmp6_;
#line 792 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = app_window_fullscreen_window;
#line 792 "/home/jens/Source/shotwell/src/AppWindow.vala"
		page_returning_from_fullscreen (_tmp7_, _tmp8_);
#line 792 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp7_);
#line 5158 "AppWindow.c"
	}
#line 794 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp9_ = app_window_fullscreen_window;
#line 794 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_hide (G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, gtk_widget_get_type (), GtkWidget));
#line 795 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp10_ = app_window_fullscreen_window;
#line 795 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_widget_destroy (G_TYPE_CHECK_INSTANCE_CAST (_tmp10_, gtk_widget_get_type (), GtkWidget));
#line 796 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (app_window_fullscreen_window);
#line 796 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_fullscreen_window = NULL;
#line 798 "/home/jens/Source/shotwell/src/AppWindow.vala"
	gtk_window_present (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow));
#line 5174 "AppWindow.c"
}


GtkAction* app_window_get_common_action (AppWindow* self, const gchar* name) {
	GtkAction* result = NULL;
	GtkActionGroup** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	const gchar* _tmp7_ = NULL;
#line 801 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 801 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = self->common_action_groups;
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0__length1 = self->common_action_groups_length1;
#line 5191 "AppWindow.c"
	{
		GtkActionGroup** group_collection = NULL;
		gint group_collection_length1 = 0;
		gint _group_collection_size_ = 0;
		gint group_it = 0;
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
		group_collection = _tmp0_;
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
		group_collection_length1 = _tmp0__length1;
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
		for (group_it = 0; group_it < _tmp0__length1; group_it = group_it + 1) {
#line 5203 "AppWindow.c"
			GtkActionGroup* _tmp1_ = NULL;
			GtkActionGroup* group = NULL;
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
			_tmp1_ = _g_object_ref0 (group_collection[group_it]);
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
			group = _tmp1_;
#line 5210 "AppWindow.c"
			{
				GtkAction* action = NULL;
				GtkActionGroup* _tmp2_ = NULL;
				const gchar* _tmp3_ = NULL;
				GtkAction* _tmp4_ = NULL;
				GtkAction* _tmp5_ = NULL;
				GtkAction* _tmp6_ = NULL;
#line 803 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp2_ = group;
#line 803 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp3_ = name;
#line 803 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp4_ = gtk_action_group_get_action (_tmp2_, _tmp3_);
#line 803 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp5_ = _g_object_ref0 (_tmp4_);
#line 803 "/home/jens/Source/shotwell/src/AppWindow.vala"
				action = _tmp5_;
#line 804 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_tmp6_ = action;
#line 804 "/home/jens/Source/shotwell/src/AppWindow.vala"
				if (_tmp6_ != NULL) {
#line 805 "/home/jens/Source/shotwell/src/AppWindow.vala"
					result = action;
#line 805 "/home/jens/Source/shotwell/src/AppWindow.vala"
					_g_object_unref0 (group);
#line 805 "/home/jens/Source/shotwell/src/AppWindow.vala"
					return result;
#line 5238 "AppWindow.c"
				}
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_g_object_unref0 (action);
#line 802 "/home/jens/Source/shotwell/src/AppWindow.vala"
				_g_object_unref0 (group);
#line 5244 "AppWindow.c"
			}
		}
	}
#line 808 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp7_ = name;
#line 808 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_warning ("AppWindow.vala:808: No common action found: %s", _tmp7_);
#line 810 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = NULL;
#line 810 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 5256 "AppWindow.c"
}


void app_window_set_common_action_sensitive (AppWindow* self, const gchar* name, gboolean sensitive) {
	GtkAction* action = NULL;
	const gchar* _tmp0_ = NULL;
	GtkAction* _tmp1_ = NULL;
	GtkAction* _tmp2_ = NULL;
#line 813 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 813 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (name != NULL);
#line 814 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = name;
#line 814 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = app_window_get_common_action (self, _tmp0_);
#line 814 "/home/jens/Source/shotwell/src/AppWindow.vala"
	action = _tmp1_;
#line 815 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = action;
#line 815 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ != NULL) {
#line 5279 "AppWindow.c"
		GtkAction* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 816 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = action;
#line 816 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = sensitive;
#line 816 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_sensitive (_tmp3_, _tmp4_);
#line 5288 "AppWindow.c"
	}
#line 813 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (action);
#line 5292 "AppWindow.c"
}


void app_window_set_common_action_important (AppWindow* self, const gchar* name, gboolean important) {
	GtkAction* action = NULL;
	const gchar* _tmp0_ = NULL;
	GtkAction* _tmp1_ = NULL;
	GtkAction* _tmp2_ = NULL;
#line 819 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 819 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (name != NULL);
#line 820 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = name;
#line 820 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = app_window_get_common_action (self, _tmp0_);
#line 820 "/home/jens/Source/shotwell/src/AppWindow.vala"
	action = _tmp1_;
#line 821 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = action;
#line 821 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ != NULL) {
#line 5315 "AppWindow.c"
		GtkAction* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 822 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = action;
#line 822 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = important;
#line 822 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_is_important (_tmp3_, _tmp4_);
#line 5324 "AppWindow.c"
	}
#line 819 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (action);
#line 5328 "AppWindow.c"
}


void app_window_set_common_action_visible (AppWindow* self, const gchar* name, gboolean visible) {
	GtkAction* action = NULL;
	const gchar* _tmp0_ = NULL;
	GtkAction* _tmp1_ = NULL;
	GtkAction* _tmp2_ = NULL;
#line 825 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 825 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (name != NULL);
#line 826 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = name;
#line 826 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = app_window_get_common_action (self, _tmp0_);
#line 826 "/home/jens/Source/shotwell/src/AppWindow.vala"
	action = _tmp1_;
#line 827 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = action;
#line 827 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ != NULL) {
#line 5351 "AppWindow.c"
		GtkAction* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
#line 828 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = action;
#line 828 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = visible;
#line 828 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_visible (_tmp3_, _tmp4_);
#line 5360 "AppWindow.c"
	}
#line 825 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (action);
#line 5364 "AppWindow.c"
}


static void _app_window_on_update_common_actions_data_collection_contents_altered (DataCollection* _sender, GeeIterable* added, GeeIterable* removed, gpointer self) {
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_update_common_actions ((AppWindow*) self);
#line 5371 "AppWindow.c"
}


static void _app_window_on_update_common_actions_view_collection_selection_group_altered (ViewCollection* _sender, gpointer self) {
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_update_common_actions ((AppWindow*) self);
#line 5378 "AppWindow.c"
}


static void _app_window_on_update_common_actions_view_collection_items_state_changed (ViewCollection* _sender, GeeIterable* changed, gpointer self) {
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_on_update_common_actions ((AppWindow*) self);
#line 5385 "AppWindow.c"
}


static void app_window_real_switched_pages (PageWindow* base, Page* old_page, Page* new_page) {
	AppWindow * self;
	Page* _tmp0_ = NULL;
	Page* _tmp1_ = NULL;
	Page* _tmp2_ = NULL;
	Page* _tmp15_ = NULL;
	Page* _tmp34_ = NULL;
	Page* _tmp35_ = NULL;
#line 831 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_APP_WINDOW, AppWindow);
#line 831 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((old_page == NULL) || IS_PAGE (old_page));
#line 831 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((new_page == NULL) || IS_PAGE (new_page));
#line 832 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = old_page;
#line 832 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = new_page;
#line 832 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_update_common_action_availability (self, _tmp0_, _tmp1_);
#line 834 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = old_page;
#line 834 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ != NULL) {
#line 5413 "AppWindow.c"
		Page* _tmp3_ = NULL;
		ViewCollection* _tmp4_ = NULL;
		ViewCollection* _tmp5_ = NULL;
		guint _tmp6_ = 0U;
		Page* _tmp7_ = NULL;
		ViewCollection* _tmp8_ = NULL;
		ViewCollection* _tmp9_ = NULL;
		guint _tmp10_ = 0U;
		Page* _tmp11_ = NULL;
		ViewCollection* _tmp12_ = NULL;
		ViewCollection* _tmp13_ = NULL;
		guint _tmp14_ = 0U;
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = old_page;
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = page_get_view (_tmp3_);
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp4_;
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_parse_name ("contents-altered", TYPE_DATA_COLLECTION, &_tmp6_, NULL, FALSE);
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_handlers_disconnect_matched (G_TYPE_CHECK_INSTANCE_CAST (_tmp5_, TYPE_DATA_COLLECTION, DataCollection), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp6_, 0, NULL, (GCallback) _app_window_on_update_common_actions_data_collection_contents_altered, self);
#line 835 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp5_);
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = old_page;
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = page_get_view (_tmp7_);
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = _tmp8_;
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_parse_name ("selection-group-altered", TYPE_VIEW_COLLECTION, &_tmp10_, NULL, FALSE);
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_handlers_disconnect_matched (_tmp9_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp10_, 0, NULL, (GCallback) _app_window_on_update_common_actions_view_collection_selection_group_altered, self);
#line 836 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp9_);
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = old_page;
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp12_ = page_get_view (_tmp11_);
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp13_ = _tmp12_;
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_parse_name ("items-state-changed", TYPE_VIEW_COLLECTION, &_tmp14_, NULL, FALSE);
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_handlers_disconnect_matched (_tmp13_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp14_, 0, NULL, (GCallback) _app_window_on_update_common_actions_view_collection_items_state_changed, self);
#line 837 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp13_);
#line 5462 "AppWindow.c"
	}
#line 840 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp15_ = new_page;
#line 840 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp15_ != NULL) {
#line 5468 "AppWindow.c"
		Page* _tmp16_ = NULL;
		ViewCollection* _tmp17_ = NULL;
		ViewCollection* _tmp18_ = NULL;
		Page* _tmp19_ = NULL;
		ViewCollection* _tmp20_ = NULL;
		ViewCollection* _tmp21_ = NULL;
		Page* _tmp22_ = NULL;
		ViewCollection* _tmp23_ = NULL;
		ViewCollection* _tmp24_ = NULL;
		Page* _tmp25_ = NULL;
		Page* _tmp26_ = NULL;
		ViewCollection* _tmp27_ = NULL;
		ViewCollection* _tmp28_ = NULL;
		gint _tmp29_ = 0;
		Page* _tmp30_ = NULL;
		ViewCollection* _tmp31_ = NULL;
		ViewCollection* _tmp32_ = NULL;
		gint _tmp33_ = 0;
#line 841 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp16_ = new_page;
#line 841 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp17_ = page_get_view (_tmp16_);
#line 841 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp18_ = _tmp17_;
#line 841 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp18_, TYPE_DATA_COLLECTION, DataCollection), "contents-altered", (GCallback) _app_window_on_update_common_actions_data_collection_contents_altered, self, 0);
#line 841 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp18_);
#line 842 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp19_ = new_page;
#line 842 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp20_ = page_get_view (_tmp19_);
#line 842 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp21_ = _tmp20_;
#line 842 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_connect_object (_tmp21_, "selection-group-altered", (GCallback) _app_window_on_update_common_actions_view_collection_selection_group_altered, self, 0);
#line 842 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp21_);
#line 843 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp22_ = new_page;
#line 843 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp23_ = page_get_view (_tmp22_);
#line 843 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp24_ = _tmp23_;
#line 843 "/home/jens/Source/shotwell/src/AppWindow.vala"
		g_signal_connect_object (_tmp24_, "items-state-changed", (GCallback) _app_window_on_update_common_actions_view_collection_items_state_changed, self, 0);
#line 843 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp24_);
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp25_ = new_page;
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp26_ = new_page;
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp27_ = page_get_view (_tmp26_);
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp28_ = _tmp27_;
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp29_ = view_collection_get_selected_count (_tmp28_);
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp30_ = new_page;
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp31_ = page_get_view (_tmp30_);
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp32_ = _tmp31_;
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp33_ = data_collection_get_count (G_TYPE_CHECK_INSTANCE_CAST (_tmp32_, TYPE_DATA_COLLECTION, DataCollection));
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_update_common_actions (self, _tmp25_, _tmp29_, _tmp33_);
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp32_);
#line 845 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp28_);
#line 5541 "AppWindow.c"
	}
#line 849 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp34_ = old_page;
#line 849 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp35_ = new_page;
#line 849 "/home/jens/Source/shotwell/src/AppWindow.vala"
	PAGE_WINDOW_CLASS (app_window_parent_class)->switched_pages (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), _tmp34_, _tmp35_);
#line 5549 "AppWindow.c"
}


static void app_window_real_update_common_action_availability (AppWindow* self, Page* old_page, Page* new_page) {
	gboolean is_checkerboard = FALSE;
	Page* _tmp0_ = NULL;
#line 855 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((old_page == NULL) || IS_PAGE (old_page));
#line 855 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((new_page == NULL) || IS_PAGE (new_page));
#line 856 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = new_page;
#line 856 "/home/jens/Source/shotwell/src/AppWindow.vala"
	is_checkerboard = G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, TYPE_CHECKERBOARD_PAGE);
#line 858 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_set_common_action_sensitive (self, "CommonSelectAll", is_checkerboard);
#line 859 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_set_common_action_sensitive (self, "CommonSelectNone", is_checkerboard);
#line 5568 "AppWindow.c"
}


void app_window_update_common_action_availability (AppWindow* self, Page* old_page, Page* new_page) {
#line 855 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 855 "/home/jens/Source/shotwell/src/AppWindow.vala"
	APP_WINDOW_GET_CLASS (self)->update_common_action_availability (self, old_page, new_page);
#line 5577 "AppWindow.c"
}


static void app_window_real_update_common_actions (AppWindow* self, Page* page, gint selected_count, gint count) {
	Page* _tmp0_ = NULL;
	gint _tmp2_ = 0;
#line 865 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 866 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page;
#line 866 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, TYPE_CHECKERBOARD_PAGE)) {
#line 5590 "AppWindow.c"
		gint _tmp1_ = 0;
#line 867 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp1_ = count;
#line 867 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_set_common_action_sensitive (self, "CommonSelectAll", _tmp1_ > 0);
#line 5596 "AppWindow.c"
	}
#line 868 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = selected_count;
#line 868 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_set_common_action_sensitive (self, "CommonJumpToFile", _tmp2_ == 1);
#line 870 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_decorate_undo_action (self);
#line 871 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_decorate_redo_action (self);
#line 5606 "AppWindow.c"
}


void app_window_update_common_actions (AppWindow* self, Page* page, gint selected_count, gint count) {
#line 865 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 865 "/home/jens/Source/shotwell/src/AppWindow.vala"
	APP_WINDOW_GET_CLASS (self)->update_common_actions (self, page, selected_count, count);
#line 5615 "AppWindow.c"
}


static void app_window_on_update_common_actions (AppWindow* self) {
	Page* page = NULL;
	Page* _tmp0_ = NULL;
	Page* _tmp1_ = NULL;
#line 874 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 875 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 875 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page = _tmp0_;
#line 876 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = page;
#line 876 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ != NULL) {
#line 5633 "AppWindow.c"
		Page* _tmp2_ = NULL;
		Page* _tmp3_ = NULL;
		ViewCollection* _tmp4_ = NULL;
		ViewCollection* _tmp5_ = NULL;
		gint _tmp6_ = 0;
		Page* _tmp7_ = NULL;
		ViewCollection* _tmp8_ = NULL;
		ViewCollection* _tmp9_ = NULL;
		gint _tmp10_ = 0;
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp2_ = page;
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = page;
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = page_get_view (_tmp3_);
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp4_;
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = view_collection_get_selected_count (_tmp5_);
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = page;
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = page_get_view (_tmp7_);
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = _tmp8_;
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = data_collection_get_count (G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, TYPE_DATA_COLLECTION, DataCollection));
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		app_window_update_common_actions (self, _tmp2_, _tmp6_, _tmp10_);
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp9_);
#line 877 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp5_);
#line 5667 "AppWindow.c"
	}
#line 874 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (page);
#line 5671 "AppWindow.c"
}


static gpointer _command_manager_ref0 (gpointer self) {
#line 881 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return self ? command_manager_ref (self) : NULL;
#line 5678 "AppWindow.c"
}


CommandManager* app_window_get_command_manager (void) {
	CommandManager* result = NULL;
	CommandManager* _tmp0_ = NULL;
	CommandManager* _tmp1_ = NULL;
#line 881 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_command_manager;
#line 881 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _command_manager_ref0 (_tmp0_);
#line 881 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp1_;
#line 881 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 5694 "AppWindow.c"
}


static void app_window_on_command_manager_altered (AppWindow* self) {
#line 884 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 885 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_decorate_undo_action (self);
#line 886 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_decorate_redo_action (self);
#line 5705 "AppWindow.c"
}


static void app_window_decorate_command_manager_action (AppWindow* self, const gchar* name, const gchar* prefix, const gchar* default_explanation, CommandDescription* desc) {
	GtkAction* action = NULL;
	const gchar* _tmp0_ = NULL;
	GtkAction* _tmp1_ = NULL;
	GtkAction* _tmp2_ = NULL;
	CommandDescription* _tmp3_ = NULL;
#line 889 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 889 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (name != NULL);
#line 889 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (prefix != NULL);
#line 889 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (default_explanation != NULL);
#line 889 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail ((desc == NULL) || IS_COMMAND_DESCRIPTION (desc));
#line 891 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = name;
#line 891 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = app_window_get_common_action (self, _tmp0_);
#line 891 "/home/jens/Source/shotwell/src/AppWindow.vala"
	action = _tmp1_;
#line 892 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = action;
#line 892 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ == NULL) {
#line 893 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (action);
#line 893 "/home/jens/Source/shotwell/src/AppWindow.vala"
		return;
#line 5739 "AppWindow.c"
	}
#line 895 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = desc;
#line 895 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp3_ != NULL) {
#line 5745 "AppWindow.c"
		GtkAction* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		CommandDescription* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		GtkAction* _tmp11_ = NULL;
		CommandDescription* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		GtkAction* _tmp15_ = NULL;
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = action;
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = prefix;
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp6_ = desc;
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp7_ = command_description_get_name (_tmp6_);
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp8_ = _tmp7_;
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp9_ = g_strdup_printf ("%s %s", _tmp5_, _tmp8_);
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp10_ = _tmp9_;
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_label (_tmp4_, _tmp10_);
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp10_);
#line 896 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp8_);
#line 897 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp11_ = action;
#line 897 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp12_ = desc;
#line 897 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp13_ = command_description_get_explanation (_tmp12_);
#line 897 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp14_ = _tmp13_;
#line 897 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_tooltip (_tmp11_, _tmp14_);
#line 897 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_free0 (_tmp14_);
#line 898 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp15_ = action;
#line 898 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_sensitive (_tmp15_, TRUE);
#line 5794 "AppWindow.c"
	} else {
		GtkAction* _tmp16_ = NULL;
		const gchar* _tmp17_ = NULL;
		GtkAction* _tmp18_ = NULL;
		const gchar* _tmp19_ = NULL;
		GtkAction* _tmp20_ = NULL;
#line 900 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp16_ = action;
#line 900 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp17_ = prefix;
#line 900 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_label (_tmp16_, _tmp17_);
#line 901 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp18_ = action;
#line 901 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp19_ = default_explanation;
#line 901 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_tooltip (_tmp18_, _tmp19_);
#line 902 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp20_ = action;
#line 902 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_action_set_sensitive (_tmp20_, FALSE);
#line 5817 "AppWindow.c"
	}
#line 889 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (action);
#line 5821 "AppWindow.c"
}


void app_window_decorate_undo_action (AppWindow* self) {
	CommandManager* _tmp0_ = NULL;
	CommandManager* _tmp1_ = NULL;
	CommandDescription* _tmp2_ = NULL;
	CommandDescription* _tmp3_ = NULL;
#line 906 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_get_command_manager ();
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = command_manager_get_undo_description (_tmp1_);
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _tmp2_;
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_decorate_command_manager_action (self, "CommonUndo", RESOURCES_UNDO_MENU, "", _tmp3_);
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp3_);
#line 907 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_command_manager_unref0 (_tmp1_);
#line 5846 "AppWindow.c"
}


void app_window_decorate_redo_action (AppWindow* self) {
	CommandManager* _tmp0_ = NULL;
	CommandManager* _tmp1_ = NULL;
	CommandDescription* _tmp2_ = NULL;
	CommandDescription* _tmp3_ = NULL;
#line 911 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_get_command_manager ();
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = _tmp0_;
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = command_manager_get_redo_description (_tmp1_);
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp3_ = _tmp2_;
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_decorate_command_manager_action (self, "CommonRedo", RESOURCES_REDO_MENU, "", _tmp3_);
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (_tmp3_);
#line 912 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_command_manager_unref0 (_tmp1_);
#line 5871 "AppWindow.c"
}


static void app_window_on_undo (AppWindow* self) {
	CommandManager* _tmp0_ = NULL;
#line 916 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 917 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_command_manager;
#line 917 "/home/jens/Source/shotwell/src/AppWindow.vala"
	command_manager_undo (_tmp0_);
#line 5883 "AppWindow.c"
}


static void app_window_on_redo (AppWindow* self) {
	CommandManager* _tmp0_ = NULL;
#line 920 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 921 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = app_window_command_manager;
#line 921 "/home/jens/Source/shotwell/src/AppWindow.vala"
	command_manager_redo (_tmp0_);
#line 5895 "AppWindow.c"
}


static void app_window_on_select_all (AppWindow* self) {
	Page* page = NULL;
	Page* _tmp0_ = NULL;
	CheckerboardPage* _tmp1_ = NULL;
	Page* _tmp2_ = NULL;
#line 924 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 925 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 925 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, TYPE_CHECKERBOARD_PAGE) ? ((CheckerboardPage*) _tmp0_) : NULL;
#line 925 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ == NULL) {
#line 925 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 5914 "AppWindow.c"
	}
#line 925 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page = G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_PAGE, Page);
#line 926 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = page;
#line 926 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ != NULL) {
#line 5922 "AppWindow.c"
		Page* _tmp3_ = NULL;
		ViewCollection* _tmp4_ = NULL;
		ViewCollection* _tmp5_ = NULL;
#line 927 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = page;
#line 927 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = page_get_view (_tmp3_);
#line 927 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp4_;
#line 927 "/home/jens/Source/shotwell/src/AppWindow.vala"
		view_collection_select_all (_tmp5_);
#line 927 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp5_);
#line 5936 "AppWindow.c"
	}
#line 924 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (page);
#line 5940 "AppWindow.c"
}


static void app_window_on_select_none (AppWindow* self) {
	Page* page = NULL;
	Page* _tmp0_ = NULL;
	CheckerboardPage* _tmp1_ = NULL;
	Page* _tmp2_ = NULL;
#line 930 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 931 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = page_window_get_current_page (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow));
#line 931 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, TYPE_CHECKERBOARD_PAGE) ? ((CheckerboardPage*) _tmp0_) : NULL;
#line 931 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp1_ == NULL) {
#line 931 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_g_object_unref0 (_tmp0_);
#line 5959 "AppWindow.c"
	}
#line 931 "/home/jens/Source/shotwell/src/AppWindow.vala"
	page = G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_PAGE, Page);
#line 932 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = page;
#line 932 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (_tmp2_ != NULL) {
#line 5967 "AppWindow.c"
		Page* _tmp3_ = NULL;
		ViewCollection* _tmp4_ = NULL;
		ViewCollection* _tmp5_ = NULL;
#line 933 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp3_ = page;
#line 933 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp4_ = page_get_view (_tmp3_);
#line 933 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_tmp5_ = _tmp4_;
#line 933 "/home/jens/Source/shotwell/src/AppWindow.vala"
		view_collection_unselect_all (_tmp5_);
#line 933 "/home/jens/Source/shotwell/src/AppWindow.vala"
		_data_collection_unref0 (_tmp5_);
#line 5981 "AppWindow.c"
	}
#line 930 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (page);
#line 5985 "AppWindow.c"
}


static gboolean app_window_real_configure_event (GtkWidget* base, GdkEventConfigure* event) {
	AppWindow * self;
	gboolean result = FALSE;
	GdkWindow* _tmp0_ = NULL;
	GdkWindowState _tmp1_ = 0;
	gboolean _tmp2_ = FALSE;
	GdkEventConfigure* _tmp5_ = NULL;
	gboolean _tmp6_ = FALSE;
#line 936 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_APP_WINDOW, AppWindow);
#line 936 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 937 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_widget_get_window (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget));
#line 937 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp1_ = gdk_window_get_state (_tmp0_);
#line 937 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->maximized = _tmp1_ == GDK_WINDOW_STATE_MAXIMIZED;
#line 939 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp2_ = self->maximized;
#line 939 "/home/jens/Source/shotwell/src/AppWindow.vala"
	if (!_tmp2_) {
#line 6011 "AppWindow.c"
		gint _tmp3_ = 0;
		gint _tmp4_ = 0;
#line 940 "/home/jens/Source/shotwell/src/AppWindow.vala"
		gtk_window_get_size (G_TYPE_CHECK_INSTANCE_CAST (self, gtk_window_get_type (), GtkWindow), &_tmp3_, &_tmp4_);
#line 940 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->dimensions.width = _tmp3_;
#line 940 "/home/jens/Source/shotwell/src/AppWindow.vala"
		self->dimensions.height = _tmp4_;
#line 6020 "AppWindow.c"
	}
#line 942 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp5_ = event;
#line 942 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp6_ = GTK_WIDGET_CLASS (app_window_parent_class)->configure_event (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE_WINDOW, PageWindow), gtk_widget_get_type (), GtkWidget), _tmp5_);
#line 942 "/home/jens/Source/shotwell/src/AppWindow.vala"
	result = _tmp6_;
#line 942 "/home/jens/Source/shotwell/src/AppWindow.vala"
	return result;
#line 6030 "AppWindow.c"
}


static void app_window_class_init (AppWindowClass * klass) {
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	app_window_parent_class = g_type_class_peek_parent (klass);
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	g_type_class_add_private (klass, sizeof (AppWindowPrivate));
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->on_fullscreen = app_window_real_on_fullscreen;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->get_app_role = app_window_real_get_app_role;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->on_quit = app_window_real_on_quit;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->destroy = app_window_real_destroy;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->create_common_action_groups = app_window_real_create_common_action_groups;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->replace_common_placeholders = app_window_real_replace_common_placeholders;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((PageWindowClass *) klass)->switched_pages = app_window_real_switched_pages;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->update_common_action_availability = app_window_real_update_common_action_availability;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((AppWindowClass *) klass)->update_common_actions = app_window_real_update_common_actions;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	((GtkWidgetClass *) klass)->configure_event = app_window_real_configure_event;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	G_OBJECT_CLASS (klass)->finalize = app_window_finalize;
#line 6061 "AppWindow.c"
}


static void app_window_instance_init (AppWindow * self) {
	GtkActionGroup* _tmp0_ = NULL;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv = APP_WINDOW_GET_PRIVATE (self);
#line 417 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->maximized = FALSE;
#line 419 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->pos_x = 0;
#line 420 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->pos_y = 0;
#line 422 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_tmp0_ = gtk_action_group_new ("AppWindowGlobalActionGroup");
#line 422 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->priv->common_action_group = _tmp0_;
#line 6079 "AppWindow.c"
}


static void app_window_finalize (GObject* obj) {
	AppWindow * self;
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_APP_WINDOW, AppWindow);
#line 416 "/home/jens/Source/shotwell/src/AppWindow.vala"
	self->common_action_groups = (_vala_array_free (self->common_action_groups, self->common_action_groups_length1, (GDestroyNotify) g_object_unref), NULL);
#line 422 "/home/jens/Source/shotwell/src/AppWindow.vala"
	_g_object_unref0 (self->priv->common_action_group);
#line 406 "/home/jens/Source/shotwell/src/AppWindow.vala"
	G_OBJECT_CLASS (app_window_parent_class)->finalize (obj);
#line 6093 "AppWindow.c"
}


GType app_window_get_type (void) {
	static volatile gsize app_window_type_id__volatile = 0;
	if (g_once_init_enter (&app_window_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AppWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) app_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppWindow), 0, (GInstanceInitFunc) app_window_instance_init, NULL };
		GType app_window_type_id;
		app_window_type_id = g_type_register_static (TYPE_PAGE_WINDOW, "AppWindow", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&app_window_type_id__volatile, app_window_type_id);
	}
	return app_window_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



