/* RajcePublishing.c generated by valac 0.32.1, the Vala compiler
 * generated from RajcePublishing.vala, do not modify */

/* Copyright 2014 rajce.net
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include <glib.h>
#include <glib-object.h>
#include <shotwell-plugin-dev-1.0.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include "shotwell-plugin-common.h"
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include <libxml/tree.h>
#include <gee.h>
#include <float.h>
#include <math.h>
#include <libxml/parser.h>
#include <libsoup/soup.h>
#include <gobject/gvaluecollector.h>


#define TYPE_RAJCE_SERVICE (rajce_service_get_type ())
#define RAJCE_SERVICE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_RAJCE_SERVICE, RajceService))
#define RAJCE_SERVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_RAJCE_SERVICE, RajceServiceClass))
#define IS_RAJCE_SERVICE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_RAJCE_SERVICE))
#define IS_RAJCE_SERVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_RAJCE_SERVICE))
#define RAJCE_SERVICE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_RAJCE_SERVICE, RajceServiceClass))

typedef struct _RajceService RajceService;
typedef struct _RajceServiceClass RajceServiceClass;
typedef struct _RajceServicePrivate RajceServicePrivate;
#define _g_free0(var) (var = (g_free (var), NULL))

#define PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER (publishing_rajce_rajce_publisher_get_type ())
#define PUBLISHING_RAJCE_RAJCE_PUBLISHER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisher))
#define PUBLISHING_RAJCE_RAJCE_PUBLISHER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisherClass))
#define PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER))
#define PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER))
#define PUBLISHING_RAJCE_RAJCE_PUBLISHER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisherClass))

typedef struct _PublishingRajceRajcePublisher PublishingRajceRajcePublisher;
typedef struct _PublishingRajceRajcePublisherClass PublishingRajceRajcePublisherClass;
typedef struct _PublishingRajceRajcePublisherPrivate PublishingRajceRajcePublisherPrivate;

#define PUBLISHING_RAJCE_TYPE_SESSION (publishing_rajce_session_get_type ())
#define PUBLISHING_RAJCE_SESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSession))
#define PUBLISHING_RAJCE_SESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSessionClass))
#define PUBLISHING_RAJCE_IS_SESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_SESSION))
#define PUBLISHING_RAJCE_IS_SESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_SESSION))
#define PUBLISHING_RAJCE_SESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSessionClass))

typedef struct _PublishingRajceSession PublishingRajceSession;
typedef struct _PublishingRajceSessionClass PublishingRajceSessionClass;

#define PUBLISHING_RAJCE_TYPE_ALBUM (publishing_rajce_album_get_type ())
#define PUBLISHING_RAJCE_ALBUM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_ALBUM, PublishingRajceAlbum))
#define PUBLISHING_RAJCE_ALBUM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_ALBUM, PublishingRajceAlbumClass))
#define PUBLISHING_RAJCE_IS_ALBUM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_ALBUM))
#define PUBLISHING_RAJCE_IS_ALBUM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_ALBUM))
#define PUBLISHING_RAJCE_ALBUM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_ALBUM, PublishingRajceAlbumClass))

typedef struct _PublishingRajceAlbum PublishingRajceAlbum;
typedef struct _PublishingRajceAlbumClass PublishingRajceAlbumClass;

#define PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS (publishing_rajce_publishing_parameters_get_type ())
#define PUBLISHING_RAJCE_PUBLISHING_PARAMETERS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS, PublishingRajcePublishingParameters))
#define PUBLISHING_RAJCE_PUBLISHING_PARAMETERS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS, PublishingRajcePublishingParametersClass))
#define PUBLISHING_RAJCE_IS_PUBLISHING_PARAMETERS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS))
#define PUBLISHING_RAJCE_IS_PUBLISHING_PARAMETERS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS))
#define PUBLISHING_RAJCE_PUBLISHING_PARAMETERS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS, PublishingRajcePublishingParametersClass))

typedef struct _PublishingRajcePublishingParameters PublishingRajcePublishingParameters;
typedef struct _PublishingRajcePublishingParametersClass PublishingRajcePublishingParametersClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _publishing_rest_support_session_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rest_support_session_unref (var), NULL)))
#define _publishing_rajce_publishing_parameters_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rajce_publishing_parameters_unref (var), NULL)))

#define PUBLISHING_RAJCE_AUTHENTICATION_PANE_TYPE_MODE (publishing_rajce_authentication_pane_mode_get_type ())

#define PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE (publishing_rajce_authentication_pane_get_type ())
#define PUBLISHING_RAJCE_AUTHENTICATION_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPane))
#define PUBLISHING_RAJCE_AUTHENTICATION_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPaneClass))
#define PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE))
#define PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE))
#define PUBLISHING_RAJCE_AUTHENTICATION_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPaneClass))

typedef struct _PublishingRajceAuthenticationPane PublishingRajceAuthenticationPane;
typedef struct _PublishingRajceAuthenticationPaneClass PublishingRajceAuthenticationPaneClass;

#define PUBLISHING_RAJCE_TYPE_TRANSACTION (publishing_rajce_transaction_get_type ())
#define PUBLISHING_RAJCE_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_TRANSACTION, PublishingRajceTransaction))
#define PUBLISHING_RAJCE_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_TRANSACTION, PublishingRajceTransactionClass))
#define PUBLISHING_RAJCE_IS_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_TRANSACTION))
#define PUBLISHING_RAJCE_IS_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_TRANSACTION))
#define PUBLISHING_RAJCE_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_TRANSACTION, PublishingRajceTransactionClass))

typedef struct _PublishingRajceTransaction PublishingRajceTransaction;
typedef struct _PublishingRajceTransactionClass PublishingRajceTransactionClass;

#define PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION (publishing_rajce_session_login_transaction_get_type ())
#define PUBLISHING_RAJCE_SESSION_LOGIN_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION, PublishingRajceSessionLoginTransaction))
#define PUBLISHING_RAJCE_SESSION_LOGIN_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION, PublishingRajceSessionLoginTransactionClass))
#define PUBLISHING_RAJCE_IS_SESSION_LOGIN_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION))
#define PUBLISHING_RAJCE_IS_SESSION_LOGIN_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION))
#define PUBLISHING_RAJCE_SESSION_LOGIN_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION, PublishingRajceSessionLoginTransactionClass))

typedef struct _PublishingRajceSessionLoginTransaction PublishingRajceSessionLoginTransaction;
typedef struct _PublishingRajceSessionLoginTransactionClass PublishingRajceSessionLoginTransactionClass;
#define _publishing_rest_support_transaction_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rest_support_transaction_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _publishing_rest_support_xml_document_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rest_support_xml_document_unref (var), NULL)))

#define PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION (publishing_rajce_get_albums_transaction_get_type ())
#define PUBLISHING_RAJCE_GET_ALBUMS_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION, PublishingRajceGetAlbumsTransaction))
#define PUBLISHING_RAJCE_GET_ALBUMS_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION, PublishingRajceGetAlbumsTransactionClass))
#define PUBLISHING_RAJCE_IS_GET_ALBUMS_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION))
#define PUBLISHING_RAJCE_IS_GET_ALBUMS_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION))
#define PUBLISHING_RAJCE_GET_ALBUMS_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION, PublishingRajceGetAlbumsTransactionClass))

typedef struct _PublishingRajceGetAlbumsTransaction PublishingRajceGetAlbumsTransaction;
typedef struct _PublishingRajceGetAlbumsTransactionClass PublishingRajceGetAlbumsTransactionClass;
#define _publishing_rajce_album_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rajce_album_unref (var), NULL)))

#define PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE (publishing_rajce_publishing_options_pane_get_type ())
#define PUBLISHING_RAJCE_PUBLISHING_OPTIONS_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPane))
#define PUBLISHING_RAJCE_PUBLISHING_OPTIONS_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPaneClass))
#define PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE))
#define PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE))
#define PUBLISHING_RAJCE_PUBLISHING_OPTIONS_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPaneClass))

typedef struct _PublishingRajcePublishingOptionsPane PublishingRajcePublishingOptionsPane;
typedef struct _PublishingRajcePublishingOptionsPaneClass PublishingRajcePublishingOptionsPaneClass;
typedef struct _PublishingRajcePublishingParametersPrivate PublishingRajcePublishingParametersPrivate;

#define PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION (publishing_rajce_create_album_transaction_get_type ())
#define PUBLISHING_RAJCE_CREATE_ALBUM_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION, PublishingRajceCreateAlbumTransaction))
#define PUBLISHING_RAJCE_CREATE_ALBUM_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION, PublishingRajceCreateAlbumTransactionClass))
#define PUBLISHING_RAJCE_IS_CREATE_ALBUM_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION))
#define PUBLISHING_RAJCE_IS_CREATE_ALBUM_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION))
#define PUBLISHING_RAJCE_CREATE_ALBUM_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION, PublishingRajceCreateAlbumTransactionClass))

typedef struct _PublishingRajceCreateAlbumTransaction PublishingRajceCreateAlbumTransaction;
typedef struct _PublishingRajceCreateAlbumTransactionClass PublishingRajceCreateAlbumTransactionClass;

#define PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION (publishing_rajce_open_album_transaction_get_type ())
#define PUBLISHING_RAJCE_OPEN_ALBUM_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION, PublishingRajceOpenAlbumTransaction))
#define PUBLISHING_RAJCE_OPEN_ALBUM_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION, PublishingRajceOpenAlbumTransactionClass))
#define PUBLISHING_RAJCE_IS_OPEN_ALBUM_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION))
#define PUBLISHING_RAJCE_IS_OPEN_ALBUM_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION))
#define PUBLISHING_RAJCE_OPEN_ALBUM_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION, PublishingRajceOpenAlbumTransactionClass))

typedef struct _PublishingRajceOpenAlbumTransaction PublishingRajceOpenAlbumTransaction;
typedef struct _PublishingRajceOpenAlbumTransactionClass PublishingRajceOpenAlbumTransactionClass;

#define PUBLISHING_RAJCE_TYPE_UPLOADER (publishing_rajce_uploader_get_type ())
#define PUBLISHING_RAJCE_UPLOADER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_UPLOADER, PublishingRajceUploader))
#define PUBLISHING_RAJCE_UPLOADER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_UPLOADER, PublishingRajceUploaderClass))
#define PUBLISHING_RAJCE_IS_UPLOADER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_UPLOADER))
#define PUBLISHING_RAJCE_IS_UPLOADER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_UPLOADER))
#define PUBLISHING_RAJCE_UPLOADER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_UPLOADER, PublishingRajceUploaderClass))

typedef struct _PublishingRajceUploader PublishingRajceUploader;
typedef struct _PublishingRajceUploaderClass PublishingRajceUploaderClass;
#define _publishing_rest_support_batch_uploader_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rest_support_batch_uploader_unref (var), NULL)))

#define PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION (publishing_rajce_get_album_url_transaction_get_type ())
#define PUBLISHING_RAJCE_GET_ALBUM_URL_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION, PublishingRajceGetAlbumUrlTransaction))
#define PUBLISHING_RAJCE_GET_ALBUM_URL_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION, PublishingRajceGetAlbumUrlTransactionClass))
#define PUBLISHING_RAJCE_IS_GET_ALBUM_URL_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION))
#define PUBLISHING_RAJCE_IS_GET_ALBUM_URL_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION))
#define PUBLISHING_RAJCE_GET_ALBUM_URL_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION, PublishingRajceGetAlbumUrlTransactionClass))

typedef struct _PublishingRajceGetAlbumUrlTransaction PublishingRajceGetAlbumUrlTransaction;
typedef struct _PublishingRajceGetAlbumUrlTransactionClass PublishingRajceGetAlbumUrlTransactionClass;

#define PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION (publishing_rajce_close_album_transaction_get_type ())
#define PUBLISHING_RAJCE_CLOSE_ALBUM_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION, PublishingRajceCloseAlbumTransaction))
#define PUBLISHING_RAJCE_CLOSE_ALBUM_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION, PublishingRajceCloseAlbumTransactionClass))
#define PUBLISHING_RAJCE_IS_CLOSE_ALBUM_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION))
#define PUBLISHING_RAJCE_IS_CLOSE_ALBUM_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION))
#define PUBLISHING_RAJCE_CLOSE_ALBUM_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION, PublishingRajceCloseAlbumTransactionClass))

typedef struct _PublishingRajceCloseAlbumTransaction PublishingRajceCloseAlbumTransaction;
typedef struct _PublishingRajceCloseAlbumTransactionClass PublishingRajceCloseAlbumTransactionClass;
typedef struct _PublishingRajceAlbumPrivate PublishingRajceAlbumPrivate;
typedef struct _PublishingRajceParamSpecAlbum PublishingRajceParamSpecAlbum;
typedef struct _PublishingRajceUploaderPrivate PublishingRajceUploaderPrivate;

#define PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION (publishing_rajce_add_photo_transaction_get_type ())
#define PUBLISHING_RAJCE_ADD_PHOTO_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION, PublishingRajceAddPhotoTransaction))
#define PUBLISHING_RAJCE_ADD_PHOTO_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION, PublishingRajceAddPhotoTransactionClass))
#define PUBLISHING_RAJCE_IS_ADD_PHOTO_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION))
#define PUBLISHING_RAJCE_IS_ADD_PHOTO_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION))
#define PUBLISHING_RAJCE_ADD_PHOTO_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION, PublishingRajceAddPhotoTransactionClass))

typedef struct _PublishingRajceAddPhotoTransaction PublishingRajceAddPhotoTransaction;
typedef struct _PublishingRajceAddPhotoTransactionClass PublishingRajceAddPhotoTransactionClass;
typedef struct _PublishingRajceAuthenticationPanePrivate PublishingRajceAuthenticationPanePrivate;
typedef struct _PublishingRajcePublishingOptionsPanePrivate PublishingRajcePublishingOptionsPanePrivate;
typedef struct _PublishingRajceParamSpecPublishingParameters PublishingRajceParamSpecPublishingParameters;
typedef struct _PublishingRajceSessionPrivate PublishingRajceSessionPrivate;

#define PUBLISHING_RAJCE_TYPE_ARG_ITEM (publishing_rajce_arg_item_get_type ())
#define PUBLISHING_RAJCE_ARG_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_ARG_ITEM, PublishingRajceArgItem))
#define PUBLISHING_RAJCE_ARG_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_ARG_ITEM, PublishingRajceArgItemClass))
#define PUBLISHING_RAJCE_IS_ARG_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_ARG_ITEM))
#define PUBLISHING_RAJCE_IS_ARG_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_ARG_ITEM))
#define PUBLISHING_RAJCE_ARG_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_ARG_ITEM, PublishingRajceArgItemClass))

typedef struct _PublishingRajceArgItem PublishingRajceArgItem;
typedef struct _PublishingRajceArgItemClass PublishingRajceArgItemClass;
typedef struct _PublishingRajceArgItemPrivate PublishingRajceArgItemPrivate;
#define _publishing_rajce_arg_item_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rajce_arg_item_unref (var), NULL)))
typedef struct _PublishingRajceParamSpecArgItem PublishingRajceParamSpecArgItem;

#define PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST (publishing_rajce_live_api_request_get_type ())
#define PUBLISHING_RAJCE_LIVE_API_REQUEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST, PublishingRajceLiveApiRequest))
#define PUBLISHING_RAJCE_LIVE_API_REQUEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST, PublishingRajceLiveApiRequestClass))
#define PUBLISHING_RAJCE_IS_LIVE_API_REQUEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST))
#define PUBLISHING_RAJCE_IS_LIVE_API_REQUEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST))
#define PUBLISHING_RAJCE_LIVE_API_REQUEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST, PublishingRajceLiveApiRequestClass))

typedef struct _PublishingRajceLiveApiRequest PublishingRajceLiveApiRequest;
typedef struct _PublishingRajceLiveApiRequestClass PublishingRajceLiveApiRequestClass;
typedef struct _PublishingRajceLiveApiRequestPrivate PublishingRajceLiveApiRequestPrivate;
typedef struct _PublishingRajceParamSpecLiveApiRequest PublishingRajceParamSpecLiveApiRequest;
typedef struct _PublishingRajceTransactionPrivate PublishingRajceTransactionPrivate;
typedef struct _PublishingRajceSessionLoginTransactionPrivate PublishingRajceSessionLoginTransactionPrivate;
#define _publishing_rajce_live_api_request_unref0(var) ((var == NULL) ? NULL : (var = (publishing_rajce_live_api_request_unref (var), NULL)))
typedef struct _PublishingRajceGetAlbumsTransactionPrivate PublishingRajceGetAlbumsTransactionPrivate;
typedef struct _PublishingRajceCreateAlbumTransactionPrivate PublishingRajceCreateAlbumTransactionPrivate;
typedef struct _PublishingRajceOpenAlbumTransactionPrivate PublishingRajceOpenAlbumTransactionPrivate;
typedef struct _PublishingRajceGetAlbumUrlTransactionPrivate PublishingRajceGetAlbumUrlTransactionPrivate;
typedef struct _PublishingRajceCloseAlbumTransactionPrivate PublishingRajceCloseAlbumTransactionPrivate;

#define PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION (publishing_rajce_get_categories_transaction_get_type ())
#define PUBLISHING_RAJCE_GET_CATEGORIES_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION, PublishingRajceGetCategoriesTransaction))
#define PUBLISHING_RAJCE_GET_CATEGORIES_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION, PublishingRajceGetCategoriesTransactionClass))
#define PUBLISHING_RAJCE_IS_GET_CATEGORIES_TRANSACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION))
#define PUBLISHING_RAJCE_IS_GET_CATEGORIES_TRANSACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION))
#define PUBLISHING_RAJCE_GET_CATEGORIES_TRANSACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION, PublishingRajceGetCategoriesTransactionClass))

typedef struct _PublishingRajceGetCategoriesTransaction PublishingRajceGetCategoriesTransaction;
typedef struct _PublishingRajceGetCategoriesTransactionClass PublishingRajceGetCategoriesTransactionClass;
typedef struct _PublishingRajceGetCategoriesTransactionPrivate PublishingRajceGetCategoriesTransactionPrivate;
typedef struct _PublishingRajceAddPhotoTransactionPrivate PublishingRajceAddPhotoTransactionPrivate;
#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _RajceService {
	GObject parent_instance;
	RajceServicePrivate * priv;
};

struct _RajceServiceClass {
	GObjectClass parent_class;
};

struct _PublishingRajceRajcePublisher {
	GObject parent_instance;
	PublishingRajceRajcePublisherPrivate * priv;
};

struct _PublishingRajceRajcePublisherClass {
	GObjectClass parent_class;
};

struct _PublishingRajceRajcePublisherPrivate {
	SpitPublishingPluginHost* host;
	SpitPublishingProgressCallback progress_reporter;
	gpointer progress_reporter_target;
	GDestroyNotify progress_reporter_target_destroy_notify;
	SpitPublishingService* service;
	gboolean running;
	PublishingRajceSession* session;
	PublishingRajceAlbum** albums;
	gint albums_length1;
	gint _albums_size_;
	PublishingRajcePublishingParameters* parameters;
	SpitPublishingPublisherMediaType media_type;
};

typedef enum  {
	PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_INTRO,
	PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER
} PublishingRajceAuthenticationPaneMode;

struct _PublishingRajcePublishingParameters {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PublishingRajcePublishingParametersPrivate * priv;
	gchar* album_name;
	gboolean* album_hidden;
	gint* album_id;
};

struct _PublishingRajcePublishingParametersClass {
	GTypeClass parent_class;
	void (*finalize) (PublishingRajcePublishingParameters *self);
};

struct _PublishingRajceAlbum {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PublishingRajceAlbumPrivate * priv;
	gint id;
	gchar* albumName;
	gchar* url;
	gchar* thumbUrl;
	gchar* createDate;
	gchar* updateDate;
	gboolean hidden;
	gboolean secure;
	gint photoCount;
};

struct _PublishingRajceAlbumClass {
	GTypeClass parent_class;
	void (*finalize) (PublishingRajceAlbum *self);
};

struct _PublishingRajceParamSpecAlbum {
	GParamSpec parent_instance;
};

struct _PublishingRajceUploader {
	PublishingRESTSupportBatchUploader parent_instance;
	PublishingRajceUploaderPrivate * priv;
};

struct _PublishingRajceUploaderClass {
	PublishingRESTSupportBatchUploaderClass parent_class;
};

struct _PublishingRajceUploaderPrivate {
	PublishingRajcePublishingParameters* parameters;
	gchar* url;
};

struct _PublishingRajceAuthenticationPane {
	GObject parent_instance;
	PublishingRajceAuthenticationPanePrivate * priv;
};

struct _PublishingRajceAuthenticationPaneClass {
	GObjectClass parent_class;
};

struct _PublishingRajceAuthenticationPanePrivate {
	GtkBox* pane_widget;
	GtkBuilder* builder;
	GtkEntry* username_entry;
	GtkEntry* password_entry;
	GtkCheckButton* remember_checkbutton;
	GtkButton* login_button;
	gboolean crypt;
};

struct _PublishingRajcePublishingOptionsPane {
	GObject parent_instance;
	PublishingRajcePublishingOptionsPanePrivate * priv;
};

struct _PublishingRajcePublishingOptionsPaneClass {
	GObjectClass parent_class;
};

struct _PublishingRajcePublishingOptionsPanePrivate {
	PublishingRajceRajcePublisher* publisher;
	PublishingRajceAlbum** albums;
	gint albums_length1;
	gint _albums_size_;
	gchar* username;
	GtkBuilder* builder;
	GtkBox* pane_widget;
	GtkLabel* login_identity_label;
	GtkLabel* publish_to_label;
	GtkRadioButton* use_existing_radio;
	GtkComboBoxText* existing_albums_combo;
	GtkRadioButton* create_new_radio;
	GtkEntry* new_album_entry;
	GtkCheckButton* hide_check;
	GtkCheckButton* show_check;
	GtkButton* publish_button;
	GtkButton* logout_button;
};

struct _PublishingRajceParamSpecPublishingParameters {
	GParamSpec parent_instance;
};

struct _PublishingRajceSession {
	PublishingRESTSupportSession parent_instance;
	PublishingRajceSessionPrivate * priv;
};

struct _PublishingRajceSessionClass {
	PublishingRESTSupportSessionClass parent_class;
};

struct _PublishingRajceSessionPrivate {
	gchar* usertoken;
	gchar* albumtoken;
	gchar* albumticket;
	gchar* username;
	gint* userid;
	gint* maxsize;
	gint* quality;
};

struct _PublishingRajceArgItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PublishingRajceArgItemPrivate * priv;
	gchar* key;
	gchar* val;
	PublishingRajceArgItem** children;
	gint children_length1;
	gint _children_size_;
};

struct _PublishingRajceArgItemClass {
	GTypeClass parent_class;
	void (*finalize) (PublishingRajceArgItem *self);
};

struct _PublishingRajceParamSpecArgItem {
	GParamSpec parent_instance;
};

struct _PublishingRajceLiveApiRequest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PublishingRajceLiveApiRequestPrivate * priv;
};

struct _PublishingRajceLiveApiRequestClass {
	GTypeClass parent_class;
	void (*finalize) (PublishingRajceLiveApiRequest *self);
};

struct _PublishingRajceLiveApiRequestPrivate {
	PublishingRajceArgItem** _params;
	gint _params_length1;
	gint __params_size_;
	gchar* _cmd;
};

struct _PublishingRajceParamSpecLiveApiRequest {
	GParamSpec parent_instance;
};

struct _PublishingRajceTransaction {
	PublishingRESTSupportTransaction parent_instance;
	PublishingRajceTransactionPrivate * priv;
};

struct _PublishingRajceTransactionClass {
	PublishingRESTSupportTransactionClass parent_class;
};

struct _PublishingRajceSessionLoginTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceSessionLoginTransactionPrivate * priv;
};

struct _PublishingRajceSessionLoginTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceGetAlbumsTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceGetAlbumsTransactionPrivate * priv;
};

struct _PublishingRajceGetAlbumsTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceCreateAlbumTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceCreateAlbumTransactionPrivate * priv;
};

struct _PublishingRajceCreateAlbumTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceOpenAlbumTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceOpenAlbumTransactionPrivate * priv;
};

struct _PublishingRajceOpenAlbumTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceGetAlbumUrlTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceGetAlbumUrlTransactionPrivate * priv;
};

struct _PublishingRajceGetAlbumUrlTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceCloseAlbumTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceCloseAlbumTransactionPrivate * priv;
};

struct _PublishingRajceCloseAlbumTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceGetCategoriesTransaction {
	PublishingRajceTransaction parent_instance;
	PublishingRajceGetCategoriesTransactionPrivate * priv;
};

struct _PublishingRajceGetCategoriesTransactionClass {
	PublishingRajceTransactionClass parent_class;
};

struct _PublishingRajceAddPhotoTransaction {
	PublishingRESTSupportUploadTransaction parent_instance;
	PublishingRajceAddPhotoTransactionPrivate * priv;
};

struct _PublishingRajceAddPhotoTransactionClass {
	PublishingRESTSupportUploadTransactionClass parent_class;
};

struct _PublishingRajceAddPhotoTransactionPrivate {
	PublishingRajcePublishingParameters* parameters;
};


static gpointer rajce_service_parent_class = NULL;
static GdkPixbuf** rajce_service_icon_pixbuf_set;
static gint rajce_service_icon_pixbuf_set_length1;
static GdkPixbuf** rajce_service_icon_pixbuf_set = NULL;
static gint rajce_service_icon_pixbuf_set_length1 = 0;
static gint _rajce_service_icon_pixbuf_set_size_ = 0;
static SpitPluggableIface* rajce_service_spit_pluggable_parent_iface = NULL;
static SpitPublishingServiceIface* rajce_service_spit_publishing_service_parent_iface = NULL;
static gpointer publishing_rajce_rajce_publisher_parent_class = NULL;
static SpitPublishingPublisherIface* publishing_rajce_rajce_publisher_spit_publishing_publisher_parent_iface = NULL;
static gpointer publishing_rajce_album_parent_class = NULL;
static gpointer publishing_rajce_uploader_parent_class = NULL;
static gpointer publishing_rajce_authentication_pane_parent_class = NULL;
static gchar* publishing_rajce_authentication_pane_INTRO_MESSAGE;
static gchar* publishing_rajce_authentication_pane_INTRO_MESSAGE = NULL;
static gchar* publishing_rajce_authentication_pane_FAILED_RETRY_USER_MESSAGE;
static gchar* publishing_rajce_authentication_pane_FAILED_RETRY_USER_MESSAGE = NULL;
static SpitPublishingDialogPaneIface* publishing_rajce_authentication_pane_spit_publishing_dialog_pane_parent_iface = NULL;
static gpointer publishing_rajce_publishing_options_pane_parent_class = NULL;
static SpitPublishingDialogPaneIface* publishing_rajce_publishing_options_pane_spit_publishing_dialog_pane_parent_iface = NULL;
static gpointer publishing_rajce_publishing_parameters_parent_class = NULL;
static gpointer publishing_rajce_session_parent_class = NULL;
static gpointer publishing_rajce_arg_item_parent_class = NULL;
static gpointer publishing_rajce_live_api_request_parent_class = NULL;
static gpointer publishing_rajce_transaction_parent_class = NULL;
static gpointer publishing_rajce_session_login_transaction_parent_class = NULL;
static gpointer publishing_rajce_get_albums_transaction_parent_class = NULL;
static gpointer publishing_rajce_create_album_transaction_parent_class = NULL;
static gpointer publishing_rajce_open_album_transaction_parent_class = NULL;
static gpointer publishing_rajce_get_album_url_transaction_parent_class = NULL;
static gpointer publishing_rajce_close_album_transaction_parent_class = NULL;
static gpointer publishing_rajce_get_categories_transaction_parent_class = NULL;
static gpointer publishing_rajce_add_photo_transaction_parent_class = NULL;

GType rajce_service_get_type (void) G_GNUC_CONST;
enum  {
	RAJCE_SERVICE_DUMMY_PROPERTY
};
#define RAJCE_SERVICE_ICON_FILENAME "rajce.png"
RajceService* rajce_service_new (GFile* resource_directory);
RajceService* rajce_service_construct (GType object_type, GFile* resource_directory);
static gint rajce_service_real_get_pluggable_interface (SpitPluggable* base, gint min_host_interface, gint max_host_interface);
static const gchar* rajce_service_real_get_id (SpitPluggable* base);
static const gchar* rajce_service_real_get_pluggable_name (SpitPluggable* base);
static void rajce_service_real_get_info (SpitPluggable* base, SpitPluggableInfo* info);
static GdkPixbuf** _vala_array_dup4 (GdkPixbuf** self, int length);
static SpitPublishingPublisher* rajce_service_real_create_publisher (SpitPublishingService* base, SpitPublishingPluginHost* host);
PublishingRajceRajcePublisher* publishing_rajce_rajce_publisher_new (SpitPublishingService* service, SpitPublishingPluginHost* host);
PublishingRajceRajcePublisher* publishing_rajce_rajce_publisher_construct (GType object_type, SpitPublishingService* service, SpitPublishingPluginHost* host);
GType publishing_rajce_rajce_publisher_get_type (void) G_GNUC_CONST;
static SpitPublishingPublisherMediaType rajce_service_real_get_supported_media (SpitPublishingService* base);
static void rajce_service_real_activation (SpitPluggable* base, gboolean enabled);
static void rajce_service_finalize (GObject* obj);
GType publishing_rajce_session_get_type (void) G_GNUC_CONST;
gpointer publishing_rajce_album_ref (gpointer instance);
void publishing_rajce_album_unref (gpointer instance);
GParamSpec* publishing_rajce_param_spec_album (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void publishing_rajce_value_set_album (GValue* value, gpointer v_object);
void publishing_rajce_value_take_album (GValue* value, gpointer v_object);
gpointer publishing_rajce_value_get_album (const GValue* value);
GType publishing_rajce_album_get_type (void) G_GNUC_CONST;
gpointer publishing_rajce_publishing_parameters_ref (gpointer instance);
void publishing_rajce_publishing_parameters_unref (gpointer instance);
GParamSpec* publishing_rajce_param_spec_publishing_parameters (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void publishing_rajce_value_set_publishing_parameters (GValue* value, gpointer v_object);
void publishing_rajce_value_take_publishing_parameters (GValue* value, gpointer v_object);
gpointer publishing_rajce_value_get_publishing_parameters (const GValue* value);
GType publishing_rajce_publishing_parameters_get_type (void) G_GNUC_CONST;
#define PUBLISHING_RAJCE_RAJCE_PUBLISHER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisherPrivate))
enum  {
	PUBLISHING_RAJCE_RAJCE_PUBLISHER_DUMMY_PROPERTY
};
PublishingRajceSession* publishing_rajce_session_new (void);
PublishingRajceSession* publishing_rajce_session_construct (GType object_type);
static gchar* publishing_rajce_rajce_publisher_get_rajce_url (PublishingRajceRajcePublisher* self);
static SpitPublishingService* publishing_rajce_rajce_publisher_real_get_service (SpitPublishingPublisher* base);
SpitPublishingPluginHost* publishing_rajce_rajce_publisher_get_host (PublishingRajceRajcePublisher* self);
static gboolean publishing_rajce_rajce_publisher_real_is_running (SpitPublishingPublisher* base);
static void publishing_rajce_rajce_publisher_real_start (SpitPublishingPublisher* base);
static void publishing_rajce_rajce_publisher_do_fetch_albums (PublishingRajceRajcePublisher* self);
gchar* publishing_rajce_rajce_publisher_get_username (PublishingRajceRajcePublisher* self);
gchar* publishing_rajce_rajce_publisher_get_token (PublishingRajceRajcePublisher* self);
gboolean publishing_rajce_rajce_publisher_get_remember (PublishingRajceRajcePublisher* self);
static gboolean* _bool_dup (gboolean* self);
static void publishing_rajce_rajce_publisher_do_network_login (PublishingRajceRajcePublisher* self, const gchar* username, const gchar* token, gboolean remember);
GType publishing_rajce_authentication_pane_mode_get_type (void) G_GNUC_CONST;
static void publishing_rajce_rajce_publisher_do_show_authentication_pane (PublishingRajceRajcePublisher* self, PublishingRajceAuthenticationPaneMode mode);
static void publishing_rajce_rajce_publisher_real_stop (SpitPublishingPublisher* base);
gchar* publishing_rajce_rajce_publisher_get_url (PublishingRajceRajcePublisher* self);
static void publishing_rajce_rajce_publisher_set_username (PublishingRajceRajcePublisher* self, const gchar* username);
static void publishing_rajce_rajce_publisher_set_token (PublishingRajceRajcePublisher* self, const gchar* token);
static void publishing_rajce_rajce_publisher_set_remember (PublishingRajceRajcePublisher* self, gboolean remember);
gboolean publishing_rajce_rajce_publisher_get_hide_album (PublishingRajceRajcePublisher* self);
void publishing_rajce_rajce_publisher_set_hide_album (PublishingRajceRajcePublisher* self, gboolean hide_album);
gboolean publishing_rajce_rajce_publisher_get_show_album (PublishingRajceRajcePublisher* self);
void publishing_rajce_rajce_publisher_set_show_album (PublishingRajceRajcePublisher* self, gboolean show_album);
GType publishing_rajce_authentication_pane_get_type (void) G_GNUC_CONST;
PublishingRajceAuthenticationPane* publishing_rajce_authentication_pane_new (PublishingRajceRajcePublisher* publisher, PublishingRajceAuthenticationPaneMode mode);
PublishingRajceAuthenticationPane* publishing_rajce_authentication_pane_construct (GType object_type, PublishingRajceRajcePublisher* publisher, PublishingRajceAuthenticationPaneMode mode);
static void publishing_rajce_rajce_publisher_on_authentication_pane_login_clicked (PublishingRajceRajcePublisher* self, const gchar* username, const gchar* token, gboolean remember);
static void _publishing_rajce_rajce_publisher_on_authentication_pane_login_clicked_publishing_rajce_authentication_pane_login (PublishingRajceAuthenticationPane* _sender, const gchar* user, const gchar* token, gboolean remember, gpointer self);
GtkWidget* publishing_rajce_authentication_pane_get_default_widget (PublishingRajceAuthenticationPane* self);
GType publishing_rajce_transaction_get_type (void) G_GNUC_CONST;
GType publishing_rajce_session_login_transaction_get_type (void) G_GNUC_CONST;
PublishingRajceSessionLoginTransaction* publishing_rajce_session_login_transaction_new (PublishingRajceSession* session, const gchar* url, const gchar* username, const gchar* token);
PublishingRajceSessionLoginTransaction* publishing_rajce_session_login_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, const gchar* username, const gchar* token);
static void publishing_rajce_rajce_publisher_on_login_network_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
static void _publishing_rajce_rajce_publisher_on_login_network_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_login_network_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn);
static void _publishing_rajce_rajce_publisher_on_login_network_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self);
static void publishing_rajce_rajce_publisher_do_show_error (PublishingRajceRajcePublisher* self, GError* e);
gchar* publishing_rajce_transaction_validate_xml (PublishingRESTSupportXmlDocument* doc);
static gchar* _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response (PublishingRESTSupportXmlDocument* doc, gpointer self);
void publishing_rajce_session_authenticate (PublishingRajceSession* self, const gchar* token, const gchar* name, gint id, gint maxsize, gint quality);
GType publishing_rajce_get_albums_transaction_get_type (void) G_GNUC_CONST;
PublishingRajceGetAlbumsTransaction* publishing_rajce_get_albums_transaction_new (PublishingRajceSession* session, const gchar* url);
PublishingRajceGetAlbumsTransaction* publishing_rajce_get_albums_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url);
static void publishing_rajce_rajce_publisher_on_albums_fetch_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
static void _publishing_rajce_rajce_publisher_on_albums_fetch_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_albums_fetch_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn);
static void _publishing_rajce_rajce_publisher_on_albums_fetch_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self);
PublishingRajceAlbum* publishing_rajce_album_new (gint id, const gchar* albumName, const gchar* url, const gchar* thumbUrl, const gchar* createDate, const gchar* updateDate, gboolean hidden, gboolean secure, gint photoCount);
PublishingRajceAlbum* publishing_rajce_album_construct (GType object_type, gint id, const gchar* albumName, const gchar* url, const gchar* thumbUrl, const gchar* createDate, const gchar* updateDate, gboolean hidden, gboolean secure, gint photoCount);
gint publishing_rajce_album_compare_albums (PublishingRajceAlbum* a, PublishingRajceAlbum* b);
static gint _publishing_rajce_album_compare_albums_gcompare_data_func (gconstpointer a, gconstpointer b, gpointer self);
void publishing_rajce_session_set_usertoken (PublishingRajceSession* self, const gchar* usertoken);
static void publishing_rajce_rajce_publisher_do_show_publishing_options_pane (PublishingRajceRajcePublisher* self);
static void publishing_rajce_rajce_publisher_on_network_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
GType publishing_rajce_publishing_options_pane_get_type (void) G_GNUC_CONST;
gchar* publishing_rajce_session_get_username (PublishingRajceSession* self);
PublishingRajcePublishingOptionsPane* publishing_rajce_publishing_options_pane_new (PublishingRajceRajcePublisher* publisher, const gchar* username, PublishingRajceAlbum** albums, int albums_length1);
PublishingRajcePublishingOptionsPane* publishing_rajce_publishing_options_pane_construct (GType object_type, PublishingRajceRajcePublisher* publisher, const gchar* username, PublishingRajceAlbum** albums, int albums_length1);
static void publishing_rajce_rajce_publisher_on_publishing_options_pane_logout_clicked (PublishingRajceRajcePublisher* self);
static void _publishing_rajce_rajce_publisher_on_publishing_options_pane_logout_clicked_publishing_rajce_publishing_options_pane_logout (PublishingRajcePublishingOptionsPane* _sender, gpointer self);
static void publishing_rajce_rajce_publisher_on_publishing_options_pane_publish_clicked (PublishingRajceRajcePublisher* self, PublishingRajcePublishingParameters* parameters);
static void _publishing_rajce_rajce_publisher_on_publishing_options_pane_publish_clicked_publishing_rajce_publishing_options_pane_publish (PublishingRajcePublishingOptionsPane* _sender, PublishingRajcePublishingParameters* parameters, gpointer self);
GtkWidget* publishing_rajce_publishing_options_pane_get_default_widget (PublishingRajcePublishingOptionsPane* self);
void publishing_rajce_session_deauthenticate (PublishingRajceSession* self);
static void publishing_rajce_rajce_publisher_do_begin_upload (PublishingRajceRajcePublisher* self);
static gboolean _int_equal (const gint* s1, const gint* s2);
GType publishing_rajce_create_album_transaction_get_type (void) G_GNUC_CONST;
PublishingRajceCreateAlbumTransaction* publishing_rajce_create_album_transaction_new (PublishingRajceSession* session, const gchar* url, const gchar* albumName, gboolean hidden);
PublishingRajceCreateAlbumTransaction* publishing_rajce_create_album_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, const gchar* albumName, gboolean hidden);
static void publishing_rajce_rajce_publisher_on_create_album_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
static void _publishing_rajce_rajce_publisher_on_create_album_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_create_album_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn);
static void _publishing_rajce_rajce_publisher_on_create_album_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self);
GType publishing_rajce_open_album_transaction_get_type (void) G_GNUC_CONST;
PublishingRajceOpenAlbumTransaction* publishing_rajce_open_album_transaction_new (PublishingRajceSession* session, const gchar* url, gint albumID);
PublishingRajceOpenAlbumTransaction* publishing_rajce_open_album_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, gint albumID);
static void publishing_rajce_rajce_publisher_on_open_album_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
static void _publishing_rajce_rajce_publisher_on_open_album_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_open_album_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn);
static void _publishing_rajce_rajce_publisher_on_open_album_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self);
static gint* _int_dup (gint* self);
void publishing_rajce_session_set_albumtoken (PublishingRajceSession* self, const gchar* albumtoken);
static void publishing_rajce_rajce_publisher_do_upload_photos (PublishingRajceRajcePublisher* self);
gint publishing_rajce_session_get_maxsize (PublishingRajceSession* self);
GType publishing_rajce_uploader_get_type (void) G_GNUC_CONST;
PublishingRajceUploader* publishing_rajce_uploader_new (PublishingRajceSession* session, const gchar* url, SpitPublishingPublishable** publishables, int publishables_length1, PublishingRajcePublishingParameters* parameters);
PublishingRajceUploader* publishing_rajce_uploader_construct (GType object_type, PublishingRajceSession* session, const gchar* url, SpitPublishingPublishable** publishables, int publishables_length1, PublishingRajcePublishingParameters* parameters);
static void publishing_rajce_rajce_publisher_on_upload_photos_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportBatchUploader* uploader, gint num_published);
static void _publishing_rajce_rajce_publisher_on_upload_photos_complete_publishing_rest_support_batch_uploader_upload_complete (PublishingRESTSupportBatchUploader* _sender, gint num_photos_published, gpointer self);
static void publishing_rajce_rajce_publisher_on_upload_photos_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportBatchUploader* uploader, GError* err);
static void _publishing_rajce_rajce_publisher_on_upload_photos_error_publishing_rest_support_batch_uploader_upload_error (PublishingRESTSupportBatchUploader* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_upload_photos_status_updated (PublishingRajceRajcePublisher* self, gint file_number, gdouble completed_fraction);
static void _publishing_rajce_rajce_publisher_on_upload_photos_status_updated_spit_publishing_progress_callback (gint file_number, gdouble fraction_complete, gpointer self);
static void publishing_rajce_rajce_publisher_do_end_upload (PublishingRajceRajcePublisher* self);
static void publishing_rajce_rajce_publisher_do_get_album_url (PublishingRajceRajcePublisher* self);
static void publishing_rajce_rajce_publisher_do_close_album (PublishingRajceRajcePublisher* self);
GType publishing_rajce_get_album_url_transaction_get_type (void) G_GNUC_CONST;
PublishingRajceGetAlbumUrlTransaction* publishing_rajce_get_album_url_transaction_new (PublishingRajceSession* session, const gchar* url);
PublishingRajceGetAlbumUrlTransaction* publishing_rajce_get_album_url_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url);
static void publishing_rajce_rajce_publisher_on_get_album_url_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
static void _publishing_rajce_rajce_publisher_on_get_album_url_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_get_album_url_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn);
static void _publishing_rajce_rajce_publisher_on_get_album_url_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self);
void publishing_rajce_session_set_albumticket (PublishingRajceSession* self, const gchar* albumticket);
GType publishing_rajce_close_album_transaction_get_type (void) G_GNUC_CONST;
PublishingRajceCloseAlbumTransaction* publishing_rajce_close_album_transaction_new (PublishingRajceSession* session, const gchar* url);
PublishingRajceCloseAlbumTransaction* publishing_rajce_close_album_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url);
static void publishing_rajce_rajce_publisher_on_close_album_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err);
static void _publishing_rajce_rajce_publisher_on_close_album_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self);
static void publishing_rajce_rajce_publisher_on_close_album_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn);
static void _publishing_rajce_rajce_publisher_on_close_album_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self);
static void publishing_rajce_rajce_publisher_do_show_success_pane (PublishingRajceRajcePublisher* self);
gchar* publishing_rajce_session_get_albumticket (PublishingRajceSession* self);
static void publishing_rajce_rajce_publisher_do_show_error_message (PublishingRajceRajcePublisher* self, const gchar* message);
static void publishing_rajce_rajce_publisher_finalize (GObject* obj);
enum  {
	PUBLISHING_RAJCE_ALBUM_DUMMY_PROPERTY
};
static void publishing_rajce_album_finalize (PublishingRajceAlbum* obj);
#define PUBLISHING_RAJCE_UPLOADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_UPLOADER, PublishingRajceUploaderPrivate))
enum  {
	PUBLISHING_RAJCE_UPLOADER_DUMMY_PROPERTY
};
static PublishingRESTSupportTransaction* publishing_rajce_uploader_real_create_transaction (PublishingRESTSupportBatchUploader* base, SpitPublishingPublishable* publishable);
PublishingRajceAddPhotoTransaction* publishing_rajce_add_photo_transaction_new (PublishingRajceSession* session, const gchar* url, PublishingRajcePublishingParameters* parameters, SpitPublishingPublishable* publishable);
PublishingRajceAddPhotoTransaction* publishing_rajce_add_photo_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, PublishingRajcePublishingParameters* parameters, SpitPublishingPublishable* publishable);
GType publishing_rajce_add_photo_transaction_get_type (void) G_GNUC_CONST;
static void publishing_rajce_uploader_finalize (PublishingRESTSupportBatchUploader* obj);
#define PUBLISHING_RAJCE_AUTHENTICATION_PANE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPanePrivate))
enum  {
	PUBLISHING_RAJCE_AUTHENTICATION_PANE_DUMMY_PROPERTY
};
static void publishing_rajce_authentication_pane_on_user_changed (PublishingRajceAuthenticationPane* self);
static void _publishing_rajce_authentication_pane_on_user_changed_gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void publishing_rajce_authentication_pane_on_password_changed (PublishingRajceAuthenticationPane* self);
static void _publishing_rajce_authentication_pane_on_password_changed_gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void publishing_rajce_authentication_pane_on_login_button_clicked (PublishingRajceAuthenticationPane* self);
static void _publishing_rajce_authentication_pane_on_login_button_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void publishing_rajce_authentication_pane_update_login_button_sensitivity (PublishingRajceAuthenticationPane* self);
static GtkWidget* publishing_rajce_authentication_pane_real_get_widget (SpitPublishingDialogPane* base);
static SpitPublishingDialogPaneGeometryOptions publishing_rajce_authentication_pane_real_get_preferred_geometry (SpitPublishingDialogPane* base);
static void publishing_rajce_authentication_pane_real_on_pane_installed (SpitPublishingDialogPane* base);
static void publishing_rajce_authentication_pane_real_on_pane_uninstalled (SpitPublishingDialogPane* base);
static void g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void publishing_rajce_authentication_pane_finalize (GObject* obj);
#define PUBLISHING_RAJCE_PUBLISHING_OPTIONS_PANE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPanePrivate))
enum  {
	PUBLISHING_RAJCE_PUBLISHING_OPTIONS_PANE_DUMMY_PROPERTY
};
static PublishingRajceAlbum** _vala_array_dup5 (PublishingRajceAlbum** self, int length);
static void publishing_rajce_publishing_options_pane_on_use_existing_radio_clicked (PublishingRajcePublishingOptionsPane* self);
static void _publishing_rajce_publishing_options_pane_on_use_existing_radio_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void publishing_rajce_publishing_options_pane_on_create_new_radio_clicked (PublishingRajcePublishingOptionsPane* self);
static void _publishing_rajce_publishing_options_pane_on_create_new_radio_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void publishing_rajce_publishing_options_pane_on_new_album_entry_changed (PublishingRajcePublishingOptionsPane* self);
static void _publishing_rajce_publishing_options_pane_on_new_album_entry_changed_gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void publishing_rajce_publishing_options_pane_on_logout_clicked (PublishingRajcePublishingOptionsPane* self);
static void _publishing_rajce_publishing_options_pane_on_logout_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void publishing_rajce_publishing_options_pane_on_publish_clicked (PublishingRajcePublishingOptionsPane* self);
static void _publishing_rajce_publishing_options_pane_on_publish_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_new_to_new_album (const gchar* album_name, gboolean album_hidden);
PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_construct_to_new_album (GType object_type, const gchar* album_name, gboolean album_hidden);
PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_new_to_existing_album (const gchar* album_name, gint album_id);
PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_construct_to_existing_album (GType object_type, const gchar* album_name, gint album_id);
static void publishing_rajce_publishing_options_pane_update_publish_button_sensitivity (PublishingRajcePublishingOptionsPane* self);
void publishing_rajce_publishing_options_pane_installed (PublishingRajcePublishingOptionsPane* self);
void publishing_rajce_publishing_options_pane_notify_publish (PublishingRajcePublishingOptionsPane* self, PublishingRajcePublishingParameters* parameters);
void publishing_rajce_publishing_options_pane_notify_logout (PublishingRajcePublishingOptionsPane* self);
static GtkWidget* publishing_rajce_publishing_options_pane_real_get_widget (SpitPublishingDialogPane* base);
static SpitPublishingDialogPaneGeometryOptions publishing_rajce_publishing_options_pane_real_get_preferred_geometry (SpitPublishingDialogPane* base);
static void publishing_rajce_publishing_options_pane_real_on_pane_installed (SpitPublishingDialogPane* base);
static void _publishing_rajce_publishing_options_pane_notify_publish_publishing_rajce_publishing_options_pane_publish (PublishingRajcePublishingOptionsPane* _sender, PublishingRajcePublishingParameters* parameters, gpointer self);
static void _publishing_rajce_publishing_options_pane_notify_logout_publishing_rajce_publishing_options_pane_logout (PublishingRajcePublishingOptionsPane* _sender, gpointer self);
static void publishing_rajce_publishing_options_pane_real_on_pane_uninstalled (SpitPublishingDialogPane* base);
static void g_cclosure_user_marshal_VOID__PUBLISHING_RAJCE_PUBLISHING_PARAMETERS (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void publishing_rajce_publishing_options_pane_finalize (GObject* obj);
enum  {
	PUBLISHING_RAJCE_PUBLISHING_PARAMETERS_DUMMY_PROPERTY
};
static PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_new (void);
static PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_construct (GType object_type);
static void publishing_rajce_publishing_parameters_finalize (PublishingRajcePublishingParameters* obj);
#define PUBLISHING_RAJCE_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSessionPrivate))
enum  {
	PUBLISHING_RAJCE_SESSION_DUMMY_PROPERTY
};
static gboolean publishing_rajce_session_real_is_authenticated (PublishingRESTSupportSession* base);
gchar* publishing_rajce_session_get_usertoken (PublishingRajceSession* self);
gchar* publishing_rajce_session_get_albumtoken (PublishingRajceSession* self);
static void publishing_rajce_session_finalize (PublishingRESTSupportSession* obj);
gpointer publishing_rajce_arg_item_ref (gpointer instance);
void publishing_rajce_arg_item_unref (gpointer instance);
GParamSpec* publishing_rajce_param_spec_arg_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void publishing_rajce_value_set_arg_item (GValue* value, gpointer v_object);
void publishing_rajce_value_take_arg_item (GValue* value, gpointer v_object);
gpointer publishing_rajce_value_get_arg_item (const GValue* value);
GType publishing_rajce_arg_item_get_type (void) G_GNUC_CONST;
enum  {
	PUBLISHING_RAJCE_ARG_ITEM_DUMMY_PROPERTY
};
PublishingRajceArgItem* publishing_rajce_arg_item_new (const gchar* k, const gchar* v);
PublishingRajceArgItem* publishing_rajce_arg_item_construct (GType object_type, const gchar* k, const gchar* v);
void publishing_rajce_arg_item_AddChild (PublishingRajceArgItem* self, PublishingRajceArgItem* child);
static void _vala_array_add6 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
void publishing_rajce_arg_item_AddChildren (PublishingRajceArgItem* self, PublishingRajceArgItem** newchildren, int newchildren_length1);
static void publishing_rajce_arg_item_finalize (PublishingRajceArgItem* obj);
gpointer publishing_rajce_live_api_request_ref (gpointer instance);
void publishing_rajce_live_api_request_unref (gpointer instance);
GParamSpec* publishing_rajce_param_spec_live_api_request (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void publishing_rajce_value_set_live_api_request (GValue* value, gpointer v_object);
void publishing_rajce_value_take_live_api_request (GValue* value, gpointer v_object);
gpointer publishing_rajce_value_get_live_api_request (const GValue* value);
GType publishing_rajce_live_api_request_get_type (void) G_GNUC_CONST;
#define PUBLISHING_RAJCE_LIVE_API_REQUEST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST, PublishingRajceLiveApiRequestPrivate))
enum  {
	PUBLISHING_RAJCE_LIVE_API_REQUEST_DUMMY_PROPERTY
};
PublishingRajceLiveApiRequest* publishing_rajce_live_api_request_new (const gchar* cmd);
PublishingRajceLiveApiRequest* publishing_rajce_live_api_request_construct (GType object_type, const gchar* cmd);
void publishing_rajce_live_api_request_AddParam (PublishingRajceLiveApiRequest* self, const gchar* name, const gchar* val);
static void _vala_array_add7 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
void publishing_rajce_live_api_request_AddParamBool (PublishingRajceLiveApiRequest* self, const gchar* name, gboolean val);
void publishing_rajce_live_api_request_AddParamInt (PublishingRajceLiveApiRequest* self, const gchar* name, gint val);
void publishing_rajce_live_api_request_AddParamNode (PublishingRajceLiveApiRequest* self, const gchar* name, PublishingRajceArgItem** val, int val_length1);
static void _vala_array_add8 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
gchar* publishing_rajce_live_api_request_Params2XmlString (PublishingRajceLiveApiRequest* self, gboolean urlencode);
static void publishing_rajce_live_api_request_WriteParam (xmlNode* node, PublishingRajceArgItem* arg);
static void publishing_rajce_live_api_request_finalize (PublishingRajceLiveApiRequest* obj);
enum  {
	PUBLISHING_RAJCE_TRANSACTION_DUMMY_PROPERTY
};
PublishingRajceTransaction* publishing_rajce_transaction_new (PublishingRajceSession* session);
PublishingRajceTransaction* publishing_rajce_transaction_construct (GType object_type, PublishingRajceSession* session);
enum  {
	PUBLISHING_RAJCE_SESSION_LOGIN_TRANSACTION_DUMMY_PROPERTY
};
enum  {
	PUBLISHING_RAJCE_GET_ALBUMS_TRANSACTION_DUMMY_PROPERTY
};
static void _vala_array_add9 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
static void _vala_array_add10 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
static void _vala_array_add11 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
static void _vala_array_add12 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
static void _vala_array_add13 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value);
enum  {
	PUBLISHING_RAJCE_CREATE_ALBUM_TRANSACTION_DUMMY_PROPERTY
};
enum  {
	PUBLISHING_RAJCE_OPEN_ALBUM_TRANSACTION_DUMMY_PROPERTY
};
enum  {
	PUBLISHING_RAJCE_GET_ALBUM_URL_TRANSACTION_DUMMY_PROPERTY
};
enum  {
	PUBLISHING_RAJCE_CLOSE_ALBUM_TRANSACTION_DUMMY_PROPERTY
};
GType publishing_rajce_get_categories_transaction_get_type (void) G_GNUC_CONST;
enum  {
	PUBLISHING_RAJCE_GET_CATEGORIES_TRANSACTION_DUMMY_PROPERTY
};
PublishingRajceGetCategoriesTransaction* publishing_rajce_get_categories_transaction_new (PublishingRajceSession* session, const gchar* url);
PublishingRajceGetCategoriesTransaction* publishing_rajce_get_categories_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url);
#define PUBLISHING_RAJCE_ADD_PHOTO_TRANSACTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION, PublishingRajceAddPhotoTransactionPrivate))
enum  {
	PUBLISHING_RAJCE_ADD_PHOTO_TRANSACTION_DUMMY_PROPERTY
};
static void _g_free0_ (gpointer var);
static void publishing_rajce_add_photo_transaction_finalize (PublishingRESTSupportTransaction* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


RajceService* rajce_service_construct (GType object_type, GFile* resource_directory) {
	RajceService * self = NULL;
	GdkPixbuf** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
#line 13 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (G_IS_FILE (resource_directory), NULL);
#line 13 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (RajceService*) g_object_new (object_type, NULL);
#line 15 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = rajce_service_icon_pixbuf_set;
#line 15 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0__length1 = rajce_service_icon_pixbuf_set_length1;
#line 15 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp0_ == NULL) {
#line 910 "RajcePublishing.c"
		gint _tmp1_ = 0;
		GdkPixbuf** _tmp2_ = NULL;
#line 16 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = resources_load_from_resource (PLUGIN_RESOURCE_PATH "/" RAJCE_SERVICE_ICON_FILENAME, &_tmp1_);
#line 16 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		rajce_service_icon_pixbuf_set = (_vala_array_free (rajce_service_icon_pixbuf_set, rajce_service_icon_pixbuf_set_length1, (GDestroyNotify) g_object_unref), NULL);
#line 16 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		rajce_service_icon_pixbuf_set = _tmp2_;
#line 16 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		rajce_service_icon_pixbuf_set_length1 = _tmp1_;
#line 16 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_rajce_service_icon_pixbuf_set_size_ = rajce_service_icon_pixbuf_set_length1;
#line 923 "RajcePublishing.c"
	}
#line 13 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 927 "RajcePublishing.c"
}


RajceService* rajce_service_new (GFile* resource_directory) {
#line 13 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return rajce_service_construct (TYPE_RAJCE_SERVICE, resource_directory);
#line 934 "RajcePublishing.c"
}


static gint rajce_service_real_get_pluggable_interface (SpitPluggable* base, gint min_host_interface, gint max_host_interface) {
	RajceService * self;
	gint result = 0;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
#line 21 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 23 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = min_host_interface;
#line 23 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = max_host_interface;
#line 23 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = spit_negotiate_interfaces (_tmp0_, _tmp1_, SPIT_PUBLISHING_CURRENT_INTERFACE);
#line 23 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp2_;
#line 23 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 956 "RajcePublishing.c"
}


static const gchar* rajce_service_real_get_id (SpitPluggable* base) {
	RajceService * self;
	const gchar* result = NULL;
#line 27 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 29 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = "org.yorba.shotwell.publishing.rajce";
#line 29 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 969 "RajcePublishing.c"
}


static const gchar* rajce_service_real_get_pluggable_name (SpitPluggable* base) {
	RajceService * self;
	const gchar* result = NULL;
#line 32 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 34 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = "Rajce";
#line 34 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 982 "RajcePublishing.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self ? g_object_ref (self) : NULL;
#line 989 "RajcePublishing.c"
}


static GdkPixbuf** _vala_array_dup4 (GdkPixbuf** self, int length) {
	GdkPixbuf** result;
	int i;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = g_new0 (GdkPixbuf*, length + 1);
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	for (i = 0; i < length; i++) {
#line 1000 "RajcePublishing.c"
		GdkPixbuf* _tmp0_ = NULL;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = _g_object_ref0 (self[i]);
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result[i] = _tmp0_;
#line 1006 "RajcePublishing.c"
	}
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1010 "RajcePublishing.c"
}


static void rajce_service_real_get_info (SpitPluggable* base, SpitPluggableInfo* info) {
	RajceService * self;
	gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	GdkPixbuf** _tmp8_ = NULL;
	gint _tmp8__length1 = 0;
	GdkPixbuf** _tmp9_ = NULL;
	gint _tmp9__length1 = 0;
#line 37 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 37 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (info != NULL);
#line 39 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = g_strdup ("rajce.net developers");
#line 39 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).authors);
#line 39 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).authors = _tmp0_;
#line 40 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _ ("Copyright (C) 2013 rajce.net");
#line 40 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 40 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).copyright);
#line 40 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).copyright = _tmp2_;
#line 41 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = g_strdup (RESOURCES_TRANSLATORS);
#line 41 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).translators);
#line 41 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).translators = _tmp3_;
#line 42 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = g_strdup (_VERSION);
#line 42 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).version);
#line 42 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).version = _tmp4_;
#line 43 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = g_strdup (RESOURCES_WEBSITE_NAME);
#line 43 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).website_name);
#line 43 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).website_name = _tmp5_;
#line 44 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = g_strdup (RESOURCES_WEBSITE_URL);
#line 44 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).website_url);
#line 44 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).website_url = _tmp6_;
#line 45 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).is_license_wordwrapped = FALSE;
#line 46 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = g_strdup (RESOURCES_LICENSE);
#line 46 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 ((*info).license);
#line 46 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).license = _tmp7_;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = rajce_service_icon_pixbuf_set;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8__length1 = rajce_service_icon_pixbuf_set_length1;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = (_tmp8_ != NULL) ? _vala_array_dup4 (_tmp8_, _tmp8__length1) : ((gpointer) _tmp8_);
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9__length1 = _tmp8__length1;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).icons = (_vala_array_free ((*info).icons, (*info).icons_length1, (GDestroyNotify) g_object_unref), NULL);
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).icons = _tmp9_;
#line 47 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*info).icons_length1 = _tmp9__length1;
#line 1092 "RajcePublishing.c"
}


static SpitPublishingPublisher* rajce_service_real_create_publisher (SpitPublishingService* base, SpitPublishingPluginHost* host) {
	RajceService * self;
	SpitPublishingPublisher* result = NULL;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajceRajcePublisher* _tmp1_ = NULL;
#line 50 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 50 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (SPIT_PUBLISHING_IS_PLUGIN_HOST (host), NULL);
#line 52 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = host;
#line 52 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = publishing_rajce_rajce_publisher_new (G_TYPE_CHECK_INSTANCE_CAST (self, SPIT_PUBLISHING_TYPE_SERVICE, SpitPublishingService), _tmp0_);
#line 52 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, SPIT_PUBLISHING_TYPE_PUBLISHER, SpitPublishingPublisher);
#line 52 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1113 "RajcePublishing.c"
}


static SpitPublishingPublisherMediaType rajce_service_real_get_supported_media (SpitPublishingService* base) {
	RajceService * self;
	SpitPublishingPublisherMediaType result = 0;
#line 55 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 57 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = SPIT_PUBLISHING_PUBLISHER_MEDIA_TYPE_PHOTO;
#line 57 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1126 "RajcePublishing.c"
}


static void rajce_service_real_activation (SpitPluggable* base, gboolean enabled) {
	RajceService * self;
#line 60 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_RAJCE_SERVICE, RajceService);
#line 1134 "RajcePublishing.c"
}


static void rajce_service_class_init (RajceServiceClass * klass) {
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	rajce_service_parent_class = g_type_class_peek_parent (klass);
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (klass)->finalize = rajce_service_finalize;
#line 1143 "RajcePublishing.c"
}


static void rajce_service_spit_pluggable_interface_init (SpitPluggableIface * iface) {
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	rajce_service_spit_pluggable_parent_iface = g_type_interface_peek_parent (iface);
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_pluggable_interface = (gint (*)(SpitPluggable*, gint, gint)) rajce_service_real_get_pluggable_interface;
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_id = (const gchar* (*)(SpitPluggable*)) rajce_service_real_get_id;
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_pluggable_name = (const gchar* (*)(SpitPluggable*)) rajce_service_real_get_pluggable_name;
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_info = (void (*)(SpitPluggable*, SpitPluggableInfo*)) rajce_service_real_get_info;
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->activation = (void (*)(SpitPluggable*, gboolean)) rajce_service_real_activation;
#line 1160 "RajcePublishing.c"
}


static void rajce_service_spit_publishing_service_interface_init (SpitPublishingServiceIface * iface) {
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	rajce_service_spit_publishing_service_parent_iface = g_type_interface_peek_parent (iface);
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->create_publisher = (SpitPublishingPublisher* (*)(SpitPublishingService*, SpitPublishingPluginHost*)) rajce_service_real_create_publisher;
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_supported_media = (SpitPublishingPublisherMediaType (*)(SpitPublishingService*)) rajce_service_real_get_supported_media;
#line 1171 "RajcePublishing.c"
}


static void rajce_service_instance_init (RajceService * self) {
}


static void rajce_service_finalize (GObject* obj) {
	RajceService * self;
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_RAJCE_SERVICE, RajceService);
#line 7 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (rajce_service_parent_class)->finalize (obj);
#line 1185 "RajcePublishing.c"
}


GType rajce_service_get_type (void) {
	static volatile gsize rajce_service_type_id__volatile = 0;
	if (g_once_init_enter (&rajce_service_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (RajceServiceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rajce_service_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RajceService), 0, (GInstanceInitFunc) rajce_service_instance_init, NULL };
		static const GInterfaceInfo spit_pluggable_info = { (GInterfaceInitFunc) rajce_service_spit_pluggable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo spit_publishing_service_info = { (GInterfaceInitFunc) rajce_service_spit_publishing_service_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType rajce_service_type_id;
		rajce_service_type_id = g_type_register_static (G_TYPE_OBJECT, "RajceService", &g_define_type_info, 0);
		g_type_add_interface_static (rajce_service_type_id, SPIT_TYPE_PLUGGABLE, &spit_pluggable_info);
		g_type_add_interface_static (rajce_service_type_id, SPIT_PUBLISHING_TYPE_SERVICE, &spit_publishing_service_info);
		g_once_init_leave (&rajce_service_type_id__volatile, rajce_service_type_id);
	}
	return rajce_service_type_id__volatile;
}


PublishingRajceRajcePublisher* publishing_rajce_rajce_publisher_construct (GType object_type, SpitPublishingService* service, SpitPublishingPluginHost* host) {
	PublishingRajceRajcePublisher * self = NULL;
	SpitPublishingService* _tmp0_ = NULL;
	SpitPublishingService* _tmp1_ = NULL;
	SpitPublishingPluginHost* _tmp2_ = NULL;
	SpitPublishingPluginHost* _tmp3_ = NULL;
	PublishingRajceSession* _tmp4_ = NULL;
	SpitPublishingPluginHost* _tmp5_ = NULL;
	gint _tmp6_ = 0;
	SpitPublishingPublishable** _tmp7_ = NULL;
#line 84 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (SPIT_PUBLISHING_IS_SERVICE (service), NULL);
#line 84 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (SPIT_PUBLISHING_IS_PLUGIN_HOST (host), NULL);
#line 84 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceRajcePublisher*) g_object_new (object_type, NULL);
#line 86 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:86: RajcePublisher created.");
#line 87 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = service;
#line 87 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 87 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->service);
#line 87 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->service = _tmp1_;
#line 88 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = host;
#line 88 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 88 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->host);
#line 88 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->host = _tmp3_;
#line 89 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_new ();
#line 89 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_session_unref0 (self->priv->session);
#line 89 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->session = _tmp4_;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = host;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = spit_publishing_plugin_host_get_publishables (_tmp5_, &_tmp6_);
#line 1249 "RajcePublishing.c"
	{
		SpitPublishingPublishable** p_collection = NULL;
		gint p_collection_length1 = 0;
		gint _p_collection_size_ = 0;
		gint p_it = 0;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		p_collection = _tmp7_;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		p_collection_length1 = _tmp6_;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		for (p_it = 0; p_it < _tmp6_; p_it = p_it + 1) {
#line 1261 "RajcePublishing.c"
			SpitPublishingPublishable* _tmp8_ = NULL;
			SpitPublishingPublishable* p = NULL;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp8_ = _g_object_ref0 (p_collection[p_it]);
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			p = _tmp8_;
#line 1268 "RajcePublishing.c"
			{
				SpitPublishingPublisherMediaType _tmp9_ = 0;
				SpitPublishingPublishable* _tmp10_ = NULL;
				SpitPublishingPublisherMediaType _tmp11_ = 0;
#line 92 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp9_ = self->priv->media_type;
#line 92 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp10_ = p;
#line 92 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp11_ = spit_publishing_publishable_get_media_type (_tmp10_);
#line 92 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				self->priv->media_type = _tmp9_ | _tmp11_;
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_g_object_unref0 (p);
#line 1283 "RajcePublishing.c"
			}
		}
#line 91 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		p_collection = (_vala_array_free (p_collection, p_collection_length1, (GDestroyNotify) g_object_unref), NULL);
#line 1288 "RajcePublishing.c"
	}
#line 84 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 1292 "RajcePublishing.c"
}


PublishingRajceRajcePublisher* publishing_rajce_rajce_publisher_new (SpitPublishingService* service, SpitPublishingPluginHost* host) {
#line 84 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_rajce_publisher_construct (PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, service, host);
#line 1299 "RajcePublishing.c"
}


static gchar* publishing_rajce_rajce_publisher_get_rajce_url (PublishingRajceRajcePublisher* self) {
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 95 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), NULL);
#line 97 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = g_strdup ("http://www.rajce.idnes.cz/liveAPI/index.php");
#line 97 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp0_;
#line 97 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1314 "RajcePublishing.c"
}


static SpitPublishingService* publishing_rajce_rajce_publisher_real_get_service (SpitPublishingPublisher* base) {
	PublishingRajceRajcePublisher * self;
	SpitPublishingService* result = NULL;
	SpitPublishingService* _tmp0_ = NULL;
	SpitPublishingService* _tmp1_ = NULL;
#line 102 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisher);
#line 102 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->service;
#line 102 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 102 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 102 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1333 "RajcePublishing.c"
}


SpitPublishingPluginHost* publishing_rajce_rajce_publisher_get_host (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* result = NULL;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	SpitPublishingPluginHost* _tmp1_ = NULL;
#line 103 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), NULL);
#line 103 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 103 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 103 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 103 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1351 "RajcePublishing.c"
}


static gboolean publishing_rajce_rajce_publisher_real_is_running (SpitPublishingPublisher* base) {
	PublishingRajceRajcePublisher * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
#line 104 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisher);
#line 104 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->running;
#line 104 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp0_;
#line 104 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1367 "RajcePublishing.c"
}


static gboolean* _bool_dup (gboolean* self) {
	gboolean* dup;
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	dup = g_new0 (gboolean, 1);
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	memcpy (dup, self, sizeof (gboolean));
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return dup;
#line 1379 "RajcePublishing.c"
}


static gpointer __bool_dup0 (gpointer self) {
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self ? _bool_dup (self) : NULL;
#line 1386 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_real_start (SpitPublishingPublisher* base) {
	PublishingRajceRajcePublisher * self;
	gboolean _tmp0_ = FALSE;
	PublishingRajceSession* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
#line 106 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisher);
#line 108 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = spit_publishing_publisher_is_running (G_TYPE_CHECK_INSTANCE_CAST (self, SPIT_PUBLISHING_TYPE_PUBLISHER, SpitPublishingPublisher));
#line 108 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp0_) {
#line 109 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 1403 "RajcePublishing.c"
	}
#line 111 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:111: RajcePublisher: start");
#line 112 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->running = TRUE;
#line 114 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->session;
#line 114 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rest_support_session_is_authenticated (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession));
#line 114 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp2_) {
#line 116 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:116: RajcePublisher: session is authenticated.");
#line 117 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_fetch_albums (self);
#line 1419 "RajcePublishing.c"
	} else {
		gchar* persistent_username = NULL;
		gchar* _tmp3_ = NULL;
		gchar* persistent_token = NULL;
		gchar* _tmp4_ = NULL;
		gboolean* persistent_remember = NULL;
		gboolean _tmp5_ = FALSE;
		gboolean* _tmp6_ = NULL;
		gboolean _tmp7_ = FALSE;
		const gchar* _tmp8_ = NULL;
#line 121 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:121: RajcePublisher: session is not authenticated" \
".");
#line 122 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = publishing_rajce_rajce_publisher_get_username (self);
#line 122 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		persistent_username = _tmp3_;
#line 123 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = publishing_rajce_rajce_publisher_get_token (self);
#line 123 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		persistent_token = _tmp4_;
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = publishing_rajce_rajce_publisher_get_remember (self);
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = __bool_dup0 (&_tmp5_);
#line 124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		persistent_remember = _tmp6_;
#line 125 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = persistent_username;
#line 125 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp8_ != NULL) {
#line 1450 "RajcePublishing.c"
			const gchar* _tmp9_ = NULL;
#line 125 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp9_ = persistent_token;
#line 125 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp7_ = _tmp9_ != NULL;
#line 1456 "RajcePublishing.c"
		} else {
#line 125 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp7_ = FALSE;
#line 1460 "RajcePublishing.c"
		}
#line 125 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp7_) {
#line 1464 "RajcePublishing.c"
			const gchar* _tmp10_ = NULL;
			const gchar* _tmp11_ = NULL;
			gboolean* _tmp12_ = NULL;
#line 126 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp10_ = persistent_username;
#line 126 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp11_ = persistent_token;
#line 126 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp12_ = persistent_remember;
#line 126 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rajce_rajce_publisher_do_network_login (self, _tmp10_, _tmp11_, *_tmp12_);
#line 1476 "RajcePublishing.c"
		} else {
#line 128 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rajce_rajce_publisher_do_show_authentication_pane (self, PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_INTRO);
#line 1480 "RajcePublishing.c"
		}
#line 114 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (persistent_remember);
#line 114 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (persistent_token);
#line 114 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (persistent_username);
#line 1488 "RajcePublishing.c"
	}
}


static void publishing_rajce_rajce_publisher_real_stop (SpitPublishingPublisher* base) {
	PublishingRajceRajcePublisher * self;
#line 132 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisher);
#line 134 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:134: RajcePublisher: stop");
#line 135 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->running = FALSE;
#line 1501 "RajcePublishing.c"
}


gchar* publishing_rajce_rajce_publisher_get_url (PublishingRajceRajcePublisher* self) {
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 140 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), NULL);
#line 140 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = publishing_rajce_rajce_publisher_get_rajce_url (self);
#line 140 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp0_;
#line 140 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1516 "RajcePublishing.c"
}


gchar* publishing_rajce_rajce_publisher_get_username (PublishingRajceRajcePublisher* self) {
	gchar* result = NULL;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 141 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), NULL);
#line 141 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 141 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = spit_host_interface_get_config_string (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "username", NULL);
#line 141 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 141 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1534 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_set_username (PublishingRajceRajcePublisher* self, const gchar* username) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 142 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 142 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (username != NULL);
#line 142 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 142 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = username;
#line 142 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_host_interface_set_config_string (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "username", _tmp1_);
#line 1551 "RajcePublishing.c"
}


gchar* publishing_rajce_rajce_publisher_get_token (PublishingRajceRajcePublisher* self) {
	gchar* result = NULL;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), NULL);
#line 143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = spit_host_interface_get_config_string (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "token", NULL);
#line 143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1569 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_set_token (PublishingRajceRajcePublisher* self, const gchar* token) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 144 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 144 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 144 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = token;
#line 144 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_host_interface_set_config_string (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "token", _tmp1_);
#line 1584 "RajcePublishing.c"
}


gboolean publishing_rajce_rajce_publisher_get_remember (PublishingRajceRajcePublisher* self) {
	gboolean result = FALSE;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 147 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), FALSE);
#line 147 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 147 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = spit_host_interface_get_config_bool (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "remember", FALSE);
#line 147 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 147 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1602 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_set_remember (PublishingRajceRajcePublisher* self, gboolean remember) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 148 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 148 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 148 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = remember;
#line 148 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_host_interface_set_config_bool (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "remember", _tmp1_);
#line 1617 "RajcePublishing.c"
}


gboolean publishing_rajce_rajce_publisher_get_hide_album (PublishingRajceRajcePublisher* self) {
	gboolean result = FALSE;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), FALSE);
#line 149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = spit_host_interface_get_config_bool (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "hide-album", FALSE);
#line 149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1635 "RajcePublishing.c"
}


void publishing_rajce_rajce_publisher_set_hide_album (PublishingRajceRajcePublisher* self, gboolean hide_album) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 150 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 150 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 150 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = hide_album;
#line 150 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_host_interface_set_config_bool (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "hide-album", _tmp1_);
#line 1650 "RajcePublishing.c"
}


gboolean publishing_rajce_rajce_publisher_get_show_album (PublishingRajceRajcePublisher* self) {
	gboolean result = FALSE;
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 151 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self), FALSE);
#line 151 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 151 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = spit_host_interface_get_config_bool (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "show-album", TRUE);
#line 151 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 151 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1668 "RajcePublishing.c"
}


void publishing_rajce_rajce_publisher_set_show_album (PublishingRajceRajcePublisher* self, gboolean show_album) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = show_album;
#line 152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_host_interface_set_config_bool (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, SPIT_TYPE_HOST_INTERFACE, SpitHostInterface), "show-album", _tmp1_);
#line 1683 "RajcePublishing.c"
}


/**
     * Action that shows the authentication pane.
     */
static void _publishing_rajce_rajce_publisher_on_authentication_pane_login_clicked_publishing_rajce_authentication_pane_login (PublishingRajceAuthenticationPane* _sender, const gchar* user, const gchar* token, gboolean remember, gpointer self) {
#line 167 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_authentication_pane_login_clicked ((PublishingRajceRajcePublisher*) self, user, token, remember);
#line 1693 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_show_authentication_pane (PublishingRajceRajcePublisher* self, PublishingRajceAuthenticationPaneMode mode) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajceAuthenticationPane* authentication_pane = NULL;
	PublishingRajceAuthenticationPaneMode _tmp1_ = 0;
	PublishingRajceAuthenticationPane* _tmp2_ = NULL;
	SpitPublishingPluginHost* _tmp3_ = NULL;
	SpitPublishingPluginHost* _tmp4_ = NULL;
	GtkWidget* _tmp5_ = NULL;
	GtkWidget* _tmp6_ = NULL;
#line 161 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 163 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:163: ACTION: installing authentication pane");
#line 165 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 165 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, FALSE);
#line 166 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = mode;
#line 166 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_authentication_pane_new (self, _tmp1_);
#line 166 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	authentication_pane = _tmp2_;
#line 167 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (authentication_pane, "login", (GCallback) _publishing_rajce_rajce_publisher_on_authentication_pane_login_clicked_publishing_rajce_authentication_pane_login, self, 0);
#line 168 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = self->priv->host;
#line 168 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_install_dialog_pane (_tmp3_, G_TYPE_CHECK_INSTANCE_CAST (authentication_pane, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane), SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CLOSE);
#line 169 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = self->priv->host;
#line 169 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rajce_authentication_pane_get_default_widget (authentication_pane);
#line 169 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 169 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_dialog_default_widget (_tmp4_, _tmp6_);
#line 169 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (_tmp6_);
#line 161 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (authentication_pane);
#line 1738 "RajcePublishing.c"
}


/**
     * Event triggered when the login button in the authentication panel is clicked.
     */
static void publishing_rajce_rajce_publisher_on_authentication_pane_login_clicked (PublishingRajceRajcePublisher* self, const gchar* username, const gchar* token, gboolean remember) {
	gboolean _tmp0_ = FALSE;
	const gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
#line 175 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 175 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (username != NULL);
#line 175 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (token != NULL);
#line 177 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:177: EVENT: on_authentication_pane_login_clicked");
#line 178 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->running;
#line 178 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!_tmp0_) {
#line 179 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 1764 "RajcePublishing.c"
	}
#line 180 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = username;
#line 180 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = token;
#line 180 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = remember;
#line 180 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_network_login (self, _tmp1_, _tmp2_, _tmp3_);
#line 1774 "RajcePublishing.c"
}


/**
     * Action to perform a network login to a Rajce service.
     */
static void _publishing_rajce_rajce_publisher_on_login_network_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self) {
#line 195 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_login_network_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 1784 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_login_network_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self) {
#line 196 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_login_network_complete ((PublishingRajceRajcePublisher*) self, _sender);
#line 1791 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_network_login (PublishingRajceRajcePublisher* self, const gchar* username, const gchar* token, gboolean remember) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	SpitPublishingPluginHost* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	const gchar* _tmp3_ = NULL;
	const gchar* _tmp4_ = NULL;
	gboolean _tmp5_ = FALSE;
	PublishingRajceSessionLoginTransaction* login_trans = NULL;
	PublishingRajceSession* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	PublishingRajceSessionLoginTransaction* _tmp12_ = NULL;
	PublishingRajceSessionLoginTransaction* _tmp13_ = NULL;
	GError * _inner_error_ = NULL;
#line 186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (username != NULL);
#line 186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (token != NULL);
#line 188 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:188: ACTION: logging in");
#line 189 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 189 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, TRUE);
#line 190 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->host;
#line 190 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_install_login_wait_pane (_tmp1_);
#line 191 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = remember;
#line 191 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_set_remember (self, _tmp2_);
#line 192 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = username;
#line 192 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_set_username (self, _tmp3_);
#line 193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = remember;
#line 193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp5_) {
#line 1839 "RajcePublishing.c"
		const gchar* _tmp6_ = NULL;
#line 193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = token;
#line 193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = _tmp6_;
#line 1845 "RajcePublishing.c"
	} else {
#line 193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = NULL;
#line 1849 "RajcePublishing.c"
	}
#line 193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_set_token (self, _tmp4_);
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = self->priv->session;
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = publishing_rajce_rajce_publisher_get_url (self);
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = _tmp8_;
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = username;
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = token;
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = publishing_rajce_session_login_transaction_new (_tmp7_, _tmp9_, _tmp10_, _tmp11_);
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13_ = _tmp12_;
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp9_);
#line 194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	login_trans = _tmp13_;
#line 195 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (login_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "network-error", (GCallback) _publishing_rajce_rajce_publisher_on_login_network_error_publishing_rest_support_transaction_network_error, self, 0);
#line 196 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (login_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "completed", (GCallback) _publishing_rajce_rajce_publisher_on_login_network_complete_publishing_rest_support_transaction_completed, self, 0);
#line 1875 "RajcePublishing.c"
	{
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rest_support_transaction_execute (G_TYPE_CHECK_INSTANCE_CAST (login_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), &_inner_error_);
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 1883 "RajcePublishing.c"
				goto __catch16_spit_publishing_publishing_error;
			}
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_transaction_unref0 (login_trans);
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 199 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 1894 "RajcePublishing.c"
		}
	}
	goto __finally16;
	__catch16_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp14_ = NULL;
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 203 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:203: ERROR: do_network_login");
#line 204 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = err;
#line 204 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp14_);
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 1914 "RajcePublishing.c"
	}
	__finally16:
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_transaction_unref0 (login_trans);
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 197 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 1927 "RajcePublishing.c"
	}
#line 186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_transaction_unref0 (login_trans);
#line 1931 "RajcePublishing.c"
}


/**
     * Event triggered when the network login action is complete and successful.
     */
static gchar* _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response (PublishingRESTSupportXmlDocument* doc, gpointer self) {
	gchar* result;
	result = publishing_rajce_transaction_validate_xml (doc);
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 1943 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_on_login_network_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	GError * _inner_error_ = NULL;
#line 211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (txn));
#line 213 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:213: EVENT: on_login_network_complete");
#line 214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = txn;
#line 214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_login_network_complete_publishing_rest_support_transaction_completed, self);
#line 215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = txn;
#line 215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_login_network_error_publishing_rest_support_transaction_network_error, self);
#line 1971 "RajcePublishing.c"
	{
		PublishingRESTSupportXmlDocument* doc = NULL;
		PublishingRESTSupportTransaction* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp7_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp8_ = NULL;
		xmlNode* response = NULL;
		PublishingRESTSupportXmlDocument* _tmp9_ = NULL;
		xmlNode* _tmp10_ = NULL;
		xmlNode* sessionToken = NULL;
		PublishingRESTSupportXmlDocument* _tmp11_ = NULL;
		xmlNode* _tmp12_ = NULL;
		xmlNode* _tmp13_ = NULL;
		xmlNode* maxWidth = NULL;
		PublishingRESTSupportXmlDocument* _tmp14_ = NULL;
		xmlNode* _tmp15_ = NULL;
		xmlNode* _tmp16_ = NULL;
		xmlNode* maxHeight = NULL;
		PublishingRESTSupportXmlDocument* _tmp17_ = NULL;
		xmlNode* _tmp18_ = NULL;
		xmlNode* _tmp19_ = NULL;
		xmlNode* quality = NULL;
		PublishingRESTSupportXmlDocument* _tmp20_ = NULL;
		xmlNode* _tmp21_ = NULL;
		xmlNode* _tmp22_ = NULL;
		xmlNode* nick = NULL;
		PublishingRESTSupportXmlDocument* _tmp23_ = NULL;
		xmlNode* _tmp24_ = NULL;
		xmlNode* _tmp25_ = NULL;
		gint maxW = 0;
		xmlNode* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gint _tmp29_ = 0;
		gint _tmp30_ = 0;
		gint maxH = 0;
		xmlNode* _tmp31_ = NULL;
		gchar* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		gint _tmp36_ = 0;
		gint _tmp37_ = 0;
		PublishingRajceSession* _tmp39_ = NULL;
		xmlNode* _tmp40_ = NULL;
		gchar* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		xmlNode* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gint _tmp46_ = 0;
		xmlNode* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		gint _tmp50_ = 0;
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = txn;
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = publishing_rest_support_transaction_get_response (_tmp4_);
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = _tmp5_;
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = publishing_rest_support_xml_document_parse_string (_tmp6_, _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response, NULL, &_inner_error_);
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = _tmp7_;
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp6_);
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		doc = _tmp8_;
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2046 "RajcePublishing.c"
				goto __catch17_spit_publishing_publishing_error;
			}
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 219 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2055 "RajcePublishing.c"
		}
#line 220 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = doc;
#line 220 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = publishing_rest_support_xml_document_get_root_node (_tmp9_);
#line 220 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		response = _tmp10_;
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = doc;
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = response;
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = publishing_rest_support_xml_document_get_named_child (_tmp11_, _tmp12_, "sessionToken", &_inner_error_);
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		sessionToken = _tmp13_;
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2077 "RajcePublishing.c"
				goto __catch17_spit_publishing_publishing_error;
			}
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 221 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2088 "RajcePublishing.c"
		}
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = doc;
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = response;
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = publishing_rest_support_xml_document_get_named_child (_tmp14_, _tmp15_, "maxWidth", &_inner_error_);
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		maxWidth = _tmp16_;
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2104 "RajcePublishing.c"
				goto __catch17_spit_publishing_publishing_error;
			}
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2115 "RajcePublishing.c"
		}
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = doc;
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = response;
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = publishing_rest_support_xml_document_get_named_child (_tmp17_, _tmp18_, "maxHeight", &_inner_error_);
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		maxHeight = _tmp19_;
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2131 "RajcePublishing.c"
				goto __catch17_spit_publishing_publishing_error;
			}
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 223 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2142 "RajcePublishing.c"
		}
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = doc;
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = response;
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = publishing_rest_support_xml_document_get_named_child (_tmp20_, _tmp21_, "quality", &_inner_error_);
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		quality = _tmp22_;
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2158 "RajcePublishing.c"
				goto __catch17_spit_publishing_publishing_error;
			}
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2169 "RajcePublishing.c"
		}
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = doc;
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = response;
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = publishing_rest_support_xml_document_get_named_child (_tmp23_, _tmp24_, "nick", &_inner_error_);
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		nick = _tmp25_;
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2185 "RajcePublishing.c"
				goto __catch17_spit_publishing_publishing_error;
			}
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2196 "RajcePublishing.c"
		}
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = maxWidth;
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = (gchar*) xmlNodeGetContent (_tmp26_);
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_ = _tmp27_;
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp29_ = atoi (_tmp28_);
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp30_ = _tmp29_;
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp28_);
#line 226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		maxW = _tmp30_;
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp31_ = maxHeight;
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp32_ = (gchar*) xmlNodeGetContent (_tmp31_);
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp33_ = _tmp32_;
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp34_ = atoi (_tmp33_);
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp35_ = _tmp34_;
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp33_);
#line 227 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		maxH = _tmp35_;
#line 228 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp36_ = maxW;
#line 228 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp37_ = maxH;
#line 228 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp36_ > _tmp37_) {
#line 2232 "RajcePublishing.c"
			gint _tmp38_ = 0;
#line 230 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp38_ = maxW;
#line 230 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			maxH = _tmp38_;
#line 2238 "RajcePublishing.c"
		}
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp39_ = self->priv->session;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp40_ = sessionToken;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp41_ = (gchar*) xmlNodeGetContent (_tmp40_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp42_ = _tmp41_;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp43_ = nick;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp44_ = (gchar*) xmlNodeGetContent (_tmp43_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp45_ = _tmp44_;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp46_ = maxH;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp47_ = quality;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp48_ = (gchar*) xmlNodeGetContent (_tmp47_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp49_ = _tmp48_;
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp50_ = atoi (_tmp49_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_authenticate (_tmp39_, _tmp42_, _tmp45_, 0, _tmp46_, _tmp50_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp49_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp45_);
#line 232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp42_);
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_xml_document_unref0 (doc);
#line 2274 "RajcePublishing.c"
	}
	goto __finally17;
	__catch17_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		gint code_int = 0;
		GError* _tmp51_ = NULL;
		const gchar* _tmp52_ = NULL;
		gint _tmp53_ = 0;
		gint _tmp54_ = 0;
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp51_ = err;
#line 236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp52_ = _tmp51_->message;
#line 236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp53_ = atoi (_tmp52_);
#line 236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		code_int = _tmp53_;
#line 237 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp54_ = code_int;
#line 237 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp54_ == 999) {
#line 239 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_debug ("RajcePublishing.vala:239: ERROR: on_login_network_complete, code 999");
#line 240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rajce_rajce_publisher_do_show_authentication_pane (self, PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER);
#line 2305 "RajcePublishing.c"
		} else {
			GError* _tmp55_ = NULL;
#line 244 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_debug ("RajcePublishing.vala:244: ERROR: on_login_network_complete");
#line 245 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp55_ = err;
#line 245 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rajce_rajce_publisher_do_show_error (self, _tmp55_);
#line 2314 "RajcePublishing.c"
		}
#line 247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 2320 "RajcePublishing.c"
	}
	__finally17:
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 217 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 2331 "RajcePublishing.c"
	}
#line 249 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_fetch_albums (self);
#line 2335 "RajcePublishing.c"
}


/**
     * Event triggered when a network login action fails due to a network error.
     */
static void publishing_rajce_rajce_publisher_on_login_network_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
#line 255 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 255 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:257: EVENT: on_login_network_error");
#line 258 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = bad_txn;
#line 258 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 258 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_login_network_complete_publishing_rest_support_transaction_completed, self);
#line 259 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = bad_txn;
#line 259 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 259 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_login_network_error_publishing_rest_support_transaction_network_error, self);
#line 260 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_authentication_pane (self, PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER);
#line 2367 "RajcePublishing.c"
}


/**
     * Action that fetches all user albums from the Rajce.
     */
static void _publishing_rajce_rajce_publisher_on_albums_fetch_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self) {
#line 273 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_albums_fetch_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 2377 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_albums_fetch_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self) {
#line 274 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_albums_fetch_complete ((PublishingRajceRajcePublisher*) self, _sender);
#line 2384 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_fetch_albums (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	SpitPublishingPluginHost* _tmp1_ = NULL;
	PublishingRajceGetAlbumsTransaction* get_albums_trans = NULL;
	PublishingRajceSession* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	PublishingRajceGetAlbumsTransaction* _tmp5_ = NULL;
	PublishingRajceGetAlbumsTransaction* _tmp6_ = NULL;
	GError * _inner_error_ = NULL;
#line 266 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 268 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:268: ACTION: fetching albums");
#line 269 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 269 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, TRUE);
#line 270 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->host;
#line 270 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_install_account_fetch_wait_pane (_tmp1_);
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->session;
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = publishing_rajce_rajce_publisher_get_url (self);
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = _tmp3_;
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rajce_get_albums_transaction_new (_tmp2_, _tmp4_);
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp4_);
#line 272 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	get_albums_trans = _tmp6_;
#line 273 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (get_albums_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "network-error", (GCallback) _publishing_rajce_rajce_publisher_on_albums_fetch_error_publishing_rest_support_transaction_network_error, self, 0);
#line 274 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (get_albums_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "completed", (GCallback) _publishing_rajce_rajce_publisher_on_albums_fetch_complete_publishing_rest_support_transaction_completed, self, 0);
#line 2428 "RajcePublishing.c"
	{
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rest_support_transaction_execute (G_TYPE_CHECK_INSTANCE_CAST (get_albums_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), &_inner_error_);
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2436 "RajcePublishing.c"
				goto __catch18_spit_publishing_publishing_error;
			}
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_transaction_unref0 (get_albums_trans);
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2447 "RajcePublishing.c"
		}
	}
	goto __finally18;
	__catch18_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp7_ = NULL;
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 282 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:282: ERROR: do_fetch_albums");
#line 283 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = err;
#line 283 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp7_);
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 2467 "RajcePublishing.c"
	}
	__finally18:
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_transaction_unref0 (get_albums_trans);
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 2480 "RajcePublishing.c"
	}
#line 266 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_transaction_unref0 (get_albums_trans);
#line 2484 "RajcePublishing.c"
}


/**
     * Event triggered when the fetch albums action completes successfully.
     */
static gint _publishing_rajce_album_compare_albums_gcompare_data_func (gconstpointer a, gconstpointer b, gpointer self) {
	gint result;
	result = publishing_rajce_album_compare_albums ((PublishingRajceAlbum*) a, (PublishingRajceAlbum*) b);
#line 320 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 2496 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_on_albums_fetch_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	PublishingRajceAlbum** _tmp7_ = NULL;
	gint _tmp7__length1 = 0;
	GeeArrayList* list = NULL;
	GeeArrayList* _tmp8_ = NULL;
	GError * _inner_error_ = NULL;
#line 290 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 290 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (txn));
#line 292 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:292: EVENT: on_albums_fetch_complete");
#line 293 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = txn;
#line 293 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 293 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_albums_fetch_complete_publishing_rest_support_transaction_completed, self);
#line 294 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = txn;
#line 294 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 294 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_albums_fetch_error_publishing_rest_support_transaction_network_error, self);
#line 295 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = txn;
#line 295 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rest_support_transaction_get_response (_tmp4_);
#line 295 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 295 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:295: RajcePlugin: list of albums: %s", _tmp6_);
#line 295 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp6_);
#line 296 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = self->priv->albums;
#line 296 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7__length1 = self->priv->albums_length1;
#line 296 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp7_ != NULL) {
#line 298 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->albums = (_vala_array_free (self->priv->albums, self->priv->albums_length1, (GDestroyNotify) publishing_rajce_album_unref), NULL);
#line 298 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->albums = NULL;
#line 298 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->albums_length1 = 0;
#line 298 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->_albums_size_ = self->priv->albums_length1;
#line 2555 "RajcePublishing.c"
	}
#line 300 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = gee_array_list_new (PUBLISHING_RAJCE_TYPE_ALBUM, (GBoxedCopyFunc) publishing_rajce_album_ref, publishing_rajce_album_unref, NULL, NULL, NULL);
#line 300 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	list = _tmp8_;
#line 2561 "RajcePublishing.c"
	{
		PublishingRESTSupportXmlDocument* doc = NULL;
		PublishingRESTSupportTransaction* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp12_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp13_ = NULL;
		xmlNode* response = NULL;
		PublishingRESTSupportXmlDocument* _tmp14_ = NULL;
		xmlNode* _tmp15_ = NULL;
		xmlNode* sessionToken = NULL;
		PublishingRESTSupportXmlDocument* _tmp16_ = NULL;
		xmlNode* _tmp17_ = NULL;
		xmlNode* _tmp18_ = NULL;
		xmlNode* nodealbums = NULL;
		PublishingRESTSupportXmlDocument* _tmp19_ = NULL;
		xmlNode* _tmp20_ = NULL;
		xmlNode* _tmp21_ = NULL;
		GeeArrayList* _tmp96_ = NULL;
		GeeArrayList* _tmp97_ = NULL;
		gint _tmp98_ = 0;
		gpointer* _tmp99_ = NULL;
		PublishingRajceSession* _tmp100_ = NULL;
		xmlNode* _tmp101_ = NULL;
		gchar* _tmp102_ = NULL;
		gchar* _tmp103_ = NULL;
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = txn;
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = publishing_rest_support_transaction_get_response (_tmp9_);
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = _tmp10_;
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = publishing_rest_support_xml_document_parse_string (_tmp11_, _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response, NULL, &_inner_error_);
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = _tmp12_;
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp11_);
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		doc = _tmp13_;
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2606 "RajcePublishing.c"
				goto __catch19_spit_publishing_publishing_error;
			}
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_object_unref0 (list);
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 303 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2617 "RajcePublishing.c"
		}
#line 304 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = doc;
#line 304 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = publishing_rest_support_xml_document_get_root_node (_tmp14_);
#line 304 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		response = _tmp15_;
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = doc;
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = response;
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = publishing_rest_support_xml_document_get_named_child (_tmp16_, _tmp17_, "sessionToken", &_inner_error_);
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		sessionToken = _tmp18_;
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2639 "RajcePublishing.c"
				goto __catch19_spit_publishing_publishing_error;
			}
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_object_unref0 (list);
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 305 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2652 "RajcePublishing.c"
		}
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = doc;
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = response;
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = publishing_rest_support_xml_document_get_named_child (_tmp19_, _tmp20_, "albums", &_inner_error_);
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		nodealbums = _tmp21_;
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2668 "RajcePublishing.c"
				goto __catch19_spit_publishing_publishing_error;
			}
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_object_unref0 (list);
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 2681 "RajcePublishing.c"
		}
		{
			xmlNode* album = NULL;
			xmlNode* _tmp22_ = NULL;
			xmlNode* _tmp23_ = NULL;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp22_ = nodealbums;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp23_ = _tmp22_->children;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			album = _tmp23_;
#line 2693 "RajcePublishing.c"
			{
				gboolean _tmp24_ = FALSE;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp24_ = TRUE;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				while (TRUE) {
#line 2700 "RajcePublishing.c"
					xmlNode* _tmp27_ = NULL;
					gint id = 0;
					xmlNode* _tmp28_ = NULL;
					gchar* _tmp29_ = NULL;
					gchar* _tmp30_ = NULL;
					gint _tmp31_ = 0;
					gint _tmp32_ = 0;
					xmlNode* _tmp33_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp34_ = NULL;
					xmlNode* _tmp35_ = NULL;
					xmlNode* _tmp36_ = NULL;
					gchar* albumName = NULL;
					gchar* _tmp37_ = NULL;
					xmlNode* _tmp38_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp39_ = NULL;
					xmlNode* _tmp40_ = NULL;
					xmlNode* _tmp41_ = NULL;
					gchar* url = NULL;
					gchar* _tmp42_ = NULL;
					xmlNode* _tmp43_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp44_ = NULL;
					xmlNode* _tmp45_ = NULL;
					xmlNode* _tmp46_ = NULL;
					gchar* thumbUrl = NULL;
					gchar* _tmp47_ = NULL;
					xmlNode* _tmp48_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp49_ = NULL;
					xmlNode* _tmp50_ = NULL;
					xmlNode* _tmp51_ = NULL;
					gchar* createDate = NULL;
					gchar* _tmp52_ = NULL;
					xmlNode* _tmp53_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp54_ = NULL;
					xmlNode* _tmp55_ = NULL;
					xmlNode* _tmp56_ = NULL;
					gchar* updateDate = NULL;
					gchar* _tmp57_ = NULL;
					gboolean _tmp58_ = FALSE;
					xmlNode* _tmp59_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp60_ = NULL;
					xmlNode* _tmp61_ = NULL;
					xmlNode* _tmp62_ = NULL;
					gchar* _tmp63_ = NULL;
					gchar* _tmp64_ = NULL;
					gint _tmp65_ = 0;
					gboolean _tmp66_ = FALSE;
					gboolean hidden = FALSE;
					gboolean _tmp67_ = FALSE;
					xmlNode* _tmp68_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp69_ = NULL;
					xmlNode* _tmp70_ = NULL;
					xmlNode* _tmp71_ = NULL;
					gchar* _tmp72_ = NULL;
					gchar* _tmp73_ = NULL;
					gint _tmp74_ = 0;
					gboolean _tmp75_ = FALSE;
					gboolean secure = FALSE;
					xmlNode* _tmp76_ = NULL;
					PublishingRESTSupportXmlDocument* _tmp77_ = NULL;
					xmlNode* _tmp78_ = NULL;
					xmlNode* _tmp79_ = NULL;
					gint photoCount = 0;
					gchar* _tmp80_ = NULL;
					gchar* _tmp81_ = NULL;
					gint _tmp82_ = 0;
					gint _tmp83_ = 0;
					GeeArrayList* _tmp84_ = NULL;
					gint _tmp85_ = 0;
					const gchar* _tmp86_ = NULL;
					const gchar* _tmp87_ = NULL;
					const gchar* _tmp88_ = NULL;
					const gchar* _tmp89_ = NULL;
					const gchar* _tmp90_ = NULL;
					gboolean _tmp91_ = FALSE;
					gboolean _tmp92_ = FALSE;
					gint _tmp93_ = 0;
					PublishingRajceAlbum* _tmp94_ = NULL;
					PublishingRajceAlbum* _tmp95_ = NULL;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (!_tmp24_) {
#line 2781 "RajcePublishing.c"
						xmlNode* _tmp25_ = NULL;
						xmlNode* _tmp26_ = NULL;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp25_ = album;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp26_ = _tmp25_->next;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						album = _tmp26_;
#line 2790 "RajcePublishing.c"
					}
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp24_ = FALSE;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp27_ = album;
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (!(_tmp27_ != NULL)) {
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						break;
#line 2800 "RajcePublishing.c"
					}
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp28_ = album;
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp29_ = (gchar*) xmlGetProp (_tmp28_, (xmlChar*) "id");
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp30_ = _tmp29_;
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp31_ = atoi (_tmp30_);
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp32_ = _tmp31_;
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (_tmp30_);
#line 309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					id = _tmp32_;
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp34_ = doc;
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp35_ = album;
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp36_ = publishing_rest_support_xml_document_get_named_child (_tmp34_, _tmp35_, "albumName", &_inner_error_);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp33_ = _tmp36_;
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2830 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 2843 "RajcePublishing.c"
					}
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp37_ = (gchar*) xmlNodeGetContent (_tmp33_);
#line 310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					albumName = _tmp37_;
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp39_ = doc;
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp40_ = album;
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp41_ = publishing_rest_support_xml_document_get_named_child (_tmp39_, _tmp40_, "url", &_inner_error_);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp38_ = _tmp41_;
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2865 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 2880 "RajcePublishing.c"
					}
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp42_ = (gchar*) xmlNodeGetContent (_tmp38_);
#line 311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					url = _tmp42_;
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp44_ = doc;
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp45_ = album;
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp46_ = publishing_rest_support_xml_document_get_named_child (_tmp44_, _tmp45_, "thumbUrl", &_inner_error_);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp43_ = _tmp46_;
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2904 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 2921 "RajcePublishing.c"
					}
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp47_ = (gchar*) xmlNodeGetContent (_tmp43_);
#line 312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					thumbUrl = _tmp47_;
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp49_ = doc;
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp50_ = album;
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp51_ = publishing_rest_support_xml_document_get_named_child (_tmp49_, _tmp50_, "createDate", &_inner_error_);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp48_ = _tmp51_;
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2947 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 2966 "RajcePublishing.c"
					}
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp52_ = (gchar*) xmlNodeGetContent (_tmp48_);
#line 313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					createDate = _tmp52_;
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp54_ = doc;
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp55_ = album;
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp56_ = publishing_rest_support_xml_document_get_named_child (_tmp54_, _tmp55_, "updateDate", &_inner_error_);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp53_ = _tmp56_;
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 2994 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 3015 "RajcePublishing.c"
					}
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp57_ = (gchar*) xmlNodeGetContent (_tmp53_);
#line 314 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					updateDate = _tmp57_;
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp60_ = doc;
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp61_ = album;
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp62_ = publishing_rest_support_xml_document_get_named_child (_tmp60_, _tmp61_, "hidden", &_inner_error_);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp59_ = _tmp62_;
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (updateDate);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3045 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (updateDate);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 3068 "RajcePublishing.c"
					}
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp63_ = (gchar*) xmlNodeGetContent (_tmp59_);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp64_ = _tmp63_;
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp65_ = atoi (_tmp64_);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp66_ = _tmp65_ > 0;
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (_tmp64_);
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (_tmp66_) {
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp58_ = TRUE;
#line 3084 "RajcePublishing.c"
					} else {
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp58_ = FALSE;
#line 3088 "RajcePublishing.c"
					}
#line 315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					hidden = _tmp58_;
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp69_ = doc;
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp70_ = album;
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp71_ = publishing_rest_support_xml_document_get_named_child (_tmp69_, _tmp70_, "secure", &_inner_error_);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp68_ = _tmp71_;
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (updateDate);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3116 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (updateDate);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 3139 "RajcePublishing.c"
					}
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp72_ = (gchar*) xmlNodeGetContent (_tmp68_);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp73_ = _tmp72_;
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp74_ = atoi (_tmp73_);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp75_ = _tmp74_ > 0;
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (_tmp73_);
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (_tmp75_) {
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp67_ = TRUE;
#line 3155 "RajcePublishing.c"
					} else {
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp67_ = FALSE;
#line 3159 "RajcePublishing.c"
					}
#line 316 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					secure = _tmp67_;
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp77_ = doc;
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp78_ = album;
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp79_ = publishing_rest_support_xml_document_get_named_child (_tmp77_, _tmp78_, "photoCount", &_inner_error_);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp76_ = _tmp79_;
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (updateDate);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3187 "RajcePublishing.c"
							goto __catch19_spit_publishing_publishing_error;
						}
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (updateDate);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (createDate);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (thumbUrl);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (url);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (albumName);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_publishing_rest_support_xml_document_unref0 (doc);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_object_unref0 (list);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						g_clear_error (&_inner_error_);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						return;
#line 3210 "RajcePublishing.c"
					}
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp80_ = (gchar*) xmlNodeGetContent (_tmp76_);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp81_ = _tmp80_;
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp82_ = atoi (_tmp81_);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp83_ = _tmp82_;
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (_tmp81_);
#line 317 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					photoCount = _tmp83_;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp84_ = list;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp85_ = id;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp86_ = albumName;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp87_ = url;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp88_ = thumbUrl;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp89_ = createDate;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp90_ = updateDate;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp91_ = hidden;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp92_ = secure;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp93_ = photoCount;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp94_ = publishing_rajce_album_new (_tmp85_, _tmp86_, _tmp87_, _tmp88_, _tmp89_, _tmp90_, _tmp91_, _tmp92_, _tmp93_);
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp95_ = _tmp94_;
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					gee_abstract_list_insert (G_TYPE_CHECK_INSTANCE_CAST (_tmp84_, GEE_TYPE_ABSTRACT_LIST, GeeAbstractList), 0, _tmp95_);
#line 318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_publishing_rajce_album_unref0 (_tmp95_);
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (updateDate);
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (createDate);
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (thumbUrl);
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (url);
#line 307 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (albumName);
#line 3262 "RajcePublishing.c"
				}
			}
		}
#line 320 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp96_ = list;
#line 320 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gee_list_sort (G_TYPE_CHECK_INSTANCE_CAST (_tmp96_, GEE_TYPE_LIST, GeeList), _publishing_rajce_album_compare_albums_gcompare_data_func, NULL, NULL);
#line 321 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp97_ = list;
#line 321 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp99_ = gee_collection_to_array (G_TYPE_CHECK_INSTANCE_CAST (_tmp97_, GEE_TYPE_COLLECTION, GeeCollection), &_tmp98_);
#line 321 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->albums = (_vala_array_free (self->priv->albums, self->priv->albums_length1, (GDestroyNotify) publishing_rajce_album_unref), NULL);
#line 321 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->albums = _tmp99_;
#line 321 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->albums_length1 = _tmp98_;
#line 321 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->_albums_size_ = self->priv->albums_length1;
#line 322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp100_ = self->priv->session;
#line 322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp101_ = sessionToken;
#line 322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp102_ = (gchar*) xmlNodeGetContent (_tmp101_);
#line 322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp103_ = _tmp102_;
#line 322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_usertoken (_tmp100_, _tmp103_);
#line 322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp103_);
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_xml_document_unref0 (doc);
#line 3296 "RajcePublishing.c"
	}
	goto __finally19;
	__catch19_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp104_ = NULL;
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 326 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:326: ERROR: on_albums_fetch_complete");
#line 327 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp104_ = err;
#line 327 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp104_);
#line 328 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 328 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (list);
#line 328 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 3319 "RajcePublishing.c"
	}
	__finally19:
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (list);
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 3332 "RajcePublishing.c"
	}
#line 330 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_publishing_options_pane (self);
#line 290 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (list);
#line 3338 "RajcePublishing.c"
}


/**
     * Event triggered when the fetch albums transaction fails due to a network error.
     */
static void publishing_rajce_rajce_publisher_on_albums_fetch_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	GError* _tmp5_ = NULL;
#line 336 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 336 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:338: EVENT: on_albums_fetch_error");
#line 339 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = bad_txn;
#line 339 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 339 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_albums_fetch_complete_publishing_rest_support_transaction_completed, self);
#line 340 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = bad_txn;
#line 340 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 340 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_albums_fetch_error_publishing_rest_support_transaction_network_error, self);
#line 341 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = bad_txn;
#line 341 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = err;
#line 341 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_network_error (self, _tmp4_, _tmp5_);
#line 3376 "RajcePublishing.c"
}


/**
     * Action that shows the publishing options pane.
     */
static void _publishing_rajce_rajce_publisher_on_publishing_options_pane_logout_clicked_publishing_rajce_publishing_options_pane_logout (PublishingRajcePublishingOptionsPane* _sender, gpointer self) {
#line 352 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_publishing_options_pane_logout_clicked ((PublishingRajceRajcePublisher*) self);
#line 3386 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_publishing_options_pane_publish_clicked_publishing_rajce_publishing_options_pane_publish (PublishingRajcePublishingOptionsPane* _sender, PublishingRajcePublishingParameters* parameters, gpointer self) {
#line 353 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_publishing_options_pane_publish_clicked ((PublishingRajceRajcePublisher*) self, parameters);
#line 3393 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_show_publishing_options_pane (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajcePublishingOptionsPane* opts_pane = NULL;
	PublishingRajceSession* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	PublishingRajceAlbum** _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
	PublishingRajcePublishingOptionsPane* _tmp5_ = NULL;
	PublishingRajcePublishingOptionsPane* _tmp6_ = NULL;
	SpitPublishingPluginHost* _tmp7_ = NULL;
	SpitPublishingPluginHost* _tmp8_ = NULL;
	GtkWidget* _tmp9_ = NULL;
	GtkWidget* _tmp10_ = NULL;
#line 347 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 349 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:349: ACTION: installing publishing options pane");
#line 350 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 350 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, FALSE);
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->session;
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_session_get_username (_tmp1_);
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _tmp2_;
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = self->priv->albums;
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4__length1 = self->priv->albums_length1;
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rajce_publishing_options_pane_new (self, _tmp3_, _tmp4_, _tmp4__length1);
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp3_);
#line 351 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	opts_pane = _tmp6_;
#line 352 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (opts_pane, "logout", (GCallback) _publishing_rajce_rajce_publisher_on_publishing_options_pane_logout_clicked_publishing_rajce_publishing_options_pane_logout, self, 0);
#line 353 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (opts_pane, "publish", (GCallback) _publishing_rajce_rajce_publisher_on_publishing_options_pane_publish_clicked_publishing_rajce_publishing_options_pane_publish, self, 0);
#line 354 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = self->priv->host;
#line 354 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_install_dialog_pane (_tmp7_, G_TYPE_CHECK_INSTANCE_CAST (opts_pane, SPIT_PUBLISHING_TYPE_DIALOG_PANE, SpitPublishingDialogPane), SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CLOSE);
#line 355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = self->priv->host;
#line 355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = publishing_rajce_publishing_options_pane_get_default_widget (opts_pane);
#line 355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = _tmp9_;
#line 355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_dialog_default_widget (_tmp8_, _tmp10_);
#line 355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (_tmp10_);
#line 347 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (opts_pane);
#line 3457 "RajcePublishing.c"
}


/**
     * Event triggered when the user clicks logout in the publishing options pane.
     */
static void publishing_rajce_rajce_publisher_on_publishing_options_pane_logout_clicked (PublishingRajceRajcePublisher* self) {
	PublishingRajceSession* _tmp0_ = NULL;
#line 361 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 363 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:363: EVENT: on_publishing_options_pane_logout_cli" \
"cked");
#line 364 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->session;
#line 364 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_session_deauthenticate (_tmp0_);
#line 365 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_authentication_pane (self, PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_INTRO);
#line 3476 "RajcePublishing.c"
}


/**
     * Event triggered when the user clicks publish in the publishing options pane.
     *
     * @param parameters the publishing parameters
     */
static gpointer _publishing_rajce_publishing_parameters_ref0 (gpointer self) {
#line 376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self ? publishing_rajce_publishing_parameters_ref (self) : NULL;
#line 3488 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_on_publishing_options_pane_publish_clicked (PublishingRajceRajcePublisher* self, PublishingRajcePublishingParameters* parameters) {
	PublishingRajcePublishingParameters* _tmp0_ = NULL;
	PublishingRajcePublishingParameters* _tmp1_ = NULL;
#line 373 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 373 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_PARAMETERS (parameters));
#line 375 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:375: EVENT: on_publishing_options_pane_publish_cl" \
"icked");
#line 376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = parameters;
#line 376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _publishing_rajce_publishing_parameters_ref0 (_tmp0_);
#line 376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_publishing_parameters_unref0 (self->priv->parameters);
#line 376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->parameters = _tmp1_;
#line 377 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_begin_upload (self);
#line 3511 "RajcePublishing.c"
}


/**
     * Begin upload action: open existing album or create a new one
     */
static gboolean _int_equal (const gint* s1, const gint* s2) {
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (s1 == s2) {
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return TRUE;
#line 3523 "RajcePublishing.c"
	}
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (s1 == NULL) {
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return FALSE;
#line 3529 "RajcePublishing.c"
	}
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (s2 == NULL) {
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return FALSE;
#line 3535 "RajcePublishing.c"
	}
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return (*s1) == (*s2);
#line 3539 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_create_album_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self) {
#line 391 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_create_album_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 3546 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_create_album_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self) {
#line 392 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_create_album_complete ((PublishingRajceRajcePublisher*) self, _sender);
#line 3553 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_open_album_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self) {
#line 408 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_open_album_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 3560 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_open_album_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self) {
#line 409 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_open_album_complete ((PublishingRajceRajcePublisher*) self, _sender);
#line 3567 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_begin_upload (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajcePublishingParameters* _tmp1_ = NULL;
	gint* _tmp2_ = NULL;
	gint _tmp3_ = 0;
	GError * _inner_error_ = NULL;
#line 383 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, TRUE);
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->parameters;
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = _tmp1_->album_id;
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = 0;
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_int_equal (_tmp2_, &_tmp3_) == TRUE) {
#line 3591 "RajcePublishing.c"
		PublishingRajceCreateAlbumTransaction* create_album_trans = NULL;
		PublishingRajceSession* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		PublishingRajcePublishingParameters* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		PublishingRajcePublishingParameters* _tmp9_ = NULL;
		gboolean* _tmp10_ = NULL;
		PublishingRajceCreateAlbumTransaction* _tmp11_ = NULL;
		PublishingRajceCreateAlbumTransaction* _tmp12_ = NULL;
		PublishingRajceCreateAlbumTransaction* _tmp13_ = NULL;
		PublishingRajceCreateAlbumTransaction* _tmp14_ = NULL;
#line 389 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:389: ACTION: closing album");
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = self->priv->session;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = publishing_rajce_rajce_publisher_get_url (self);
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = _tmp5_;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = self->priv->parameters;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = _tmp7_->album_name;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = self->priv->parameters;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = _tmp9_->album_hidden;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = publishing_rajce_create_album_transaction_new (_tmp4_, _tmp6_, _tmp8_, *_tmp10_);
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = _tmp11_;
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp6_);
#line 390 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		create_album_trans = _tmp12_;
#line 391 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = create_album_trans;
#line 391 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp13_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "network-error", (GCallback) _publishing_rajce_rajce_publisher_on_create_album_error_publishing_rest_support_transaction_network_error, self, 0);
#line 392 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = create_album_trans;
#line 392 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp14_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "completed", (GCallback) _publishing_rajce_rajce_publisher_on_create_album_complete_publishing_rest_support_transaction_completed, self, 0);
#line 3636 "RajcePublishing.c"
		{
			PublishingRajceCreateAlbumTransaction* _tmp15_ = NULL;
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp15_ = create_album_trans;
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rest_support_transaction_execute (G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), &_inner_error_);
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3647 "RajcePublishing.c"
					goto __catch20_spit_publishing_publishing_error;
				}
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_publishing_rest_support_transaction_unref0 (create_album_trans);
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				g_clear_error (&_inner_error_);
#line 395 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				return;
#line 3658 "RajcePublishing.c"
			}
		}
		goto __finally20;
		__catch20_spit_publishing_publishing_error:
		{
			GError* err = NULL;
			GError* _tmp16_ = NULL;
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			err = _inner_error_;
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_inner_error_ = NULL;
#line 399 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_debug ("RajcePublishing.vala:399: ERROR: create album");
#line 400 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp16_ = err;
#line 400 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rajce_rajce_publisher_do_show_error (self, _tmp16_);
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_error_free0 (err);
#line 3678 "RajcePublishing.c"
		}
		__finally20:
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_transaction_unref0 (create_album_trans);
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 393 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 3691 "RajcePublishing.c"
		}
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_transaction_unref0 (create_album_trans);
#line 3695 "RajcePublishing.c"
	} else {
		PublishingRajceOpenAlbumTransaction* open_album_trans = NULL;
		PublishingRajceSession* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		PublishingRajcePublishingParameters* _tmp20_ = NULL;
		gint* _tmp21_ = NULL;
		PublishingRajceOpenAlbumTransaction* _tmp22_ = NULL;
		PublishingRajceOpenAlbumTransaction* _tmp23_ = NULL;
		PublishingRajceOpenAlbumTransaction* _tmp24_ = NULL;
		PublishingRajceOpenAlbumTransaction* _tmp25_ = NULL;
#line 406 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:406: ACTION: opening album");
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = self->priv->session;
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = publishing_rajce_rajce_publisher_get_url (self);
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = _tmp18_;
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = self->priv->parameters;
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = _tmp20_->album_id;
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = publishing_rajce_open_album_transaction_new (_tmp17_, _tmp19_, *_tmp21_);
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = _tmp22_;
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp19_);
#line 407 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		open_album_trans = _tmp23_;
#line 408 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = open_album_trans;
#line 408 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp24_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "network-error", (GCallback) _publishing_rajce_rajce_publisher_on_open_album_error_publishing_rest_support_transaction_network_error, self, 0);
#line 409 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = open_album_trans;
#line 409 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp25_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "completed", (GCallback) _publishing_rajce_rajce_publisher_on_open_album_complete_publishing_rest_support_transaction_completed, self, 0);
#line 3735 "RajcePublishing.c"
		{
			PublishingRajceOpenAlbumTransaction* _tmp26_ = NULL;
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp26_ = open_album_trans;
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rest_support_transaction_execute (G_TYPE_CHECK_INSTANCE_CAST (_tmp26_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), &_inner_error_);
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3746 "RajcePublishing.c"
					goto __catch21_spit_publishing_publishing_error;
				}
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_publishing_rest_support_transaction_unref0 (open_album_trans);
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				g_clear_error (&_inner_error_);
#line 412 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				return;
#line 3757 "RajcePublishing.c"
			}
		}
		goto __finally21;
		__catch21_spit_publishing_publishing_error:
		{
			GError* err = NULL;
			GError* _tmp27_ = NULL;
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			err = _inner_error_;
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_inner_error_ = NULL;
#line 416 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_debug ("RajcePublishing.vala:416: ERROR: open album");
#line 417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp27_ = err;
#line 417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			publishing_rajce_rajce_publisher_do_show_error (self, _tmp27_);
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_error_free0 (err);
#line 3777 "RajcePublishing.c"
		}
		__finally21:
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_transaction_unref0 (open_album_trans);
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 410 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 3790 "RajcePublishing.c"
		}
#line 386 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_transaction_unref0 (open_album_trans);
#line 3794 "RajcePublishing.c"
	}
}


/**
     * Event triggered when the create album completes successfully.
     */
static gint* _int_dup (gint* self) {
	gint* dup;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	dup = g_new0 (gint, 1);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	memcpy (dup, self, sizeof (gint));
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return dup;
#line 3810 "RajcePublishing.c"
}


static gpointer __int_dup0 (gpointer self) {
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self ? _int_dup (self) : NULL;
#line 3817 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_on_create_album_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	GError * _inner_error_ = NULL;
#line 425 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 425 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (txn));
#line 427 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:427: EVENT: on_create_album_complete");
#line 428 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = txn;
#line 428 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 428 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_create_album_complete_publishing_rest_support_transaction_completed, self);
#line 429 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = txn;
#line 429 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 429 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_create_album_error_publishing_rest_support_transaction_network_error, self);
#line 430 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = txn;
#line 430 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rest_support_transaction_get_response (_tmp4_);
#line 430 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 430 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:430: RajcePlugin: create album: %s", _tmp6_);
#line 430 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp6_);
#line 3858 "RajcePublishing.c"
	{
		PublishingRESTSupportXmlDocument* doc = NULL;
		PublishingRESTSupportTransaction* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp10_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp11_ = NULL;
		xmlNode* response = NULL;
		PublishingRESTSupportXmlDocument* _tmp12_ = NULL;
		xmlNode* _tmp13_ = NULL;
		xmlNode* _tmp14_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp15_ = NULL;
		xmlNode* _tmp16_ = NULL;
		xmlNode* _tmp17_ = NULL;
		gchar* sessionToken = NULL;
		gchar* _tmp18_ = NULL;
		xmlNode* _tmp19_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp20_ = NULL;
		xmlNode* _tmp21_ = NULL;
		xmlNode* _tmp22_ = NULL;
		gchar* albumToken = NULL;
		gchar* _tmp23_ = NULL;
		xmlNode* _tmp24_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp25_ = NULL;
		xmlNode* _tmp26_ = NULL;
		xmlNode* _tmp27_ = NULL;
		PublishingRajcePublishingParameters* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gint _tmp31_ = 0;
		gint* _tmp32_ = NULL;
		PublishingRajceSession* _tmp33_ = NULL;
		const gchar* _tmp34_ = NULL;
		PublishingRajceSession* _tmp35_ = NULL;
		const gchar* _tmp36_ = NULL;
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = txn;
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = publishing_rest_support_transaction_get_response (_tmp7_);
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = _tmp8_;
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = publishing_rest_support_xml_document_parse_string (_tmp9_, _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response, NULL, &_inner_error_);
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = _tmp10_;
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp9_);
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		doc = _tmp11_;
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3912 "RajcePublishing.c"
				goto __catch22_spit_publishing_publishing_error;
			}
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 3921 "RajcePublishing.c"
		}
#line 434 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = doc;
#line 434 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = publishing_rest_support_xml_document_get_root_node (_tmp12_);
#line 434 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		response = _tmp13_;
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = doc;
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = response;
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = publishing_rest_support_xml_document_get_named_child (_tmp15_, _tmp16_, "sessionToken", &_inner_error_);
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = _tmp17_;
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3943 "RajcePublishing.c"
				goto __catch22_spit_publishing_publishing_error;
			}
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 3954 "RajcePublishing.c"
		}
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = (gchar*) xmlNodeGetContent (_tmp14_);
#line 435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		sessionToken = _tmp18_;
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = doc;
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = response;
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = publishing_rest_support_xml_document_get_named_child (_tmp20_, _tmp21_, "albumToken", &_inner_error_);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = _tmp22_;
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 3976 "RajcePublishing.c"
				goto __catch22_spit_publishing_publishing_error;
			}
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 3989 "RajcePublishing.c"
		}
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = (gchar*) xmlNodeGetContent (_tmp19_);
#line 436 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		albumToken = _tmp23_;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = doc;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = response;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = publishing_rest_support_xml_document_get_named_child (_tmp25_, _tmp26_, "albumID", &_inner_error_);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = _tmp27_;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (albumToken);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4013 "RajcePublishing.c"
				goto __catch22_spit_publishing_publishing_error;
			}
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (albumToken);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4028 "RajcePublishing.c"
		}
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_ = self->priv->parameters;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp29_ = (gchar*) xmlNodeGetContent (_tmp24_);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp30_ = _tmp29_;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp31_ = atoi (_tmp30_);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp32_ = __int_dup0 (&_tmp31_);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp28_->album_id);
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_->album_id = _tmp32_;
#line 437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp30_);
#line 438 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp33_ = self->priv->session;
#line 438 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp34_ = sessionToken;
#line 438 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_usertoken (_tmp33_, _tmp34_);
#line 439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp35_ = self->priv->session;
#line 439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp36_ = albumToken;
#line 439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_albumtoken (_tmp35_, _tmp36_);
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (albumToken);
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (sessionToken);
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_xml_document_unref0 (doc);
#line 4064 "RajcePublishing.c"
	}
	goto __finally22;
	__catch22_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp37_ = NULL;
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 443 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:443: ERROR: on_create_album_complete");
#line 444 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp37_ = err;
#line 444 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp37_);
#line 445 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 445 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 4085 "RajcePublishing.c"
	}
	__finally22:
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 431 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 4096 "RajcePublishing.c"
	}
#line 447 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_upload_photos (self);
#line 4100 "RajcePublishing.c"
}


/**
     * Event triggered when the create album transaction fails due to a network error.
     */
static void publishing_rajce_rajce_publisher_on_create_album_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	GError* _tmp5_ = NULL;
#line 453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 455 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:455: EVENT: on_create_album_error");
#line 456 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = bad_txn;
#line 456 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 456 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_create_album_complete_publishing_rest_support_transaction_completed, self);
#line 457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = bad_txn;
#line 457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_create_album_error_publishing_rest_support_transaction_network_error, self);
#line 458 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = bad_txn;
#line 458 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = err;
#line 458 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_network_error (self, _tmp4_, _tmp5_);
#line 4138 "RajcePublishing.c"
}


/**
     * Event triggered when the open album completes successfully.
     */
static void publishing_rajce_rajce_publisher_on_open_album_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	GError * _inner_error_ = NULL;
#line 464 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 464 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (txn));
#line 466 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:466: EVENT: on_open_album_complete");
#line 467 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = txn;
#line 467 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 467 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_open_album_complete_publishing_rest_support_transaction_completed, self);
#line 468 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = txn;
#line 468 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 468 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_open_album_error_publishing_rest_support_transaction_network_error, self);
#line 469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = txn;
#line 469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rest_support_transaction_get_response (_tmp4_);
#line 469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:469: RajcePlugin: open album: %s", _tmp6_);
#line 469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp6_);
#line 4182 "RajcePublishing.c"
	{
		PublishingRESTSupportXmlDocument* doc = NULL;
		PublishingRESTSupportTransaction* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp10_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp11_ = NULL;
		xmlNode* response = NULL;
		PublishingRESTSupportXmlDocument* _tmp12_ = NULL;
		xmlNode* _tmp13_ = NULL;
		xmlNode* _tmp14_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp15_ = NULL;
		xmlNode* _tmp16_ = NULL;
		xmlNode* _tmp17_ = NULL;
		gchar* sessionToken = NULL;
		gchar* _tmp18_ = NULL;
		xmlNode* _tmp19_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp20_ = NULL;
		xmlNode* _tmp21_ = NULL;
		xmlNode* _tmp22_ = NULL;
		gchar* albumToken = NULL;
		gchar* _tmp23_ = NULL;
		PublishingRajceSession* _tmp24_ = NULL;
		const gchar* _tmp25_ = NULL;
		PublishingRajceSession* _tmp26_ = NULL;
		const gchar* _tmp27_ = NULL;
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = txn;
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = publishing_rest_support_transaction_get_response (_tmp7_);
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = _tmp8_;
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = publishing_rest_support_xml_document_parse_string (_tmp9_, _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response, NULL, &_inner_error_);
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = _tmp10_;
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp9_);
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		doc = _tmp11_;
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4227 "RajcePublishing.c"
				goto __catch23_spit_publishing_publishing_error;
			}
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4236 "RajcePublishing.c"
		}
#line 473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = doc;
#line 473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = publishing_rest_support_xml_document_get_root_node (_tmp12_);
#line 473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		response = _tmp13_;
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = doc;
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = response;
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = publishing_rest_support_xml_document_get_named_child (_tmp15_, _tmp16_, "sessionToken", &_inner_error_);
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = _tmp17_;
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4258 "RajcePublishing.c"
				goto __catch23_spit_publishing_publishing_error;
			}
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4269 "RajcePublishing.c"
		}
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = (gchar*) xmlNodeGetContent (_tmp14_);
#line 474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		sessionToken = _tmp18_;
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = doc;
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = response;
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = publishing_rest_support_xml_document_get_named_child (_tmp20_, _tmp21_, "albumToken", &_inner_error_);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = _tmp22_;
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4291 "RajcePublishing.c"
				goto __catch23_spit_publishing_publishing_error;
			}
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4304 "RajcePublishing.c"
		}
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = (gchar*) xmlNodeGetContent (_tmp19_);
#line 475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		albumToken = _tmp23_;
#line 476 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = self->priv->session;
#line 476 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = sessionToken;
#line 476 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_usertoken (_tmp24_, _tmp25_);
#line 477 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = self->priv->session;
#line 477 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = albumToken;
#line 477 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_albumtoken (_tmp26_, _tmp27_);
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (albumToken);
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (sessionToken);
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_xml_document_unref0 (doc);
#line 4328 "RajcePublishing.c"
	}
	goto __finally23;
	__catch23_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp28_ = NULL;
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 481 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:481: ERROR: on_open_album_complete");
#line 482 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_ = err;
#line 482 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp28_);
#line 483 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 483 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 4349 "RajcePublishing.c"
	}
	__finally23:
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 470 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 4360 "RajcePublishing.c"
	}
#line 485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_upload_photos (self);
#line 4364 "RajcePublishing.c"
}


/**
     * Event triggered when the open album transaction fails due to a network error.
     */
static void publishing_rajce_rajce_publisher_on_open_album_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	GError* _tmp5_ = NULL;
#line 491 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 491 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 493 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:493: EVENT: on_open_album_error");
#line 494 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = bad_txn;
#line 494 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 494 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_open_album_complete_publishing_rest_support_transaction_completed, self);
#line 495 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = bad_txn;
#line 495 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 495 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_open_album_error_publishing_rest_support_transaction_network_error, self);
#line 496 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = bad_txn;
#line 496 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = err;
#line 496 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_network_error (self, _tmp4_, _tmp5_);
#line 4402 "RajcePublishing.c"
}


/**
     * Upload photos: the key part of the plugin
     */
static void _publishing_rajce_rajce_publisher_on_upload_photos_complete_publishing_rest_support_batch_uploader_upload_complete (PublishingRESTSupportBatchUploader* _sender, gint num_photos_published, gpointer self) {
#line 509 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_upload_photos_complete ((PublishingRajceRajcePublisher*) self, _sender, num_photos_published);
#line 4412 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_upload_photos_error_publishing_rest_support_batch_uploader_upload_error (PublishingRESTSupportBatchUploader* _sender, GError* err, gpointer self) {
#line 510 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_upload_photos_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 4419 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_upload_photos_status_updated_spit_publishing_progress_callback (gint file_number, gdouble fraction_complete, gpointer self) {
#line 511 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_upload_photos_status_updated ((PublishingRajceRajcePublisher*) self, file_number, fraction_complete);
#line 4426 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_upload_photos (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajceSession* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	void* _tmp3_ = NULL;
	GDestroyNotify _tmp4_ = NULL;
	SpitPublishingProgressCallback _tmp5_ = NULL;
	SpitPublishingPublishable** publishables = NULL;
	SpitPublishingPluginHost* _tmp6_ = NULL;
	gint _tmp7_ = 0;
	SpitPublishingPublishable** _tmp8_ = NULL;
	gint publishables_length1 = 0;
	gint _publishables_size_ = 0;
	PublishingRajceUploader* uploader = NULL;
	PublishingRajceSession* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	PublishingRajcePublishingParameters* _tmp12_ = NULL;
	PublishingRajceUploader* _tmp13_ = NULL;
	PublishingRajceUploader* _tmp14_ = NULL;
#line 502 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 504 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:504: ACTION: uploading photos");
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->session;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_session_get_maxsize (_tmp1_);
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = spit_publishing_plugin_host_serialize_publishables (_tmp0_, _tmp2_, FALSE, &_tmp3_, &_tmp4_);
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(self->priv->progress_reporter_target_destroy_notify == NULL) ? NULL : (self->priv->progress_reporter_target_destroy_notify (self->priv->progress_reporter_target), NULL);
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter = NULL;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter_target = NULL;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter_target_destroy_notify = NULL;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter = _tmp5_;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter_target = _tmp3_;
#line 505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter_target_destroy_notify = _tmp4_;
#line 506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = self->priv->host;
#line 506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = spit_publishing_plugin_host_get_publishables (_tmp6_, &_tmp7_);
#line 506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishables = _tmp8_;
#line 506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishables_length1 = _tmp7_;
#line 506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishables_size_ = publishables_length1;
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = self->priv->session;
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = publishing_rajce_rajce_publisher_get_url (self);
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = _tmp10_;
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = self->priv->parameters;
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13_ = publishing_rajce_uploader_new (_tmp9_, _tmp11_, publishables, publishables_length1, _tmp12_);
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp14_ = _tmp13_;
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp11_);
#line 508 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	uploader = _tmp14_;
#line 509 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (uploader, PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, PublishingRESTSupportBatchUploader), "upload-complete", (GCallback) _publishing_rajce_rajce_publisher_on_upload_photos_complete_publishing_rest_support_batch_uploader_upload_complete, self, 0);
#line 510 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (uploader, PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, PublishingRESTSupportBatchUploader), "upload-error", (GCallback) _publishing_rajce_rajce_publisher_on_upload_photos_error_publishing_rest_support_batch_uploader_upload_error, self, 0);
#line 511 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_batch_uploader_upload (G_TYPE_CHECK_INSTANCE_CAST (uploader, PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, PublishingRESTSupportBatchUploader), _publishing_rajce_rajce_publisher_on_upload_photos_status_updated_spit_publishing_progress_callback, self);
#line 502 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_batch_uploader_unref0 (uploader);
#line 502 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishables = (_vala_array_free (publishables, publishables_length1, (GDestroyNotify) g_object_unref), NULL);
#line 4512 "RajcePublishing.c"
}


/**
     * Event triggered when the batch uploader reports that at least one of the
     * network transactions encapsulating uploads has completed successfully
     */
static void publishing_rajce_rajce_publisher_on_upload_photos_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportBatchUploader* uploader, gint num_published) {
	PublishingRESTSupportBatchUploader* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportBatchUploader* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
#line 518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_BATCH_UPLOADER (uploader));
#line 520 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:520: EVENT: on_upload_photos_complete");
#line 521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = uploader;
#line 521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("upload-complete", PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, &_tmp1_, NULL, FALSE);
#line 521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_upload_photos_complete_publishing_rest_support_batch_uploader_upload_complete, self);
#line 522 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = uploader;
#line 522 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("upload-error", PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, &_tmp3_, NULL, FALSE);
#line 522 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_upload_photos_error_publishing_rest_support_batch_uploader_upload_error, self);
#line 525 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_end_upload (self);
#line 4545 "RajcePublishing.c"
}


/**
     * Event triggered when the batch uploader reports that at least one of the
     * network transactions encapsulating uploads has caused a network error
     */
static void publishing_rajce_rajce_publisher_on_upload_photos_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportBatchUploader* uploader, GError* err) {
	PublishingRESTSupportBatchUploader* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportBatchUploader* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	GError* _tmp4_ = NULL;
#line 532 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 532 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_BATCH_UPLOADER (uploader));
#line 534 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:534: EVENT: on_upload_photos_error");
#line 535 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = uploader;
#line 535 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("upload-complete", PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, &_tmp1_, NULL, FALSE);
#line 535 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_upload_photos_complete_publishing_rest_support_batch_uploader_upload_complete, self);
#line 536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = uploader;
#line 536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("upload-error", PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, &_tmp3_, NULL, FALSE);
#line 536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_upload_photos_error_publishing_rest_support_batch_uploader_upload_error, self);
#line 537 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = err;
#line 537 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_error (self, _tmp4_);
#line 4581 "RajcePublishing.c"
}


/**
     * Event triggered when upload progresses and the status needs to be updated.
     */
static void publishing_rajce_rajce_publisher_on_upload_photos_status_updated (PublishingRajceRajcePublisher* self, gint file_number, gdouble completed_fraction) {
	gboolean _tmp0_ = FALSE;
#line 543 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 545 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = spit_publishing_publisher_is_running (G_TYPE_CHECK_INSTANCE_CAST (self, SPIT_PUBLISHING_TYPE_PUBLISHER, SpitPublishingPublisher));
#line 545 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp0_) {
#line 4596 "RajcePublishing.c"
		gdouble _tmp1_ = 0.0;
		SpitPublishingProgressCallback _tmp2_ = NULL;
		void* _tmp2__target = NULL;
		SpitPublishingProgressCallback _tmp3_ = NULL;
		void* _tmp3__target = NULL;
		gint _tmp4_ = 0;
		gdouble _tmp5_ = 0.0;
#line 547 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp1_ = completed_fraction;
#line 547 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:547: EVENT: uploader reports upload %.2f percent " \
"complete.", 100.0 * _tmp1_);
#line 548 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = self->priv->progress_reporter;
#line 548 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2__target = self->priv->progress_reporter_target;
#line 548 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_vala_assert (_tmp2_ != NULL, "progress_reporter != null");
#line 549 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = self->priv->progress_reporter;
#line 549 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3__target = self->priv->progress_reporter_target;
#line 549 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = file_number;
#line 549 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = completed_fraction;
#line 549 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ (_tmp4_, _tmp5_, _tmp3__target);
#line 4624 "RajcePublishing.c"
	}
}


static void publishing_rajce_rajce_publisher_do_end_upload (PublishingRajceRajcePublisher* self) {
	gboolean _tmp0_ = FALSE;
#line 553 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 555 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = publishing_rajce_rajce_publisher_get_show_album (self);
#line 555 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp0_) {
#line 557 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_get_album_url (self);
#line 4639 "RajcePublishing.c"
	} else {
#line 561 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_close_album (self);
#line 4643 "RajcePublishing.c"
	}
}


/**
     * End upload action: get album url
     */
static void _publishing_rajce_rajce_publisher_on_get_album_url_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self) {
#line 573 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_get_album_url_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 4654 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_get_album_url_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self) {
#line 574 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_get_album_url_complete ((PublishingRajceRajcePublisher*) self, _sender);
#line 4661 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_get_album_url (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajceGetAlbumUrlTransaction* get_album_url_trans = NULL;
	PublishingRajceSession* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	PublishingRajceGetAlbumUrlTransaction* _tmp4_ = NULL;
	PublishingRajceGetAlbumUrlTransaction* _tmp5_ = NULL;
	GError * _inner_error_ = NULL;
#line 568 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 570 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:570: ACTION: getting album URL");
#line 571 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 571 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, TRUE);
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->session;
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_rajce_publisher_get_url (self);
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _tmp2_;
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_get_album_url_transaction_new (_tmp1_, _tmp3_);
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp3_);
#line 572 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	get_album_url_trans = _tmp5_;
#line 573 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (get_album_url_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "network-error", (GCallback) _publishing_rajce_rajce_publisher_on_get_album_url_error_publishing_rest_support_transaction_network_error, self, 0);
#line 574 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (get_album_url_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "completed", (GCallback) _publishing_rajce_rajce_publisher_on_get_album_url_complete_publishing_rest_support_transaction_completed, self, 0);
#line 4700 "RajcePublishing.c"
	{
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rest_support_transaction_execute (G_TYPE_CHECK_INSTANCE_CAST (get_album_url_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), &_inner_error_);
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4708 "RajcePublishing.c"
				goto __catch24_spit_publishing_publishing_error;
			}
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_transaction_unref0 (get_album_url_trans);
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 577 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4719 "RajcePublishing.c"
		}
	}
	goto __finally24;
	__catch24_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp6_ = NULL;
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 581 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:581: ERROR: close album");
#line 582 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = err;
#line 582 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp6_);
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 4739 "RajcePublishing.c"
	}
	__finally24:
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_transaction_unref0 (get_album_url_trans);
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 575 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 4752 "RajcePublishing.c"
	}
#line 568 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_transaction_unref0 (get_album_url_trans);
#line 4756 "RajcePublishing.c"
}


/**
     * Event triggered when the get album url completes successfully.
     */
static void publishing_rajce_rajce_publisher_on_get_album_url_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	GError * _inner_error_ = NULL;
#line 589 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 589 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (txn));
#line 591 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:591: EVENT: on_get_album_url_complete");
#line 592 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = txn;
#line 592 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 592 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_get_album_url_complete_publishing_rest_support_transaction_completed, self);
#line 593 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = txn;
#line 593 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 593 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_get_album_url_error_publishing_rest_support_transaction_network_error, self);
#line 594 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = txn;
#line 594 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rest_support_transaction_get_response (_tmp4_);
#line 594 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 594 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:594: RajcePlugin: get album url: %s", _tmp6_);
#line 594 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp6_);
#line 4800 "RajcePublishing.c"
	{
		PublishingRESTSupportXmlDocument* doc = NULL;
		PublishingRESTSupportTransaction* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp10_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp11_ = NULL;
		xmlNode* response = NULL;
		PublishingRESTSupportXmlDocument* _tmp12_ = NULL;
		xmlNode* _tmp13_ = NULL;
		xmlNode* _tmp14_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp15_ = NULL;
		xmlNode* _tmp16_ = NULL;
		xmlNode* _tmp17_ = NULL;
		gchar* sessionToken = NULL;
		gchar* _tmp18_ = NULL;
		xmlNode* _tmp19_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp20_ = NULL;
		xmlNode* _tmp21_ = NULL;
		xmlNode* _tmp22_ = NULL;
		gchar* url = NULL;
		gchar* _tmp23_ = NULL;
		PublishingRajceSession* _tmp24_ = NULL;
		const gchar* _tmp25_ = NULL;
		PublishingRajceSession* _tmp26_ = NULL;
		const gchar* _tmp27_ = NULL;
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = txn;
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = publishing_rest_support_transaction_get_response (_tmp7_);
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = _tmp8_;
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = publishing_rest_support_xml_document_parse_string (_tmp9_, _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response, NULL, &_inner_error_);
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = _tmp10_;
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp9_);
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		doc = _tmp11_;
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4845 "RajcePublishing.c"
				goto __catch25_spit_publishing_publishing_error;
			}
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 597 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4854 "RajcePublishing.c"
		}
#line 598 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = doc;
#line 598 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = publishing_rest_support_xml_document_get_root_node (_tmp12_);
#line 598 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		response = _tmp13_;
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = doc;
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = response;
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = publishing_rest_support_xml_document_get_named_child (_tmp15_, _tmp16_, "sessionToken", &_inner_error_);
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = _tmp17_;
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4876 "RajcePublishing.c"
				goto __catch25_spit_publishing_publishing_error;
			}
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4887 "RajcePublishing.c"
		}
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = (gchar*) xmlNodeGetContent (_tmp14_);
#line 599 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		sessionToken = _tmp18_;
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = doc;
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = response;
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = publishing_rest_support_xml_document_get_named_child (_tmp20_, _tmp21_, "url", &_inner_error_);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = _tmp22_;
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 4909 "RajcePublishing.c"
				goto __catch25_spit_publishing_publishing_error;
			}
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (sessionToken);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 4922 "RajcePublishing.c"
		}
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = (gchar*) xmlNodeGetContent (_tmp19_);
#line 600 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		url = _tmp23_;
#line 601 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = self->priv->session;
#line 601 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = sessionToken;
#line 601 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_usertoken (_tmp24_, _tmp25_);
#line 602 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = self->priv->session;
#line 602 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = url;
#line 602 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_albumticket (_tmp26_, _tmp27_);
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (url);
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (sessionToken);
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_xml_document_unref0 (doc);
#line 4946 "RajcePublishing.c"
	}
	goto __finally25;
	__catch25_spit_publishing_publishing_error:
	{
		GError* err = NULL;
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 606 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:606: ERROR: on_get_album_url_complete");
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 4960 "RajcePublishing.c"
	}
	__finally25:
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 595 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 4971 "RajcePublishing.c"
	}
#line 611 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_close_album (self);
#line 4975 "RajcePublishing.c"
}


/**
     * Event triggered when the get album url transaction fails due to a network error.
     */
static void publishing_rajce_rajce_publisher_on_get_album_url_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
#line 617 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 617 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 619 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:619: EVENT: on_get_album_url_error");
#line 620 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = bad_txn;
#line 620 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 620 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_get_album_url_complete_publishing_rest_support_transaction_completed, self);
#line 621 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = bad_txn;
#line 621 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 621 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_get_album_url_error_publishing_rest_support_transaction_network_error, self);
#line 624 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_close_album (self);
#line 5007 "RajcePublishing.c"
}


/**
     * End upload action: close album
     */
static void _publishing_rajce_rajce_publisher_on_close_album_error_publishing_rest_support_transaction_network_error (PublishingRESTSupportTransaction* _sender, GError* err, gpointer self) {
#line 636 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_close_album_error ((PublishingRajceRajcePublisher*) self, _sender, err);
#line 5017 "RajcePublishing.c"
}


static void _publishing_rajce_rajce_publisher_on_close_album_complete_publishing_rest_support_transaction_completed (PublishingRESTSupportTransaction* _sender, gpointer self) {
#line 637 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_on_close_album_complete ((PublishingRajceRajcePublisher*) self, _sender);
#line 5024 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_do_close_album (PublishingRajceRajcePublisher* self) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	PublishingRajceCloseAlbumTransaction* close_album_trans = NULL;
	PublishingRajceSession* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	PublishingRajceCloseAlbumTransaction* _tmp4_ = NULL;
	PublishingRajceCloseAlbumTransaction* _tmp5_ = NULL;
	GError * _inner_error_ = NULL;
#line 631 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 633 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:633: ACTION: closing album");
#line 634 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 634 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp0_, TRUE);
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->session;
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_rajce_publisher_get_url (self);
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _tmp2_;
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_close_album_transaction_new (_tmp1_, _tmp3_);
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp3_);
#line 635 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	close_album_trans = _tmp5_;
#line 636 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (close_album_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "network-error", (GCallback) _publishing_rajce_rajce_publisher_on_close_album_error_publishing_rest_support_transaction_network_error, self, 0);
#line 637 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (close_album_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "completed", (GCallback) _publishing_rajce_rajce_publisher_on_close_album_complete_publishing_rest_support_transaction_completed, self, 0);
#line 5063 "RajcePublishing.c"
	{
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rest_support_transaction_execute (G_TYPE_CHECK_INSTANCE_CAST (close_album_trans, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), &_inner_error_);
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 5071 "RajcePublishing.c"
				goto __catch26_spit_publishing_publishing_error;
			}
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_transaction_unref0 (close_album_trans);
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 640 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 5082 "RajcePublishing.c"
		}
	}
	goto __finally26;
	__catch26_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp6_ = NULL;
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 644 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:644: ERROR: close album");
#line 645 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = err;
#line 645 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp6_);
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 5102 "RajcePublishing.c"
	}
	__finally26:
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_transaction_unref0 (close_album_trans);
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 638 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 5115 "RajcePublishing.c"
	}
#line 631 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_transaction_unref0 (close_album_trans);
#line 5119 "RajcePublishing.c"
}


/**
     * Event triggered when the close album completes successfully.
     */
static void publishing_rajce_rajce_publisher_on_close_album_complete (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* txn) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	PublishingRESTSupportTransaction* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	GError * _inner_error_ = NULL;
#line 652 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 652 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (txn));
#line 654 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:654: EVENT: on_close_album_complete");
#line 655 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = txn;
#line 655 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 655 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_close_album_complete_publishing_rest_support_transaction_completed, self);
#line 656 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = txn;
#line 656 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 656 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_close_album_error_publishing_rest_support_transaction_network_error, self);
#line 657 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = txn;
#line 657 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rest_support_transaction_get_response (_tmp4_);
#line 657 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 657 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:657: RajcePlugin: close album: %s", _tmp6_);
#line 657 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp6_);
#line 5163 "RajcePublishing.c"
	{
		PublishingRESTSupportXmlDocument* doc = NULL;
		PublishingRESTSupportTransaction* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp10_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp11_ = NULL;
		xmlNode* response = NULL;
		PublishingRESTSupportXmlDocument* _tmp12_ = NULL;
		xmlNode* _tmp13_ = NULL;
		xmlNode* _tmp14_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp15_ = NULL;
		xmlNode* _tmp16_ = NULL;
		xmlNode* _tmp17_ = NULL;
		gchar* sessionToken = NULL;
		gchar* _tmp18_ = NULL;
		PublishingRajceSession* _tmp19_ = NULL;
		const gchar* _tmp20_ = NULL;
		PublishingRajceSession* _tmp21_ = NULL;
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = txn;
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = publishing_rest_support_transaction_get_response (_tmp7_);
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = _tmp8_;
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = publishing_rest_support_xml_document_parse_string (_tmp9_, _publishing_rajce_transaction_validate_xml_publishing_rest_support_xml_document_check_for_error_response, NULL, &_inner_error_);
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = _tmp10_;
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp9_);
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		doc = _tmp11_;
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 5201 "RajcePublishing.c"
				goto __catch27_spit_publishing_publishing_error;
			}
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 660 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 5210 "RajcePublishing.c"
		}
#line 661 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = doc;
#line 661 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = publishing_rest_support_xml_document_get_root_node (_tmp12_);
#line 661 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		response = _tmp13_;
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = doc;
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = response;
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = publishing_rest_support_xml_document_get_named_child (_tmp15_, _tmp16_, "sessionToken", &_inner_error_);
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = _tmp17_;
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 5232 "RajcePublishing.c"
				goto __catch27_spit_publishing_publishing_error;
			}
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_publishing_rest_support_xml_document_unref0 (doc);
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 5243 "RajcePublishing.c"
		}
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = (gchar*) xmlNodeGetContent (_tmp14_);
#line 662 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		sessionToken = _tmp18_;
#line 663 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = self->priv->session;
#line 663 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = sessionToken;
#line 663 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_usertoken (_tmp19_, _tmp20_);
#line 664 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = self->priv->session;
#line 664 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_session_set_albumtoken (_tmp21_, NULL);
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (sessionToken);
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rest_support_xml_document_unref0 (doc);
#line 5263 "RajcePublishing.c"
	}
	goto __finally27;
	__catch27_spit_publishing_publishing_error:
	{
		GError* err = NULL;
		GError* _tmp22_ = NULL;
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 668 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_debug ("RajcePublishing.vala:668: ERROR: on_close_album_complete");
#line 669 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = err;
#line 669 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_error (self, _tmp22_);
#line 670 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 670 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 5284 "RajcePublishing.c"
	}
	__finally27:
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 658 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 5295 "RajcePublishing.c"
	}
#line 672 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_success_pane (self);
#line 5299 "RajcePublishing.c"
}


/**
     * Event triggered when the close album transaction fails due to a network error.
     */
static void publishing_rajce_rajce_publisher_on_close_album_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	PublishingRESTSupportTransaction* _tmp0_ = NULL;
	guint _tmp1_ = 0U;
	PublishingRESTSupportTransaction* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
#line 678 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 678 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 680 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:680: EVENT: on_close_album_error");
#line 681 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = bad_txn;
#line 681 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("completed", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp1_, NULL, FALSE);
#line 681 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_close_album_complete_publishing_rest_support_transaction_completed, self);
#line 682 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = bad_txn;
#line 682 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("network-error", PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, &_tmp3_, NULL, FALSE);
#line 682 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _publishing_rajce_rajce_publisher_on_close_album_error_publishing_rest_support_transaction_network_error, self);
#line 685 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_success_pane (self);
#line 5331 "RajcePublishing.c"
}


/**
     * Action to display the success pane in the publishing dialog.
     */
static void publishing_rajce_rajce_publisher_do_show_success_pane (PublishingRajceRajcePublisher* self) {
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	SpitPublishingPluginHost* _tmp10_ = NULL;
	SpitPublishingPluginHost* _tmp11_ = NULL;
	GError * _inner_error_ = NULL;
#line 692 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 694 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:694: ACTION: installing success pane");
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = publishing_rajce_rajce_publisher_get_show_album (self);
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp1_) {
#line 5352 "RajcePublishing.c"
		PublishingRajceSession* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = self->priv->session;
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = publishing_rajce_session_get_albumticket (_tmp2_);
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = _tmp3_;
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = _tmp4_ != NULL;
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp4_);
#line 5366 "RajcePublishing.c"
	} else {
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = FALSE;
#line 5370 "RajcePublishing.c"
	}
#line 695 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp0_) {
#line 5374 "RajcePublishing.c"
		{
			PublishingRajceSession* _tmp5_ = NULL;
			gchar* _tmp6_ = NULL;
			gchar* _tmp7_ = NULL;
			gchar* _tmp8_ = NULL;
			gchar* _tmp9_ = NULL;
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp5_ = self->priv->session;
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp6_ = publishing_rajce_session_get_albumticket (_tmp5_);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp7_ = _tmp6_;
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp8_ = g_strconcat ("xdg-open ", _tmp7_, NULL);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp9_ = _tmp8_;
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_spawn_command_line_async (_tmp9_, &_inner_error_);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (_tmp9_);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (_tmp7_);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				if (_inner_error_->domain == G_SPAWN_ERROR) {
#line 5401 "RajcePublishing.c"
					goto __catch28_g_spawn_error;
				}
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				g_clear_error (&_inner_error_);
#line 699 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				return;
#line 5410 "RajcePublishing.c"
			}
		}
		goto __finally28;
		__catch28_g_spawn_error:
		{
			GError* e = NULL;
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			e = _inner_error_;
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_inner_error_ = NULL;
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_error_free0 (e);
#line 5423 "RajcePublishing.c"
		}
		__finally28:
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 697 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return;
#line 5434 "RajcePublishing.c"
		}
	}
#line 705 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = self->priv->host;
#line 705 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_set_service_locked (_tmp10_, FALSE);
#line 706 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = self->priv->host;
#line 706 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_install_success_pane (_tmp11_);
#line 5445 "RajcePublishing.c"
}


/**
     * Helper event to handle network errors.
     */
static void publishing_rajce_rajce_publisher_on_network_error (PublishingRajceRajcePublisher* self, PublishingRESTSupportTransaction* bad_txn, GError* err) {
	GError* _tmp0_ = NULL;
#line 712 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 712 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_REST_SUPPORT_IS_TRANSACTION (bad_txn));
#line 714 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:714: EVENT: on_network_error");
#line 715 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = err;
#line 715 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_error (self, _tmp0_);
#line 5464 "RajcePublishing.c"
}


/**
     * Action to display an error to the user.
     */
static void publishing_rajce_rajce_publisher_do_show_error (PublishingRajceRajcePublisher* self, GError* e) {
	gchar* error_type = NULL;
	gchar* _tmp0_ = NULL;
	GError* _tmp1_ = NULL;
	const gchar* _tmp14_ = NULL;
	GError* _tmp15_ = NULL;
	const gchar* _tmp16_ = NULL;
	gchar* _tmp17_ = NULL;
	gchar* _tmp18_ = NULL;
	const gchar* _tmp19_ = NULL;
#line 721 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 723 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:723: ACTION: do_show_error");
#line 724 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = g_strdup ("UNKNOWN");
#line 724 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	error_type = _tmp0_;
#line 725 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = e;
#line 725 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (g_error_matches (_tmp1_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_NO_ANSWER)) {
#line 727 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_do_show_authentication_pane (self, PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER);
#line 728 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (error_type);
#line 728 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return;
#line 5499 "RajcePublishing.c"
	} else {
		GError* _tmp2_ = NULL;
#line 729 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = e;
#line 729 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (g_error_matches (_tmp2_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_COMMUNICATION_FAILED)) {
#line 5506 "RajcePublishing.c"
			gchar* _tmp3_ = NULL;
#line 730 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp3_ = g_strdup ("COMMUNICATION_FAILED");
#line 730 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (error_type);
#line 730 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			error_type = _tmp3_;
#line 5514 "RajcePublishing.c"
		} else {
			GError* _tmp4_ = NULL;
#line 731 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp4_ = e;
#line 731 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (g_error_matches (_tmp4_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_PROTOCOL_ERROR)) {
#line 5521 "RajcePublishing.c"
				gchar* _tmp5_ = NULL;
#line 732 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp5_ = g_strdup ("PROTOCOL_ERROR");
#line 732 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_g_free0 (error_type);
#line 732 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				error_type = _tmp5_;
#line 5529 "RajcePublishing.c"
			} else {
				GError* _tmp6_ = NULL;
#line 733 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp6_ = e;
#line 733 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				if (g_error_matches (_tmp6_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_SERVICE_ERROR)) {
#line 5536 "RajcePublishing.c"
					gchar* _tmp7_ = NULL;
#line 734 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp7_ = g_strdup ("SERVICE_ERROR");
#line 734 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_g_free0 (error_type);
#line 734 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					error_type = _tmp7_;
#line 5544 "RajcePublishing.c"
				} else {
					GError* _tmp8_ = NULL;
#line 735 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp8_ = e;
#line 735 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					if (g_error_matches (_tmp8_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_MALFORMED_RESPONSE)) {
#line 5551 "RajcePublishing.c"
						gchar* _tmp9_ = NULL;
#line 736 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp9_ = g_strdup ("MALFORMED_RESPONSE");
#line 736 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_g_free0 (error_type);
#line 736 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						error_type = _tmp9_;
#line 5559 "RajcePublishing.c"
					} else {
						GError* _tmp10_ = NULL;
#line 737 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						_tmp10_ = e;
#line 737 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
						if (g_error_matches (_tmp10_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_LOCAL_FILE_ERROR)) {
#line 5566 "RajcePublishing.c"
							gchar* _tmp11_ = NULL;
#line 738 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
							_tmp11_ = g_strdup ("LOCAL_FILE_ERROR");
#line 738 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
							_g_free0 (error_type);
#line 738 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
							error_type = _tmp11_;
#line 5574 "RajcePublishing.c"
						} else {
							GError* _tmp12_ = NULL;
#line 739 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
							_tmp12_ = e;
#line 739 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
							if (g_error_matches (_tmp12_, SPIT_PUBLISHING_PUBLISHING_ERROR, SPIT_PUBLISHING_PUBLISHING_ERROR_EXPIRED_SESSION)) {
#line 5581 "RajcePublishing.c"
								gchar* _tmp13_ = NULL;
#line 740 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
								_tmp13_ = g_strdup ("EXPIRED_SESSION");
#line 740 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
								_g_free0 (error_type);
#line 740 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
								error_type = _tmp13_;
#line 5589 "RajcePublishing.c"
							}
						}
					}
				}
			}
		}
	}
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp14_ = error_type;
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp15_ = e;
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp16_ = _tmp15_->message;
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp17_ = g_strdup_printf ("Unhandled error: type=%s; message='%s'", _tmp14_, _tmp16_);
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp18_ = _tmp17_;
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:743: %s", _tmp18_);
#line 743 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp18_);
#line 744 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp19_ = _ ("An error message occurred when publishing to Rajce. Please try again.");
#line 744 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_do_show_error_message (self, _tmp19_);
#line 721 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (error_type);
#line 5617 "RajcePublishing.c"
}


/**
     * Action to display an error message to the user.
     */
static void publishing_rajce_rajce_publisher_do_show_error_message (PublishingRajceRajcePublisher* self, const gchar* message) {
	SpitPublishingPluginHost* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 750 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (self));
#line 750 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (message != NULL);
#line 752 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:752: ACTION: do_show_error_message");
#line 753 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->host;
#line 753 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = message;
#line 753 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spit_publishing_plugin_host_install_static_message_pane (_tmp0_, _tmp1_, SPIT_PUBLISHING_PLUGIN_HOST_BUTTON_MODE_CLOSE);
#line 5639 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_class_init (PublishingRajceRajcePublisherClass * klass) {
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_parent_class = g_type_class_peek_parent (klass);
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajceRajcePublisherPrivate));
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (klass)->finalize = publishing_rajce_rajce_publisher_finalize;
#line 5650 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_spit_publishing_publisher_interface_init (SpitPublishingPublisherIface * iface) {
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_spit_publishing_publisher_parent_iface = g_type_interface_peek_parent (iface);
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_service = (SpitPublishingService* (*)(SpitPublishingPublisher*)) publishing_rajce_rajce_publisher_real_get_service;
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->is_running = (gboolean (*)(SpitPublishingPublisher*)) publishing_rajce_rajce_publisher_real_is_running;
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->start = (void (*)(SpitPublishingPublisher*)) publishing_rajce_rajce_publisher_real_start;
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->stop = (void (*)(SpitPublishingPublisher*)) publishing_rajce_rajce_publisher_real_stop;
#line 5665 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_instance_init (PublishingRajceRajcePublisher * self) {
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_RAJCE_PUBLISHER_GET_PRIVATE (self);
#line 68 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->host = NULL;
#line 69 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter = NULL;
#line 70 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->service = NULL;
#line 71 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->running = FALSE;
#line 80 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums = NULL;
#line 80 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums_length1 = 0;
#line 80 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_albums_size_ = self->priv->albums_length1;
#line 81 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->parameters = NULL;
#line 82 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->media_type = SPIT_PUBLISHING_PUBLISHER_MEDIA_TYPE_NONE;
#line 5690 "RajcePublishing.c"
}


static void publishing_rajce_rajce_publisher_finalize (GObject* obj) {
	PublishingRajceRajcePublisher * self;
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_RAJCE_PUBLISHER, PublishingRajceRajcePublisher);
#line 68 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->host);
#line 69 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(self->priv->progress_reporter_target_destroy_notify == NULL) ? NULL : (self->priv->progress_reporter_target_destroy_notify (self->priv->progress_reporter_target), NULL);
#line 69 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter = NULL;
#line 69 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter_target = NULL;
#line 69 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->progress_reporter_target_destroy_notify = NULL;
#line 70 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->service);
#line 72 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_session_unref0 (self->priv->session);
#line 80 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums = (_vala_array_free (self->priv->albums, self->priv->albums_length1, (GDestroyNotify) publishing_rajce_album_unref), NULL);
#line 81 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_publishing_parameters_unref0 (self->priv->parameters);
#line 66 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (publishing_rajce_rajce_publisher_parent_class)->finalize (obj);
#line 5718 "RajcePublishing.c"
}


GType publishing_rajce_rajce_publisher_get_type (void) {
	static volatile gsize publishing_rajce_rajce_publisher_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_rajce_publisher_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceRajcePublisherClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_rajce_publisher_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceRajcePublisher), 0, (GInstanceInitFunc) publishing_rajce_rajce_publisher_instance_init, NULL };
		static const GInterfaceInfo spit_publishing_publisher_info = { (GInterfaceInitFunc) publishing_rajce_rajce_publisher_spit_publishing_publisher_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType publishing_rajce_rajce_publisher_type_id;
		publishing_rajce_rajce_publisher_type_id = g_type_register_static (G_TYPE_OBJECT, "PublishingRajceRajcePublisher", &g_define_type_info, 0);
		g_type_add_interface_static (publishing_rajce_rajce_publisher_type_id, SPIT_PUBLISHING_TYPE_PUBLISHER, &spit_publishing_publisher_info);
		g_once_init_leave (&publishing_rajce_rajce_publisher_type_id__volatile, publishing_rajce_rajce_publisher_type_id);
	}
	return publishing_rajce_rajce_publisher_type_id__volatile;
}


PublishingRajceAlbum* publishing_rajce_album_construct (GType object_type, gint id, const gchar* albumName, const gchar* url, const gchar* thumbUrl, const gchar* createDate, const gchar* updateDate, gboolean hidden, gboolean secure, gint photoCount) {
	PublishingRajceAlbum* self = NULL;
	gint _tmp0_ = 0;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	const gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gboolean _tmp11_ = FALSE;
	gboolean _tmp12_ = FALSE;
	gint _tmp13_ = 0;
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (albumName != NULL, NULL);
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (thumbUrl != NULL, NULL);
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (createDate != NULL, NULL);
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (updateDate != NULL, NULL);
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceAlbum*) g_type_create_instance (object_type);
#line 773 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = id;
#line 773 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->id = _tmp0_;
#line 774 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = albumName;
#line 774 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 774 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->albumName);
#line 774 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->albumName = _tmp2_;
#line 775 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = url;
#line 775 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = g_strdup (_tmp3_);
#line 775 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->url);
#line 775 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->url = _tmp4_;
#line 776 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = thumbUrl;
#line 776 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = g_strdup (_tmp5_);
#line 776 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->thumbUrl);
#line 776 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->thumbUrl = _tmp6_;
#line 777 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = createDate;
#line 777 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = g_strdup (_tmp7_);
#line 777 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->createDate);
#line 777 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->createDate = _tmp8_;
#line 778 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = updateDate;
#line 778 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = g_strdup (_tmp9_);
#line 778 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->updateDate);
#line 778 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->updateDate = _tmp10_;
#line 779 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = hidden;
#line 779 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->hidden = _tmp11_;
#line 780 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = secure;
#line 780 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->secure = _tmp12_;
#line 781 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13_ = photoCount;
#line 781 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->photoCount = _tmp13_;
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 5822 "RajcePublishing.c"
}


PublishingRajceAlbum* publishing_rajce_album_new (gint id, const gchar* albumName, const gchar* url, const gchar* thumbUrl, const gchar* createDate, const gchar* updateDate, gboolean hidden, gboolean secure, gint photoCount) {
#line 771 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_album_construct (PUBLISHING_RAJCE_TYPE_ALBUM, id, albumName, url, thumbUrl, createDate, updateDate, hidden, secure, photoCount);
#line 5829 "RajcePublishing.c"
}


gint publishing_rajce_album_compare_albums (PublishingRajceAlbum* a, PublishingRajceAlbum* b) {
	gint result = 0;
	gboolean _tmp0_ = FALSE;
	PublishingRajceAlbum* _tmp1_ = NULL;
	PublishingRajceAlbum* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	PublishingRajceAlbum* _tmp11_ = NULL;
	const gchar* _tmp12_ = NULL;
	gint _tmp13_ = 0;
#line 783 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail ((a == NULL) || PUBLISHING_RAJCE_IS_ALBUM (a), 0);
#line 783 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail ((b == NULL) || PUBLISHING_RAJCE_IS_ALBUM (b), 0);
#line 785 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = a;
#line 785 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp1_ == NULL) {
#line 5850 "RajcePublishing.c"
		PublishingRajceAlbum* _tmp2_ = NULL;
#line 785 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = b;
#line 785 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = _tmp2_ == NULL;
#line 5856 "RajcePublishing.c"
	} else {
#line 785 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = FALSE;
#line 5860 "RajcePublishing.c"
	}
#line 785 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp0_) {
#line 787 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result = 0;
#line 787 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return result;
#line 5868 "RajcePublishing.c"
	} else {
		gboolean _tmp3_ = FALSE;
		PublishingRajceAlbum* _tmp4_ = NULL;
#line 789 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = a;
#line 789 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp4_ == NULL) {
#line 5876 "RajcePublishing.c"
			PublishingRajceAlbum* _tmp5_ = NULL;
#line 789 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp5_ = b;
#line 789 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp3_ = _tmp5_ != NULL;
#line 5882 "RajcePublishing.c"
		} else {
#line 789 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp3_ = FALSE;
#line 5886 "RajcePublishing.c"
		}
#line 789 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp3_) {
#line 791 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			result = 1;
#line 791 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return result;
#line 5894 "RajcePublishing.c"
		} else {
			gboolean _tmp6_ = FALSE;
			PublishingRajceAlbum* _tmp7_ = NULL;
#line 793 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp7_ = a;
#line 793 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_tmp7_ != NULL) {
#line 5902 "RajcePublishing.c"
				PublishingRajceAlbum* _tmp8_ = NULL;
#line 793 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp8_ = b;
#line 793 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp6_ = _tmp8_ == NULL;
#line 5908 "RajcePublishing.c"
			} else {
#line 793 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp6_ = FALSE;
#line 5912 "RajcePublishing.c"
			}
#line 793 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_tmp6_) {
#line 795 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				result = -1;
#line 795 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				return result;
#line 5920 "RajcePublishing.c"
			}
		}
	}
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = b;
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = _tmp9_->updateDate;
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = a;
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = _tmp11_->updateDate;
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13_ = g_ascii_strcasecmp (_tmp10_, _tmp12_);
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp13_;
#line 797 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 5938 "RajcePublishing.c"
}


static void publishing_rajce_value_album_init (GValue* value) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	value->data[0].v_pointer = NULL;
#line 5945 "RajcePublishing.c"
}


static void publishing_rajce_value_album_free_value (GValue* value) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (value->data[0].v_pointer) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_album_unref (value->data[0].v_pointer);
#line 5954 "RajcePublishing.c"
	}
}


static void publishing_rajce_value_album_copy_value (const GValue* src_value, GValue* dest_value) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (src_value->data[0].v_pointer) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = publishing_rajce_album_ref (src_value->data[0].v_pointer);
#line 5964 "RajcePublishing.c"
	} else {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = NULL;
#line 5968 "RajcePublishing.c"
	}
}


static gpointer publishing_rajce_value_album_peek_pointer (const GValue* value) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 5976 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_album_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (collect_values[0].v_pointer) {
#line 5983 "RajcePublishing.c"
		PublishingRajceAlbum* object;
		object = collect_values[0].v_pointer;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (object->parent_instance.g_class == NULL) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 5990 "RajcePublishing.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 5994 "RajcePublishing.c"
		}
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = publishing_rajce_album_ref (object);
#line 5998 "RajcePublishing.c"
	} else {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 6002 "RajcePublishing.c"
	}
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 6006 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_album_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	PublishingRajceAlbum** object_p;
	object_p = collect_values[0].v_pointer;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!object_p) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 6017 "RajcePublishing.c"
	}
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!value->data[0].v_pointer) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = NULL;
#line 6023 "RajcePublishing.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = value->data[0].v_pointer;
#line 6027 "RajcePublishing.c"
	} else {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = publishing_rajce_album_ref (value->data[0].v_pointer);
#line 6031 "RajcePublishing.c"
	}
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 6035 "RajcePublishing.c"
}


GParamSpec* publishing_rajce_param_spec_album (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	PublishingRajceParamSpecAlbum* spec;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (g_type_is_a (object_type, PUBLISHING_RAJCE_TYPE_ALBUM), NULL);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return G_PARAM_SPEC (spec);
#line 6049 "RajcePublishing.c"
}


gpointer publishing_rajce_value_get_album (const GValue* value) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_ALBUM), NULL);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 6058 "RajcePublishing.c"
}


void publishing_rajce_value_set_album (GValue* value, gpointer v_object) {
	PublishingRajceAlbum* old;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_ALBUM));
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_ALBUM));
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_album_ref (value->data[0].v_pointer);
#line 6078 "RajcePublishing.c"
	} else {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 6082 "RajcePublishing.c"
	}
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_album_unref (old);
#line 6088 "RajcePublishing.c"
	}
}


void publishing_rajce_value_take_album (GValue* value, gpointer v_object) {
	PublishingRajceAlbum* old;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_ALBUM));
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_ALBUM));
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 6107 "RajcePublishing.c"
	} else {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 6111 "RajcePublishing.c"
	}
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_album_unref (old);
#line 6117 "RajcePublishing.c"
	}
}


static void publishing_rajce_album_class_init (PublishingRajceAlbumClass * klass) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_album_parent_class = g_type_class_peek_parent (klass);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRajceAlbumClass *) klass)->finalize = publishing_rajce_album_finalize;
#line 6127 "RajcePublishing.c"
}


static void publishing_rajce_album_instance_init (PublishingRajceAlbum * self) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->ref_count = 1;
#line 6134 "RajcePublishing.c"
}


static void publishing_rajce_album_finalize (PublishingRajceAlbum* obj) {
	PublishingRajceAlbum * self;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_ALBUM, PublishingRajceAlbum);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_destroy (self);
#line 762 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->albumName);
#line 763 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->url);
#line 764 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->thumbUrl);
#line 765 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->createDate);
#line 766 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->updateDate);
#line 6154 "RajcePublishing.c"
}


GType publishing_rajce_album_get_type (void) {
	static volatile gsize publishing_rajce_album_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_album_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { publishing_rajce_value_album_init, publishing_rajce_value_album_free_value, publishing_rajce_value_album_copy_value, publishing_rajce_value_album_peek_pointer, "p", publishing_rajce_value_album_collect_value, "p", publishing_rajce_value_album_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceAlbumClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_album_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceAlbum), 0, (GInstanceInitFunc) publishing_rajce_album_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType publishing_rajce_album_type_id;
		publishing_rajce_album_type_id = g_type_register_fundamental (g_type_fundamental_next (), "PublishingRajceAlbum", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&publishing_rajce_album_type_id__volatile, publishing_rajce_album_type_id);
	}
	return publishing_rajce_album_type_id__volatile;
}


gpointer publishing_rajce_album_ref (gpointer instance) {
	PublishingRajceAlbum* self;
	self = instance;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_atomic_int_inc (&self->ref_count);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return instance;
#line 6179 "RajcePublishing.c"
}


void publishing_rajce_album_unref (gpointer instance) {
	PublishingRajceAlbum* self;
	self = instance;
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		PUBLISHING_RAJCE_ALBUM_GET_CLASS (self)->finalize (self);
#line 759 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 6192 "RajcePublishing.c"
	}
}


PublishingRajceUploader* publishing_rajce_uploader_construct (GType object_type, PublishingRajceSession* session, const gchar* url, SpitPublishingPublishable** publishables, int publishables_length1, PublishingRajcePublishingParameters* parameters) {
	PublishingRajceUploader* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	SpitPublishingPublishable** _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
	PublishingRajcePublishingParameters* _tmp2_ = NULL;
	PublishingRajcePublishingParameters* _tmp3_ = NULL;
	const gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
#line 807 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 807 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 807 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_PARAMETERS (parameters), NULL);
#line 809 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 809 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = publishables;
#line 809 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1__length1 = publishables_length1;
#line 809 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceUploader*) publishing_rest_support_batch_uploader_construct (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, _tmp1__length1);
#line 810 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = parameters;
#line 810 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _publishing_rajce_publishing_parameters_ref0 (_tmp2_);
#line 810 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_publishing_parameters_unref0 (self->priv->parameters);
#line 810 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->parameters = _tmp3_;
#line 811 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = url;
#line 811 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = g_strdup (_tmp4_);
#line 811 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->url);
#line 811 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->url = _tmp5_;
#line 807 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 6238 "RajcePublishing.c"
}


PublishingRajceUploader* publishing_rajce_uploader_new (PublishingRajceSession* session, const gchar* url, SpitPublishingPublishable** publishables, int publishables_length1, PublishingRajcePublishingParameters* parameters) {
#line 807 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_uploader_construct (PUBLISHING_RAJCE_TYPE_UPLOADER, session, url, publishables, publishables_length1, parameters);
#line 6245 "RajcePublishing.c"
}


static PublishingRESTSupportTransaction* publishing_rajce_uploader_real_create_transaction (PublishingRESTSupportBatchUploader* base, SpitPublishingPublishable* publishable) {
	PublishingRajceUploader * self;
	PublishingRESTSupportTransaction* result = NULL;
	PublishingRESTSupportSession* _tmp0_ = NULL;
	PublishingRajceSession* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	PublishingRajcePublishingParameters* _tmp3_ = NULL;
	SpitPublishingPublishable* _tmp4_ = NULL;
	PublishingRajceAddPhotoTransaction* _tmp5_ = NULL;
	PublishingRESTSupportTransaction* _tmp6_ = NULL;
#line 814 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_UPLOADER, PublishingRajceUploader);
#line 814 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (SPIT_PUBLISHING_IS_PUBLISHABLE (publishable), NULL);
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = publishing_rest_support_batch_uploader_get_session (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, PublishingRESTSupportBatchUploader));
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSession);
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->url;
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = self->priv->parameters;
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishable;
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = publishing_rajce_add_photo_transaction_new (_tmp1_, _tmp2_, _tmp3_, _tmp4_);
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp5_, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction);
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rest_support_session_unref0 (_tmp1_);
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp6_;
#line 816 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 6283 "RajcePublishing.c"
}


static void publishing_rajce_uploader_class_init (PublishingRajceUploaderClass * klass) {
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_uploader_parent_class = g_type_class_peek_parent (klass);
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRESTSupportBatchUploaderClass *) klass)->finalize = publishing_rajce_uploader_finalize;
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajceUploaderPrivate));
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRESTSupportBatchUploaderClass *) klass)->create_transaction = publishing_rajce_uploader_real_create_transaction;
#line 6296 "RajcePublishing.c"
}


static void publishing_rajce_uploader_instance_init (PublishingRajceUploader * self) {
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_UPLOADER_GET_PRIVATE (self);
#line 6303 "RajcePublishing.c"
}


static void publishing_rajce_uploader_finalize (PublishingRESTSupportBatchUploader* obj) {
	PublishingRajceUploader * self;
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_UPLOADER, PublishingRajceUploader);
#line 804 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_publishing_parameters_unref0 (self->priv->parameters);
#line 805 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->url);
#line 802 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	PUBLISHING_REST_SUPPORT_BATCH_UPLOADER_CLASS (publishing_rajce_uploader_parent_class)->finalize (obj);
#line 6317 "RajcePublishing.c"
}


GType publishing_rajce_uploader_get_type (void) {
	static volatile gsize publishing_rajce_uploader_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_uploader_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceUploaderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_uploader_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceUploader), 0, (GInstanceInitFunc) publishing_rajce_uploader_instance_init, NULL };
		GType publishing_rajce_uploader_type_id;
		publishing_rajce_uploader_type_id = g_type_register_static (PUBLISHING_REST_SUPPORT_TYPE_BATCH_UPLOADER, "PublishingRajceUploader", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_uploader_type_id__volatile, publishing_rajce_uploader_type_id);
	}
	return publishing_rajce_uploader_type_id__volatile;
}


GType publishing_rajce_authentication_pane_mode_get_type (void) {
	static volatile gsize publishing_rajce_authentication_pane_mode_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_authentication_pane_mode_type_id__volatile)) {
		static const GEnumValue values[] = {{PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_INTRO, "PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_INTRO", "intro"}, {PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER, "PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER", "failed-retry-user"}, {0, NULL, NULL}};
		GType publishing_rajce_authentication_pane_mode_type_id;
		publishing_rajce_authentication_pane_mode_type_id = g_enum_register_static ("PublishingRajceAuthenticationPaneMode", values);
		g_once_init_leave (&publishing_rajce_authentication_pane_mode_type_id__volatile, publishing_rajce_authentication_pane_mode_type_id);
	}
	return publishing_rajce_authentication_pane_mode_type_id__volatile;
}


static void _publishing_rajce_authentication_pane_on_user_changed_gtk_editable_changed (GtkEditable* _sender, gpointer self) {
#line 897 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_on_user_changed ((PublishingRajceAuthenticationPane*) self);
#line 6348 "RajcePublishing.c"
}


static void _publishing_rajce_authentication_pane_on_password_changed_gtk_editable_changed (GtkEditable* _sender, gpointer self) {
#line 898 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_on_password_changed ((PublishingRajceAuthenticationPane*) self);
#line 6355 "RajcePublishing.c"
}


static void _publishing_rajce_authentication_pane_on_login_button_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 899 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_on_login_button_clicked ((PublishingRajceAuthenticationPane*) self);
#line 6362 "RajcePublishing.c"
}


PublishingRajceAuthenticationPane* publishing_rajce_authentication_pane_construct (GType object_type, PublishingRajceRajcePublisher* publisher, PublishingRajceAuthenticationPaneMode mode) {
	PublishingRajceAuthenticationPane * self = NULL;
	GtkBox* _tmp0_ = NULL;
	GError * _inner_error_ = NULL;
#line 846 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (publisher), NULL);
#line 846 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceAuthenticationPane*) g_object_new (object_type, NULL);
#line 848 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
#line 848 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_object_ref_sink (_tmp0_);
#line 848 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->pane_widget);
#line 848 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->pane_widget = _tmp0_;
#line 6382 "RajcePublishing.c"
	{
		GtkBuilder* _tmp1_ = NULL;
		GtkBuilder* _tmp2_ = NULL;
		GtkBuilder* _tmp3_ = NULL;
		GtkAlignment* align = NULL;
		GtkBuilder* _tmp4_ = NULL;
		GObject* _tmp5_ = NULL;
		GtkAlignment* _tmp6_ = NULL;
		GtkLabel* message_label = NULL;
		GtkBuilder* _tmp7_ = NULL;
		GObject* _tmp8_ = NULL;
		GtkLabel* _tmp9_ = NULL;
		PublishingRajceAuthenticationPaneMode _tmp10_ = 0;
		GtkBuilder* _tmp18_ = NULL;
		GObject* _tmp19_ = NULL;
		GtkEntry* _tmp20_ = NULL;
		gchar* persistent_username = NULL;
		PublishingRajceRajcePublisher* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		const gchar* _tmp23_ = NULL;
		GtkBuilder* _tmp26_ = NULL;
		GObject* _tmp27_ = NULL;
		GtkEntry* _tmp28_ = NULL;
		gchar* persistent_token = NULL;
		PublishingRajceRajcePublisher* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		const gchar* _tmp31_ = NULL;
		GtkBuilder* _tmp34_ = NULL;
		GObject* _tmp35_ = NULL;
		GtkCheckButton* _tmp36_ = NULL;
		GtkCheckButton* _tmp37_ = NULL;
		PublishingRajceRajcePublisher* _tmp38_ = NULL;
		gboolean _tmp39_ = FALSE;
		GtkBuilder* _tmp40_ = NULL;
		GObject* _tmp41_ = NULL;
		GtkButton* _tmp42_ = NULL;
		GtkLabel* label2 = NULL;
		GtkBuilder* _tmp43_ = NULL;
		GObject* _tmp44_ = NULL;
		GtkLabel* _tmp45_ = NULL;
		GtkLabel* label3 = NULL;
		GtkBuilder* _tmp46_ = NULL;
		GObject* _tmp47_ = NULL;
		GtkLabel* _tmp48_ = NULL;
		GtkLabel* _tmp49_ = NULL;
		const gchar* _tmp50_ = NULL;
		GtkLabel* _tmp51_ = NULL;
		const gchar* _tmp52_ = NULL;
		GtkCheckButton* _tmp53_ = NULL;
		const gchar* _tmp54_ = NULL;
		GtkButton* _tmp55_ = NULL;
		const gchar* _tmp56_ = NULL;
		GtkEntry* _tmp57_ = NULL;
		GtkEntry* _tmp58_ = NULL;
		GtkButton* _tmp59_ = NULL;
		GtkAlignment* _tmp60_ = NULL;
		GtkBox* _tmp61_ = NULL;
		PublishingRajceRajcePublisher* _tmp62_ = NULL;
		SpitPublishingPluginHost* _tmp63_ = NULL;
		SpitPublishingPluginHost* _tmp64_ = NULL;
		GtkButton* _tmp65_ = NULL;
#line 851 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp1_ = gtk_builder_new ();
#line 851 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->builder);
#line 851 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->builder = _tmp1_;
#line 852 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = self->priv->builder;
#line 852 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_builder_add_from_resource (_tmp2_, PLUGIN_RESOURCE_PATH "/rajce_authentication_pane.ui", &_inner_error_);
#line 852 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 6456 "RajcePublishing.c"
			goto __catch29_g_error;
		}
#line 854 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = self->priv->builder;
#line 854 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_builder_connect_signals (_tmp3_, NULL);
#line 855 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = self->priv->builder;
#line 855 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = gtk_builder_get_object (_tmp4_, "alignment");
#line 855 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, gtk_alignment_get_type ()) ? ((GtkAlignment*) _tmp5_) : NULL);
#line 855 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		align = _tmp6_;
#line 856 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = self->priv->builder;
#line 856 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = gtk_builder_get_object (_tmp7_, "message_label");
#line 856 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, gtk_label_get_type ()) ? ((GtkLabel*) _tmp8_) : NULL);
#line 856 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		message_label = _tmp9_;
#line 857 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = mode;
#line 857 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		switch (_tmp10_) {
#line 857 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			case PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_INTRO:
#line 6485 "RajcePublishing.c"
			{
				GtkLabel* _tmp11_ = NULL;
				const gchar* _tmp12_ = NULL;
#line 860 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp11_ = message_label;
#line 860 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp12_ = publishing_rajce_authentication_pane_INTRO_MESSAGE;
#line 860 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				gtk_label_set_text (_tmp11_, _tmp12_);
#line 861 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				break;
#line 6497 "RajcePublishing.c"
			}
#line 857 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			case PUBLISHING_RAJCE_AUTHENTICATION_PANE_MODE_FAILED_RETRY_USER:
#line 6501 "RajcePublishing.c"
			{
				GtkLabel* _tmp13_ = NULL;
				const gchar* _tmp14_ = NULL;
				const gchar* _tmp15_ = NULL;
				gchar* _tmp16_ = NULL;
				gchar* _tmp17_ = NULL;
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp13_ = message_label;
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp14_ = _ ("Invalid User Email or Password");
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp15_ = publishing_rajce_authentication_pane_FAILED_RETRY_USER_MESSAGE;
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp16_ = g_strdup_printf ("<b>%s</b>\n\n%s", _tmp14_, _tmp15_);
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp17_ = _tmp16_;
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				gtk_label_set_markup (_tmp13_, _tmp17_);
#line 864 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_g_free0 (_tmp17_);
#line 866 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				break;
#line 6524 "RajcePublishing.c"
			}
			default:
#line 857 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			break;
#line 6529 "RajcePublishing.c"
		}
#line 868 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = self->priv->builder;
#line 868 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = gtk_builder_get_object (_tmp18_, "username_entry");
#line 868 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp19_, gtk_entry_get_type ()) ? ((GtkEntry*) _tmp19_) : NULL);
#line 868 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->username_entry);
#line 868 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->username_entry = _tmp20_;
#line 869 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = publisher;
#line 869 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = publishing_rajce_rajce_publisher_get_username (_tmp21_);
#line 869 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		persistent_username = _tmp22_;
#line 870 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = persistent_username;
#line 870 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp23_ != NULL) {
#line 6551 "RajcePublishing.c"
			GtkEntry* _tmp24_ = NULL;
			const gchar* _tmp25_ = NULL;
#line 872 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp24_ = self->priv->username_entry;
#line 872 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp25_ = persistent_username;
#line 872 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			gtk_entry_set_text (_tmp24_, _tmp25_);
#line 6560 "RajcePublishing.c"
		}
#line 874 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = self->priv->builder;
#line 874 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = gtk_builder_get_object (_tmp26_, "password_entry");
#line 874 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp27_, gtk_entry_get_type ()) ? ((GtkEntry*) _tmp27_) : NULL);
#line 874 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->password_entry);
#line 874 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->password_entry = _tmp28_;
#line 875 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp29_ = publisher;
#line 875 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp30_ = publishing_rajce_rajce_publisher_get_token (_tmp29_);
#line 875 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		persistent_token = _tmp30_;
#line 876 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp31_ = persistent_token;
#line 876 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (_tmp31_ != NULL) {
#line 6582 "RajcePublishing.c"
			GtkEntry* _tmp32_ = NULL;
			const gchar* _tmp33_ = NULL;
#line 878 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp32_ = self->priv->password_entry;
#line 878 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp33_ = persistent_token;
#line 878 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			gtk_entry_set_text (_tmp32_, _tmp33_);
#line 879 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			self->priv->crypt = FALSE;
#line 6593 "RajcePublishing.c"
		} else {
#line 883 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			self->priv->crypt = TRUE;
#line 6597 "RajcePublishing.c"
		}
#line 885 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp34_ = self->priv->builder;
#line 885 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp35_ = gtk_builder_get_object (_tmp34_, "remember_checkbutton");
#line 885 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp36_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp35_, gtk_check_button_get_type ()) ? ((GtkCheckButton*) _tmp35_) : NULL);
#line 885 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->remember_checkbutton);
#line 885 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->remember_checkbutton = _tmp36_;
#line 886 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp37_ = self->priv->remember_checkbutton;
#line 886 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp38_ = publisher;
#line 886 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp39_ = publishing_rajce_rajce_publisher_get_remember (_tmp38_);
#line 886 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_toggle_button_set_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp37_, gtk_toggle_button_get_type (), GtkToggleButton), _tmp39_);
#line 887 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp40_ = self->priv->builder;
#line 887 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp41_ = gtk_builder_get_object (_tmp40_, "login_button");
#line 887 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp42_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp41_, gtk_button_get_type ()) ? ((GtkButton*) _tmp41_) : NULL);
#line 887 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->login_button);
#line 887 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->login_button = _tmp42_;
#line 889 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp43_ = self->priv->builder;
#line 889 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp44_ = gtk_builder_get_object (_tmp43_, "label2");
#line 889 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp45_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp44_, gtk_label_get_type ()) ? ((GtkLabel*) _tmp44_) : NULL);
#line 889 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		label2 = _tmp45_;
#line 890 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp46_ = self->priv->builder;
#line 890 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp47_ = gtk_builder_get_object (_tmp46_, "label3");
#line 890 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp48_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp47_, gtk_label_get_type ()) ? ((GtkLabel*) _tmp47_) : NULL);
#line 890 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		label3 = _tmp48_;
#line 892 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp49_ = label2;
#line 892 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp50_ = _ ("_Email address");
#line 892 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_label_set_label (_tmp49_, _tmp50_);
#line 893 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp51_ = label3;
#line 893 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp52_ = _ ("_Password");
#line 893 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_label_set_label (_tmp51_, _tmp52_);
#line 894 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp53_ = self->priv->remember_checkbutton;
#line 894 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp54_ = _ ("_Remember");
#line 894 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (G_TYPE_CHECK_INSTANCE_CAST (_tmp53_, gtk_button_get_type (), GtkButton), _tmp54_);
#line 895 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp55_ = self->priv->login_button;
#line 895 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp56_ = _ ("Login");
#line 895 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (_tmp55_, _tmp56_);
#line 897 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp57_ = self->priv->username_entry;
#line 897 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp57_, GTK_TYPE_EDITABLE, GtkEditable), "changed", (GCallback) _publishing_rajce_authentication_pane_on_user_changed_gtk_editable_changed, self, 0);
#line 898 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp58_ = self->priv->password_entry;
#line 898 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp58_, GTK_TYPE_EDITABLE, GtkEditable), "changed", (GCallback) _publishing_rajce_authentication_pane_on_password_changed_gtk_editable_changed, self, 0);
#line 899 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp59_ = self->priv->login_button;
#line 899 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (_tmp59_, "clicked", (GCallback) _publishing_rajce_authentication_pane_on_login_button_clicked_gtk_button_clicked, self, 0);
#line 900 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp60_ = align;
#line 900 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp61_ = self->priv->pane_widget;
#line 900 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_widget_reparent (G_TYPE_CHECK_INSTANCE_CAST (_tmp60_, gtk_widget_get_type (), GtkWidget), G_TYPE_CHECK_INSTANCE_CAST (_tmp61_, gtk_widget_get_type (), GtkWidget));
#line 901 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp62_ = publisher;
#line 901 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp63_ = publishing_rajce_rajce_publisher_get_host (_tmp62_);
#line 901 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp64_ = _tmp63_;
#line 901 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp65_ = self->priv->login_button;
#line 901 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		spit_publishing_plugin_host_set_dialog_default_widget (_tmp64_, G_TYPE_CHECK_INSTANCE_CAST (_tmp65_, gtk_widget_get_type (), GtkWidget));
#line 901 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (_tmp64_);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (label3);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (label2);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (persistent_token);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (persistent_username);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (message_label);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (align);
#line 6709 "RajcePublishing.c"
	}
	goto __finally29;
	__catch29_g_error:
	{
		GError* e = NULL;
		GError* _tmp66_ = NULL;
		const gchar* _tmp67_ = NULL;
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		e = _inner_error_;
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 905 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp66_ = e;
#line 905 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp67_ = _tmp66_->message;
#line 905 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_warning ("RajcePublishing.vala:905: Could not load UI: %s", _tmp67_);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (e);
#line 6729 "RajcePublishing.c"
	}
	__finally29:
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 849 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return NULL;
#line 6740 "RajcePublishing.c"
	}
#line 846 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 6744 "RajcePublishing.c"
}


PublishingRajceAuthenticationPane* publishing_rajce_authentication_pane_new (PublishingRajceRajcePublisher* publisher, PublishingRajceAuthenticationPaneMode mode) {
#line 846 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_authentication_pane_construct (PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, publisher, mode);
#line 6751 "RajcePublishing.c"
}


GtkWidget* publishing_rajce_authentication_pane_get_default_widget (PublishingRajceAuthenticationPane* self) {
	GtkWidget* result = NULL;
	GtkButton* _tmp0_ = NULL;
	GtkWidget* _tmp1_ = NULL;
#line 909 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE (self), NULL);
#line 911 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->login_button;
#line 911 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 911 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 911 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 6769 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_on_login_button_clicked (PublishingRajceAuthenticationPane* self) {
	gchar* token = NULL;
	GtkEntry* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
	GtkEntry* _tmp6_ = NULL;
	const gchar* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	GtkCheckButton* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
#line 914 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE (self));
#line 916 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->password_entry;
#line 916 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = gtk_entry_get_text (_tmp0_);
#line 916 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 916 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	token = _tmp2_;
#line 917 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = self->priv->crypt;
#line 917 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp3_) {
#line 6798 "RajcePublishing.c"
		const gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
#line 919 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = token;
#line 919 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = g_compute_checksum_for_string (G_CHECKSUM_MD5, _tmp4_, (gsize) -1);
#line 919 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (token);
#line 919 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		token = _tmp5_;
#line 6809 "RajcePublishing.c"
	}
#line 921 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = self->priv->username_entry;
#line 921 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = gtk_entry_get_text (_tmp6_);
#line 921 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = token;
#line 921 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = self->priv->remember_checkbutton;
#line 921 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = gtk_toggle_button_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, gtk_toggle_button_get_type (), GtkToggleButton));
#line 921 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_emit_by_name (self, "login", _tmp7_, _tmp8_, _tmp10_);
#line 914 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (token);
#line 6825 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_on_user_changed (PublishingRajceAuthenticationPane* self) {
#line 924 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE (self));
#line 926 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_update_login_button_sensitivity (self);
#line 6834 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_on_password_changed (PublishingRajceAuthenticationPane* self) {
#line 929 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE (self));
#line 931 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->crypt = TRUE;
#line 932 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_update_login_button_sensitivity (self);
#line 6845 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_update_login_button_sensitivity (PublishingRajceAuthenticationPane* self) {
	gboolean _tmp0_ = FALSE;
	GtkEntry* _tmp1_ = NULL;
	guint _tmp2_ = 0U;
	guint _tmp3_ = 0U;
	GtkButton* _tmp7_ = NULL;
#line 935 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_AUTHENTICATION_PANE (self));
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->username_entry;
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = gtk_entry_get_text_length (_tmp1_);
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _tmp2_;
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp3_ > ((guint) 0)) {
#line 6865 "RajcePublishing.c"
		GtkEntry* _tmp4_ = NULL;
		guint _tmp5_ = 0U;
		guint _tmp6_ = 0U;
#line 938 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = self->priv->password_entry;
#line 938 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = gtk_entry_get_text_length (_tmp4_);
#line 938 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = _tmp5_;
#line 938 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = _tmp6_ > ((guint) 0);
#line 6877 "RajcePublishing.c"
	} else {
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = FALSE;
#line 6881 "RajcePublishing.c"
	}
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = self->priv->login_button;
#line 937 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp7_, gtk_widget_get_type (), GtkWidget), _tmp0_);
#line 6887 "RajcePublishing.c"
}


static GtkWidget* publishing_rajce_authentication_pane_real_get_widget (SpitPublishingDialogPane* base) {
	PublishingRajceAuthenticationPane * self;
	GtkWidget* result = NULL;
	GtkBox* _tmp0_ = NULL;
	GtkWidget* _tmp1_ = NULL;
#line 941 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPane);
#line 943 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->pane_widget;
#line 943 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 943 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 943 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 6906 "RajcePublishing.c"
}


static SpitPublishingDialogPaneGeometryOptions publishing_rajce_authentication_pane_real_get_preferred_geometry (SpitPublishingDialogPane* base) {
	PublishingRajceAuthenticationPane * self;
	SpitPublishingDialogPaneGeometryOptions result = 0;
#line 946 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPane);
#line 948 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = SPIT_PUBLISHING_DIALOG_PANE_GEOMETRY_OPTIONS_NONE;
#line 948 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 6919 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_real_on_pane_installed (SpitPublishingDialogPane* base) {
	PublishingRajceAuthenticationPane * self;
	GtkEntry* _tmp0_ = NULL;
	GtkEntry* _tmp1_ = NULL;
	GtkButton* _tmp2_ = NULL;
#line 951 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPane);
#line 953 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->username_entry;
#line 953 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_grab_focus (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 954 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->password_entry;
#line 954 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_entry_set_activates_default (_tmp1_, TRUE);
#line 955 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->login_button;
#line 955 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_can_default (G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, gtk_widget_get_type (), GtkWidget), TRUE);
#line 956 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_update_login_button_sensitivity (self);
#line 6944 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_real_on_pane_uninstalled (SpitPublishingDialogPane* base) {
	PublishingRajceAuthenticationPane * self;
#line 958 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPane);
#line 6952 "RajcePublishing.c"
}


static void g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN) (gpointer data1, const char* arg_1, const char* arg_2, gboolean arg_3, gpointer data2);
	register GMarshalFunc_VOID__STRING_STRING_BOOLEAN callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (n_param_values == 4);
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data1 = closure->data;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data2 = param_values->data[0].v_pointer;
#line 6971 "RajcePublishing.c"
	} else {
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data1 = param_values->data[0].v_pointer;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data2 = closure->data;
#line 6977 "RajcePublishing.c"
	}
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	callback = (GMarshalFunc_VOID__STRING_STRING_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	callback (data1, g_value_get_string (param_values + 1), g_value_get_string (param_values + 2), g_value_get_boolean (param_values + 3), data2);
#line 6983 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_class_init (PublishingRajceAuthenticationPaneClass * klass) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_parent_class = g_type_class_peek_parent (klass);
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajceAuthenticationPanePrivate));
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (klass)->finalize = publishing_rajce_authentication_pane_finalize;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_new ("login", PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN, G_TYPE_NONE, 3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
#line 833 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = _ ("Enter email and password associated with your Rajce account.");
#line 833 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 833 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_INTRO_MESSAGE = _tmp1_;
#line 834 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = _ ("Invalid email and/or password. Please try again");
#line 834 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 834 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_FAILED_RETRY_USER_MESSAGE = _tmp3_;
#line 7012 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_spit_publishing_dialog_pane_interface_init (SpitPublishingDialogPaneIface * iface) {
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_authentication_pane_spit_publishing_dialog_pane_parent_iface = g_type_interface_peek_parent (iface);
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_widget = (GtkWidget* (*)(SpitPublishingDialogPane*)) publishing_rajce_authentication_pane_real_get_widget;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_preferred_geometry = (SpitPublishingDialogPaneGeometryOptions (*)(SpitPublishingDialogPane*)) publishing_rajce_authentication_pane_real_get_preferred_geometry;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->on_pane_installed = (void (*)(SpitPublishingDialogPane*)) publishing_rajce_authentication_pane_real_on_pane_installed;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->on_pane_uninstalled = (void (*)(SpitPublishingDialogPane*)) publishing_rajce_authentication_pane_real_on_pane_uninstalled;
#line 7027 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_instance_init (PublishingRajceAuthenticationPane * self) {
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_AUTHENTICATION_PANE_GET_PRIVATE (self);
#line 836 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->pane_widget = NULL;
#line 842 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->crypt = TRUE;
#line 7038 "RajcePublishing.c"
}


static void publishing_rajce_authentication_pane_finalize (GObject* obj) {
	PublishingRajceAuthenticationPane * self;
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_AUTHENTICATION_PANE, PublishingRajceAuthenticationPane);
#line 836 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->pane_widget);
#line 837 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->builder);
#line 838 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->username_entry);
#line 839 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->password_entry);
#line 840 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->remember_checkbutton);
#line 841 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->login_button);
#line 826 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (publishing_rajce_authentication_pane_parent_class)->finalize (obj);
#line 7060 "RajcePublishing.c"
}


/**
 * The authentication pane used when asking service URL, user name and password
 * from the user.
 */
GType publishing_rajce_authentication_pane_get_type (void) {
	static volatile gsize publishing_rajce_authentication_pane_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_authentication_pane_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceAuthenticationPaneClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_authentication_pane_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceAuthenticationPane), 0, (GInstanceInitFunc) publishing_rajce_authentication_pane_instance_init, NULL };
		static const GInterfaceInfo spit_publishing_dialog_pane_info = { (GInterfaceInitFunc) publishing_rajce_authentication_pane_spit_publishing_dialog_pane_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType publishing_rajce_authentication_pane_type_id;
		publishing_rajce_authentication_pane_type_id = g_type_register_static (G_TYPE_OBJECT, "PublishingRajceAuthenticationPane", &g_define_type_info, 0);
		g_type_add_interface_static (publishing_rajce_authentication_pane_type_id, SPIT_PUBLISHING_TYPE_DIALOG_PANE, &spit_publishing_dialog_pane_info);
		g_once_init_leave (&publishing_rajce_authentication_pane_type_id__volatile, publishing_rajce_authentication_pane_type_id);
	}
	return publishing_rajce_authentication_pane_type_id__volatile;
}


static gpointer _publishing_rajce_album_ref0 (gpointer self) {
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self ? publishing_rajce_album_ref (self) : NULL;
#line 7085 "RajcePublishing.c"
}


static PublishingRajceAlbum** _vala_array_dup5 (PublishingRajceAlbum** self, int length) {
	PublishingRajceAlbum** result;
	int i;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = g_new0 (PublishingRajceAlbum*, length + 1);
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	for (i = 0; i < length; i++) {
#line 7096 "RajcePublishing.c"
		PublishingRajceAlbum* _tmp0_ = NULL;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = _publishing_rajce_album_ref0 (self[i]);
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result[i] = _tmp0_;
#line 7102 "RajcePublishing.c"
	}
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 7106 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_on_use_existing_radio_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1020 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_on_use_existing_radio_clicked ((PublishingRajcePublishingOptionsPane*) self);
#line 7113 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_on_create_new_radio_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1021 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_on_create_new_radio_clicked ((PublishingRajcePublishingOptionsPane*) self);
#line 7120 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_on_new_album_entry_changed_gtk_editable_changed (GtkEditable* _sender, gpointer self) {
#line 1022 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_on_new_album_entry_changed ((PublishingRajcePublishingOptionsPane*) self);
#line 7127 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_on_logout_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1023 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_on_logout_clicked ((PublishingRajcePublishingOptionsPane*) self);
#line 7134 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_on_publish_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1024 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_on_publish_clicked ((PublishingRajcePublishingOptionsPane*) self);
#line 7141 "RajcePublishing.c"
}


PublishingRajcePublishingOptionsPane* publishing_rajce_publishing_options_pane_construct (GType object_type, PublishingRajceRajcePublisher* publisher, const gchar* username, PublishingRajceAlbum** albums, int albums_length1) {
	PublishingRajcePublishingOptionsPane * self = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	PublishingRajceAlbum** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	PublishingRajceAlbum** _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	PublishingRajceRajcePublisher* _tmp4_ = NULL;
	PublishingRajceRajcePublisher* _tmp5_ = NULL;
	GtkBox* _tmp6_ = NULL;
	GError * _inner_error_ = NULL;
#line 984 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_RAJCE_PUBLISHER (publisher), NULL);
#line 984 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (username != NULL, NULL);
#line 984 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajcePublishingOptionsPane*) g_object_new (object_type, NULL);
#line 986 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = username;
#line 986 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 986 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->username);
#line 986 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->username = _tmp1_;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = albums;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2__length1 = albums_length1;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = (_tmp2_ != NULL) ? _vala_array_dup5 (_tmp2_, _tmp2__length1) : ((gpointer) _tmp2_);
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3__length1 = _tmp2__length1;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums = (_vala_array_free (self->priv->albums, self->priv->albums_length1, (GDestroyNotify) publishing_rajce_album_unref), NULL);
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums = _tmp3_;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums_length1 = _tmp3__length1;
#line 987 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_albums_size_ = self->priv->albums_length1;
#line 988 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publisher;
#line 988 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _g_object_ref0 (_tmp4_);
#line 988 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->publisher);
#line 988 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->publisher = _tmp5_;
#line 989 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
#line 989 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_object_ref_sink (_tmp6_);
#line 989 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->pane_widget);
#line 989 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->pane_widget = _tmp6_;
#line 7203 "RajcePublishing.c"
	{
		GtkBuilder* _tmp7_ = NULL;
		GtkBuilder* _tmp8_ = NULL;
		GtkBuilder* _tmp9_ = NULL;
		GtkBuilder* _tmp10_ = NULL;
		GObject* _tmp11_ = NULL;
		GtkBox* _tmp12_ = NULL;
		GtkBuilder* _tmp13_ = NULL;
		GObject* _tmp14_ = NULL;
		GtkLabel* _tmp15_ = NULL;
		GtkBuilder* _tmp16_ = NULL;
		GObject* _tmp17_ = NULL;
		GtkLabel* _tmp18_ = NULL;
		GtkBuilder* _tmp19_ = NULL;
		GObject* _tmp20_ = NULL;
		GtkRadioButton* _tmp21_ = NULL;
		GtkBuilder* _tmp22_ = NULL;
		GObject* _tmp23_ = NULL;
		GtkComboBoxText* _tmp24_ = NULL;
		GtkBuilder* _tmp25_ = NULL;
		GObject* _tmp26_ = NULL;
		GtkRadioButton* _tmp27_ = NULL;
		GtkBuilder* _tmp28_ = NULL;
		GObject* _tmp29_ = NULL;
		GtkEntry* _tmp30_ = NULL;
		GtkBuilder* _tmp31_ = NULL;
		GObject* _tmp32_ = NULL;
		GtkCheckButton* _tmp33_ = NULL;
		GtkCheckButton* _tmp34_ = NULL;
		const gchar* _tmp35_ = NULL;
		GtkBuilder* _tmp36_ = NULL;
		GObject* _tmp37_ = NULL;
		GtkCheckButton* _tmp38_ = NULL;
		GtkBuilder* _tmp39_ = NULL;
		GObject* _tmp40_ = NULL;
		GtkButton* _tmp41_ = NULL;
		GtkBuilder* _tmp42_ = NULL;
		GObject* _tmp43_ = NULL;
		GtkButton* _tmp44_ = NULL;
		GtkCheckButton* _tmp45_ = NULL;
		PublishingRajceRajcePublisher* _tmp46_ = NULL;
		gboolean _tmp47_ = FALSE;
		GtkCheckButton* _tmp48_ = NULL;
		PublishingRajceRajcePublisher* _tmp49_ = NULL;
		gboolean _tmp50_ = FALSE;
		GtkLabel* _tmp51_ = NULL;
		const gchar* _tmp52_ = NULL;
		const gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
		GtkLabel* _tmp56_ = NULL;
		const gchar* _tmp57_ = NULL;
		GtkRadioButton* _tmp58_ = NULL;
		const gchar* _tmp59_ = NULL;
		GtkRadioButton* _tmp60_ = NULL;
		const gchar* _tmp61_ = NULL;
		GtkCheckButton* _tmp62_ = NULL;
		const gchar* _tmp63_ = NULL;
		GtkButton* _tmp64_ = NULL;
		const gchar* _tmp65_ = NULL;
		GtkButton* _tmp66_ = NULL;
		const gchar* _tmp67_ = NULL;
		GtkRadioButton* _tmp68_ = NULL;
		GtkRadioButton* _tmp69_ = NULL;
		GtkEntry* _tmp70_ = NULL;
		GtkButton* _tmp71_ = NULL;
		GtkButton* _tmp72_ = NULL;
#line 993 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = gtk_builder_new ();
#line 993 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->builder);
#line 993 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->builder = _tmp7_;
#line 994 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = self->priv->builder;
#line 994 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_builder_add_from_resource (_tmp8_, PLUGIN_RESOURCE_PATH "/rajce_publishing_options_pane.ui", &_inner_error_);
#line 994 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7283 "RajcePublishing.c"
			goto __catch30_g_error;
		}
#line 995 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = self->priv->builder;
#line 995 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_builder_connect_signals (_tmp9_, NULL);
#line 997 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = self->priv->builder;
#line 997 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = gtk_builder_get_object (_tmp10_, "rajce_pane_widget");
#line 997 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp11_, gtk_box_get_type (), GtkBox));
#line 997 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->pane_widget);
#line 997 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->pane_widget = _tmp12_;
#line 998 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = self->priv->builder;
#line 998 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = gtk_builder_get_object (_tmp13_, "login_identity_label");
#line 998 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp14_, gtk_label_get_type (), GtkLabel));
#line 998 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->login_identity_label);
#line 998 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->login_identity_label = _tmp15_;
#line 999 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = self->priv->builder;
#line 999 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = gtk_builder_get_object (_tmp16_, "publish_to_label");
#line 999 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp17_, gtk_label_get_type (), GtkLabel));
#line 999 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->publish_to_label);
#line 999 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->publish_to_label = _tmp18_;
#line 1000 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = self->priv->builder;
#line 1000 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = gtk_builder_get_object (_tmp19_, "use_existing_radio");
#line 1000 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp20_, gtk_radio_button_get_type (), GtkRadioButton));
#line 1000 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->use_existing_radio);
#line 1000 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->use_existing_radio = _tmp21_;
#line 1001 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = self->priv->builder;
#line 1001 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = gtk_builder_get_object (_tmp22_, "existing_albums_combo");
#line 1001 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp23_, gtk_combo_box_text_get_type (), GtkComboBoxText));
#line 1001 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->existing_albums_combo);
#line 1001 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->existing_albums_combo = _tmp24_;
#line 1002 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = self->priv->builder;
#line 1002 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = gtk_builder_get_object (_tmp25_, "create_new_radio");
#line 1002 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp26_, gtk_radio_button_get_type (), GtkRadioButton));
#line 1002 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->create_new_radio);
#line 1002 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->create_new_radio = _tmp27_;
#line 1003 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_ = self->priv->builder;
#line 1003 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp29_ = gtk_builder_get_object (_tmp28_, "new_album_entry");
#line 1003 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp30_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp29_, gtk_entry_get_type (), GtkEntry));
#line 1003 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->new_album_entry);
#line 1003 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->new_album_entry = _tmp30_;
#line 1004 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp31_ = self->priv->builder;
#line 1004 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp32_ = gtk_builder_get_object (_tmp31_, "hide_check");
#line 1004 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp33_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp32_, gtk_check_button_get_type (), GtkCheckButton));
#line 1004 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->hide_check);
#line 1004 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->hide_check = _tmp33_;
#line 1005 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp34_ = self->priv->hide_check;
#line 1005 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp35_ = _ ("_Hide album");
#line 1005 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (G_TYPE_CHECK_INSTANCE_CAST (_tmp34_, gtk_button_get_type (), GtkButton), _tmp35_);
#line 1006 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp36_ = self->priv->builder;
#line 1006 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp37_ = gtk_builder_get_object (_tmp36_, "show_check");
#line 1006 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp38_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp37_, gtk_check_button_get_type (), GtkCheckButton));
#line 1006 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->show_check);
#line 1006 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->show_check = _tmp38_;
#line 1007 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp39_ = self->priv->builder;
#line 1007 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp40_ = gtk_builder_get_object (_tmp39_, "publish_button");
#line 1007 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp41_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp40_, gtk_button_get_type (), GtkButton));
#line 1007 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->publish_button);
#line 1007 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->publish_button = _tmp41_;
#line 1008 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp42_ = self->priv->builder;
#line 1008 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp43_ = gtk_builder_get_object (_tmp42_, "logout_button");
#line 1008 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp44_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp43_, gtk_button_get_type (), GtkButton));
#line 1008 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_object_unref0 (self->priv->logout_button);
#line 1008 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		self->priv->logout_button = _tmp44_;
#line 1010 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp45_ = self->priv->hide_check;
#line 1010 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp46_ = publisher;
#line 1010 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp47_ = publishing_rajce_rajce_publisher_get_hide_album (_tmp46_);
#line 1010 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_toggle_button_set_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp45_, gtk_toggle_button_get_type (), GtkToggleButton), _tmp47_);
#line 1011 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp48_ = self->priv->show_check;
#line 1011 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp49_ = publisher;
#line 1011 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp50_ = publishing_rajce_rajce_publisher_get_show_album (_tmp49_);
#line 1011 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_toggle_button_set_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp48_, gtk_toggle_button_get_type (), GtkToggleButton), _tmp50_);
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp51_ = self->priv->login_identity_label;
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp52_ = _ ("You are logged into Rajce as %s.");
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp53_ = username;
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp54_ = g_strdup_printf (_tmp52_, _tmp53_);
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp55_ = _tmp54_;
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_label_set_label (_tmp51_, _tmp55_);
#line 1012 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (_tmp55_);
#line 1013 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp56_ = self->priv->publish_to_label;
#line 1013 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp57_ = _ ("Photos will appear in:");
#line 1013 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_label_set_label (_tmp56_, _tmp57_);
#line 1014 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp58_ = self->priv->use_existing_radio;
#line 1014 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp59_ = _ ("An _existing album:");
#line 1014 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (G_TYPE_CHECK_INSTANCE_CAST (_tmp58_, gtk_button_get_type (), GtkButton), _tmp59_);
#line 1015 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp60_ = self->priv->create_new_radio;
#line 1015 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp61_ = _ ("A _new album named:");
#line 1015 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (G_TYPE_CHECK_INSTANCE_CAST (_tmp60_, gtk_button_get_type (), GtkButton), _tmp61_);
#line 1016 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp62_ = self->priv->show_check;
#line 1016 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp63_ = _ ("Open target _album in browser");
#line 1016 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (G_TYPE_CHECK_INSTANCE_CAST (_tmp62_, gtk_button_get_type (), GtkButton), _tmp63_);
#line 1017 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp64_ = self->priv->publish_button;
#line 1017 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp65_ = _ ("_Publish");
#line 1017 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (_tmp64_, _tmp65_);
#line 1018 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp66_ = self->priv->logout_button;
#line 1018 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp67_ = _ ("_Logout");
#line 1018 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_button_set_label (_tmp66_, _tmp67_);
#line 1020 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp68_ = self->priv->use_existing_radio;
#line 1020 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp68_, gtk_button_get_type (), GtkButton), "clicked", (GCallback) _publishing_rajce_publishing_options_pane_on_use_existing_radio_clicked_gtk_button_clicked, self, 0);
#line 1021 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp69_ = self->priv->create_new_radio;
#line 1021 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp69_, gtk_button_get_type (), GtkButton), "clicked", (GCallback) _publishing_rajce_publishing_options_pane_on_create_new_radio_clicked_gtk_button_clicked, self, 0);
#line 1022 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp70_ = self->priv->new_album_entry;
#line 1022 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp70_, GTK_TYPE_EDITABLE, GtkEditable), "changed", (GCallback) _publishing_rajce_publishing_options_pane_on_new_album_entry_changed_gtk_editable_changed, self, 0);
#line 1023 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp71_ = self->priv->logout_button;
#line 1023 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (_tmp71_, "clicked", (GCallback) _publishing_rajce_publishing_options_pane_on_logout_clicked_gtk_button_clicked, self, 0);
#line 1024 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp72_ = self->priv->publish_button;
#line 1024 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_connect_object (_tmp72_, "clicked", (GCallback) _publishing_rajce_publishing_options_pane_on_publish_clicked_gtk_button_clicked, self, 0);
#line 7492 "RajcePublishing.c"
	}
	goto __finally30;
	__catch30_g_error:
	{
		GError* e = NULL;
		GError* _tmp73_ = NULL;
		const gchar* _tmp74_ = NULL;
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		e = _inner_error_;
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 1028 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp73_ = e;
#line 1028 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp74_ = _tmp73_->message;
#line 1028 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_warning ("RajcePublishing.vala:1028: Could not load UI: %s", _tmp74_);
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (e);
#line 7512 "RajcePublishing.c"
	}
	__finally30:
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 991 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return NULL;
#line 7523 "RajcePublishing.c"
	}
#line 984 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 7527 "RajcePublishing.c"
}


PublishingRajcePublishingOptionsPane* publishing_rajce_publishing_options_pane_new (PublishingRajceRajcePublisher* publisher, const gchar* username, PublishingRajceAlbum** albums, int albums_length1) {
#line 984 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_publishing_options_pane_construct (PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, publisher, username, albums, albums_length1);
#line 7534 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_on_publish_clicked (PublishingRajcePublishingOptionsPane* self) {
	gboolean show_album = FALSE;
	GtkCheckButton* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	PublishingRajceRajcePublisher* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
	GtkRadioButton* _tmp4_ = NULL;
	gboolean _tmp5_ = FALSE;
#line 1033 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1035 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->show_check;
#line 1035 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = gtk_toggle_button_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_toggle_button_get_type (), GtkToggleButton));
#line 1035 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	show_album = _tmp1_;
#line 1036 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->publisher;
#line 1036 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = show_album;
#line 1036 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_rajce_publisher_set_show_album (_tmp2_, _tmp3_);
#line 1037 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = self->priv->create_new_radio;
#line 1037 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = gtk_toggle_button_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_toggle_button_get_type (), GtkToggleButton));
#line 1037 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp5_) {
#line 7566 "RajcePublishing.c"
		gchar* album_name = NULL;
		GtkEntry* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gboolean hide_album = FALSE;
		GtkCheckButton* _tmp9_ = NULL;
		gboolean _tmp10_ = FALSE;
		PublishingRajceRajcePublisher* _tmp11_ = NULL;
		gboolean _tmp12_ = FALSE;
		const gchar* _tmp13_ = NULL;
		gboolean _tmp14_ = FALSE;
		PublishingRajcePublishingParameters* _tmp15_ = NULL;
		PublishingRajcePublishingParameters* _tmp16_ = NULL;
#line 1039 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = self->priv->new_album_entry;
#line 1039 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = gtk_entry_get_text (_tmp6_);
#line 1039 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = g_strdup (_tmp7_);
#line 1039 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		album_name = _tmp8_;
#line 1040 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = self->priv->hide_check;
#line 1040 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = gtk_toggle_button_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, gtk_toggle_button_get_type (), GtkToggleButton));
#line 1040 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		hide_album = _tmp10_;
#line 1041 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = self->priv->publisher;
#line 1041 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = hide_album;
#line 1041 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_rajce_publisher_set_hide_album (_tmp11_, _tmp12_);
#line 1042 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = album_name;
#line 1042 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = hide_album;
#line 1042 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = publishing_rajce_publishing_parameters_new_to_new_album (_tmp13_, _tmp14_);
#line 1042 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = _tmp15_;
#line 1042 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_emit_by_name (self, "publish", _tmp16_);
#line 1042 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rajce_publishing_parameters_unref0 (_tmp16_);
#line 1037 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (album_name);
#line 7614 "RajcePublishing.c"
	} else {
		gint id = 0;
		PublishingRajceAlbum** _tmp17_ = NULL;
		gint _tmp17__length1 = 0;
		GtkComboBoxText* _tmp18_ = NULL;
		gint _tmp19_ = 0;
		PublishingRajceAlbum* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gchar* album_name = NULL;
		PublishingRajceAlbum** _tmp22_ = NULL;
		gint _tmp22__length1 = 0;
		GtkComboBoxText* _tmp23_ = NULL;
		gint _tmp24_ = 0;
		PublishingRajceAlbum* _tmp25_ = NULL;
		const gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		const gchar* _tmp28_ = NULL;
		gint _tmp29_ = 0;
		PublishingRajcePublishingParameters* _tmp30_ = NULL;
		PublishingRajcePublishingParameters* _tmp31_ = NULL;
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = self->priv->albums;
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17__length1 = self->priv->albums_length1;
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = self->priv->existing_albums_combo;
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp19_ = gtk_combo_box_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp18_, gtk_combo_box_get_type (), GtkComboBox));
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp20_ = _tmp17_[_tmp19_];
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp21_ = _tmp20_->id;
#line 1046 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		id = _tmp21_;
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22_ = self->priv->albums;
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp22__length1 = self->priv->albums_length1;
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp23_ = self->priv->existing_albums_combo;
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp24_ = gtk_combo_box_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp23_, gtk_combo_box_get_type (), GtkComboBox));
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp25_ = _tmp22_[_tmp24_];
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp26_ = _tmp25_->albumName;
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp27_ = g_strdup (_tmp26_);
#line 1047 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		album_name = _tmp27_;
#line 1048 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp28_ = album_name;
#line 1048 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp29_ = id;
#line 1048 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp30_ = publishing_rajce_publishing_parameters_new_to_existing_album (_tmp28_, _tmp29_);
#line 1048 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp31_ = _tmp30_;
#line 1048 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_signal_emit_by_name (self, "publish", _tmp31_);
#line 1048 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_publishing_rajce_publishing_parameters_unref0 (_tmp31_);
#line 1037 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (album_name);
#line 7679 "RajcePublishing.c"
	}
}


static void publishing_rajce_publishing_options_pane_on_use_existing_radio_clicked (PublishingRajcePublishingOptionsPane* self) {
	GtkComboBoxText* _tmp0_ = NULL;
	GtkEntry* _tmp1_ = NULL;
	GtkComboBoxText* _tmp2_ = NULL;
	GtkCheckButton* _tmp3_ = NULL;
#line 1052 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1054 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->existing_albums_combo;
#line 1054 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget), TRUE);
#line 1055 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->new_album_entry;
#line 1055 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, gtk_widget_get_type (), GtkWidget), FALSE);
#line 1056 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->existing_albums_combo;
#line 1056 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_grab_focus (G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, gtk_widget_get_type (), GtkWidget));
#line 1057 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_update_publish_button_sensitivity (self);
#line 1058 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = self->priv->hide_check;
#line 1058 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, gtk_widget_get_type (), GtkWidget), FALSE);
#line 7709 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_on_create_new_radio_clicked (PublishingRajcePublishingOptionsPane* self) {
	GtkEntry* _tmp0_ = NULL;
	GtkComboBoxText* _tmp1_ = NULL;
	GtkEntry* _tmp2_ = NULL;
	GtkCheckButton* _tmp3_ = NULL;
#line 1061 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1063 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->new_album_entry;
#line 1063 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget), TRUE);
#line 1064 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = self->priv->existing_albums_combo;
#line 1064 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, gtk_widget_get_type (), GtkWidget), FALSE);
#line 1065 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->new_album_entry;
#line 1065 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_grab_focus (G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, gtk_widget_get_type (), GtkWidget));
#line 1066 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_update_publish_button_sensitivity (self);
#line 1067 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = self->priv->hide_check;
#line 1067 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, gtk_widget_get_type (), GtkWidget), TRUE);
#line 7738 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_on_logout_clicked (PublishingRajcePublishingOptionsPane* self) {
#line 1070 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1072 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_emit_by_name (self, "logout");
#line 7747 "RajcePublishing.c"
}


static gchar* string_strip (const gchar* self) {
	gchar* result = NULL;
	gchar* _result_ = NULL;
	gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 1207 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1208 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1208 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1209 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	_tmp1_ = _result_;
#line 1209 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	g_strstrip (_tmp1_);
#line 1210 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	result = _result_;
#line 1210 "/usr/share/vala-0.32/vapi/glib-2.0.vapi"
	return result;
#line 7770 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_update_publish_button_sensitivity (PublishingRajcePublishingOptionsPane* self) {
	gchar* album_name = NULL;
	GtkEntry* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
	const gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gboolean _tmp7_ = FALSE;
	GtkButton* _tmp10_ = NULL;
#line 1074 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1076 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->new_album_entry;
#line 1076 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = gtk_entry_get_text (_tmp0_);
#line 1076 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 1076 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	album_name = _tmp2_;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = album_name;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = string_strip (_tmp4_);
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = _tmp5_;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = g_strcmp0 (_tmp6_, "") != 0;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp6_);
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp7_) {
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = TRUE;
#line 7809 "RajcePublishing.c"
	} else {
		GtkRadioButton* _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = self->priv->create_new_radio;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = gtk_toggle_button_get_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp8_, gtk_toggle_button_get_type (), GtkToggleButton));
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = !_tmp9_;
#line 7819 "RajcePublishing.c"
	}
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = self->priv->publish_button;
#line 1077 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp10_, gtk_widget_get_type (), GtkWidget), _tmp3_);
#line 1074 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (album_name);
#line 7827 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_on_new_album_entry_changed (PublishingRajcePublishingOptionsPane* self) {
#line 1079 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1081 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_update_publish_button_sensitivity (self);
#line 7836 "RajcePublishing.c"
}


void publishing_rajce_publishing_options_pane_installed (PublishingRajcePublishingOptionsPane* self) {
	PublishingRajceAlbum** _tmp9_ = NULL;
	gint _tmp9__length1 = 0;
	GtkRadioButton* _tmp15_ = NULL;
#line 1083 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 7846 "RajcePublishing.c"
	{
		gint i = 0;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		i = 0;
#line 7851 "RajcePublishing.c"
		{
			gboolean _tmp0_ = FALSE;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp0_ = TRUE;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			while (TRUE) {
#line 7858 "RajcePublishing.c"
				gint _tmp2_ = 0;
				PublishingRajceAlbum** _tmp3_ = NULL;
				gint _tmp3__length1 = 0;
				GtkComboBoxText* _tmp4_ = NULL;
				PublishingRajceAlbum** _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gint _tmp6_ = 0;
				PublishingRajceAlbum* _tmp7_ = NULL;
				const gchar* _tmp8_ = NULL;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				if (!_tmp0_) {
#line 7870 "RajcePublishing.c"
					gint _tmp1_ = 0;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp1_ = i;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					i = _tmp1_ + 1;
#line 7876 "RajcePublishing.c"
				}
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp0_ = FALSE;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp2_ = i;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp3_ = self->priv->albums;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp3__length1 = self->priv->albums_length1;
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				if (!(_tmp2_ < _tmp3__length1)) {
#line 1085 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					break;
#line 7890 "RajcePublishing.c"
				}
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp4_ = self->priv->existing_albums_combo;
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp5_ = self->priv->albums;
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp5__length1 = self->priv->albums_length1;
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp6_ = i;
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp7_ = _tmp5_[_tmp6_];
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp8_ = _tmp7_->albumName;
#line 1088 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				gtk_combo_box_text_append_text (_tmp4_, _tmp8_);
#line 7906 "RajcePublishing.c"
			}
		}
	}
#line 1090 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = self->priv->albums;
#line 1090 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9__length1 = self->priv->albums_length1;
#line 1090 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp9__length1 == 0) {
#line 7916 "RajcePublishing.c"
		GtkComboBoxText* _tmp10_ = NULL;
		GtkRadioButton* _tmp11_ = NULL;
#line 1092 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = self->priv->existing_albums_combo;
#line 1092 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp10_, gtk_widget_get_type (), GtkWidget), FALSE);
#line 1093 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = self->priv->use_existing_radio;
#line 1093 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp11_, gtk_widget_get_type (), GtkWidget), FALSE);
#line 7927 "RajcePublishing.c"
	} else {
		GtkComboBoxText* _tmp12_ = NULL;
		GtkComboBoxText* _tmp13_ = NULL;
		GtkRadioButton* _tmp14_ = NULL;
#line 1097 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = self->priv->existing_albums_combo;
#line 1097 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_combo_box_set_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp12_, gtk_combo_box_get_type (), GtkComboBox), 0);
#line 1098 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = self->priv->existing_albums_combo;
#line 1098 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp13_, gtk_widget_get_type (), GtkWidget), TRUE);
#line 1099 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = self->priv->use_existing_radio;
#line 1099 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		gtk_widget_set_sensitive (G_TYPE_CHECK_INSTANCE_CAST (_tmp14_, gtk_widget_get_type (), GtkWidget), TRUE);
#line 7944 "RajcePublishing.c"
	}
#line 1101 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp15_ = self->priv->create_new_radio;
#line 1101 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	gtk_toggle_button_set_active (G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, gtk_toggle_button_get_type (), GtkToggleButton), TRUE);
#line 1102 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_on_create_new_radio_clicked (self);
#line 7952 "RajcePublishing.c"
}


void publishing_rajce_publishing_options_pane_notify_publish (PublishingRajcePublishingOptionsPane* self, PublishingRajcePublishingParameters* parameters) {
	PublishingRajcePublishingParameters* _tmp0_ = NULL;
#line 1105 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1105 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_PARAMETERS (parameters));
#line 1107 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = parameters;
#line 1107 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_emit_by_name (self, "publish", _tmp0_);
#line 7966 "RajcePublishing.c"
}


void publishing_rajce_publishing_options_pane_notify_logout (PublishingRajcePublishingOptionsPane* self) {
#line 1110 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self));
#line 1112 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_emit_by_name (self, "logout");
#line 7975 "RajcePublishing.c"
}


GtkWidget* publishing_rajce_publishing_options_pane_get_default_widget (PublishingRajcePublishingOptionsPane* self) {
	GtkWidget* result = NULL;
	GtkButton* _tmp0_ = NULL;
	GtkWidget* _tmp1_ = NULL;
#line 1115 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_OPTIONS_PANE (self), NULL);
#line 1117 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->logout_button;
#line 1117 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 1117 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 1117 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 7993 "RajcePublishing.c"
}


static GtkWidget* publishing_rajce_publishing_options_pane_real_get_widget (SpitPublishingDialogPane* base) {
	PublishingRajcePublishingOptionsPane * self;
	GtkWidget* result = NULL;
	GtkBox* _tmp0_ = NULL;
	GtkWidget* _tmp1_ = NULL;
#line 1119 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPane);
#line 1121 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->pane_widget;
#line 1121 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, gtk_widget_get_type (), GtkWidget));
#line 1121 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 1121 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8012 "RajcePublishing.c"
}


static SpitPublishingDialogPaneGeometryOptions publishing_rajce_publishing_options_pane_real_get_preferred_geometry (SpitPublishingDialogPane* base) {
	PublishingRajcePublishingOptionsPane * self;
	SpitPublishingDialogPaneGeometryOptions result = 0;
#line 1124 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPane);
#line 1126 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = SPIT_PUBLISHING_DIALOG_PANE_GEOMETRY_OPTIONS_NONE;
#line 1126 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8025 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_notify_publish_publishing_rajce_publishing_options_pane_publish (PublishingRajcePublishingOptionsPane* _sender, PublishingRajcePublishingParameters* parameters, gpointer self) {
#line 1132 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_notify_publish ((PublishingRajcePublishingOptionsPane*) self, parameters);
#line 8032 "RajcePublishing.c"
}


static void _publishing_rajce_publishing_options_pane_notify_logout_publishing_rajce_publishing_options_pane_logout (PublishingRajcePublishingOptionsPane* _sender, gpointer self) {
#line 1133 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_notify_logout ((PublishingRajcePublishingOptionsPane*) self);
#line 8039 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_real_on_pane_installed (SpitPublishingDialogPane* base) {
	PublishingRajcePublishingOptionsPane * self;
#line 1129 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPane);
#line 1131 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_installed (self);
#line 1132 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (self, "publish", (GCallback) _publishing_rajce_publishing_options_pane_notify_publish_publishing_rajce_publishing_options_pane_publish, self, 0);
#line 1133 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_connect_object (self, "logout", (GCallback) _publishing_rajce_publishing_options_pane_notify_logout_publishing_rajce_publishing_options_pane_logout, self, 0);
#line 8053 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_real_on_pane_uninstalled (SpitPublishingDialogPane* base) {
	PublishingRajcePublishingOptionsPane * self;
	guint _tmp0_ = 0U;
	guint _tmp1_ = 0U;
#line 1136 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPane);
#line 1138 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("publish", PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, &_tmp0_, NULL, FALSE);
#line 1138 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (self, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp0_, 0, NULL, (GCallback) _publishing_rajce_publishing_options_pane_notify_publish_publishing_rajce_publishing_options_pane_publish, self);
#line 1139 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_parse_name ("logout", PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, &_tmp1_, NULL, FALSE);
#line 1139 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_disconnect_matched (self, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _publishing_rajce_publishing_options_pane_notify_logout_publishing_rajce_publishing_options_pane_logout, self);
#line 8071 "RajcePublishing.c"
}


static void g_cclosure_user_marshal_VOID__PUBLISHING_RAJCE_PUBLISHING_PARAMETERS (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__PUBLISHING_RAJCE_PUBLISHING_PARAMETERS) (gpointer data1, gpointer arg_1, gpointer data2);
	register GMarshalFunc_VOID__PUBLISHING_RAJCE_PUBLISHING_PARAMETERS callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (n_param_values == 2);
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data1 = closure->data;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data2 = param_values->data[0].v_pointer;
#line 8090 "RajcePublishing.c"
	} else {
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data1 = param_values->data[0].v_pointer;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		data2 = closure->data;
#line 8096 "RajcePublishing.c"
	}
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	callback = (GMarshalFunc_VOID__PUBLISHING_RAJCE_PUBLISHING_PARAMETERS) (marshal_data ? marshal_data : cc->callback);
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	callback (data1, publishing_rajce_value_get_publishing_parameters (param_values + 1), data2);
#line 8102 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_class_init (PublishingRajcePublishingOptionsPaneClass * klass) {
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_parent_class = g_type_class_peek_parent (klass);
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajcePublishingOptionsPanePrivate));
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (klass)->finalize = publishing_rajce_publishing_options_pane_finalize;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_new ("publish", PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__PUBLISHING_RAJCE_PUBLISHING_PARAMETERS, G_TYPE_NONE, 1, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS);
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_new ("logout", PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 8117 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_spit_publishing_dialog_pane_interface_init (SpitPublishingDialogPaneIface * iface) {
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_options_pane_spit_publishing_dialog_pane_parent_iface = g_type_interface_peek_parent (iface);
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_widget = (GtkWidget* (*)(SpitPublishingDialogPane*)) publishing_rajce_publishing_options_pane_real_get_widget;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->get_preferred_geometry = (SpitPublishingDialogPaneGeometryOptions (*)(SpitPublishingDialogPane*)) publishing_rajce_publishing_options_pane_real_get_preferred_geometry;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->on_pane_installed = (void (*)(SpitPublishingDialogPane*)) publishing_rajce_publishing_options_pane_real_on_pane_installed;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	iface->on_pane_uninstalled = (void (*)(SpitPublishingDialogPane*)) publishing_rajce_publishing_options_pane_real_on_pane_uninstalled;
#line 8132 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_instance_init (PublishingRajcePublishingOptionsPane * self) {
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_PUBLISHING_OPTIONS_PANE_GET_PRIVATE (self);
#line 968 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->builder = NULL;
#line 969 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->pane_widget = NULL;
#line 970 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->login_identity_label = NULL;
#line 971 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->publish_to_label = NULL;
#line 972 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->use_existing_radio = NULL;
#line 973 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->existing_albums_combo = NULL;
#line 974 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->create_new_radio = NULL;
#line 975 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->new_album_entry = NULL;
#line 976 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->hide_check = NULL;
#line 977 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->show_check = NULL;
#line 978 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->publish_button = NULL;
#line 979 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->logout_button = NULL;
#line 8163 "RajcePublishing.c"
}


static void publishing_rajce_publishing_options_pane_finalize (GObject* obj) {
	PublishingRajcePublishingOptionsPane * self;
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_PUBLISHING_OPTIONS_PANE, PublishingRajcePublishingOptionsPane);
#line 964 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->publisher);
#line 965 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albums = (_vala_array_free (self->priv->albums, self->priv->albums_length1, (GDestroyNotify) publishing_rajce_album_unref), NULL);
#line 966 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->username);
#line 968 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->builder);
#line 969 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->pane_widget);
#line 970 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->login_identity_label);
#line 971 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->publish_to_label);
#line 972 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->use_existing_radio);
#line 973 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->existing_albums_combo);
#line 974 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->create_new_radio);
#line 975 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->new_album_entry);
#line 976 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->hide_check);
#line 977 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->show_check);
#line 978 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->publish_button);
#line 979 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (self->priv->logout_button);
#line 962 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_OBJECT_CLASS (publishing_rajce_publishing_options_pane_parent_class)->finalize (obj);
#line 8203 "RajcePublishing.c"
}


GType publishing_rajce_publishing_options_pane_get_type (void) {
	static volatile gsize publishing_rajce_publishing_options_pane_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_publishing_options_pane_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajcePublishingOptionsPaneClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_publishing_options_pane_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajcePublishingOptionsPane), 0, (GInstanceInitFunc) publishing_rajce_publishing_options_pane_instance_init, NULL };
		static const GInterfaceInfo spit_publishing_dialog_pane_info = { (GInterfaceInitFunc) publishing_rajce_publishing_options_pane_spit_publishing_dialog_pane_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType publishing_rajce_publishing_options_pane_type_id;
		publishing_rajce_publishing_options_pane_type_id = g_type_register_static (G_TYPE_OBJECT, "PublishingRajcePublishingOptionsPane", &g_define_type_info, 0);
		g_type_add_interface_static (publishing_rajce_publishing_options_pane_type_id, SPIT_PUBLISHING_TYPE_DIALOG_PANE, &spit_publishing_dialog_pane_info);
		g_once_init_leave (&publishing_rajce_publishing_options_pane_type_id__volatile, publishing_rajce_publishing_options_pane_type_id);
	}
	return publishing_rajce_publishing_options_pane_type_id__volatile;
}


static PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_construct (GType object_type) {
	PublishingRajcePublishingParameters* self = NULL;
#line 1149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajcePublishingParameters*) g_type_create_instance (object_type);
#line 1149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 8227 "RajcePublishing.c"
}


static PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_new (void) {
#line 1149 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_publishing_parameters_construct (PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS);
#line 8234 "RajcePublishing.c"
}


PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_construct_to_new_album (GType object_type, const gchar* album_name, gboolean album_hidden) {
	PublishingRajcePublishingParameters* self = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	gboolean* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint* _tmp5_ = NULL;
#line 1152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (album_name != NULL, NULL);
#line 1152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajcePublishingParameters*) g_type_create_instance (object_type);
#line 1154 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = album_name;
#line 1154 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1154 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_name);
#line 1154 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->album_name = _tmp1_;
#line 1155 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = album_hidden;
#line 1155 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = __bool_dup0 (&_tmp2_);
#line 1155 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_hidden);
#line 1155 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->album_hidden = _tmp3_;
#line 1156 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = 0;
#line 1156 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = __int_dup0 (&_tmp4_);
#line 1156 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_id);
#line 1156 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->album_id = _tmp5_;
#line 1152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 8276 "RajcePublishing.c"
}


PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_new_to_new_album (const gchar* album_name, gboolean album_hidden) {
#line 1152 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_publishing_parameters_construct_to_new_album (PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS, album_name, album_hidden);
#line 8283 "RajcePublishing.c"
}


PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_construct_to_existing_album (GType object_type, const gchar* album_name, gint album_id) {
	PublishingRajcePublishingParameters* self = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gint* _tmp3_ = NULL;
#line 1158 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (album_name != NULL, NULL);
#line 1158 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajcePublishingParameters*) g_type_create_instance (object_type);
#line 1160 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = album_name;
#line 1160 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1160 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_name);
#line 1160 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->album_name = _tmp1_;
#line 1161 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_hidden);
#line 1161 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->album_hidden = NULL;
#line 1162 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = album_id;
#line 1162 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = __int_dup0 (&_tmp2_);
#line 1162 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_id);
#line 1162 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->album_id = _tmp3_;
#line 1158 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 8319 "RajcePublishing.c"
}


PublishingRajcePublishingParameters* publishing_rajce_publishing_parameters_new_to_existing_album (const gchar* album_name, gint album_id) {
#line 1158 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_publishing_parameters_construct_to_existing_album (PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS, album_name, album_id);
#line 8326 "RajcePublishing.c"
}


static void publishing_rajce_value_publishing_parameters_init (GValue* value) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	value->data[0].v_pointer = NULL;
#line 8333 "RajcePublishing.c"
}


static void publishing_rajce_value_publishing_parameters_free_value (GValue* value) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (value->data[0].v_pointer) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_publishing_parameters_unref (value->data[0].v_pointer);
#line 8342 "RajcePublishing.c"
	}
}


static void publishing_rajce_value_publishing_parameters_copy_value (const GValue* src_value, GValue* dest_value) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (src_value->data[0].v_pointer) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = publishing_rajce_publishing_parameters_ref (src_value->data[0].v_pointer);
#line 8352 "RajcePublishing.c"
	} else {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = NULL;
#line 8356 "RajcePublishing.c"
	}
}


static gpointer publishing_rajce_value_publishing_parameters_peek_pointer (const GValue* value) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 8364 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_publishing_parameters_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (collect_values[0].v_pointer) {
#line 8371 "RajcePublishing.c"
		PublishingRajcePublishingParameters* object;
		object = collect_values[0].v_pointer;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (object->parent_instance.g_class == NULL) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 8378 "RajcePublishing.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 8382 "RajcePublishing.c"
		}
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = publishing_rajce_publishing_parameters_ref (object);
#line 8386 "RajcePublishing.c"
	} else {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 8390 "RajcePublishing.c"
	}
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 8394 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_publishing_parameters_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	PublishingRajcePublishingParameters** object_p;
	object_p = collect_values[0].v_pointer;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!object_p) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 8405 "RajcePublishing.c"
	}
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!value->data[0].v_pointer) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = NULL;
#line 8411 "RajcePublishing.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = value->data[0].v_pointer;
#line 8415 "RajcePublishing.c"
	} else {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = publishing_rajce_publishing_parameters_ref (value->data[0].v_pointer);
#line 8419 "RajcePublishing.c"
	}
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 8423 "RajcePublishing.c"
}


GParamSpec* publishing_rajce_param_spec_publishing_parameters (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	PublishingRajceParamSpecPublishingParameters* spec;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (g_type_is_a (object_type, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS), NULL);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return G_PARAM_SPEC (spec);
#line 8437 "RajcePublishing.c"
}


gpointer publishing_rajce_value_get_publishing_parameters (const GValue* value) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS), NULL);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 8446 "RajcePublishing.c"
}


void publishing_rajce_value_set_publishing_parameters (GValue* value, gpointer v_object) {
	PublishingRajcePublishingParameters* old;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS));
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS));
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_publishing_parameters_ref (value->data[0].v_pointer);
#line 8466 "RajcePublishing.c"
	} else {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 8470 "RajcePublishing.c"
	}
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_publishing_parameters_unref (old);
#line 8476 "RajcePublishing.c"
	}
}


void publishing_rajce_value_take_publishing_parameters (GValue* value, gpointer v_object) {
	PublishingRajcePublishingParameters* old;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS));
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS));
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 8495 "RajcePublishing.c"
	} else {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 8499 "RajcePublishing.c"
	}
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_publishing_parameters_unref (old);
#line 8505 "RajcePublishing.c"
	}
}


static void publishing_rajce_publishing_parameters_class_init (PublishingRajcePublishingParametersClass * klass) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_publishing_parameters_parent_class = g_type_class_peek_parent (klass);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRajcePublishingParametersClass *) klass)->finalize = publishing_rajce_publishing_parameters_finalize;
#line 8515 "RajcePublishing.c"
}


static void publishing_rajce_publishing_parameters_instance_init (PublishingRajcePublishingParameters * self) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->ref_count = 1;
#line 8522 "RajcePublishing.c"
}


static void publishing_rajce_publishing_parameters_finalize (PublishingRajcePublishingParameters* obj) {
	PublishingRajcePublishingParameters * self;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_PUBLISHING_PARAMETERS, PublishingRajcePublishingParameters);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_destroy (self);
#line 1145 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_name);
#line 1146 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_hidden);
#line 1147 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->album_id);
#line 8538 "RajcePublishing.c"
}


GType publishing_rajce_publishing_parameters_get_type (void) {
	static volatile gsize publishing_rajce_publishing_parameters_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_publishing_parameters_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { publishing_rajce_value_publishing_parameters_init, publishing_rajce_value_publishing_parameters_free_value, publishing_rajce_value_publishing_parameters_copy_value, publishing_rajce_value_publishing_parameters_peek_pointer, "p", publishing_rajce_value_publishing_parameters_collect_value, "p", publishing_rajce_value_publishing_parameters_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajcePublishingParametersClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_publishing_parameters_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajcePublishingParameters), 0, (GInstanceInitFunc) publishing_rajce_publishing_parameters_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType publishing_rajce_publishing_parameters_type_id;
		publishing_rajce_publishing_parameters_type_id = g_type_register_fundamental (g_type_fundamental_next (), "PublishingRajcePublishingParameters", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&publishing_rajce_publishing_parameters_type_id__volatile, publishing_rajce_publishing_parameters_type_id);
	}
	return publishing_rajce_publishing_parameters_type_id__volatile;
}


gpointer publishing_rajce_publishing_parameters_ref (gpointer instance) {
	PublishingRajcePublishingParameters* self;
	self = instance;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_atomic_int_inc (&self->ref_count);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return instance;
#line 8563 "RajcePublishing.c"
}


void publishing_rajce_publishing_parameters_unref (gpointer instance) {
	PublishingRajcePublishingParameters* self;
	self = instance;
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		PUBLISHING_RAJCE_PUBLISHING_PARAMETERS_GET_CLASS (self)->finalize (self);
#line 1143 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 8576 "RajcePublishing.c"
	}
}


PublishingRajceSession* publishing_rajce_session_construct (GType object_type) {
	PublishingRajceSession* self = NULL;
#line 1181 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceSession*) publishing_rest_support_session_construct (object_type, "");
#line 1179 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 8587 "RajcePublishing.c"
}


PublishingRajceSession* publishing_rajce_session_new (void) {
#line 1179 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_session_construct (PUBLISHING_RAJCE_TYPE_SESSION);
#line 8594 "RajcePublishing.c"
}


static gboolean publishing_rajce_session_real_is_authenticated (PublishingRESTSupportSession* base) {
	PublishingRajceSession * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gint* _tmp2_ = NULL;
#line 1184 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSession);
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = self->priv->userid;
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp2_ != NULL) {
#line 8610 "RajcePublishing.c"
		const gchar* _tmp3_ = NULL;
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = self->priv->usertoken;
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp1_ = _tmp3_ != NULL;
#line 8616 "RajcePublishing.c"
	} else {
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp1_ = FALSE;
#line 8620 "RajcePublishing.c"
	}
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp1_) {
#line 8624 "RajcePublishing.c"
		const gchar* _tmp4_ = NULL;
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = self->priv->username;
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = _tmp4_ != NULL;
#line 8630 "RajcePublishing.c"
	} else {
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = FALSE;
#line 8634 "RajcePublishing.c"
	}
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp0_;
#line 1186 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8640 "RajcePublishing.c"
}


void publishing_rajce_session_authenticate (PublishingRajceSession* self, const gchar* token, const gchar* name, gint id, gint maxsize, gint quality) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint* _tmp5_ = NULL;
	gint _tmp6_ = 0;
	gint* _tmp7_ = NULL;
	gint _tmp8_ = 0;
	gint* _tmp9_ = NULL;
#line 1189 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_SESSION (self));
#line 1189 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (token != NULL);
#line 1189 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (name != NULL);
#line 1191 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = token;
#line 1191 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1191 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->usertoken);
#line 1191 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->usertoken = _tmp1_;
#line 1192 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = name;
#line 1192 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 1192 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->username);
#line 1192 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->username = _tmp3_;
#line 1193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = id;
#line 1193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = __int_dup0 (&_tmp4_);
#line 1193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->userid);
#line 1193 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->userid = _tmp5_;
#line 1194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = maxsize;
#line 1194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = __int_dup0 (&_tmp6_);
#line 1194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->maxsize);
#line 1194 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->maxsize = _tmp7_;
#line 1195 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = quality;
#line 1195 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = __int_dup0 (&_tmp8_);
#line 1195 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->quality);
#line 1195 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->quality = _tmp9_;
#line 8701 "RajcePublishing.c"
}


void publishing_rajce_session_deauthenticate (PublishingRajceSession* self) {
#line 1198 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_SESSION (self));
#line 1200 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->usertoken);
#line 1200 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->usertoken = NULL;
#line 1201 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->albumtoken);
#line 1201 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albumtoken = NULL;
#line 1202 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->albumticket);
#line 1202 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albumticket = NULL;
#line 1203 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->username);
#line 1203 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->username = NULL;
#line 1204 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->userid);
#line 1204 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->userid = NULL;
#line 1205 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->maxsize);
#line 1205 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->maxsize = NULL;
#line 1206 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->quality);
#line 1206 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->quality = NULL;
#line 8736 "RajcePublishing.c"
}


void publishing_rajce_session_set_usertoken (PublishingRajceSession* self, const gchar* usertoken) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1209 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_SESSION (self));
#line 1209 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = usertoken;
#line 1209 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1209 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->usertoken);
#line 1209 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->usertoken = _tmp1_;
#line 8753 "RajcePublishing.c"
}


void publishing_rajce_session_set_albumtoken (PublishingRajceSession* self, const gchar* albumtoken) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1210 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_SESSION (self));
#line 1210 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = albumtoken;
#line 1210 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1210 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->albumtoken);
#line 1210 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albumtoken = _tmp1_;
#line 8770 "RajcePublishing.c"
}


void publishing_rajce_session_set_albumticket (PublishingRajceSession* self, const gchar* albumticket) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_SESSION (self));
#line 1211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = albumticket;
#line 1211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->albumticket);
#line 1211 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albumticket = _tmp1_;
#line 8787 "RajcePublishing.c"
}


gchar* publishing_rajce_session_get_usertoken (PublishingRajceSession* self) {
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1213 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (self), NULL);
#line 1213 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->usertoken;
#line 1213 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1213 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 1213 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8805 "RajcePublishing.c"
}


gchar* publishing_rajce_session_get_albumtoken (PublishingRajceSession* self) {
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (self), NULL);
#line 1214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->albumtoken;
#line 1214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 1214 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8823 "RajcePublishing.c"
}


gchar* publishing_rajce_session_get_albumticket (PublishingRajceSession* self) {
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (self), NULL);
#line 1215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->albumticket;
#line 1215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 1215 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8841 "RajcePublishing.c"
}


gchar* publishing_rajce_session_get_username (PublishingRajceSession* self) {
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1216 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (self), NULL);
#line 1216 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->username;
#line 1216 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1216 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp1_;
#line 1216 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8859 "RajcePublishing.c"
}


gint publishing_rajce_session_get_maxsize (PublishingRajceSession* self) {
	gint result = 0;
	gint* _tmp0_ = NULL;
#line 1218 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (self), 0);
#line 1218 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->maxsize;
#line 1218 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = *_tmp0_;
#line 1218 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 8874 "RajcePublishing.c"
}


static void publishing_rajce_session_class_init (PublishingRajceSessionClass * klass) {
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_session_parent_class = g_type_class_peek_parent (klass);
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRESTSupportSessionClass *) klass)->finalize = publishing_rajce_session_finalize;
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajceSessionPrivate));
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRESTSupportSessionClass *) klass)->is_authenticated = publishing_rajce_session_real_is_authenticated;
#line 8887 "RajcePublishing.c"
}


static void publishing_rajce_session_instance_init (PublishingRajceSession * self) {
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_SESSION_GET_PRIVATE (self);
#line 1171 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->usertoken = NULL;
#line 1172 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albumtoken = NULL;
#line 1173 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->albumticket = NULL;
#line 1174 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->username = NULL;
#line 1175 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->userid = NULL;
#line 1176 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->maxsize = NULL;
#line 1177 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->quality = NULL;
#line 8908 "RajcePublishing.c"
}


static void publishing_rajce_session_finalize (PublishingRESTSupportSession* obj) {
	PublishingRajceSession * self;
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_SESSION, PublishingRajceSession);
#line 1171 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->usertoken);
#line 1172 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->albumtoken);
#line 1173 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->albumticket);
#line 1174 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->username);
#line 1175 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->userid);
#line 1176 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->maxsize);
#line 1177 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->quality);
#line 1170 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	PUBLISHING_REST_SUPPORT_SESSION_CLASS (publishing_rajce_session_parent_class)->finalize (obj);
#line 8932 "RajcePublishing.c"
}


/**
 * Session class that keeps track of the credentials
 */
GType publishing_rajce_session_get_type (void) {
	static volatile gsize publishing_rajce_session_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_session_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceSessionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_session_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceSession), 0, (GInstanceInitFunc) publishing_rajce_session_instance_init, NULL };
		GType publishing_rajce_session_type_id;
		publishing_rajce_session_type_id = g_type_register_static (PUBLISHING_REST_SUPPORT_TYPE_SESSION, "PublishingRajceSession", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_session_type_id__volatile, publishing_rajce_session_type_id);
	}
	return publishing_rajce_session_type_id__volatile;
}


static gpointer _publishing_rajce_arg_item_ref0 (gpointer self) {
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self ? publishing_rajce_arg_item_ref (self) : NULL;
#line 8954 "RajcePublishing.c"
}


PublishingRajceArgItem* publishing_rajce_arg_item_construct (GType object_type, const gchar* k, const gchar* v) {
	PublishingRajceArgItem* self = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	PublishingRajceArgItem** _tmp4_ = NULL;
#line 1228 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceArgItem*) g_type_create_instance (object_type);
#line 1230 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = k;
#line 1230 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 1230 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->key);
#line 1230 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->key = _tmp1_;
#line 1231 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = v;
#line 1231 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 1231 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->val);
#line 1231 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->val = _tmp3_;
#line 1232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = g_new0 (PublishingRajceArgItem*, 0 + 1);
#line 1232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->children = (_vala_array_free (self->children, self->children_length1, (GDestroyNotify) publishing_rajce_arg_item_unref), NULL);
#line 1232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->children = _tmp4_;
#line 1232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->children_length1 = 0;
#line 1232 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->_children_size_ = self->children_length1;
#line 1228 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 8995 "RajcePublishing.c"
}


PublishingRajceArgItem* publishing_rajce_arg_item_new (const gchar* k, const gchar* v) {
#line 1228 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_arg_item_construct (PUBLISHING_RAJCE_TYPE_ARG_ITEM, k, v);
#line 9002 "RajcePublishing.c"
}


static void _vala_array_add6 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 9013 "RajcePublishing.c"
	}
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 9019 "RajcePublishing.c"
}


void publishing_rajce_arg_item_AddChild (PublishingRajceArgItem* self, PublishingRajceArgItem* child) {
	PublishingRajceArgItem** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	PublishingRajceArgItem* _tmp1_ = NULL;
	PublishingRajceArgItem* _tmp2_ = NULL;
#line 1234 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_ARG_ITEM (self));
#line 1234 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_ARG_ITEM (child));
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->children;
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0__length1 = self->children_length1;
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = child;
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = _publishing_rajce_arg_item_ref0 (_tmp1_);
#line 1236 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add6 (&self->children, &self->children_length1, &self->_children_size_, _tmp2_);
#line 9042 "RajcePublishing.c"
}


void publishing_rajce_arg_item_AddChildren (PublishingRajceArgItem* self, PublishingRajceArgItem** newchildren, int newchildren_length1) {
	PublishingRajceArgItem** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
#line 1238 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_ARG_ITEM (self));
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = newchildren;
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0__length1 = newchildren_length1;
#line 9055 "RajcePublishing.c"
	{
		PublishingRajceArgItem** child_collection = NULL;
		gint child_collection_length1 = 0;
		gint _child_collection_size_ = 0;
		gint child_it = 0;
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		child_collection = _tmp0_;
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		child_collection_length1 = _tmp0__length1;
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		for (child_it = 0; child_it < _tmp0__length1; child_it = child_it + 1) {
#line 9067 "RajcePublishing.c"
			PublishingRajceArgItem* _tmp1_ = NULL;
			PublishingRajceArgItem* child = NULL;
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp1_ = _publishing_rajce_arg_item_ref0 (child_collection[child_it]);
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			child = _tmp1_;
#line 9074 "RajcePublishing.c"
			{
				PublishingRajceArgItem* _tmp2_ = NULL;
#line 1242 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp2_ = child;
#line 1242 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				publishing_rajce_arg_item_AddChild (self, _tmp2_);
#line 1240 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_publishing_rajce_arg_item_unref0 (child);
#line 9083 "RajcePublishing.c"
			}
		}
	}
}


static void publishing_rajce_value_arg_item_init (GValue* value) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	value->data[0].v_pointer = NULL;
#line 9093 "RajcePublishing.c"
}


static void publishing_rajce_value_arg_item_free_value (GValue* value) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (value->data[0].v_pointer) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_arg_item_unref (value->data[0].v_pointer);
#line 9102 "RajcePublishing.c"
	}
}


static void publishing_rajce_value_arg_item_copy_value (const GValue* src_value, GValue* dest_value) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (src_value->data[0].v_pointer) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = publishing_rajce_arg_item_ref (src_value->data[0].v_pointer);
#line 9112 "RajcePublishing.c"
	} else {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = NULL;
#line 9116 "RajcePublishing.c"
	}
}


static gpointer publishing_rajce_value_arg_item_peek_pointer (const GValue* value) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 9124 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_arg_item_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (collect_values[0].v_pointer) {
#line 9131 "RajcePublishing.c"
		PublishingRajceArgItem* object;
		object = collect_values[0].v_pointer;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (object->parent_instance.g_class == NULL) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 9138 "RajcePublishing.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 9142 "RajcePublishing.c"
		}
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = publishing_rajce_arg_item_ref (object);
#line 9146 "RajcePublishing.c"
	} else {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 9150 "RajcePublishing.c"
	}
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 9154 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_arg_item_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	PublishingRajceArgItem** object_p;
	object_p = collect_values[0].v_pointer;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!object_p) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 9165 "RajcePublishing.c"
	}
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!value->data[0].v_pointer) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = NULL;
#line 9171 "RajcePublishing.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = value->data[0].v_pointer;
#line 9175 "RajcePublishing.c"
	} else {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = publishing_rajce_arg_item_ref (value->data[0].v_pointer);
#line 9179 "RajcePublishing.c"
	}
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 9183 "RajcePublishing.c"
}


GParamSpec* publishing_rajce_param_spec_arg_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	PublishingRajceParamSpecArgItem* spec;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (g_type_is_a (object_type, PUBLISHING_RAJCE_TYPE_ARG_ITEM), NULL);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return G_PARAM_SPEC (spec);
#line 9197 "RajcePublishing.c"
}


gpointer publishing_rajce_value_get_arg_item (const GValue* value) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_ARG_ITEM), NULL);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 9206 "RajcePublishing.c"
}


void publishing_rajce_value_set_arg_item (GValue* value, gpointer v_object) {
	PublishingRajceArgItem* old;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_ARG_ITEM));
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_ARG_ITEM));
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_arg_item_ref (value->data[0].v_pointer);
#line 9226 "RajcePublishing.c"
	} else {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 9230 "RajcePublishing.c"
	}
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_arg_item_unref (old);
#line 9236 "RajcePublishing.c"
	}
}


void publishing_rajce_value_take_arg_item (GValue* value, gpointer v_object) {
	PublishingRajceArgItem* old;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_ARG_ITEM));
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_ARG_ITEM));
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 9255 "RajcePublishing.c"
	} else {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 9259 "RajcePublishing.c"
	}
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_arg_item_unref (old);
#line 9265 "RajcePublishing.c"
	}
}


static void publishing_rajce_arg_item_class_init (PublishingRajceArgItemClass * klass) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_arg_item_parent_class = g_type_class_peek_parent (klass);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRajceArgItemClass *) klass)->finalize = publishing_rajce_arg_item_finalize;
#line 9275 "RajcePublishing.c"
}


static void publishing_rajce_arg_item_instance_init (PublishingRajceArgItem * self) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->ref_count = 1;
#line 9282 "RajcePublishing.c"
}


static void publishing_rajce_arg_item_finalize (PublishingRajceArgItem* obj) {
	PublishingRajceArgItem * self;
	PublishingRajceArgItem** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_ARG_ITEM, PublishingRajceArgItem);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_destroy (self);
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->children;
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0__length1 = self->children_length1;
#line 9298 "RajcePublishing.c"
	{
		PublishingRajceArgItem** child_collection = NULL;
		gint child_collection_length1 = 0;
		gint _child_collection_size_ = 0;
		gint child_it = 0;
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		child_collection = _tmp0_;
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		child_collection_length1 = _tmp0__length1;
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		for (child_it = 0; child_it < _tmp0__length1; child_it = child_it + 1) {
#line 9310 "RajcePublishing.c"
			PublishingRajceArgItem* _tmp1_ = NULL;
			PublishingRajceArgItem* child = NULL;
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp1_ = _publishing_rajce_arg_item_ref0 (child_collection[child_it]);
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			child = _tmp1_;
#line 9317 "RajcePublishing.c"
			{
#line 1249 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_publishing_rajce_arg_item_unref0 (child);
#line 1249 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				child = NULL;
#line 1247 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_publishing_rajce_arg_item_unref0 (child);
#line 9325 "RajcePublishing.c"
			}
		}
	}
#line 1224 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->key);
#line 1225 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->val);
#line 1226 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->children = (_vala_array_free (self->children, self->children_length1, (GDestroyNotify) publishing_rajce_arg_item_unref), NULL);
#line 9335 "RajcePublishing.c"
}


GType publishing_rajce_arg_item_get_type (void) {
	static volatile gsize publishing_rajce_arg_item_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_arg_item_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { publishing_rajce_value_arg_item_init, publishing_rajce_value_arg_item_free_value, publishing_rajce_value_arg_item_copy_value, publishing_rajce_value_arg_item_peek_pointer, "p", publishing_rajce_value_arg_item_collect_value, "p", publishing_rajce_value_arg_item_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceArgItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_arg_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceArgItem), 0, (GInstanceInitFunc) publishing_rajce_arg_item_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType publishing_rajce_arg_item_type_id;
		publishing_rajce_arg_item_type_id = g_type_register_fundamental (g_type_fundamental_next (), "PublishingRajceArgItem", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&publishing_rajce_arg_item_type_id__volatile, publishing_rajce_arg_item_type_id);
	}
	return publishing_rajce_arg_item_type_id__volatile;
}


gpointer publishing_rajce_arg_item_ref (gpointer instance) {
	PublishingRajceArgItem* self;
	self = instance;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_atomic_int_inc (&self->ref_count);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return instance;
#line 9360 "RajcePublishing.c"
}


void publishing_rajce_arg_item_unref (gpointer instance) {
	PublishingRajceArgItem* self;
	self = instance;
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		PUBLISHING_RAJCE_ARG_ITEM_GET_CLASS (self)->finalize (self);
#line 1222 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 9373 "RajcePublishing.c"
	}
}


PublishingRajceLiveApiRequest* publishing_rajce_live_api_request_construct (GType object_type, const gchar* cmd) {
	PublishingRajceLiveApiRequest* self = NULL;
	PublishingRajceArgItem** _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
#line 1261 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (cmd != NULL, NULL);
#line 1261 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceLiveApiRequest*) g_type_create_instance (object_type);
#line 1263 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = g_new0 (PublishingRajceArgItem*, 0 + 1);
#line 1263 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_params = (_vala_array_free (self->priv->_params, self->priv->_params_length1, (GDestroyNotify) publishing_rajce_arg_item_unref), NULL);
#line 1263 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_params = _tmp0_;
#line 1263 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_params_length1 = 0;
#line 1263 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->__params_size_ = self->priv->_params_length1;
#line 1264 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = cmd;
#line 1264 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 1264 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->_cmd);
#line 1264 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_cmd = _tmp2_;
#line 1261 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 9407 "RajcePublishing.c"
}


PublishingRajceLiveApiRequest* publishing_rajce_live_api_request_new (const gchar* cmd) {
#line 1261 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_live_api_request_construct (PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST, cmd);
#line 9414 "RajcePublishing.c"
}


static void _vala_array_add7 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 9425 "RajcePublishing.c"
	}
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 9431 "RajcePublishing.c"
}


void publishing_rajce_live_api_request_AddParam (PublishingRajceLiveApiRequest* self, const gchar* name, const gchar* val) {
	PublishingRajceArgItem** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	const gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	PublishingRajceArgItem* _tmp3_ = NULL;
#line 1269 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_LIVE_API_REQUEST (self));
#line 1269 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (name != NULL);
#line 1269 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (val != NULL);
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = self->priv->_params;
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0__length1 = self->priv->_params_length1;
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = name;
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = val;
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = publishing_rajce_arg_item_new (_tmp1_, _tmp2_);
#line 1271 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add7 (&self->priv->_params, &self->priv->_params_length1, &self->priv->__params_size_, _tmp3_);
#line 9459 "RajcePublishing.c"
}


void publishing_rajce_live_api_request_AddParamBool (PublishingRajceLiveApiRequest* self, const gchar* name, gboolean val) {
	const gchar* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	const gchar* _tmp2_ = NULL;
#line 1276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_LIVE_API_REQUEST (self));
#line 1276 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (name != NULL);
#line 1278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = val;
#line 1278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp1_) {
#line 1278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = "1";
#line 9477 "RajcePublishing.c"
	} else {
#line 1278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp0_ = "0";
#line 9481 "RajcePublishing.c"
	}
#line 1278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = name;
#line 1278 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (self, _tmp2_, _tmp0_);
#line 9487 "RajcePublishing.c"
}


void publishing_rajce_live_api_request_AddParamInt (PublishingRajceLiveApiRequest* self, const gchar* name, gint val) {
	const gchar* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
#line 1283 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_LIVE_API_REQUEST (self));
#line 1283 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (name != NULL);
#line 1285 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = name;
#line 1285 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = val;
#line 1285 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = g_strdup_printf ("%i", _tmp1_);
#line 1285 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = _tmp2_;
#line 1285 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (self, _tmp0_, _tmp3_);
#line 1285 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp3_);
#line 9512 "RajcePublishing.c"
}


static void _vala_array_add8 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 9523 "RajcePublishing.c"
	}
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 9529 "RajcePublishing.c"
}


void publishing_rajce_live_api_request_AddParamNode (PublishingRajceLiveApiRequest* self, const gchar* name, PublishingRajceArgItem** val, int val_length1) {
	PublishingRajceArgItem* newItem = NULL;
	const gchar* _tmp0_ = NULL;
	PublishingRajceArgItem* _tmp1_ = NULL;
	PublishingRajceArgItem** _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	PublishingRajceArgItem** _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	PublishingRajceArgItem* _tmp4_ = NULL;
#line 1297 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_LIVE_API_REQUEST (self));
#line 1297 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (name != NULL);
#line 1299 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = name;
#line 1299 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = publishing_rajce_arg_item_new (_tmp0_, NULL);
#line 1299 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	newItem = _tmp1_;
#line 1300 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = val;
#line 1300 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2__length1 = val_length1;
#line 1300 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_arg_item_AddChildren (newItem, _tmp2_, _tmp2__length1);
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = self->priv->_params;
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3__length1 = self->priv->_params_length1;
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = _publishing_rajce_arg_item_ref0 (newItem);
#line 1301 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add8 (&self->priv->_params, &self->priv->_params_length1, &self->priv->__params_size_, _tmp4_);
#line 1297 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_arg_item_unref0 (newItem);
#line 9568 "RajcePublishing.c"
}


gchar* publishing_rajce_live_api_request_Params2XmlString (PublishingRajceLiveApiRequest* self, gboolean urlencode) {
	gchar* result = NULL;
	xmlDoc* doc = NULL;
	xmlDoc* _tmp0_ = NULL;
	xmlNode* root = NULL;
	xmlNode* _tmp1_ = NULL;
	xmlDoc* _tmp2_ = NULL;
	xmlNode* _tmp3_ = NULL;
	xmlNode* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	xmlNode* par = NULL;
	xmlNode* _tmp6_ = NULL;
	xmlNode* _tmp7_ = NULL;
	PublishingRajceArgItem** _tmp8_ = NULL;
	gint _tmp8__length1 = 0;
	gchar* xmlstr = NULL;
	xmlDoc* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
	xmlDoc* _tmp14_ = NULL;
	gboolean _tmp15_ = FALSE;
#line 1306 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_LIVE_API_REQUEST (self), NULL);
#line 1308 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = xmlNewDoc ((xmlChar*) "1.0");
#line 1308 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	doc = _tmp0_;
#line 1309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = xmlNewNode (NULL, (xmlChar*) "request");
#line 1309 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	root = _tmp1_;
#line 1310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = doc;
#line 1310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = root;
#line 1310 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xmlDocSetRootElement (_tmp2_, _tmp3_);
#line 1311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = root;
#line 1311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = self->priv->_cmd;
#line 1311 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xmlNewTextChild (_tmp4_, NULL, (xmlChar*) "command", (xmlChar*) _tmp5_);
#line 1312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = root;
#line 1312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = xmlNewTextChild (_tmp6_, NULL, (xmlChar*) "parameters", (xmlChar*) "");
#line 1312 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	par = _tmp7_;
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = self->priv->_params;
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8__length1 = self->priv->_params_length1;
#line 9624 "RajcePublishing.c"
	{
		PublishingRajceArgItem** arg_collection = NULL;
		gint arg_collection_length1 = 0;
		gint _arg_collection_size_ = 0;
		gint arg_it = 0;
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		arg_collection = _tmp8_;
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		arg_collection_length1 = _tmp8__length1;
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		for (arg_it = 0; arg_it < _tmp8__length1; arg_it = arg_it + 1) {
#line 9636 "RajcePublishing.c"
			PublishingRajceArgItem* _tmp9_ = NULL;
			PublishingRajceArgItem* arg = NULL;
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_tmp9_ = _publishing_rajce_arg_item_ref0 (arg_collection[arg_it]);
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			arg = _tmp9_;
#line 9643 "RajcePublishing.c"
			{
				xmlNode* _tmp10_ = NULL;
				PublishingRajceArgItem* _tmp11_ = NULL;
#line 1315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp10_ = par;
#line 1315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp11_ = arg;
#line 1315 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				publishing_rajce_live_api_request_WriteParam (_tmp10_, _tmp11_);
#line 1313 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_publishing_rajce_arg_item_unref0 (arg);
#line 9655 "RajcePublishing.c"
			}
		}
	}
#line 1318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = doc;
#line 1318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xmlDocDumpMemoryEnc (_tmp12_, (xmlChar**) (&_tmp13_), NULL, "UTF-8");
#line 1318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xmlstr);
#line 1318 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xmlstr = _tmp13_;
#line 1319 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp14_ = doc;
#line 1319 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xmlFreeDoc (_tmp14_);
#line 1320 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp15_ = urlencode;
#line 1320 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp15_) {
#line 9675 "RajcePublishing.c"
		const gchar* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
#line 1322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = xmlstr;
#line 1322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = soup_uri_encode (_tmp16_, "&;");
#line 1322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result = _tmp17_;
#line 1322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (xmlstr);
#line 1322 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return result;
#line 9688 "RajcePublishing.c"
	}
#line 1324 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = xmlstr;
#line 1324 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 9694 "RajcePublishing.c"
}


static void publishing_rajce_live_api_request_WriteParam (xmlNode* node, PublishingRajceArgItem* arg) {
	PublishingRajceArgItem* _tmp0_ = NULL;
	PublishingRajceArgItem** _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
#line 1329 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (PUBLISHING_RAJCE_IS_ARG_ITEM (arg));
#line 1331 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = arg;
#line 1331 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = _tmp0_->children;
#line 1331 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1__length1 = _tmp0_->children_length1;
#line 1331 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp1__length1 == 0) {
#line 9712 "RajcePublishing.c"
		xmlNode* _tmp2_ = NULL;
		PublishingRajceArgItem* _tmp3_ = NULL;
		const gchar* _tmp4_ = NULL;
		PublishingRajceArgItem* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
#line 1333 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp2_ = node;
#line 1333 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = arg;
#line 1333 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp4_ = _tmp3_->key;
#line 1333 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = arg;
#line 1333 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp6_ = _tmp5_->val;
#line 1333 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		xmlNewTextChild (_tmp2_, NULL, (xmlChar*) _tmp4_, (xmlChar*) _tmp6_);
#line 9730 "RajcePublishing.c"
	} else {
		xmlNode* subnode = NULL;
		xmlNode* _tmp7_ = NULL;
		PublishingRajceArgItem* _tmp8_ = NULL;
		const gchar* _tmp9_ = NULL;
		xmlNode* _tmp10_ = NULL;
		PublishingRajceArgItem* _tmp11_ = NULL;
		PublishingRajceArgItem** _tmp12_ = NULL;
		gint _tmp12__length1 = 0;
#line 1337 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = node;
#line 1337 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp8_ = arg;
#line 1337 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = _tmp8_->key;
#line 1337 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = xmlNewTextChild (_tmp7_, NULL, (xmlChar*) _tmp9_, (xmlChar*) "");
#line 1337 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		subnode = _tmp10_;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = arg;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = _tmp11_->children;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12__length1 = _tmp11_->children_length1;
#line 9756 "RajcePublishing.c"
		{
			PublishingRajceArgItem** child_collection = NULL;
			gint child_collection_length1 = 0;
			gint _child_collection_size_ = 0;
			gint child_it = 0;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			child_collection = _tmp12_;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			child_collection_length1 = _tmp12__length1;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			for (child_it = 0; child_it < _tmp12__length1; child_it = child_it + 1) {
#line 9768 "RajcePublishing.c"
				PublishingRajceArgItem* _tmp13_ = NULL;
				PublishingRajceArgItem* child = NULL;
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				_tmp13_ = _publishing_rajce_arg_item_ref0 (child_collection[child_it]);
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
				child = _tmp13_;
#line 9775 "RajcePublishing.c"
				{
					xmlNode* _tmp14_ = NULL;
					PublishingRajceArgItem* _tmp15_ = NULL;
#line 1340 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp14_ = subnode;
#line 1340 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_tmp15_ = child;
#line 1340 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					publishing_rajce_live_api_request_WriteParam (_tmp14_, _tmp15_);
#line 1338 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
					_publishing_rajce_arg_item_unref0 (child);
#line 9787 "RajcePublishing.c"
				}
			}
		}
	}
}


static void publishing_rajce_value_live_api_request_init (GValue* value) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	value->data[0].v_pointer = NULL;
#line 9798 "RajcePublishing.c"
}


static void publishing_rajce_value_live_api_request_free_value (GValue* value) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (value->data[0].v_pointer) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_live_api_request_unref (value->data[0].v_pointer);
#line 9807 "RajcePublishing.c"
	}
}


static void publishing_rajce_value_live_api_request_copy_value (const GValue* src_value, GValue* dest_value) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (src_value->data[0].v_pointer) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = publishing_rajce_live_api_request_ref (src_value->data[0].v_pointer);
#line 9817 "RajcePublishing.c"
	} else {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		dest_value->data[0].v_pointer = NULL;
#line 9821 "RajcePublishing.c"
	}
}


static gpointer publishing_rajce_value_live_api_request_peek_pointer (const GValue* value) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 9829 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_live_api_request_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (collect_values[0].v_pointer) {
#line 9836 "RajcePublishing.c"
		PublishingRajceLiveApiRequest* object;
		object = collect_values[0].v_pointer;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (object->parent_instance.g_class == NULL) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 9843 "RajcePublishing.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 9847 "RajcePublishing.c"
		}
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = publishing_rajce_live_api_request_ref (object);
#line 9851 "RajcePublishing.c"
	} else {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 9855 "RajcePublishing.c"
	}
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 9859 "RajcePublishing.c"
}


static gchar* publishing_rajce_value_live_api_request_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	PublishingRajceLiveApiRequest** object_p;
	object_p = collect_values[0].v_pointer;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!object_p) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 9870 "RajcePublishing.c"
	}
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (!value->data[0].v_pointer) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = NULL;
#line 9876 "RajcePublishing.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = value->data[0].v_pointer;
#line 9880 "RajcePublishing.c"
	} else {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*object_p = publishing_rajce_live_api_request_ref (value->data[0].v_pointer);
#line 9884 "RajcePublishing.c"
	}
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return NULL;
#line 9888 "RajcePublishing.c"
}


GParamSpec* publishing_rajce_param_spec_live_api_request (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	PublishingRajceParamSpecLiveApiRequest* spec;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (g_type_is_a (object_type, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST), NULL);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return G_PARAM_SPEC (spec);
#line 9902 "RajcePublishing.c"
}


gpointer publishing_rajce_value_get_live_api_request (const GValue* value) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST), NULL);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return value->data[0].v_pointer;
#line 9911 "RajcePublishing.c"
}


void publishing_rajce_value_set_live_api_request (GValue* value, gpointer v_object) {
	PublishingRajceLiveApiRequest* old;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST));
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST));
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_live_api_request_ref (value->data[0].v_pointer);
#line 9931 "RajcePublishing.c"
	} else {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 9935 "RajcePublishing.c"
	}
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_live_api_request_unref (old);
#line 9941 "RajcePublishing.c"
	}
}


void publishing_rajce_value_take_live_api_request (GValue* value, gpointer v_object) {
	PublishingRajceLiveApiRequest* old;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST));
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	old = value->data[0].v_pointer;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (v_object) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST));
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = v_object;
#line 9960 "RajcePublishing.c"
	} else {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		value->data[0].v_pointer = NULL;
#line 9964 "RajcePublishing.c"
	}
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (old) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		publishing_rajce_live_api_request_unref (old);
#line 9970 "RajcePublishing.c"
	}
}


static void publishing_rajce_live_api_request_class_init (PublishingRajceLiveApiRequestClass * klass) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_parent_class = g_type_class_peek_parent (klass);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRajceLiveApiRequestClass *) klass)->finalize = publishing_rajce_live_api_request_finalize;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajceLiveApiRequestPrivate));
#line 9982 "RajcePublishing.c"
}


static void publishing_rajce_live_api_request_instance_init (PublishingRajceLiveApiRequest * self) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_LIVE_API_REQUEST_GET_PRIVATE (self);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->ref_count = 1;
#line 9991 "RajcePublishing.c"
}


static void publishing_rajce_live_api_request_finalize (PublishingRajceLiveApiRequest* obj) {
	PublishingRajceLiveApiRequest * self;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_LIVE_API_REQUEST, PublishingRajceLiveApiRequest);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_signal_handlers_destroy (self);
#line 1259 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->_params = (_vala_array_free (self->priv->_params, self->priv->_params_length1, (GDestroyNotify) publishing_rajce_arg_item_unref), NULL);
#line 1260 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (self->priv->_cmd);
#line 10005 "RajcePublishing.c"
}


GType publishing_rajce_live_api_request_get_type (void) {
	static volatile gsize publishing_rajce_live_api_request_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_live_api_request_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { publishing_rajce_value_live_api_request_init, publishing_rajce_value_live_api_request_free_value, publishing_rajce_value_live_api_request_copy_value, publishing_rajce_value_live_api_request_peek_pointer, "p", publishing_rajce_value_live_api_request_collect_value, "p", publishing_rajce_value_live_api_request_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceLiveApiRequestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_live_api_request_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceLiveApiRequest), 0, (GInstanceInitFunc) publishing_rajce_live_api_request_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType publishing_rajce_live_api_request_type_id;
		publishing_rajce_live_api_request_type_id = g_type_register_fundamental (g_type_fundamental_next (), "PublishingRajceLiveApiRequest", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&publishing_rajce_live_api_request_type_id__volatile, publishing_rajce_live_api_request_type_id);
	}
	return publishing_rajce_live_api_request_type_id__volatile;
}


gpointer publishing_rajce_live_api_request_ref (gpointer instance) {
	PublishingRajceLiveApiRequest* self;
	self = instance;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_atomic_int_inc (&self->ref_count);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return instance;
#line 10030 "RajcePublishing.c"
}


void publishing_rajce_live_api_request_unref (gpointer instance) {
	PublishingRajceLiveApiRequest* self;
	self = instance;
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		PUBLISHING_RAJCE_LIVE_API_REQUEST_GET_CLASS (self)->finalize (self);
#line 1257 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 10043 "RajcePublishing.c"
	}
}


PublishingRajceTransaction* publishing_rajce_transaction_construct (GType object_type, PublishingRajceSession* session) {
	PublishingRajceTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
#line 1355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1357 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1357 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceTransaction*) publishing_rest_support_transaction_construct (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10059 "RajcePublishing.c"
}


PublishingRajceTransaction* publishing_rajce_transaction_new (PublishingRajceSession* session) {
#line 1355 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_transaction_construct (PUBLISHING_RAJCE_TYPE_TRANSACTION, session);
#line 10066 "RajcePublishing.c"
}


gchar* publishing_rajce_transaction_validate_xml (PublishingRESTSupportXmlDocument* doc) {
	gchar* result = NULL;
	xmlNode* root = NULL;
	PublishingRESTSupportXmlDocument* _tmp0_ = NULL;
	xmlNode* _tmp1_ = NULL;
	xmlNode* _tmp2_ = NULL;
	gchar* name = NULL;
	xmlNode* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gboolean _tmp7_ = FALSE;
	const gchar* _tmp8_ = NULL;
	xmlNode* errcode = NULL;
	xmlNode* _result_ = NULL;
	xmlNode* _tmp19_ = NULL;
	gchar* _tmp20_ = NULL;
	gchar* _tmp21_ = NULL;
	gint _tmp22_ = 0;
	xmlNode* _tmp23_ = NULL;
	gchar* _tmp24_ = NULL;
	gchar* _tmp25_ = NULL;
	gchar* _tmp26_ = NULL;
	gchar* _tmp27_ = NULL;
	GError * _inner_error_ = NULL;
#line 1360 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_REST_SUPPORT_IS_XML_DOCUMENT (doc), NULL);
#line 1362 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = doc;
#line 1362 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = publishing_rest_support_xml_document_get_root_node (_tmp0_);
#line 1362 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	root = _tmp1_;
#line 1363 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = root;
#line 1363 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp2_ == NULL) {
#line 10106 "RajcePublishing.c"
		gchar* _tmp3_ = NULL;
#line 1365 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp3_ = g_strdup ("No XML returned from server");
#line 1365 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result = _tmp3_;
#line 1365 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return result;
#line 10114 "RajcePublishing.c"
	}
#line 1367 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = root;
#line 1367 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_->name;
#line 1367 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = g_strdup (_tmp5_);
#line 1367 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	name = _tmp6_;
#line 1370 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = name;
#line 1370 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp8_ == NULL) {
#line 1370 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = TRUE;
#line 10130 "RajcePublishing.c"
	} else {
		const gchar* _tmp9_ = NULL;
#line 1370 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp9_ = name;
#line 1370 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp7_ = g_strcmp0 (_tmp9_, "response") != 0;
#line 10137 "RajcePublishing.c"
	}
#line 1370 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp7_) {
#line 10141 "RajcePublishing.c"
		gchar* _tmp10_ = NULL;
#line 1372 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp10_ = g_strdup ("No response from Rajce in XML");
#line 1372 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result = _tmp10_;
#line 1372 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (name);
#line 1372 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return result;
#line 10151 "RajcePublishing.c"
	}
	{
		xmlNode* _tmp11_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp12_ = NULL;
		xmlNode* _tmp13_ = NULL;
		xmlNode* _tmp14_ = NULL;
		xmlNode* _tmp15_ = NULL;
		PublishingRESTSupportXmlDocument* _tmp16_ = NULL;
		xmlNode* _tmp17_ = NULL;
		xmlNode* _tmp18_ = NULL;
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp12_ = doc;
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp13_ = root;
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp14_ = publishing_rest_support_xml_document_get_named_child (_tmp12_, _tmp13_, "errorCode", &_inner_error_);
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp11_ = _tmp14_;
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 10174 "RajcePublishing.c"
				goto __catch31_spit_publishing_publishing_error;
			}
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (name);
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return NULL;
#line 10185 "RajcePublishing.c"
		}
#line 1378 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		errcode = _tmp11_;
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp16_ = doc;
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp17_ = root;
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp18_ = publishing_rest_support_xml_document_get_named_child (_tmp16_, _tmp17_, "result", &_inner_error_);
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp15_ = _tmp18_;
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			if (_inner_error_->domain == SPIT_PUBLISHING_PUBLISHING_ERROR) {
#line 10201 "RajcePublishing.c"
				goto __catch31_spit_publishing_publishing_error;
			}
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			_g_free0 (name);
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			g_clear_error (&_inner_error_);
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
			return NULL;
#line 10212 "RajcePublishing.c"
		}
#line 1379 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_result_ = _tmp15_;
#line 10216 "RajcePublishing.c"
	}
	goto __finally31;
	__catch31_spit_publishing_publishing_error:
	{
		GError* err = NULL;
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		err = _inner_error_;
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_inner_error_ = NULL;
#line 1383 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		result = NULL;
#line 1383 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_error_free0 (err);
#line 1383 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (name);
#line 1383 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return result;
#line 10234 "RajcePublishing.c"
	}
	__finally31:
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_g_free0 (name);
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		g_clear_error (&_inner_error_);
#line 1376 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		return NULL;
#line 10247 "RajcePublishing.c"
	}
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp19_ = errcode;
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp20_ = (gchar*) xmlNodeGetContent (_tmp19_);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp21_ = _tmp20_;
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp22_ = atoi (_tmp21_);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp23_ = _result_;
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp24_ = (gchar*) xmlNodeGetContent (_tmp23_);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp25_ = _tmp24_;
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp26_ = g_strdup_printf ("999 Rajce Error [%d]: %s", _tmp22_, _tmp25_);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp27_ = _tmp26_;
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp25_);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp21_);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	result = _tmp27_;
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (name);
#line 1385 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return result;
#line 10277 "RajcePublishing.c"
}


static void publishing_rajce_transaction_class_init (PublishingRajceTransactionClass * klass) {
#line 1353 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_transaction_parent_class = g_type_class_peek_parent (klass);
#line 10284 "RajcePublishing.c"
}


static void publishing_rajce_transaction_instance_init (PublishingRajceTransaction * self) {
}


/**
 * Generic REST transaction class.
 *
 * This class implements the generic logic for all REST transactions used
 * by the Rajce publishing plugin.
 */
GType publishing_rajce_transaction_get_type (void) {
	static volatile gsize publishing_rajce_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceTransaction), 0, (GInstanceInitFunc) publishing_rajce_transaction_instance_init, NULL };
		GType publishing_rajce_transaction_type_id;
		publishing_rajce_transaction_type_id = g_type_register_static (PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, "PublishingRajceTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_transaction_type_id__volatile, publishing_rajce_transaction_type_id);
	}
	return publishing_rajce_transaction_type_id__volatile;
}


PublishingRajceSessionLoginTransaction* publishing_rajce_session_login_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, const gchar* username, const gchar* token) {
	PublishingRajceSessionLoginTransaction* self = NULL;
	const gchar* _tmp0_ = NULL;
	PublishingRajceSession* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp3_ = NULL;
	const gchar* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	gchar* xml = NULL;
	gchar* _tmp6_ = NULL;
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (username != NULL, NULL);
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (token != NULL, NULL);
#line 1396 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = url;
#line 1396 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:1396: SessionLoginTransaction: URL: %s", _tmp0_);
#line 1397 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = session;
#line 1397 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = url;
#line 1397 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceSessionLoginTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp2_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1398 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = publishing_rajce_live_api_request_new ("login");
#line 1398 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp3_;
#line 1399 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "clientID", "RajceShotwellPlugin");
#line 1400 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "currentVersion", "1.1.1.1");
#line 1401 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = username;
#line 1401 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "login", _tmp4_);
#line 1402 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = token;
#line 1402 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "password", _tmp5_);
#line 1403 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1403 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp6_;
#line 1404 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10367 "RajcePublishing.c"
}


PublishingRajceSessionLoginTransaction* publishing_rajce_session_login_transaction_new (PublishingRajceSession* session, const gchar* url, const gchar* username, const gchar* token) {
#line 1394 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_session_login_transaction_construct (PUBLISHING_RAJCE_TYPE_SESSION_LOGIN_TRANSACTION, session, url, username, token);
#line 10374 "RajcePublishing.c"
}


static void publishing_rajce_session_login_transaction_class_init (PublishingRajceSessionLoginTransactionClass * klass) {
#line 1392 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_session_login_transaction_parent_class = g_type_class_peek_parent (klass);
#line 10381 "RajcePublishing.c"
}


static void publishing_rajce_session_login_transaction_instance_init (PublishingRajceSessionLoginTransaction * self) {
}


/**
 * Transaction used to implement the network login interaction.
 */
GType publishing_rajce_session_login_transaction_get_type (void) {
	static volatile gsize publishing_rajce_session_login_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_session_login_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceSessionLoginTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_session_login_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceSessionLoginTransaction), 0, (GInstanceInitFunc) publishing_rajce_session_login_transaction_instance_init, NULL };
		GType publishing_rajce_session_login_transaction_type_id;
		publishing_rajce_session_login_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceSessionLoginTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_session_login_transaction_type_id__volatile, publishing_rajce_session_login_transaction_type_id);
	}
	return publishing_rajce_session_login_transaction_type_id__volatile;
}


static void _vala_array_add9 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 10411 "RajcePublishing.c"
	}
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 10417 "RajcePublishing.c"
}


static void _vala_array_add10 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 10428 "RajcePublishing.c"
	}
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 10434 "RajcePublishing.c"
}


static void _vala_array_add11 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 10445 "RajcePublishing.c"
	}
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 10451 "RajcePublishing.c"
}


static void _vala_array_add12 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 10462 "RajcePublishing.c"
	}
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 10468 "RajcePublishing.c"
}


static void _vala_array_add13 (PublishingRajceArgItem*** array, int* length, int* size, PublishingRajceArgItem* value) {
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*length) == (*size)) {
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		*array = g_renew (PublishingRajceArgItem*, *array, (*size) + 1);
#line 10479 "RajcePublishing.c"
	}
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[(*length)++] = value;
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	(*array)[*length] = NULL;
#line 10485 "RajcePublishing.c"
}


PublishingRajceGetAlbumsTransaction* publishing_rajce_get_albums_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url) {
	PublishingRajceGetAlbumsTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp2_ = NULL;
	PublishingRajceSession* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	PublishingRajceArgItem** columns = NULL;
	PublishingRajceArgItem** _tmp6_ = NULL;
	gint columns_length1 = 0;
	gint _columns_size_ = 0;
	PublishingRajceArgItem** _tmp7_ = NULL;
	gint _tmp7__length1 = 0;
	PublishingRajceArgItem* _tmp8_ = NULL;
	PublishingRajceArgItem** _tmp9_ = NULL;
	gint _tmp9__length1 = 0;
	PublishingRajceArgItem* _tmp10_ = NULL;
	PublishingRajceArgItem** _tmp11_ = NULL;
	gint _tmp11__length1 = 0;
	PublishingRajceArgItem* _tmp12_ = NULL;
	PublishingRajceArgItem** _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	PublishingRajceArgItem* _tmp14_ = NULL;
	PublishingRajceArgItem** _tmp15_ = NULL;
	gint _tmp15__length1 = 0;
	PublishingRajceArgItem* _tmp16_ = NULL;
	PublishingRajceArgItem** _tmp17_ = NULL;
	gint _tmp17__length1 = 0;
	gchar* xml = NULL;
	gchar* _tmp18_ = NULL;
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1415 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1415 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = url;
#line 1415 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceGetAlbumsTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1416 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_live_api_request_new ("getAlbumList");
#line 1416 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp2_;
#line 1417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = session;
#line 1417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_get_usertoken (_tmp3_);
#line 1417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 1417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "token", _tmp5_);
#line 1417 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp5_);
#line 1418 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = g_new0 (PublishingRajceArgItem*, 0 + 1);
#line 1418 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	columns = _tmp6_;
#line 1418 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	columns_length1 = 0;
#line 1418 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_columns_size_ = columns_length1;
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = columns;
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7__length1 = columns_length1;
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = publishing_rajce_arg_item_new ("column", "viewCount");
#line 1419 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add9 (&columns, &columns_length1, &_columns_size_, _tmp8_);
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = columns;
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9__length1 = columns_length1;
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = publishing_rajce_arg_item_new ("column", "isFavourite");
#line 1420 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add10 (&columns, &columns_length1, &_columns_size_, _tmp10_);
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = columns;
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11__length1 = columns_length1;
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = publishing_rajce_arg_item_new ("column", "descriptionHtml");
#line 1421 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add11 (&columns, &columns_length1, &_columns_size_, _tmp12_);
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13_ = columns;
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13__length1 = columns_length1;
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp14_ = publishing_rajce_arg_item_new ("column", "coverPhotoID");
#line 1422 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add12 (&columns, &columns_length1, &_columns_size_, _tmp14_);
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp15_ = columns;
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp15__length1 = columns_length1;
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp16_ = publishing_rajce_arg_item_new ("column", "localPath");
#line 1423 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_vala_array_add13 (&columns, &columns_length1, &_columns_size_, _tmp16_);
#line 1424 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp17_ = columns;
#line 1424 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp17__length1 = columns_length1;
#line 1424 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParamNode (req, "columns", _tmp17_, _tmp17__length1);
#line 1425 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp18_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1425 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp18_;
#line 1426 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	columns = (_vala_array_free (columns, columns_length1, (GDestroyNotify) publishing_rajce_arg_item_unref), NULL);
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10613 "RajcePublishing.c"
}


PublishingRajceGetAlbumsTransaction* publishing_rajce_get_albums_transaction_new (PublishingRajceSession* session, const gchar* url) {
#line 1413 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_get_albums_transaction_construct (PUBLISHING_RAJCE_TYPE_GET_ALBUMS_TRANSACTION, session, url);
#line 10620 "RajcePublishing.c"
}


static void publishing_rajce_get_albums_transaction_class_init (PublishingRajceGetAlbumsTransactionClass * klass) {
#line 1411 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_get_albums_transaction_parent_class = g_type_class_peek_parent (klass);
#line 10627 "RajcePublishing.c"
}


static void publishing_rajce_get_albums_transaction_instance_init (PublishingRajceGetAlbumsTransaction * self) {
}


/**
 * Transaction used to implement the get albums interaction.
 */
GType publishing_rajce_get_albums_transaction_get_type (void) {
	static volatile gsize publishing_rajce_get_albums_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_get_albums_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceGetAlbumsTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_get_albums_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceGetAlbumsTransaction), 0, (GInstanceInitFunc) publishing_rajce_get_albums_transaction_instance_init, NULL };
		GType publishing_rajce_get_albums_transaction_type_id;
		publishing_rajce_get_albums_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceGetAlbumsTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_get_albums_transaction_type_id__volatile, publishing_rajce_get_albums_transaction_type_id);
	}
	return publishing_rajce_get_albums_transaction_type_id__volatile;
}


PublishingRajceCreateAlbumTransaction* publishing_rajce_create_album_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, const gchar* albumName, gboolean hidden) {
	PublishingRajceCreateAlbumTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp2_ = NULL;
	PublishingRajceSession* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	gboolean _tmp7_ = FALSE;
	gchar* xml = NULL;
	gchar* _tmp8_ = NULL;
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (albumName != NULL, NULL);
#line 1437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = url;
#line 1437 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceCreateAlbumTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1438 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_live_api_request_new ("createAlbum");
#line 1438 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp2_;
#line 1439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = session;
#line 1439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_get_usertoken (_tmp3_);
#line 1439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 1439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "token", _tmp5_);
#line 1439 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp5_);
#line 1440 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = albumName;
#line 1440 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "albumName", _tmp6_);
#line 1441 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "albumDescription", "");
#line 1442 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = hidden;
#line 1442 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParamBool (req, "albumVisible", !_tmp7_);
#line 1443 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1443 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp8_;
#line 1444 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10711 "RajcePublishing.c"
}


PublishingRajceCreateAlbumTransaction* publishing_rajce_create_album_transaction_new (PublishingRajceSession* session, const gchar* url, const gchar* albumName, gboolean hidden) {
#line 1435 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_create_album_transaction_construct (PUBLISHING_RAJCE_TYPE_CREATE_ALBUM_TRANSACTION, session, url, albumName, hidden);
#line 10718 "RajcePublishing.c"
}


static void publishing_rajce_create_album_transaction_class_init (PublishingRajceCreateAlbumTransactionClass * klass) {
#line 1433 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_create_album_transaction_parent_class = g_type_class_peek_parent (klass);
#line 10725 "RajcePublishing.c"
}


static void publishing_rajce_create_album_transaction_instance_init (PublishingRajceCreateAlbumTransaction * self) {
}


/**
 * Transaction used to implement the create album interaction.
 */
GType publishing_rajce_create_album_transaction_get_type (void) {
	static volatile gsize publishing_rajce_create_album_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_create_album_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceCreateAlbumTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_create_album_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceCreateAlbumTransaction), 0, (GInstanceInitFunc) publishing_rajce_create_album_transaction_instance_init, NULL };
		GType publishing_rajce_create_album_transaction_type_id;
		publishing_rajce_create_album_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceCreateAlbumTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_create_album_transaction_type_id__volatile, publishing_rajce_create_album_transaction_type_id);
	}
	return publishing_rajce_create_album_transaction_type_id__volatile;
}


PublishingRajceOpenAlbumTransaction* publishing_rajce_open_album_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, gint albumID) {
	PublishingRajceOpenAlbumTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp2_ = NULL;
	PublishingRajceSession* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gint _tmp6_ = 0;
	gchar* xml = NULL;
	gchar* _tmp7_ = NULL;
#line 1453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1455 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1455 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = url;
#line 1455 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceOpenAlbumTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1456 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_live_api_request_new ("openAlbum");
#line 1456 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp2_;
#line 1457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = session;
#line 1457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_get_usertoken (_tmp3_);
#line 1457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 1457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "token", _tmp5_);
#line 1457 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp5_);
#line 1458 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = albumID;
#line 1458 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParamInt (req, "albumID", _tmp6_);
#line 1459 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1459 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp7_;
#line 1460 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10800 "RajcePublishing.c"
}


PublishingRajceOpenAlbumTransaction* publishing_rajce_open_album_transaction_new (PublishingRajceSession* session, const gchar* url, gint albumID) {
#line 1453 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_open_album_transaction_construct (PUBLISHING_RAJCE_TYPE_OPEN_ALBUM_TRANSACTION, session, url, albumID);
#line 10807 "RajcePublishing.c"
}


static void publishing_rajce_open_album_transaction_class_init (PublishingRajceOpenAlbumTransactionClass * klass) {
#line 1451 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_open_album_transaction_parent_class = g_type_class_peek_parent (klass);
#line 10814 "RajcePublishing.c"
}


static void publishing_rajce_open_album_transaction_instance_init (PublishingRajceOpenAlbumTransaction * self) {
}


/**
 * Transaction used to implement the open album interaction.
 */
GType publishing_rajce_open_album_transaction_get_type (void) {
	static volatile gsize publishing_rajce_open_album_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_open_album_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceOpenAlbumTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_open_album_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceOpenAlbumTransaction), 0, (GInstanceInitFunc) publishing_rajce_open_album_transaction_instance_init, NULL };
		GType publishing_rajce_open_album_transaction_type_id;
		publishing_rajce_open_album_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceOpenAlbumTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_open_album_transaction_type_id__volatile, publishing_rajce_open_album_transaction_type_id);
	}
	return publishing_rajce_open_album_transaction_type_id__volatile;
}


PublishingRajceGetAlbumUrlTransaction* publishing_rajce_get_album_url_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url) {
	PublishingRajceGetAlbumUrlTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp2_ = NULL;
	PublishingRajceSession* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	PublishingRajceSession* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* xml = NULL;
	gchar* _tmp9_ = NULL;
#line 1469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1471 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1471 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = url;
#line 1471 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceGetAlbumUrlTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_live_api_request_new ("getAlbumUrl");
#line 1472 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp2_;
#line 1473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = session;
#line 1473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_get_usertoken (_tmp3_);
#line 1473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 1473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "token", _tmp5_);
#line 1473 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp5_);
#line 1474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = session;
#line 1474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = publishing_rajce_session_get_albumtoken (_tmp6_);
#line 1474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = _tmp7_;
#line 1474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "albumToken", _tmp8_);
#line 1474 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp8_);
#line 1475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1475 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp9_;
#line 1476 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10897 "RajcePublishing.c"
}


PublishingRajceGetAlbumUrlTransaction* publishing_rajce_get_album_url_transaction_new (PublishingRajceSession* session, const gchar* url) {
#line 1469 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_get_album_url_transaction_construct (PUBLISHING_RAJCE_TYPE_GET_ALBUM_URL_TRANSACTION, session, url);
#line 10904 "RajcePublishing.c"
}


static void publishing_rajce_get_album_url_transaction_class_init (PublishingRajceGetAlbumUrlTransactionClass * klass) {
#line 1467 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_get_album_url_transaction_parent_class = g_type_class_peek_parent (klass);
#line 10911 "RajcePublishing.c"
}


static void publishing_rajce_get_album_url_transaction_instance_init (PublishingRajceGetAlbumUrlTransaction * self) {
}


/**
 * Transaction used to implement the close album interaction.
 */
GType publishing_rajce_get_album_url_transaction_get_type (void) {
	static volatile gsize publishing_rajce_get_album_url_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_get_album_url_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceGetAlbumUrlTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_get_album_url_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceGetAlbumUrlTransaction), 0, (GInstanceInitFunc) publishing_rajce_get_album_url_transaction_instance_init, NULL };
		GType publishing_rajce_get_album_url_transaction_type_id;
		publishing_rajce_get_album_url_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceGetAlbumUrlTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_get_album_url_transaction_type_id__volatile, publishing_rajce_get_album_url_transaction_type_id);
	}
	return publishing_rajce_get_album_url_transaction_type_id__volatile;
}


PublishingRajceCloseAlbumTransaction* publishing_rajce_close_album_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url) {
	PublishingRajceCloseAlbumTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp2_ = NULL;
	PublishingRajceSession* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	PublishingRajceSession* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* xml = NULL;
	gchar* _tmp9_ = NULL;
#line 1485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1487 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1487 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = url;
#line 1487 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceCloseAlbumTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1488 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_live_api_request_new ("closeAlbum");
#line 1488 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp2_;
#line 1489 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = session;
#line 1489 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_get_usertoken (_tmp3_);
#line 1489 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 1489 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "token", _tmp5_);
#line 1489 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp5_);
#line 1490 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = session;
#line 1490 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = publishing_rajce_session_get_albumtoken (_tmp6_);
#line 1490 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = _tmp7_;
#line 1490 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "albumToken", _tmp8_);
#line 1490 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp8_);
#line 1491 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1491 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp9_;
#line 1492 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 10994 "RajcePublishing.c"
}


PublishingRajceCloseAlbumTransaction* publishing_rajce_close_album_transaction_new (PublishingRajceSession* session, const gchar* url) {
#line 1485 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_close_album_transaction_construct (PUBLISHING_RAJCE_TYPE_CLOSE_ALBUM_TRANSACTION, session, url);
#line 11001 "RajcePublishing.c"
}


static void publishing_rajce_close_album_transaction_class_init (PublishingRajceCloseAlbumTransactionClass * klass) {
#line 1483 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_close_album_transaction_parent_class = g_type_class_peek_parent (klass);
#line 11008 "RajcePublishing.c"
}


static void publishing_rajce_close_album_transaction_instance_init (PublishingRajceCloseAlbumTransaction * self) {
}


/**
 * Transaction used to implement the close album interaction.
 */
GType publishing_rajce_close_album_transaction_get_type (void) {
	static volatile gsize publishing_rajce_close_album_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_close_album_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceCloseAlbumTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_close_album_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceCloseAlbumTransaction), 0, (GInstanceInitFunc) publishing_rajce_close_album_transaction_instance_init, NULL };
		GType publishing_rajce_close_album_transaction_type_id;
		publishing_rajce_close_album_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceCloseAlbumTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_close_album_transaction_type_id__volatile, publishing_rajce_close_album_transaction_type_id);
	}
	return publishing_rajce_close_album_transaction_type_id__volatile;
}


PublishingRajceGetCategoriesTransaction* publishing_rajce_get_categories_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url) {
	PublishingRajceGetCategoriesTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp2_ = NULL;
	PublishingRajceSession* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* xml = NULL;
	gchar* _tmp6_ = NULL;
#line 1501 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1501 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1503 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1503 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = url;
#line 1503 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceGetCategoriesTransaction*) publishing_rest_support_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, PUBLISHING_REST_SUPPORT_HTTP_METHOD_POST);
#line 1504 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = publishing_rajce_live_api_request_new ("getCategories");
#line 1504 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp2_;
#line 1505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = session;
#line 1505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = publishing_rajce_session_get_usertoken (_tmp3_);
#line 1505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp5_ = _tmp4_;
#line 1505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (req, "token", _tmp5_);
#line 1505 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp5_);
#line 1506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = publishing_rajce_live_api_request_Params2XmlString (req, TRUE);
#line 1506 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp6_;
#line 1507 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1501 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1501 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1501 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 11078 "RajcePublishing.c"
}


PublishingRajceGetCategoriesTransaction* publishing_rajce_get_categories_transaction_new (PublishingRajceSession* session, const gchar* url) {
#line 1501 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_get_categories_transaction_construct (PUBLISHING_RAJCE_TYPE_GET_CATEGORIES_TRANSACTION, session, url);
#line 11085 "RajcePublishing.c"
}


static void publishing_rajce_get_categories_transaction_class_init (PublishingRajceGetCategoriesTransactionClass * klass) {
#line 1499 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_get_categories_transaction_parent_class = g_type_class_peek_parent (klass);
#line 11092 "RajcePublishing.c"
}


static void publishing_rajce_get_categories_transaction_instance_init (PublishingRajceGetCategoriesTransaction * self) {
}


/**
 * Transaction used to implement the get categories interaction.
 */
GType publishing_rajce_get_categories_transaction_get_type (void) {
	static volatile gsize publishing_rajce_get_categories_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_get_categories_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceGetCategoriesTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_get_categories_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceGetCategoriesTransaction), 0, (GInstanceInitFunc) publishing_rajce_get_categories_transaction_instance_init, NULL };
		GType publishing_rajce_get_categories_transaction_type_id;
		publishing_rajce_get_categories_transaction_type_id = g_type_register_static (PUBLISHING_RAJCE_TYPE_TRANSACTION, "PublishingRajceGetCategoriesTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_get_categories_transaction_type_id__volatile, publishing_rajce_get_categories_transaction_type_id);
	}
	return publishing_rajce_get_categories_transaction_type_id__volatile;
}


static void _g_free0_ (gpointer var) {
#line 1543 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	var = (g_free (var), NULL);
#line 11118 "RajcePublishing.c"
}


PublishingRajceAddPhotoTransaction* publishing_rajce_add_photo_transaction_construct (GType object_type, PublishingRajceSession* session, const gchar* url, PublishingRajcePublishingParameters* parameters, SpitPublishingPublishable* publishable) {
	PublishingRajceAddPhotoTransaction* self = NULL;
	PublishingRajceSession* _tmp0_ = NULL;
	SpitPublishingPublishable* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	PublishingRajcePublishingParameters* _tmp3_ = NULL;
	PublishingRajcePublishingParameters* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	PublishingRajcePublishingParameters* _tmp6_ = NULL;
	gint* _tmp7_ = NULL;
	SpitPublishingPublishable* _tmp8_ = NULL;
	GFile* _tmp9_ = NULL;
	GFile* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	PublishingRajcePublishingParameters* _tmp13_ = NULL;
	const gchar* _tmp14_ = NULL;
	gchar* basename = NULL;
	SpitPublishingPublishable* _tmp15_ = NULL;
	gchar* _tmp16_ = NULL;
	gchar* comment = NULL;
	SpitPublishingPublishable* _tmp17_ = NULL;
	gchar* _tmp18_ = NULL;
	gchar* pubname = NULL;
	SpitPublishingPublishable* _tmp19_ = NULL;
	gchar* _tmp20_ = NULL;
	gint width = 0;
	PublishingRajceSession* _tmp21_ = NULL;
	gint _tmp22_ = 0;
	gint height = 0;
	PublishingRajceSession* _tmp23_ = NULL;
	gint _tmp24_ = 0;
	PublishingRajceLiveApiRequest* req = NULL;
	PublishingRajceLiveApiRequest* _tmp25_ = NULL;
	PublishingRajceLiveApiRequest* _tmp26_ = NULL;
	PublishingRajceSession* _tmp27_ = NULL;
	gchar* _tmp28_ = NULL;
	gchar* _tmp29_ = NULL;
	PublishingRajceLiveApiRequest* _tmp30_ = NULL;
	gint _tmp31_ = 0;
	PublishingRajceLiveApiRequest* _tmp32_ = NULL;
	gint _tmp33_ = 0;
	PublishingRajceLiveApiRequest* _tmp34_ = NULL;
	PublishingRajceSession* _tmp35_ = NULL;
	gchar* _tmp36_ = NULL;
	gchar* _tmp37_ = NULL;
	PublishingRajceLiveApiRequest* _tmp38_ = NULL;
	const gchar* _tmp39_ = NULL;
	PublishingRajceLiveApiRequest* _tmp40_ = NULL;
	const gchar* _tmp41_ = NULL;
	const gchar* _tmp42_ = NULL;
	const gchar* _tmp43_ = NULL;
	PublishingRajceLiveApiRequest* _tmp45_ = NULL;
	gchar* xml = NULL;
	PublishingRajceLiveApiRequest* _tmp46_ = NULL;
	gchar* _tmp47_ = NULL;
	GHashTable* disposition_table = NULL;
	GHashFunc _tmp48_ = NULL;
	GEqualFunc _tmp49_ = NULL;
	GHashTable* _tmp50_ = NULL;
	gchar* _tmp51_ = NULL;
	gchar* _tmp52_ = NULL;
	gchar* _tmp53_ = NULL;
	const gchar* _tmp54_ = NULL;
	gchar* _tmp55_ = NULL;
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_SESSION (session), NULL);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (url != NULL, NULL);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (PUBLISHING_RAJCE_IS_PUBLISHING_PARAMETERS (parameters), NULL);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_return_val_if_fail (SPIT_PUBLISHING_IS_PUBLISHABLE (publishable), NULL);
#line 1520 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp0_ = session;
#line 1520 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp1_ = publishable;
#line 1520 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp2_ = url;
#line 1520 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = (PublishingRajceAddPhotoTransaction*) publishing_rest_support_upload_transaction_construct_with_endpoint_url (object_type, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, PUBLISHING_REST_SUPPORT_TYPE_SESSION, PublishingRESTSupportSession), _tmp1_, _tmp2_);
#line 1521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp3_ = parameters;
#line 1521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp4_ = _publishing_rajce_publishing_parameters_ref0 (_tmp3_);
#line 1521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_publishing_parameters_unref0 (self->priv->parameters);
#line 1521 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->parameters = _tmp4_;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp6_ = parameters;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp7_ = _tmp6_->album_id;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if ((*_tmp7_) > 0) {
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = "";
#line 11219 "RajcePublishing.c"
	} else {
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp5_ = " new";
#line 11223 "RajcePublishing.c"
	}
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp8_ = publishable;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp9_ = spit_publishing_publishable_get_serialized_file (_tmp8_);
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp10_ = _tmp9_;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp11_ = g_file_get_basename (_tmp10_);
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp12_ = _tmp11_;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp13_ = parameters;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp14_ = _tmp13_->album_name;
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_debug ("RajcePublishing.vala:1523: RajcePlugin: Uploading photo %s to%s album " \
"%s", _tmp12_, _tmp5_, _tmp14_);
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp12_);
#line 1523 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_object_unref0 (_tmp10_);
#line 1525 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp15_ = publishable;
#line 1525 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp16_ = spit_publishing_publishable_get_param_string (_tmp15_, SPIT_PUBLISHING_PUBLISHABLE_PARAM_STRING_BASENAME);
#line 1525 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	basename = _tmp16_;
#line 1526 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp17_ = publishable;
#line 1526 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp18_ = spit_publishing_publishable_get_param_string (_tmp17_, SPIT_PUBLISHING_PUBLISHABLE_PARAM_STRING_COMMENT);
#line 1526 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	comment = _tmp18_;
#line 1527 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp19_ = publishable;
#line 1527 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp20_ = spit_publishing_publishable_get_publishing_name (_tmp19_);
#line 1527 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	pubname = _tmp20_;
#line 1529 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp21_ = session;
#line 1529 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp22_ = publishing_rajce_session_get_maxsize (_tmp21_);
#line 1529 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	width = _tmp22_;
#line 1530 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp23_ = session;
#line 1530 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp24_ = publishing_rajce_session_get_maxsize (_tmp23_);
#line 1530 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	height = _tmp24_;
#line 1532 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp25_ = publishing_rajce_live_api_request_new ("addPhoto");
#line 1532 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	req = _tmp25_;
#line 1533 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp26_ = req;
#line 1533 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp27_ = session;
#line 1533 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp28_ = publishing_rajce_session_get_usertoken (_tmp27_);
#line 1533 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp29_ = _tmp28_;
#line 1533 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (_tmp26_, "token", _tmp29_);
#line 1533 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp29_);
#line 1534 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp30_ = req;
#line 1534 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp31_ = width;
#line 1534 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParamInt (_tmp30_, "width", _tmp31_);
#line 1535 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp32_ = req;
#line 1535 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp33_ = height;
#line 1535 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParamInt (_tmp32_, "height", _tmp33_);
#line 1536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp34_ = req;
#line 1536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp35_ = session;
#line 1536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp36_ = publishing_rajce_session_get_albumtoken (_tmp35_);
#line 1536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp37_ = _tmp36_;
#line 1536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (_tmp34_, "albumToken", _tmp37_);
#line 1536 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (_tmp37_);
#line 1537 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp38_ = req;
#line 1537 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp39_ = pubname;
#line 1537 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (_tmp38_, "photoName", _tmp39_);
#line 1538 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp40_ = req;
#line 1538 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp41_ = basename;
#line 1538 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (_tmp40_, "fullFileName", _tmp41_);
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp43_ = comment;
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	if (_tmp43_ != NULL) {
#line 11331 "RajcePublishing.c"
		const gchar* _tmp44_ = NULL;
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp44_ = comment;
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp42_ = _tmp44_;
#line 11337 "RajcePublishing.c"
	} else {
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
		_tmp42_ = "";
#line 11341 "RajcePublishing.c"
	}
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp45_ = req;
#line 1539 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_live_api_request_AddParam (_tmp45_, "description", _tmp42_);
#line 1540 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp46_ = req;
#line 1540 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp47_ = publishing_rajce_live_api_request_Params2XmlString (_tmp46_, FALSE);
#line 1540 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	xml = _tmp47_;
#line 1541 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_transaction_add_argument (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_TRANSACTION, PublishingRESTSupportTransaction), "data", xml);
#line 1543 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp48_ = g_str_hash;
#line 1543 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp49_ = g_str_equal;
#line 1543 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp50_ = g_hash_table_new_full (_tmp48_, _tmp49_, _g_free0_, _g_free0_);
#line 1543 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	disposition_table = _tmp50_;
#line 1544 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp51_ = g_strdup ("name");
#line 1544 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp52_ = g_strdup ("photo");
#line 1544 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_hash_table_insert (disposition_table, _tmp51_, _tmp52_);
#line 1545 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp53_ = g_strdup ("filename");
#line 1545 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp54_ = basename;
#line 1545 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_tmp55_ = soup_uri_encode (_tmp54_, NULL);
#line 1545 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_hash_table_insert (disposition_table, _tmp53_, _tmp55_);
#line 1546 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rest_support_upload_transaction_set_binary_disposition_table (G_TYPE_CHECK_INSTANCE_CAST (self, PUBLISHING_REST_SUPPORT_TYPE_UPLOAD_TRANSACTION, PublishingRESTSupportUploadTransaction), disposition_table);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_hash_table_unref0 (disposition_table);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (xml);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_live_api_request_unref0 (req);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (pubname);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (comment);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_g_free0 (basename);
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return self;
#line 11393 "RajcePublishing.c"
}


PublishingRajceAddPhotoTransaction* publishing_rajce_add_photo_transaction_new (PublishingRajceSession* session, const gchar* url, PublishingRajcePublishingParameters* parameters, SpitPublishingPublishable* publishable) {
#line 1518 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	return publishing_rajce_add_photo_transaction_construct (PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION, session, url, parameters, publishable);
#line 11400 "RajcePublishing.c"
}


static void publishing_rajce_add_photo_transaction_class_init (PublishingRajceAddPhotoTransactionClass * klass) {
#line 1514 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	publishing_rajce_add_photo_transaction_parent_class = g_type_class_peek_parent (klass);
#line 1514 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	((PublishingRESTSupportTransactionClass *) klass)->finalize = publishing_rajce_add_photo_transaction_finalize;
#line 1514 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	g_type_class_add_private (klass, sizeof (PublishingRajceAddPhotoTransactionPrivate));
#line 11411 "RajcePublishing.c"
}


static void publishing_rajce_add_photo_transaction_instance_init (PublishingRajceAddPhotoTransaction * self) {
#line 1514 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv = PUBLISHING_RAJCE_ADD_PHOTO_TRANSACTION_GET_PRIVATE (self);
#line 1516 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self->priv->parameters = NULL;
#line 11420 "RajcePublishing.c"
}


static void publishing_rajce_add_photo_transaction_finalize (PublishingRESTSupportTransaction* obj) {
	PublishingRajceAddPhotoTransaction * self;
#line 1514 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, PUBLISHING_RAJCE_TYPE_ADD_PHOTO_TRANSACTION, PublishingRajceAddPhotoTransaction);
#line 1516 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	_publishing_rajce_publishing_parameters_unref0 (self->priv->parameters);
#line 1514 "/home/jens/Source/shotwell/plugins/shotwell-publishing-extras/RajcePublishing.vala"
	PUBLISHING_REST_SUPPORT_TRANSACTION_CLASS (publishing_rajce_add_photo_transaction_parent_class)->finalize (obj);
#line 11432 "RajcePublishing.c"
}


/**
 * Transaction used to implement the upload photo.
 */
GType publishing_rajce_add_photo_transaction_get_type (void) {
	static volatile gsize publishing_rajce_add_photo_transaction_type_id__volatile = 0;
	if (g_once_init_enter (&publishing_rajce_add_photo_transaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PublishingRajceAddPhotoTransactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) publishing_rajce_add_photo_transaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PublishingRajceAddPhotoTransaction), 0, (GInstanceInitFunc) publishing_rajce_add_photo_transaction_instance_init, NULL };
		GType publishing_rajce_add_photo_transaction_type_id;
		publishing_rajce_add_photo_transaction_type_id = g_type_register_static (PUBLISHING_REST_SUPPORT_TYPE_UPLOAD_TRANSACTION, "PublishingRajceAddPhotoTransaction", &g_define_type_info, 0);
		g_once_init_leave (&publishing_rajce_add_photo_transaction_type_id__volatile, publishing_rajce_add_photo_transaction_type_id);
	}
	return publishing_rajce_add_photo_transaction_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



